#include "stdafx.h"

#include "Gfx/Buffers_TriList.h"
#include "Gfx/Gfx_Tex.h"
#include "Gfx/VertexTypes_XyzUvRgb.h"

#include "Effects_Skydome.h"

namespace Effects
{
	Skydome::Skydome (const SkydomeParams * params)
		: buffer (NULL)
	{
		const int segmentCount = 36;
		const int ringsCount = 72;
		const int vertCount = segmentCount * (ringsCount * 6 - 3);

		Gfx::VertexTypes::XyzUvRgb * verts = aah_new Gfx::VertexTypes::XyzUvRgb [vertCount];
		Gfx::VertexTypes::XyzUvRgb * current = verts;

		for (int segmentIdx = 0; segmentIdx < segmentCount; ++segmentIdx)
		{
			Vec4 rgba (1.0f, 0.0f, 0.0f, 1.0f);

			SetVertex (current, segmentIdx, segmentCount, 0, ringsCount); ++current;
			SetVertex (current, segmentIdx, segmentCount, 1, ringsCount); ++current;
			SetVertex (current, segmentIdx + 1, segmentCount, 1, ringsCount); ++current;

			for (int ringIdx = 1; ringIdx < ringsCount; ++ringIdx)
			{
				SetVertex (current, segmentIdx, segmentCount, ringIdx, ringsCount); ++current;
				SetVertex (current, segmentIdx + 1, segmentCount, ringIdx, ringsCount); ++current;
				SetVertex (current, segmentIdx, segmentCount, ringIdx + 1, ringsCount); ++current;				

				SetVertex (current, segmentIdx + 1, segmentCount, ringIdx, ringsCount); ++current;
				SetVertex (current, segmentIdx, segmentCount, ringIdx + 1, ringsCount); ++current;				
				SetVertex (current, segmentIdx + 1, segmentCount, ringIdx + 1, ringsCount); ++current;
			}
		}

		assert (current == verts + vertCount);

		buffer = aah_new Gfx::Buffers::TriList ();
		buffer->SetVerts (verts, vertCount);

		delete [] verts;

		tex = aah_new Gfx::GfxTex ("gfx/white.png");
	}

	Skydome::~Skydome ()
	{
		delete tex;
		delete buffer;
	}

	void Skydome::Render (Vec3 cameraPos)
	{
		glDisable (GL_DEPTH_TEST);
		glDisable (GL_CULL_FACE);

		tex->Bind ();

		glPushMatrix ();
		glTranslatef (cameraPos.x, cameraPos.y, cameraPos.z);
		buffer->Render ();
		glPopMatrix ();

		tex->Unbind ();

		glEnable (GL_DEPTH_TEST);
		glEnable (GL_CULL_FACE);
	}

	void Skydome::SetVertex (Gfx::VertexTypes::XyzUvRgb * vert, int segIdx, int segCount, int ringIdx, int ringCount)
	{
		Vec4 xyz (0.0f, 1.0f, 0.0f, 0.0f);
		Vec2 uv (0.0f, 0.0f);

		Mat44 mat;
		mat = glm::rotate (mat, 180.0f * ringIdx / ringCount, Vec3 (0.0f, 0.0f, 1.0f));
		mat = glm::rotate (mat, 360.0f * segIdx / segCount, Vec3 (0.0f, 1.0f, 0.0f));
		xyz = xyz * mat;

		Vec3 xyz3 (xyz.x, xyz.y, xyz.z);
		Vec4 rgba = CalcColour (xyz3);

		vert->Set (xyz3, uv, rgba);
	}

	Vec4 Skydome::CalcColour (Vec3 dir)
	{
		Vec4 rgba (94 / 255.0, 148 / 255.0f, 201 / 255.0f, 1.0f);

		rgba = ApplyRange (rgba, dir.y, 0.0f, 0.2f, Int4 (255, 64, 0, 255), Int4 (255, 255, 0, 0));
		rgba = ApplyRange (rgba, -dir.x, -0.2f, 0.3f, Int4 (94, 148, 201, 0), Int4 (94, 148, 201, 255));
		rgba = ApplyRange (rgba, dir.y, 0.1f, 0.4f, Int4 (208, 221, 224, 192), Int4 (208, 221, 224, 0));
		rgba = ApplyRange (rgba, -dir.x, -0.8f, 0.9f, Int4 (94, 148, 201, 0), Int4 (32, 50, 66, 80));
		rgba = ApplyRange (rgba, dir.y, 0.2f, 0.8f, Int4 (0, 0, 114, 0), Int4 (0, 0, 114, 64));

		return rgba;
	}

	Vec4 Skydome::ApplyRange (Vec4 rgba, float weight, float min, float max, Int4 minRgba, Int4 maxRgba)
	{
		Vec4 minRgbaf ((float) minRgba.r / 255.0f, (float) minRgba.g / 255.0f, (float) minRgba.b / 255.0f, (float) minRgba.a / 255.0f);
		Vec4 maxRgbaf ((float) maxRgba.r / 255.0f, (float) maxRgba.g / 255.0f, (float) maxRgba.b / 255.0f, (float) maxRgba.a / 255.0f);

		float v = (weight - min) / (max - min);
		v = glm::min (v, 1.0f);
		v = glm::max (v, 0.0f);
		
		Vec4 apply = minRgbaf * (1.0f - v) + maxRgbaf * v;
		rgba.r = rgba.r * (1.0f - apply.a) + apply.r * apply.a;
		rgba.g = rgba.g * (1.0f - apply.a) + apply.g * apply.a;
		rgba.b = rgba.b * (1.0f - apply.a) + apply.b * apply.a;

		return rgba;
	}
}
