#include "stdafx.h"

#include "Collision/Collision_Pick.h"
#include "Collision/Collision_Segment_Room.h"
#include "Collision/Collision_Result.h"
#include "Gfx/Gfx_Viewport.h"
#include "Input/Input_Keys.h"
#include "Input/Input_InputMngr.h"
#include "Input/Input_Mouse.h"
#include "Particles/Particles_BillboardMngr.h"
#include "SceneHelper/SceneHelper_Helper.h"
#include "SceneSim/Query_Cell.h"
#include "SceneSim/SceneSim_Column.h"
#include "SceneSim/SceneSim_ColumnCells.h"
#include "SceneSim/SceneSim_Room.h"
#include "SceneSim/SceneSim_World.h"

#include "../ClientEnts/ClientEnts_UpdateData.h"
#include "../Layers/Layers_All.h"
#include "../LayerPlaying/LayerPlaying_WorldSyncMngr.h"
#include "../LayerPlaying/LayerPlaying_PlayingDialog.h"
#include "../Session/Session_Settings.h"
#include "../Tiny/Tiny_TinyWorlds.h"

#include "Ents_Player.h"
#include "Ents_PlayerOsd.h"

namespace Ents
{
	Player::Player (bool isEditing, CellMesh::Mngr * meshes, Layers::All * all)
		: Ents::CMO_Box (0),
		isCrouching (false),
		cellTypeSelected (1),
		hasAxe (false),
		hasHeatShield (false),
		hasIceShield (false),
		hasBelt (false),
		hasTeleporter (false),
		hasSeeds (false),
		hasBigAxe (false),
		artifactCount (0),
		all (all),
		isDead (false),
		isInit (false),
		isCameraDone (false)
	{
		// TODO:
		//clientMoveSpeed = Settings::Get ()->lua->ReadFloat ("settings/worlds/default/physics/player/move_speed");
		//clientTurnSpeed = Settings::Get ()->lua->ReadFloat ("settings/worlds/default/physics/player/turn_speed");
		//clientJumpVel = Settings::Get ()->lua->ReadFloat ("settings/worlds/default/physics/player/jump_vel");
		//clientGravity = Settings::Get ()->lua->ReadFloat ("settings/worlds/default/physics/player/gravity");
		clientMoveSpeed = 3.0f * 1.0f;
		clientTurnSpeed = 0.07f;
		clientJumpVel = 6.0f * 1.0f;
		clientGravity = -0.2f * 1.0f;
		//clientSprayBullets = false;//Settings::Get ()->lua->ReadBool ("settings/worlds/default/physics/player/spray_bullets");

		//hasAxe = true;
		//clientJumpVel = 7.3f;
		//clientJumpVel = 8.5f;
		//hasBelt = true;
		//hasIceShield = true;
		//hasHeatShield = true;
		//hasTeleporter = true;
		//hasSeeds = true;
		//hasBigAxe = true;

		osd = aah_new PlayerOsd (this, meshes);

		state = (isEditing) ? S_EDITING : S_PLAYING;
	}

	Player::~Player ()
	{
		delete osd;
		delete pickResult;
	}

	void Player::Init (Int3 columnId, Vec3 colRelPos)
	{
		homeOnlyColumnId = Int3 (1, 0, 1);
		homeOnlyColumnPos = Vec3 (0.0f, 128.0f, 0.0f);

		coord.MoveTo (columnId, colRelPos);
		orient.SetYpr (Vec3 (-30.0f, 0.0f, 0.0f));
		
		motion.SetSize (Vec3 (0.5f, 1.75f, 0.5f));

		//isUnsafePos = true;

		canJump = false;
		justJumped = false;
		isCrouching = false;

		pickResult = aah_new Collision::Result ();

		isInit = true;
	}

	void Player::Update (ClientEnts::UpdateData * update, bool allowUserInput)
	{
		if (!isInit)
		{
			return;
		}

		if (update->sim->IsRoomLoaded (Text ("hub")))
		{
			SceneSim::Room * room = update->sim->GetRoom (Text ("hub"));

			{
				prevPos = coord;
				renderEye = coord;
				Client_UpdateInput (update, room, allowUserInput && (!isDead));
			}

			if (!isDead && isCameraDone)
			{
				UpdatePicking (update->inputMngr, room, update->columnMask);
			}

			osd->Update (update, allowUserInput);
		}
	}

	void Player::UpdateFractionBlend (float weight)
	{
		//Vec3 delta = prevPos.DeltaTo (&coord) * 1.0f;
		//renderEye = coord + delta;
	}

	void Player::Render (Gfx::Camera * camera)
	{
		osd->Render (camera);
	}

	void Player::CalcCamera (Gfx::Camera * camera, Int3 * cameraColumnId, Vec3 * cameraColResPos)
	{
		assert (camera != NULL);
		assert (cameraColumnId != NULL);
		assert (cameraColResPos != NULL);

		switch (state)
		{
		case S_EDITING:
		case S_PLAYING:
			{
				Vec3 camOffset = motion.GetSize ();
				camOffset = Vec3 (camOffset.x * 0.5f, (isCrouching) ? 0.65f : 1.65f, camOffset.z * 0.5f);

				orient.SetCamera (camera, camOffset);

				Ents::Comps::Coord camCoord (coord.GetColumnId (), coord.GetColumnRelPos ());
				camCoord.MoveBy (camOffset);

				*cameraColumnId = camCoord.GetColumnId ();
				*cameraColResPos = camCoord.GetColumnRelPos ();
			}
			break;

		case S_FLYING:
			{
				Vec3 camOffset = motion.GetSize ();
				camOffset = Vec3 (4.0f, 0.0f, 4.0f);

				Ents::Comps::Coord camCoord (homeOnlyColumnId, Vec3 (-4.0f, 40.0f, -14.0f));
				camCoord.MoveBy (camOffset);

				Comps::Orient orient2 (&camCoord, Vec3 (-35.0f, -20.0f, 0.0f));
				orient2.SetCamera (camera, Vec3 (0.0f, 0.0f, 0.0f));

				*cameraColumnId = camCoord.GetColumnId ();
				*cameraColResPos = camCoord.GetColumnRelPos ();
			}
			break;
		}

		isCameraDone = true;
	}

	void Player::GetEyeCoord (Ents::Comps::Coord * out) const
	{
		assert (out != NULL);

		float yEye = (isCrouching) ? 0.65f : 1.65f;

		*out = coord;
		out->MoveBy (Vec3 (0.25f, yEye, 0.25f));
	}

	const Collision::Result * Player::GetPickResult () const
	{
		return pickResult;
	}

	void Player::Client_UpdateInput (ClientEnts::UpdateData * update, SceneSim::Room * room, bool allowUserInput)
	{
		switch (state)
		{
		case S_EDITING:
		case S_PLAYING:
			UpdateInput_Character (update, room, allowUserInput);
			break;

		case S_FLYING:
			UpdateInput_Flying (update, room, allowUserInput);
			break;
		}

		if (coord.GetColumnRelPos ().y < -90.0f)
		{
			all->playingDialog->ShowDead ();
			isDead = true;
		}
	}

	SceneSim::Cell * Player::GetCellUnder (Vec3 offset, SceneSim::Room * room)
	{
		Ents::Comps::Coord c (coord.GetColumnId (), coord.GetColumnRelPos () + offset);
		c.Fix ();
		Int3 cellId (glm::floor (c.GetColumnRelPos ().x), glm::floor (c.GetColumnRelPos ().y), glm::floor (c.GetColumnRelPos ().z));
		SceneSim::Cell * cell = room->GetColumn (c.GetColumnId ())->GetCells ()->GetCell (cellId);
		return cell;
	}

	void Player::UpdateInput_Character (ClientEnts::UpdateData * update, SceneSim::Room * room, bool allowUserInput)
	{
		const Input::Keys * keys = update->inputMngr->GetKeys ();
		const Input::Mouse * mouse = update->inputMngr->GetMouse ();

		DoMouseMove (update, room, allowUserInput);

		const bool isInAir = false;
		Vec3 velocity = motion.GetVelocity ();

		// move
		if (isInAir)
		{
			const float damp = 1.0f;
			velocity.x *= damp;
			velocity.z *= damp;
		}
		else
		{
			velocity.x = velocity.z = 0.0f;
		}

		justJumped = false;
		if (allowUserInput)
		{
			Vec3 velModifier;
			if (keys->IsDown (Session::Settings::Get ().up))
			{
				++velModifier.z;
			}
			if (keys->IsDown (Session::Settings::Get ().down))
			{
				--velModifier.z;
			}
			if (keys->IsDown (Session::Settings::Get ().left))
			{
				--velModifier.x;
			}
			if (keys->IsDown (Session::Settings::Get ().right))
			{
				++velModifier.x;
			}

			// crouch
			if (!canJump)
			{
				isCrouching = false;
			}
			else
			{
				isCrouching = keys->IsDown (Session::Settings::Get ().crouch);
				isCrouching = false;
			}

			if (canJump)
			{
				SceneSim::Cell * cell1 = GetCellUnder (Vec3 (0.0f, -0.5f, 0.0f), room);
				SceneSim::Cell * cell2 = GetCellUnder (Vec3 (0.5f, -0.5f, 0.0f), room);
				SceneSim::Cell * cell3 = GetCellUnder (Vec3 (0.0f, -0.5f, 0.5f), room);
				SceneSim::Cell * cell4 = GetCellUnder (Vec3 (0.5f, -0.5f, 0.5f), room);

				if ((state == S_PLAYING) &&
					((cell1 != NULL && cell1->type == 61) ||
					(cell2 != NULL && cell2->type == 61) ||
					(cell3 != NULL && cell3->type == 61) ||
					(cell4 != NULL && cell4->type == 61)))
				{
					all->playingDialog->ShowDead ();
					isDead = true;
					return;
				}
				else if (keys->IsDown (Session::Settings::Get ().jump))
				{
					Vec3 footOffset = motion.GetSize () * 0.5f;
					footOffset.y = -0.5f;
					SceneSim::Cell * cell = GetCellUnder (footOffset, room);

					float jumpVel = clientJumpVel;

					// check if on jump pad
					if (cell != NULL)
					{
						if (cell->type == 38)
						{
							jumpVel = 15.0f;
						}
						else if (cell->type == 60)
						{
							jumpVel = 22.0f;
						}
					}

					canJump = false;
					justJumped = true;
					velocity.y = jumpVel;
					isCrouching = false;
				}
			}

			if (velModifier.x != 0.0f || velModifier.z != 0.0f)
			{
				float speed = clientMoveSpeed;
				if (isCrouching)
				{
					speed *= 0.5f;
				}
				else if (isInAir && !justJumped)
				{
					speed *= 0.01f;
				}	
				Vec2 xz (velModifier.x, velModifier.z);

				xz = glm::normalize (xz);
				xz *= speed;
				//velocity.x += xz.x;
				//velocity.z += xz.y;

				{
					Vec4 xzMotion (xz.x, 0.0f, xz.y, 0.0f);
					Mat44 rotation;
					orient.GetYawMat (&rotation);
					xzMotion = xzMotion * rotation;
					velocity += Vec3 (xzMotion.x, 0.0f, xzMotion.z);
				}
			}
		}

		velocity.y += clientGravity;

		motion.SetVelocity (velocity);

		// attempt to move in this direction
		const bool canClimb = !isInAir;
		CMO_Box::Update (update->timeStep, room, canClimb, update->columnMask);

		// use the velocity before we attempt to move otherwise ability to determine up/down movement is lost
		if (motionResult & Ents::Comps::MR_BLOCKEDY && velocity.y <= 0.0f)
		{
			canJump = true;
		}
		else
		{
			justJumped = canJump;
			canJump = false;
		}

		if (allowUserInput)
		{
			DoLeftMouse (update, room, allowUserInput);
			DoMiddleMouse (update, room, allowUserInput);
			DoRightMouse (update, room, allowUserInput);

			DoMouseWheel (mouse->WheelDelta);
		}
	}

	void Player::UpdateInput_Flying (ClientEnts::UpdateData * update, SceneSim::Room * room, bool allowUserInput)
	{
		const Input::Keys * keys = update->inputMngr->GetKeys ();
		const Input::Mouse * mouse = update->inputMngr->GetMouse ();

		if (allowUserInput)
		{
			if (mouse->Left.IsJustOn ())
			{
				SetState (S_PLAYING);
			}

			Session::Settings & s = Session::Settings::Get ();
			if (keys->IsPressed (s.up) && homeOnlyColumnId.z < SceneSim::FixedSize - 1)
			{
				++homeOnlyColumnId.z;
				if (update->columnMask->use [homeOnlyColumnId.x][homeOnlyColumnId.z] || CheckForProhibitedFlying ()) 
				{		
					--homeOnlyColumnId.z;
				}
			}
			if (keys->IsPressed (s.down) && homeOnlyColumnId.z > 0)
			{
				--homeOnlyColumnId.z;
				if (update->columnMask->use [homeOnlyColumnId.x][homeOnlyColumnId.z] || CheckForProhibitedFlying ()) 
				{
					++homeOnlyColumnId.z;
				}
			}
			if (keys->IsPressed (s.left) && homeOnlyColumnId.x > 0)
			{
				--homeOnlyColumnId.x;
				if (update->columnMask->use [homeOnlyColumnId.x][homeOnlyColumnId.z] || CheckForProhibitedFlying ()) 
				{	
					++homeOnlyColumnId.x;
				}
			}
			if (keys->IsPressed (s.right) && homeOnlyColumnId.x < SceneSim::FixedSize - 1)
			{
				++homeOnlyColumnId.x;
				if (update->columnMask->use [homeOnlyColumnId.x][homeOnlyColumnId.z] || CheckForProhibitedFlying ()) 
				{
					--homeOnlyColumnId.x;
				}
			}

			pickResult->Clear ();
		}
	}

	int GetDist (Int3 a, Int3 b)
	{
		Int3 res (abs (a.x - b.x), 0, abs (a.z - b.z));
		return (res.x > res.z) ? res.x : res.z;
	}

	bool Player::CheckForProhibitedFlying ()
	{
		// check for sun
		if (!hasHeatShield)
		{
			if (GetDist (homeOnlyColumnId, Int3 (10, 0, 10)) < 6)
			{
				all->playingDialog->ShowWarnHotDialog ();
				return true;
			}
		}
		if (!hasIceShield)
		{
			if (GetDist (homeOnlyColumnId, Int3 (15, 0, 0)) < 2)
			{
				all->playingDialog->ShowWarnColdDialog ();
				return true;
			}
		}
		return false;
	}

	void Player::UpdatePicking (const Input::InputMngr * mngr, SceneSim::Room * room, Ld::ColumnMask * mask)
	{
		pickResult->Clear ();

		//float pickLength = inventory->GetPickLength ();
		float pickLength = 50.0f;
		if (pickLength >= 0.0f)
		{
			Vec3 rayPos;
			Vec3 rayDir;

			const Gfx::Viewport * viewport = mngr->GetViewport ();

			Collision::Pick_RayFromCamera (mngr->GetViewport (), viewport->size.x / 2, viewport->size.y / 2, mngr->GetCamera (), &rayPos, &rayDir);
			rayDir *= pickLength;

			// note the ray position may not be in the column that the player is (because camera origin is offset from player position)
			// need to recalculate - stick it in a coord and get it to sort it out

			Ents::Comps::Coord eyeCoord;
			GetEyeCoord (&eyeCoord);
			Ents::Comps::Coord camCoord (eyeCoord.GetColumnId (), rayPos);// + Vec3 (1000.0f, 0.0f, 0.0f));
			camCoord.Fix ();

			Collision::Segment_Room (camCoord.GetColumnRelPos (), rayDir, room, camCoord.GetColumnId (), mask, pickResult);
		}
	}

	void Player::DoMouseMove (ClientEnts::UpdateData * update, SceneSim::Room * room, bool allowUserInput)
	{
		const Input::Mouse * mouse = update->inputMngr->GetMouse ();
		if (allowUserInput && mouse->GetIsFpsMouse ())
		{
			// turn
			orient.TurnYawBy (mouse->GetDelta ().x * clientTurnSpeed * 4.0f);


			if (Session::Settings::Get ().isCameraInverted)
			{
				orient.TurnPitchBy (-mouse->GetDelta ().y * clientTurnSpeed * 4.0f);
			}
			else
			{
				orient.TurnPitchBy (mouse->GetDelta ().y * clientTurnSpeed * 4.0f);
			}
			orient.ClampPitch ();
		}
	}

	void Player::DoLeftMouse (ClientEnts::UpdateData * update, SceneSim::Room * room, bool allowUserInput)
	{
		const Input::Mouse * mouse = update->inputMngr->GetMouse ();
		if (mouse->Left.IsJustOn ())
		{
			if (pickResult->resultType == Collision::RT_COLLISION)
			{
				Int3 columnId = pickResult->entryPos.GetColumnId ();
				Int3 cellCoord = pickResult->entryCell + SceneSim::GetDeltaFromFace (pickResult->entryFace);
				SceneSim::FixCell (&columnId, &cellCoord);

				Int3 columnId2 = pickResult->entryPos.GetColumnId ();
				Int3 cellCoord2 = pickResult->entryCell;
				
				float radius = 0.5f;
				if (columnId.y == 0)
				{
					if (state == S_EDITING)
					{
						if (update->inputMngr->GetKeys ()->IsDown (SDLK_LCTRL)) // erase
						{
							update->worldSyncMngr->ApplyCsg (&update->timeNow, "hub", columnId2, cellCoord2, radius, 0, 0);
						}
						else if (update->inputMngr->GetKeys ()->IsDown (SDLK_LSHIFT)) // pick
						{
							cellTypeSelected = room->GetColumn (columnId2)->GetCells ()->GetCell (cellCoord2)->type;
						}
						else // place
						{
							float angle = orient.GetYpr ().y;
							while (angle >= 360.0f) angle -= 360.0f;
							while (angle < 0.0f) angle += 360.0f;

							int cellData = 2;

							if (angle > 315.0f) cellData = 2;
							else if (angle > 315.0f - 90.0f) cellData = 3;
							else if (angle > 315.0f - 180.0f) cellData = 0;
							else if (angle > 315.0f - 270.0f) cellData = 1;

							update->worldSyncMngr->ApplyCsg (&update->timeNow, "hub", columnId, cellCoord, radius, cellTypeSelected, cellData);
						}
					}
					else if (state == S_PLAYING)
					{
						bool handled = false;
						const bool wasShortPick = (pickResult->entryDistanceSqr < 2.0f * 2.0f);
						cellTypeSelected = room->GetColumn (columnId2)->GetCells ()->GetCell (cellCoord2)->type;
						switch (cellTypeSelected)
						{
						case 3: // computer terminal
							if (wasShortPick)
							{
								SetState (S_FLYING); 
							}
							break;

						case 12: // treasure chest
							if (wasShortPick)
							{
								Tiny::TINYITEMS item = update->tinyWorlds->FindItem (columnId);
								if (item == Tiny::TI_AXE) {hasAxe = true; all->playingDialog->ShowDialog (item);}
								else if (item == Tiny::TI_JUMP_1) {clientJumpVel = 7.3f; all->playingDialog->ShowDialog (item);}
								else if (item == Tiny::TI_SHIELDCOLD) {hasIceShield = true; all->playingDialog->ShowDialog (item);}
								else if (item == Tiny::TI_BELT) {hasBelt = true; all->playingDialog->ShowDialog (item);}
								else if (item == Tiny::TI_SHIELDHOT) {hasHeatShield = true; all->playingDialog->ShowDialog (item);}
								else if (item == Tiny::TI_TELEPORTER) {hasTeleporter = true; all->playingDialog->ShowDialog (item);}
								else if (item == Tiny::TI_JUMP_2) {clientJumpVel = 8.5f; all->playingDialog->ShowDialog (item);}
								else if (item == Tiny::TI_SEEDS) {hasSeeds = true; all->playingDialog->ShowDialog (item);}
								else if (item == Tiny::TI_BIGAXE) {hasBigAxe = true; all->playingDialog->ShowDialog (item);}
								

								if (item != Tiny::TI_NONE)
								{
									// dull the chest
									int oldData = cellTypeSelected = room->GetColumn (columnId2)->GetCells ()->GetCell (cellCoord2)->blockData;
									update->worldSyncMngr->ApplyCsg (&update->timeNow, "hub", columnId, cellCoord2, radius, 14, oldData);
								}
							}
							break;

						case 63:
							if (wasShortPick)
							{
								++artifactCount;
								all->playingDialog->ShowDialogArtifacts (artifactCount, 6);

								// dull the chest
								int oldData = cellTypeSelected = room->GetColumn (columnId2)->GetCells ()->GetCell (cellCoord2)->blockData;
								update->worldSyncMngr->ApplyCsg (&update->timeNow, "hub", columnId, cellCoord2, radius, 14, oldData);
							}
							break;

						case 5:
						case 23:
						case 51:
							if (wasShortPick && hasAxe)
							{
								update->worldSyncMngr->ApplyCsg (&update->timeNow, "hub", columnId2, cellCoord2, 0.5f, 0, 0);
							}
							break;
						case 13:
						case 55:
							if (wasShortPick && hasBigAxe)
							{
								update->worldSyncMngr->ApplyCsg (&update->timeNow, "hub", columnId2, cellCoord2, 0.5f, 0, 0);
							}
							break;

						case 25:
						case 27:
						case 37:
						case 42:
							if (wasShortPick && hasBelt)
							{
								update->worldSyncMngr->ApplyCsg (&update->timeNow, "hub", columnId2, cellCoord2, 0.5f, 0, 0);
							}
							break;

						case 32:
						case 40:
						case 62:
						case 68:
							if (pickResult->entryFace == SceneSim::F_T)
							{
								coord.MoveTo (columnId, Vec3 (cellCoord.x + 0.25f, cellCoord.y, cellCoord.z + 0.25f));
								motion.SetVelocity (Vec3 (0.0f, 0.0f, 0.0f));
								canJump = false;
							}
							break;

						case 59:
							if (hasSeeds && pickResult->entryFace == SceneSim::F_T)
							{
								update->worldSyncMngr->ApplyCsg (&update->timeNow, "hub", columnId2, cellCoord2, 0.5f, cellTypeSelected + 1, 0);
							}
							break;

						}

						//Int4 rgba (35, 200, 127, 255);
						//update->billboards->CreateExplosion (
						//	(int) update->timeNow,
						//	columnId, 
						//	Vec3 (cellCoord.x + 0.5f, cellCoord.y + 0.5f, cellCoord.z + 0.5f), 
						//	radius,
						//	rgba);
					}
				}	
			}
		}
	}

	void Player::DoMiddleMouse (ClientEnts::UpdateData * update, SceneSim::Room * room, bool allowUserInput)
	{
		if (state != S_EDITING)
		{
			return;
		}
		const Input::Mouse * mouse = update->inputMngr->GetMouse ();
		if (mouse->Middle.IsJustOn ())
		{
			if (pickResult->resultType == Collision::RT_COLLISION)
			{
				Int3 columnId = pickResult->entryPos.GetColumnId ();
				Int3 cellCoord = pickResult->entryCell + Int3 (0, 1, 0);
				const SceneSim::ColumnCells * cells = room->GetColumn (columnId)->GetCells ();

				if (cellCoord.y < SceneSim::CellsPerColY - 2)
				{
					for (int y = cellCoord.y; y < SceneSim::CellsPerColY; ++y)
					{
						if (cells->GetCell (cellCoord)->type == 0 && cells->GetCell (cellCoord + Int3 (0, 1, 0))->type == 0)
						{
							break;
						}
					}
				}
				else
				{
					cellCoord.y = SceneSim::CellsPerColY;
				}

				coord.MoveTo (columnId, Vec3 (cellCoord.x + 0.25f, cellCoord.y, cellCoord.z + 0.25f));
				motion.SetVelocity (Vec3 (0.0f, 0.0f, 0.0f));
				canJump = false;
			}
		}
	}

	void Player::DoRightMouse (ClientEnts::UpdateData * update, SceneSim::Room * room, bool allowUserInput)
	{
		return;

		const Input::Mouse * mouse = update->inputMngr->GetMouse ();

		if (mouse->Right.IsJustOn ())
		{
			if (pickResult->resultType == Collision::RT_COLLISION)
			{
				Int3 columnId = pickResult->entryPos.GetColumnId ();
				Int3 cellCoord = pickResult->entryCell;
				SceneSim::FixCell (&columnId, &cellCoord);
				
				float radius = 0.5f;
				if (columnId.y == 0)
				{
					SceneSim::Cell * cell = room->GetColumn (columnId)->GetCells ()->GetCell (cellCoord);

					switch (state)
					{
					case S_PLAYING: DoRightMouse_Playing (update, columnId, cellCoord, cell); break;
					case S_EDITING: break;
					}
				}	
			}
		}
	}

	void Player::DoMouseWheel (int wheelDelta)
	{
		if (state == S_EDITING)
		{
			cellTypeSelected -= wheelDelta;
			cellTypeSelected = (cellTypeSelected < 1) ? 1 : cellTypeSelected;
			cellTypeSelected = (cellTypeSelected > 128) ? 128 : cellTypeSelected;
		}
	}

	void Player::DoRightMouse_Playing (ClientEnts::UpdateData * update, Int3 columnId, Int3 cellId, SceneSim::Cell * cell)
	{
		switch (cell->type)
		{
		case 3: SetState (S_FLYING); break;
		default: break;
		}
	}

	void Player::SetState (STATES newState)
	{
		if (state == S_FLYING && newState == S_PLAYING)
		{
			coord.SetColumnId (homeOnlyColumnId);
			motion.SetVelocity (Vec3 (0.0f, 0.0f, 0.0f));
		}
		state = newState;
	}
}