#pragma once

#include "../Tiny/Tiny_TinyWorlds.h"
#include "Ents/Ents_CMO_Box.h"

namespace CellMesh {class Mngr;}
namespace ClientEnts {struct UpdateData;}
namespace Collision {class Result;}
namespace Ents {class PlayerOsd;}
namespace Gfx {class Camera;}
namespace Input {class InputMngr;}
namespace Inventory {class ItemCollection;}
namespace Layers {class All;}
namespace Ld {class ColumnMask;}
namespace SceneSim {class Cell;}

namespace Ents
{
	class Player : public Ents::CMO_Box
	{
	public:
		
		enum STATES
		{
			S_EDITING,
			S_PLAYING,
			S_FLYING,
			S_DEAD,
		};

		Player (bool isEditing, CellMesh::Mngr * meshes, Layers::All * all);
		virtual ~Player ();

		void Init (Int3 columnId, Vec3 colRelPos);

		void Update (ClientEnts::UpdateData * update, bool allowUserInput);
		void UpdateFractionBlend (float weight);

		void Render (Gfx::Camera * camera);

		void CalcCamera (Gfx::Camera * camera, Int3 * cameraColumnId, Vec3 * cameraColResPos);

		void GetEyeCoord (Ents::Comps::Coord * out) const;

		const Collision::Result * GetPickResult () const;

		STATES state;
		bool isCrouching;

		Int3 homeOnlyColumnId;
		Vec3 homeOnlyColumnPos;

	private:

		friend class PlayerOsd;

		SceneSim::Cell * GetCellUnder (Vec3 offset, SceneSim::Room * room);
		void Client_UpdateInput (ClientEnts::UpdateData * update, SceneSim::Room * room, bool allowUserInput);
		void UpdateInput_Character (ClientEnts::UpdateData * update, SceneSim::Room * room, bool allowUserInput);
		void UpdateInput_Flying (ClientEnts::UpdateData * update, SceneSim::Room * room, bool allowUserInput);
		bool CheckForProhibitedFlying ();

		void UpdatePicking (const Input::InputMngr * mngr, SceneSim::Room * room, Ld::ColumnMask * mask);

		void DoMouseMove (ClientEnts::UpdateData * update, SceneSim::Room * room, bool allowUserInput);
		void DoLeftMouse (ClientEnts::UpdateData * update, SceneSim::Room * room, bool allowUserInput);
		void DoMiddleMouse (ClientEnts::UpdateData * update, SceneSim::Room * room, bool allowUserInput);
		void DoRightMouse (ClientEnts::UpdateData * update, SceneSim::Room * room, bool allowUserInput);
		void DoMouseWheel (int wheelDelta);

		void DoRightMouse_Playing (ClientEnts::UpdateData * update, Int3 columnId, Int3 cellId, SceneSim::Cell * cell);

		void SetState (STATES newState);

		Layers::All * all;

		bool canJump;
		bool justJumped;

		Collision::Result * pickResult;
		
		float clientMoveSpeed;
		float clientTurnSpeed;
		float clientJumpVel;
		float clientGravity;
		bool clientSprayBullets;

		Ents::Comps::Coord prevPos;
		Ents::Comps::Coord renderEye;

		PlayerOsd * osd;

		int cellTypeSelected;

		bool hasAxe;
		bool hasHeatShield;
		bool hasIceShield;
		bool hasBelt;
		bool hasTeleporter;
		bool hasSeeds;
		bool hasBigAxe;

	public:
		int artifactCount;
		bool isDead;
		bool isInit;
		bool isCameraDone;
	};
}
