#include "stdafx.h"

#include "CellMesh/CellMesh_ShapeBase.h"
#include "CellMesh/CellMesh_Mngr.h"
#include "Gfx/Gfx_Camera.h"
#include "Gfx/Gfx_Tex.h"
#include "Plot/Plot_Prims.h"

#include "Ents_Player.h"
#include "Ents_PlayerOsd.h"

namespace Ents
{
	PlayerOsd::PlayerOsd (Player * player, CellMesh::Mngr * meshes)
		: player (player),
		meshes (meshes)
	{
		crosshair = aah_new Gfx::GfxTex ("gfx/crosshair_normal.png");
		editTex = aah_new Gfx::GfxTex ("gfx/terrain_inprogress_00.png");
	}

	PlayerOsd::~PlayerOsd ()
	{
		delete editTex;
		delete crosshair;
	}

	void PlayerOsd::Update (ClientEnts::UpdateData * update, bool allowUserInput)
	{
	}

	void PlayerOsd::Render (Gfx::Camera * camera)
	{
		Int2 size (camera->viewportSize.x, camera->viewportSize.y);

		Int2 xy (camera->viewportSize.x - 80, camera->viewportSize.y - 80);

		if (player->state == Player::S_EDITING)
		{
			editTex->Bind ();
			Plot::Prims_BatchBegin (Plot::PT_XYUVRGB);

			const float pixel = 1.0f / 256.0f;
			Vec2 buv1 (0.0f, 0.0f);
			Vec2 buv2 (pixel * 16.0f, pixel * 16.0f);

			for (int idx = 0; idx < 11; ++idx)
			{
				int type = player->cellTypeSelected - 6 + idx;

				if (type > 0 && type < 128)
				{
					CellMesh::ShapeBase * shape = meshes->GetShape (type);
					int texId = shape->GetInventoryIcon ();

					Vec2 uv1 (pixel * (texId % 16) * 16.0f, pixel * (texId / 16) * 16.0f);
					Vec2 uv2 (uv1.x + pixel * 16.0f, uv1.y + pixel * 16.0f);
					Int2 border (4, 4);
					if (idx == 6)
					{
						Int2 grow (16, 16);
						Plot::Prims_QuadTex (xy - grow - border, xy + Int2 (48, 48) + grow + border, buv1, buv2, Int4 (255, 0, 0, 255));
						Plot::Prims_QuadTex (xy - grow, xy + Int2 (48, 48) + grow, uv1, uv2, Int4 (255, 255, 255, 255));
					}
					else
					{
						Plot::Prims_QuadTex (xy - border, xy + Int2 (48, 48) + border, buv1, buv2, Int4 (255, 255, 255, 255));
						Plot::Prims_QuadTex (xy, xy + Int2 (48, 48), uv1, uv2, Int4 (255, 255, 255, 255));
					}
				}
				
				xy.y -= 64 + 6;
			}

			Plot::Prims_BatchEnd ();
			editTex->Unbind ();
		}

		if (player->state != Player::S_FLYING)
		{
			crosshair->Bind ();
			Plot::Prims_BatchBegin (Plot::PT_XYUVRGB);
			Plot::Prims_QuadTex (size / 2 - crosshair->GetWh (), crosshair, 2);
			Plot::Prims_BatchEnd ();
			crosshair->Unbind ();
		}
	}
}
