#include "stdafx.h"

#include "Gfx/Gfx_Camera.h"
#include "Gfx/Gfx_Font.h"
#include "Gfx/Gfx_Viewport.h"
#include "Input/Input_InputMngr.h"
#include "Input/Input_Keys.h"
#include "Input/Input_Mouse.h"
#include "Ld/Ld_ColumnMask.h"
#include "Particles/Particles_BillboardMngr.h"
#include "Profile/Profile_ProfileScope.h"
#include "SceneGfx/SceneGfx_World.h"
#include "SceneHelper/SceneHelper_SimMngr.h"
#include "SceneGfx/SceneGfx_Room.h"
#include "SceneSim/SceneSim_Defs.h"
#include "SceneSim/SceneSim_Room.h"
#include "SceneSim/SceneSim_World.h"

#include "../Ents/Ents_Player.h"
#include "../Layers/Layers_All.h"
#include "../Layers/Layers_RenderStates.h"
#include "../LayerPlaying/LayerPlaying_WorldSyncMngr.h"

#include "LayerEditing_Editing.h"
#include "LayerEditing_EditingMenus.h"

namespace LayerEditing
{
	Editing::Editing (Layers::All * all)
		: isRunningRequested (false),
		state (S_NONE),
		isRunning (false),
		all (all)
	{
		font = aah_new Gfx::Font ("fonts/nokia_medium.glf");
		update = NULL;
		worldSyncMngr = NULL;
		gfxWorld = NULL;
		simWorld = NULL;

		columnMask = aah_new Ld::ColumnMask ();
	}

	Editing::~Editing ()
	{
		delete columnMask;
		columnMask = NULL;

		if (update != NULL)
		{
			delete update->billboards;
			update->billboards = NULL;
		}

		delete worldSyncMngr; 
		worldSyncMngr = NULL;

		delete gfxWorld; 
		gfxWorld = NULL;

		delete simWorld; 
		simWorld = NULL;

		delete font;
	}

	void Editing::Preupdate (ClientEnts::UpdateData * update)
	{
		if (isRunningRequested && !isRunning)
		{
			assert (!isRunning);
			
			state = S_LOADINGSIM;
			isGraphicsComplete = false;

			update->inputMngr->GetMouse ()->SetIsFpsMouse (true);

			update->sim = simWorld = aah_new SceneSim::World ();
			update->renderer = gfxWorld = aah_new SceneGfx::World ();
			update->worldSyncMngr = worldSyncMngr = aah_new LayerPlaying::WorldSyncMngr (simWorld, gfxWorld, true);
			update->billboards = aah_new Particles::BillboardMngr (300);

			Debug_WriteLine (1, "Loading Tiny Galaxy...");
		}
		else if (!isRunningRequested && isRunning)
		{
			assert (isRunning);

			delete update->billboards;
			update->billboards = NULL;

			delete worldSyncMngr; 
			worldSyncMngr = NULL;

			delete gfxWorld; 
			gfxWorld = NULL;

			delete simWorld; 
			simWorld = NULL;
		}
		isRunning = isRunningRequested;
	}

	void Editing::Update (ClientEnts::UpdateData * update)
	{
		if (isRunning)
		{
			switch (state)
			{
			case S_NONE:
				assert (false);
				break;

			case S_LOADINGSIM:
				worldSyncMngr->Update ();
				if (worldSyncMngr->IsSimReady)
				{
					InitEnts (update);
					UpdateWorld (update, false);
					Debug_WriteLine (1, "SIM READY");
					state = S_GENERATINGGFX;
				}
				break;

			case S_GENERATINGGFX:
				UpdateWorld (update, false);
				worldSyncMngr->Update ();
				if (worldSyncMngr->IsGfxReady)
				{
					//Debug_WriteLine (1, "GRAPHICS READY");
					state = S_PLAYING;
				}
				break;

			case S_PLAYING:
				worldSyncMngr->Update ();
				UpdateWorld (update, true);
				break;
			}

			const bool allowUserInput = !all->editingMenus->GetIsRunning ();

			if (allowUserInput)
			{
				if (update->inputMngr->GetMouse ()->GetIsFpsMouse ())
				{
					if (update->inputMngr->GetKeys ()->IsPressed (SDLK_ESCAPE))
					{
						all->editingMenus->isRunningRequested = true;
					}
				}
			}
		}
	}

	void Editing::InitEnts (ClientEnts::UpdateData * update)
	{
		update->player = aah_new Ents::Player (true, update->renderer->GetPrimaryRoom ()->GetCellMesh (), all);
		update->player->Init (Int3 (1, 0, 1), Vec3 (1.0f, 32.0f, 1.0f));
	}

	void Editing::UpdateWorld (ClientEnts::UpdateData * update, bool isLoadingFinished)
	{
		this->update = update;
		
		update->columnMask = columnMask;

		update->billboards->Update (update->timeStep);
		update->player->Update (update, !all->editingMenus->GetIsRunning ());

		update->columnMask = NULL;
	}

	void Editing::UpdateFractionBlend (ClientEnts::UpdateData * update, float weight)
	{
		if (isRunning)
		{
			//update->you->UpdateFractionBlend (update, weight);
		}
	}

	void Editing::Render ()
	{
		if (isRunning)
		{
			glClearColor (0.5f, 0.5f, 0.5f, 1.0f);
			glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

			Layers::SetRenderStates_3d ();

			switch (state)
			{
			case S_NONE:
			case S_LOADINGSIM:
				break;

			case S_GENERATINGGFX:
			case S_PLAYING:
				{
					Int3 cameraColumnId (0, 0, 0);
					Vec3 cameraColResPos (0.0f, 0.0f, 0.0f);

					Gfx::Camera * camera = update->inputMngr->GetCamera ();
					camera->viewportSize = update->inputMngr->GetViewport ()->size;
					update->player->CalcCamera (camera, &cameraColumnId, &cameraColResPos);

					glMatrixMode (GL_PROJECTION);
					glLoadMatrixf ((GLfloat *) &camera->matProjection);
					glMatrixMode (GL_MODELVIEW);
					glLoadMatrixf ((GLfloat *) &camera->matView);

					//skydome->Render (cameraColResPos);
					//glCullFace (GL_BACK);

					gfxWorld->Render (0, columnMask, cameraColumnId, cameraColResPos, 10.0f * 16.0f * 2.0f, Int3 (0, 0, 0));

					glDisable (GL_CULL_FACE); 

					if (update->player != NULL)
					{
						Gfx::Camera * camera = update->inputMngr->GetCamera ();

						const Mat44 & m = camera->matView;
						Vec3 right (m [0].x, m [1].x, m [2].x);
						Vec3 up (m [0].y, m [1].y, m [2].y);
						Vec3 in (m [0].z, m [1].z, m [2].z);

						update->billboards->Render (cameraColumnId, right, up, in);
					}

					update->player->Render (camera);
				}
				break;
			}
		}
	}

	bool Editing::GetIsRunning () const
	{
		return isRunning;
	}
}
