#include "stdafx.h"

#include "Gfx/Gfx_Font.h"
#include "Gfx/Gfx_Tex.h"
#include "Gfx/Gfx_Viewport.h"
#include "Plot/Plot_Prims.h"
#include "Input/Input_InputMngr.h"
#include "Input/Input_Mouse.h"
#include "Utils/Utils_AppVersion.h"
#include "Utils/Utils_File.h"

#include "../Layers/Layers_All.h"
#include "../Layers/Layers_RenderStates.h"
#include "../LayerEditing/LayerEditing_Editing.h"
#include "../LayerPlaying/LayerPlaying_Playing.h"
#include "../Menus/Controls_Button.h"
#include "../Menus/Controls_ButtonCycle.h"
#include "../Menus/Controls_ButtonToPage.h"
#include "../Menus/Controls_Checkbox.h"
#include "../Menus/Controls_RecordKey.h"
#include "../Menus/Controls_TextEntry.h"
#include "../Menus/Controls_Title1.h"
#include "../Menus/Controls_Title2.h"
#include "../Menus/Menus_Page.h"
#include "../Menus/Menus_Pages.h"
#include "../Menus/Menus_Palette.h"
#include "../Session/Session_Settings.h"

#include "LayerFrontEnd_FrontEnd.h"

namespace LayerFrontEnd
{
	bool IsUniqueKey (SDLKey key)
	{
		int count = 0;
		count += (key == Session::Settings::Get ().up) ? 1 : 0;
		count += (key == Session::Settings::Get ().down) ? 1 : 0;
		count += (key == Session::Settings::Get ().left) ? 1 : 0;
		count += (key == Session::Settings::Get ().right) ? 1 : 0;
		count += (key == Session::Settings::Get ().jump) ? 1 : 0;
		count += (key == Session::Settings::Get ().chat) ? 1 : 0;
		count += (key == Session::Settings::Get ().showScores) ? 1 : 0;
		return (count < 2);
	}

	FrontEnd::FrontEnd (Layers::All * all, Input::InputMngr * inputMngr)
		: isRunningRequested (false),
		isRunning (false),
		all (all),
		inputMngr (inputMngr),
		font (NULL),
		pages (NULL)
	{
		font = aah_new Gfx::Font ("fonts/nokia_medium.glf");

		// create the menu pageSinglePlayerMain

		pages = aah_new Menus::Pages ();

		const Menus::Palette * p = Menus::Palette_Get (false);

		Menus::Page * page = pages->AddPage (aah_new Menus::Page (inputMngr, font, "main"));
		Int2 xy (0, 0);
		page->Add (aah_new Menus::Controls::Title2 (xy, "WELCOME TO TINY GALAXY")); xy += p->vertSpacing;
		page->Add (aah_new Menus::Controls::ButtonToPage (xy, "Play!", "singleplayer", pages)); xy += p->vertSpacing * 2;
		page->Add (aah_new Menus::Controls::Title2 (xy, "OPTIONS")); xy += p->vertSpacing;
		page->Add (aah_new Menus::Controls::ButtonToPage (xy, "Edit Controls", "controls_edit", pages)); xy += p->vertSpacing * 2;
		page->Add (aah_new Menus::Controls::Button (xy, "Quit to Desktop", OnQuitToDesktop, this)); xy += p->vertSpacing * 2;
		page->Add (aah_new Menus::Controls::Title1 (xy, "Robert Swan. Ludum Dare 23. www.aahgames.com")); xy += p->vertSpacing;
		page->MarkBottomRight (Int2 (p->labelControl.x + p->labelSize.x, xy.y));



		page = pages->AddPage (aah_new Menus::Page (inputMngr, font, "singleplayer"));
		xy = Int2 (0, 0);
		page->Add (aah_new Menus::Controls::Title2 (xy, "Setting the Scene... badly")); xy += p->vertSpacing;
		page->Add (aah_new Menus::Controls::Title1 (xy, "You live on a tiny, tiny world... in a tiny, tiny galaxy. If only you")); xy += p->vertSpacing;
		page->Add (aah_new Menus::Controls::Title1 (xy, "could collect all the hidden artifacts then maybe the \"collect all the")); xy += p->vertSpacing;
		page->Add (aah_new Menus::Controls::Title1 (xy, "artifacts and be happy for ever\" legend could come true. All your tiny,")); xy += p->vertSpacing;
		page->Add (aah_new Menus::Controls::Title1 (xy, "tiny world has is a computer and thrusters. Use your computer and scour")); xy += p->vertSpacing;
		page->Add (aah_new Menus::Controls::Title1 (xy, "the galaxy...! You have but one life.")); xy += p->vertSpacing * 2;

		//page->Add (aah_new Menus::Controls::ButtonToPage (xy, "Begin Scouring", "world_play", pages)); xy += p->vertSpacing;

		page->Add (aah_new Menus::Controls::Button (xy, "Begin Scouring", OnPlayLevelSelected, this)); xy += p->vertSpacing;

		//page->Add (aah_new Menus::Controls::ButtonToPage (xy, "Edit World", "world_edit", pages)); xy += p->vertSpacing;
		//page->Add (aah_new Menus::Controls::ButtonToPage (xy, "Create World", "world_infinite_new", pages)); xy += p->vertSpacing;

		//xy += p->vertSpacing;
		//page->Add (aah_new Menus::Controls::ButtonToPage (xy, "Return to Main Menu", "main", pages)); xy += p->vertSpacing;
		page->MarkBottomRight (Int2 (p->labelControl.x + p->labelSize.x, xy.y));



		page = pages->AddPage (aah_new Menus::Page (inputMngr, font, "world_edit"));
		page->SetOnEntry (OnWorldEditEntry, this);




		page = pages->AddPage (aah_new Menus::Page (inputMngr, font, "world_play"));
		page->SetOnEntry (OnWorldPlayEntry, this);



		page = pages->AddPage (aah_new Menus::Page (inputMngr, font, "world_infinite_new"));
		xy = Int2 (0, 0);
		page->Add (aah_new Menus::Controls::Title2 (xy, "CREATE NEW WORLD")); xy += p->vertSpacing;
		page->Add (aah_new Menus::Controls::TextEntry (Text::VT_FILENAME, xy, "World Name", 24, &Session::Settings::Get ().worldInitParams.name)); xy += p->vertSpacing;
		page->Add (aah_new Menus::Controls::TextEntry (Text::VT_TEXT, xy, "Random Seed", 24, &Session::Settings::Get ().worldInitParams.randomSeed)); xy += p->vertSpacing * 2;
		page->Add (aah_new Menus::Controls::Button (xy, "PLAY!", OnCreateNewInfiniteLevel, this)); xy += p->vertSpacing * 2;
		page->Add (aah_new Menus::Controls::ButtonToPage (xy, "Return to Main Menu", "singleplayer", pages)); xy += p->vertSpacing;
		page->MarkBottomRight (Int2 (p->labelControl.x + p->labelSize.x, xy.y));


		page = pages->AddPage (aah_new Menus::Page (inputMngr, font, "controls_edit"));
		xy = Int2 (0, 0);
		page->Add (aah_new Menus::Controls::Title2 (xy, "EDIT CONTROLS")); xy += p->vertSpacing;
		page->Add (aah_new Menus::Controls::ButtonToPage (xy, "Edit Mouse Controls", "controls_mouse", pages)); xy += p->vertSpacing;
		page->Add (aah_new Menus::Controls::ButtonToPage (xy, "Edit Keyboard Controls", "controls_keys", pages)); xy += p->vertSpacing * 2;
		page->Add (aah_new Menus::Controls::ButtonToPage (xy, "Return to Main Menu", "main", pages)); xy += p->vertSpacing;
		page->MarkBottomRight (Int2 (p->labelControl.x + p->labelSize.x, xy.y));



		page = pages->AddPage (aah_new Menus::Page (inputMngr, font, "controls_mouse"));
		page->SetOnExit (OnSaveSettings, NULL);
		xy = Int2 (0, 0);
		page->Add (aah_new Menus::Controls::Title2 (xy, "EDIT MOUSE CONTROLS")); xy += p->vertSpacing;
		page->Add (aah_new Menus::Controls::Checkbox (xy, "Invert Mouse", &Session::Settings::Get ().isCameraInverted)); xy += p->vertSpacing * 2;
		page->Add (aah_new Menus::Controls::ButtonToPage (xy, "Return to Main Menu", "controls_edit", pages)); xy += p->vertSpacing;
		page->MarkBottomRight (Int2 (p->labelControl.x + p->labelSize.x, xy.y));



		page = pages->AddPage (aah_new Menus::Page (inputMngr, font, "controls_keys"));
		page->SetOnExit (OnSaveSettings, NULL);
		xy = Int2 (0, 0);
		page->Add (aah_new Menus::Controls::Title2 (xy, "EDIT KEYBOARD CONTROLS")); xy += p->vertSpacing;
		page->Add (aah_new Menus::Controls::RecordKey (xy, "Up", &Session::Settings::Get ().up, IsUniqueKey)); xy += p->vertSpacing;
		page->Add (aah_new Menus::Controls::RecordKey (xy, "Down", &Session::Settings::Get ().down, IsUniqueKey)); xy += p->vertSpacing;
		page->Add (aah_new Menus::Controls::RecordKey (xy, "Left", &Session::Settings::Get ().left, IsUniqueKey)); xy += p->vertSpacing;
		page->Add (aah_new Menus::Controls::RecordKey (xy, "Right", &Session::Settings::Get ().right, IsUniqueKey)); xy += p->vertSpacing;
		page->Add (aah_new Menus::Controls::RecordKey (xy, "Jump", &Session::Settings::Get ().jump, IsUniqueKey)); xy += p->vertSpacing;
		//page->Add (aah_new Menus::Controls::RecordKey (xy, "Crouch", &Session::Settings::Get ().crouch, IsUniqueKey)); xy += p->vertSpacing;
		//page->Add (aah_new Menus::Controls::RecordKey (xy, "Chat", &Session::Settings::Get ().chat, IsUniqueKey)); xy += p->vertSpacing;
		//page->Add (aah_new Menus::Controls::RecordKey (xy, "Show Scores", &Session::Settings::Get ().showScores, IsUniqueKey)); xy += p->vertSpacing * 2;
		page->Add (aah_new Menus::Controls::ButtonToPage (xy, "Return to Main Menu", "controls_edit", pages)); xy += p->vertSpacing;
		page->MarkBottomRight (Int2 (p->labelControl.x + p->labelSize.x, xy.y));
	}

	FrontEnd::~FrontEnd ()
	{
		delete pages;

		delete font;
	}

	void FrontEnd::Preupdate ()
	{
		if (isRunningRequested && !isRunning)
		{
			Debug_Clear (1);
			inputMngr->GetMouse ()->SetIsFpsMouse (false);
			pages->OpenPage ("main", true);
		}
		else if (!isRunningRequested && isRunning)
		{
		}
		isRunning = isRunningRequested;
	}

	void FrontEnd::Update ()
	{
		if (isRunning)
		{
			pages->Update ();
		}
	}

	void FrontEnd::Render ()
	{
		if (isRunning)
		{
			glClearColor (110.0f / 255.0f, 170.0f / 255.0f, 243.0f / 255.0f, 1.0f);
			glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

			Layers::SetRenderStates_2d ();

			font->Begin ();

			//Utils::AppVersion serverVersion;
			//font->DrawString (serverVersion.ToText (), 8, inputMngr->GetViewport ()->size.y - 22);

			font->End ();

			pages->Render ();
		}
	}

	bool FrontEnd::OnCreateNewInfiniteLevel (Menus::Controls::Button * button, void * data)
	{
		if (Utils::File_Exists (Session::Settings::Get ().worldsFolder + Utils::File_NeatenFolder (Session::Settings::Get ().worldInitParams.name) + "data/world.lua"))
		{
			return false;
		}

		FrontEnd * state = (FrontEnd *) data;
		state->all->editing->isRunningRequested = true;
		state->isRunningRequested = false;

		Session::Settings::Get ().worldInitParams.isNew = true;
		Session::Settings::Get ().worldInitParams.folder = Session::Settings::Get ().worldsFolder;
		Session::Settings::Get ().gamemode = Session::Settings::GM_EDIT;

		return true;
	}

	bool FrontEnd::OnConnectToServer (Menus::Controls::Button * button, void * data)
	{
		FrontEnd * state = (FrontEnd *) data;
		state->all->playing->isRunningRequested = true;
		state->isRunningRequested = false;

		Session::Settings::Get ().gamemode = Session::Settings::GM_EDIT;

		Session::Settings::Save ();

		return true;
	}

	bool FrontEnd::OnQuitToDesktop (Menus::Controls::Button * button, void * data)
	{
		FrontEnd * state = (FrontEnd *) data;
		state->all->isQuitRequested = true;

		return true;
	}

	void FrontEnd::OnWorldEditEntry (Menus::Page * page, void * data)
	{
		FrontEnd * frontEnd = (FrontEnd *) data;

		const Menus::Palette * p = Menus::Palette_Get (false);

		page->RemoveControls ();

		Int2 xy = Int2 (0, 0);
		page->Add (aah_new Menus::Controls::Title2 (xy, "LOAD WORLD")); xy += p->vertSpacing;

		std::vector <Text> folders;
		Utils::File_ListFolders (Session::Settings::Get ().worldsFolder, &folders);

		int validWorldCount = 0;
		for (int idx = 0; idx < folders.size (); ++idx)
		{
			// check if it has a "world.lua" file
			Text worldLua = Session::Settings::Get ().worldsFolder + folders [idx] + "/data/world.lua";
			if (Utils::File_Exists (worldLua))
			{

				//Menus::Controls::ButtonToPage * control = aah_new Menus::Controls::ButtonToPage (xy, folders [idx], "level_loaded", frontEnd->pages, OnLoadLevelSelected, frontEnd);
				Menus::Controls::Button * control = aah_new Menus::Controls::Button (xy, folders [idx], OnEditLevelSelected, frontEnd);
				page->Add (control); xy += p->vertSpacing;

				++validWorldCount;
			}
		}

		if (validWorldCount == 0)
		{
			page->Add (aah_new Menus::Controls::Title1 (xy, "No worlds to load")); xy += p->vertSpacing;
		}

		xy += p->vertSpacing;

		page->Add (aah_new Menus::Controls::ButtonToPage (xy, "Return to Main Menu", "singleplayer", frontEnd->pages)); xy += p->vertSpacing;
		page->MarkBottomRight (Int2 (p->labelControl.x + p->labelSize.x, xy.y));	
	}

	bool FrontEnd::OnEditLevelSelected (Menus::Controls::Button * button, void * data)
	{
		FrontEnd * state = (FrontEnd *) data;
		state->all->editing->isRunningRequested = true;
		state->isRunningRequested = false;

		Session::Settings::Get ().worldInitParams.isNew = false;
		Session::Settings::Get ().worldInitParams.name = Utils::File_NeatenFolder (button->label);
		Session::Settings::Get ().worldInitParams.folder = Session::Settings::Get ().worldsFolder;
		Session::Settings::Get ().gamemode = Session::Settings::GM_EDIT;

		return true;
	}

	void FrontEnd::OnWorldPlayEntry (Menus::Page * page, void * data)
	{
		FrontEnd * frontEnd = (FrontEnd *) data;

		const Menus::Palette * p = Menus::Palette_Get (false);

		page->RemoveControls ();

		Int2 xy = Int2 (0, 0);
		page->Add (aah_new Menus::Controls::Title2 (xy, "PLAY WORLD")); xy += p->vertSpacing;

		std::vector <Text> folders;
		Utils::File_ListFolders (Session::Settings::Get ().worldsFolder, &folders);

		int validWorldCount = 0;
		for (int idx = 0; idx < folders.size (); ++idx)
		{
			// check if it has a "world.lua" file
			Text worldLua = Session::Settings::Get ().worldsFolder + folders [idx] + "/data/world.lua";
			if (Utils::File_Exists (worldLua))
			{

				//Menus::Controls::ButtonToPage * control = aah_new Menus::Controls::ButtonToPage (xy, folders [idx], "level_loaded", frontEnd->pages, OnLoadLevelSelected, frontEnd);
				Menus::Controls::Button * control = aah_new Menus::Controls::Button (xy, folders [idx], OnPlayLevelSelected, frontEnd);
				page->Add (control); xy += p->vertSpacing;

				++validWorldCount;
			}
		}

		if (validWorldCount == 0)
		{
			page->Add (aah_new Menus::Controls::Title1 (xy, "No worlds to load")); xy += p->vertSpacing;
		}

		xy += p->vertSpacing;

		page->Add (aah_new Menus::Controls::ButtonToPage (xy, "Return to Main Menu", "singleplayer", frontEnd->pages)); xy += p->vertSpacing;
		page->MarkBottomRight (Int2 (p->labelControl.x + p->labelSize.x, xy.y));	
	}

	bool FrontEnd::OnPlayLevelSelected (Menus::Controls::Button * button, void * data)
	{
		FrontEnd * state = (FrontEnd *) data;
		state->all->playing->isRunningRequested = true;
		state->isRunningRequested = false;

		Session::Settings::Get ().worldInitParams.isNew = false;
		Session::Settings::Get ().worldInitParams.name = Utils::File_NeatenFolder ("tinyworld");//button->label);
		Session::Settings::Get ().worldInitParams.folder = Session::Settings::Get ().worldsFolder;
		Session::Settings::Get ().gamemode = Session::Settings::GM_PLAY;

		return true;
	}

	void FrontEnd::OnSaveSettings (Menus::Page * page, void * data)
	{
		Session::Settings::Save ();
	}

	bool FrontEnd::OnDrawDistanceToggled (Menus::Controls::Button * button, void * data)
	{
		int * idx = (int *) data;
		++*idx;
		*idx = *idx % 5; // hard coded crap for now

		return true;
	}
}
