#include "stdafx.h"

#include "Gfx/Gfx_Font.h"
#include "Input/Input_InputMngr.h"
#include "Input/Input_Keys.h"
#include "Input/Input_Mouse.h"

#include "../LayerFrontEnd/LayerFrontEnd_FrontEnd.h"
#include "../Layers/Layers_All.h"
#include "../Menus/Controls_Button.h"
#include "../Menus/Controls_ButtonCycle.h"
#include "../Menus/Controls_Checkbox.h"
#include "../Menus/Controls_TextEntry.h"
#include "../Menus/Controls_Title1.h"
#include "../Menus/Controls_Title2.h"
#include "../Menus/Menus_Page.h"
#include "../Menus/Menus_Palette.h"
#include "../Session/Session_Settings.h"

#include "LayerPlaying_Playing.h"
#include "LayerPlaying_PlayingDialog.h"

namespace LayerPlaying
{
	PlayingDialog::PlayingDialog (Layers::All * all, Input::InputMngr * inputMngr)
		: isRunningRequested (false),
		isRunning (false),
		all (all),
		inputMngr (inputMngr),
		font (NULL),
		page (NULL),
		returnToGameButton (NULL),
		hasShownHot (false),
		hasShownCold (false),
		hasWon (false)
	{
		font = aah_new Gfx::Font ("fonts/nokia_medium.glf");

		// create the menu page

		const Menus::Palette * p = Menus::Palette_Get (false);
		page = aah_new Menus::Page (inputMngr, font, "pause");

		Int2 xy (0, 0);
		page->Add (title = aah_new Menus::Controls::Title2 (xy, "a")); xy += p->vertSpacing;
		page->Add (text1 = aah_new Menus::Controls::Title1 (xy, "b")); xy += p->vertSpacing;
		page->Add (text2 = aah_new Menus::Controls::Title1 (xy, "c")); xy += p->vertSpacing;
		page->Add (text3 = aah_new Menus::Controls::Title1 (xy, "d")); xy += p->vertSpacing * 2;

		returnToGameButton = page->Add (aah_new Menus::Controls::Button (xy, "Continue")); xy += p->vertSpacing * 2;

		page->MarkBottomRight (Int2 (p->labelControl.x + p->labelSize.x, xy.y));
	}

	PlayingDialog::~PlayingDialog ()
	{
		delete page;
		delete font;
	}

	void PlayingDialog::Reset ()
	{
		hasShownHot = false;
		hasShownCold = false;
		hasWon = false;
	}

	void PlayingDialog::ShowDialog (Tiny::TINYITEMS item)
	{
		isRunningRequested = true;
		this->item = item;

		switch (item)
		{
		case Tiny::TI_AXE:
			title->SetText ("ITEM COLLECTED!");
			text1->SetText ("You have found an AXE.");
			text2->SetText ("You can chop thin tree trunks");
			text3->SetText ("with the left mouse button!");
			break;

		case Tiny::TI_JUMP_1:
			title->SetText ("ITEM COLLECTED!");
			text1->SetText ("You have found JUMP BOOTS.");
			text2->SetText ("You can jump just a");
			text3->SetText ("little bit higher!");
			break;

		case Tiny::TI_SHIELDCOLD:
			title->SetText ("ITEM COLLECTED!");
			text1->SetText ("You have found an ICE SHIELD.");
			text2->SetText ("You can get close");
			text3->SetText ("to ice worlds!");
			break;

		case Tiny::TI_BELT:
			title->SetText ("ITEM COLLECTED!");
			text1->SetText ("You have found a BELT OF STRENGTH.");
			text2->SetText ("You can get now crush broken rocks");
			text3->SetText ("with the left mouse button!");
			break;

		case Tiny::TI_SHIELDHOT:
			title->SetText ("ITEM COLLECTED!");
			text1->SetText ("You have found a HEAT SHIELD.");
			text2->SetText ("You can get closer to the sun");
			text3->SetText ("and pass the asteroid belt!");
			break;

		case Tiny::TI_TELEPORTER:
			title->SetText ("ITEM COLLECTED!");
			text1->SetText ("You have found a TELEPORTER.");
			text2->SetText ("Look at a red target and use the");
			text3->SetText ("left mouse button to teleport!");
			break;

		case Tiny::TI_JUMP_2:
			title->SetText ("ITEM COLLECTED!");
			text1->SetText ("You have found SUPER JUMP BOOTS.");
			text2->SetText ("You can jump even");
			text3->SetText ("higher still!");
			break;

		case Tiny::TI_SEEDS:
			title->SetText ("ITEM COLLECTED!");
			text1->SetText ("You have found some MAGIC BEANS.");
			text2->SetText ("Plant on BEAN squares with the left");
			text3->SetText ("mouse button to grow a jump pad plant!");
			break;

		case Tiny::TI_BIGAXE:
			title->SetText ("ITEM COLLECTED!");
			text1->SetText ("You have found the BIG AXE.");
			text2->SetText ("Chop down bigger tree trunks! You have all");
			text3->SetText ("the items. Now collect all the artifacts!");
			break;

		default: assert (false); break;
		}
	}

	void PlayingDialog::ShowWarnHotDialog ()
	{
		if (!hasShownHot)
		{
			isRunningRequested = true;
			title->SetText ("WARNING");
			text1->SetText ("You are too close to the SUN.");
			text2->SetText ("You can not pass the asteroid belt");
			text3->SetText ("until you have found a HEAT SHIELD.");
			hasShownHot = true;
		}
	}

	void PlayingDialog::ShowWarnColdDialog ()
	{
		if (!hasShownCold)
		{
			isRunningRequested = true;
			title->SetText ("WARNING");
			text1->SetText ("You are too close to an ICE WORLD.");
			text2->SetText ("You can not get any closer until");
			text3->SetText ("you have found an ICE SHIELD.");
			hasShownCold = true;
		}
	}

	void PlayingDialog::ShowDialogArtifacts (int artifacts, int maxArtifacts)
	{
		if (artifacts < maxArtifacts)
		{
			char buffer [256];
			

			isRunningRequested = true;
			title->SetText ("CONGRATULATIONS");
			sprintf (buffer,  "You have found artifact %d of %d!", artifacts, maxArtifacts);
			text1->SetText (buffer);
			text2->SetText ("Remember - collect them all");
			text3->SetText ("and be happy for ever! YMMV");
		}
		else
		{
			isRunningRequested = true;
			title->SetText ("CONGRATULATIONS");
			text1->SetText ("You found all the artifacts. But are you any");
			text2->SetText ("happier, really? You shouldn't believe in legends");
			text3->SetText ("at your age. Still, YOU BEAT THE TINY GAME!");
			hasWon = true;
		}
	}

	void PlayingDialog::ShowDead ()
	{
		isRunningRequested = true;
		title->SetText ("BOO HISS YOU DIED");
		text1->SetText ("You died. Any you were warned this game");
		text2->SetText ("only gives you one life per attempt. Don't");
		text3->SetText ("like it? Add the code yourself.");
		hasWon = true;
	}

	void PlayingDialog::Preupdate ()
	{
		if (isRunningRequested && !isRunning)
		{
			inputMngr->GetMouse ()->SetIsFpsMouse (false);
			page->Init (1);
			page->SetActive (true);
		}
		else if (!isRunningRequested && isRunning && all->playing->GetIsRunning ())
		{
			page->SetActive (false);
			inputMngr->GetMouse ()->SetIsFpsMouse (true);
		}
		isRunning = isRunningRequested;
	}

	void PlayingDialog::Update ()
	{
		if (isRunning)
		{
			page->Update ();
			if (inputMngr->GetKeys ()->IsPressed (SDLK_ESCAPE) || page->GetButtonPressed () == returnToGameButton)
			{
				isRunningRequested = false;
				if (hasWon)
				{
					all->frontEnd->isRunningRequested = true;
					all->playing->isRunningRequested = false;
				}
			}
		}
	}

	void PlayingDialog::Render ()
	{
		if (isRunning)
		{
			page->Render ();
		}
	}

	bool PlayingDialog::GetIsRunning () const
	{
		return isRunning;
	}

	bool PlayingDialog::OnDrawDistanceToggled (Menus::Controls::Button * button, void * data)
	{
		int * idx = (int *) data;
		++*idx;
		*idx = *idx % 5; // hard coded crap for now

		return true;
	}
}
