#include "stdafx.h"

#include "Gfx/Gfx_Font.h"
#include "Input/Input_InputMngr.h"
#include "Input/Input_Keys.h"
#include "Input/Input_Mouse.h"

#include "../LayerFrontEnd/LayerFrontEnd_FrontEnd.h"
#include "../Layers/Layers_All.h"
#include "../Menus/Controls_Button.h"
#include "../Menus/Controls_ButtonCycle.h"
#include "../Menus/Controls_Checkbox.h"
#include "../Menus/Controls_TextEntry.h"
#include "../Menus/Controls_Title1.h"
#include "../Menus/Controls_Title2.h"
#include "../Menus/Menus_Page.h"
#include "../Menus/Menus_Palette.h"
#include "../Session/Session_Settings.h"

#include "LayerPlaying_Playing.h"
#include "LayerPlaying_PlayingMenus.h"

namespace LayerPlaying
{
	PlayingMenus::PlayingMenus (Layers::All * all, Input::InputMngr * inputMngr)
		: isRunningRequested (false),
		isRunning (false),
		all (all),
		inputMngr (inputMngr),
		font (NULL),
		page (NULL),
		returnToGameButton (NULL),
		quitToFrontEndButton (NULL)
	{
		font = aah_new Gfx::Font ("fonts/nokia_medium.glf");

		// create the menu page

		const Menus::Palette * p = Menus::Palette_Get (false);
		page = aah_new Menus::Page (inputMngr, font, "pause");
		page->SetOnExit (OnSaveSettings, NULL);

		Int2 xy (0, 0);
		page->Add (aah_new Menus::Controls::Title2 (xy, "PLAYER SETTINGS")); xy += p->vertSpacing;
		page->Add (aah_new Menus::Controls::Checkbox (xy, "Invert Mouse", &Session::Settings::Get ().isCameraInverted)); xy += p->vertSpacing * 2;

		//Text * options = aah_new Text [5];
		//options [0] = "Small";
		//options [1] = "Medium";
		//options [2] = "Large";
		//options [3] = "Supersize";
		//options [4] = "Welcome To Your Doom";
		//page->Add (aah_new Menus::Controls::ButtonCycle (xy, options, 5, OnDrawDistanceToggled, &Session::Settings::Get ().drawDistance)); xy += p->vertSpacing * 2;

		returnToGameButton = page->Add (aah_new Menus::Controls::Button (xy, "Return to Game")); xy += p->vertSpacing * 2;

		quitToFrontEndButton = page->Add (aah_new Menus::Controls::Button (xy, "Quit to Front End")); xy += p->vertSpacing;

		page->MarkBottomRight (Int2 (p->labelControl.x + p->labelSize.x, xy.y));
	}

	PlayingMenus::~PlayingMenus ()
	{
		delete page;
		delete font;
	}

	void PlayingMenus::Preupdate ()
	{
		if (isRunningRequested && !isRunning)
		{
			inputMngr->GetMouse ()->SetIsFpsMouse (false);
			page->Init (1);
			page->SetActive (true);
		}
		else if (!isRunningRequested && isRunning && all->playing->GetIsRunning ())
		{
			page->SetActive (false);
			inputMngr->GetMouse ()->SetIsFpsMouse (true);
		}
		isRunning = isRunningRequested;
	}

	void PlayingMenus::Update ()
	{
		if (isRunning)
		{
			page->Update ();
			if (inputMngr->GetKeys ()->IsPressed (SDLK_ESCAPE) || page->GetButtonPressed () == returnToGameButton)
			{
				isRunningRequested = false;
			}
			else if (page->GetButtonPressed () == quitToFrontEndButton)
			{
				isRunningRequested = false;
				all->frontEnd->isRunningRequested = true;
				all->playing->isRunningRequested = false;
			}
		}
	}

	void PlayingMenus::Render ()
	{
		if (isRunning)
		{
			page->Render ();
		}
	}

	bool PlayingMenus::GetIsRunning () const
	{
		return isRunning;
	}

	void PlayingMenus::OnSaveSettings (Menus::Page * page, void * data)
	{
		Session::Settings::Save ();
	}

	bool PlayingMenus::OnDrawDistanceToggled (Menus::Controls::Button * button, void * data)
	{
		int * idx = (int *) data;
		++*idx;
		*idx = *idx % 5; // hard coded crap for now

		return true;
	}
}
