#include "stdafx.h"

#include "../LayerEditing/LayerEditing_Editing.h"
#include "../LayerEditing/LayerEditing_EditingMenus.h"
#include "../LayerFrontEnd/LayerFrontEnd_FrontEnd.h"
#include "../LayerPlaying/LayerPlaying_Playing.h"
#include "../LayerPlaying/LayerPlaying_PlayingMenus.h"
#include "../LayerPlaying/LayerPlaying_PlayingDialog.h"

#include "Layers_All.h"

namespace Layers
{
	All::All (Input::InputMngr * inputMngr)
		: isQuitRequested (false)
	{
		update.inputMngr = inputMngr;

		frontEnd = aah_new LayerFrontEnd::FrontEnd (this, inputMngr);
		editing = aah_new LayerEditing::Editing (this);
		editingMenus = aah_new LayerEditing::EditingMenus (this, inputMngr);
		playing = aah_new LayerPlaying::Playing (this);
		playingMenus = aah_new LayerPlaying::PlayingMenus (this, inputMngr);
		playingDialog = aah_new LayerPlaying::PlayingDialog (this, inputMngr);

		frontEnd->isRunningRequested = true;
	}

	All::~All ()
	{
		delete editing;
		delete editingMenus;
		delete playingDialog;
		delete playingMenus;
		delete playing;
		delete frontEnd;
	}

	void All::Update (RakNet::Time timeNow, float timeStepInSeconds)
	{	
		update.timeNow = timeNow;
		update.timeStep = timeStepInSeconds;

		frontEnd->Preupdate ();
		editing->Preupdate (&update);
		editingMenus->Preupdate ();
		playing->Preupdate (&update);
		playingMenus->Preupdate ();
		playingDialog->Preupdate ();

		frontEnd->Update ();
		editing->Update (&update);
		editingMenus->Update ();
		playing->Update (&update);
		playingMenus->Update ();
		playingDialog->Update ();
	}

	void All::UpdateFractionBlend (RakNet::Time timeNow, float weight)
	{
		//playing->UpdateFractionBlend (&update, weight);
	}

	void All::Render ()
	{
		frontEnd->Render ();
		editing->Render ();
		editingMenus->Render ();
		playing->Render ();
		playingMenus->Render ();
		playingDialog->Render ();
	}
}
