#include "stdafx.h"

#include "Gfx/Gfx_Font.h"
#include "Input/Input_InputMngr.h"
#include "Input/Input_Keys.h"
#include "Input/Input_Mouse.h"

#include "Controls_Button.h"
#include "Menus_Draw.h"
#include "Menus_Palette.h"

namespace Menus
{
	namespace Controls
	{
		Button::Button (Int2 tl, TextRef label) 
			: Base (tl),
			label (label),
			wasPressed (false),
			onExecuted (NULL),
			data (NULL)
		{
		}

		Button::Button (Int2 tl, TextRef label, OnExecuted onExecuted, void * data) 
			: Base (tl),
			label (label),
			wasPressed (false),
			onExecuted (onExecuted),
			data (data)
		{
		}

		Button::~Button () 
		{
		}

		bool Button::IsHighlighted (Int2 cursor) const
		{
			const Palette * p = Palette_Get (isFocused);
			return (cursor.y >= tl.y && cursor.y <= tl.y + p->labelSize.y);
		}

		void Button::Update (Input::InputMngr * inputMngr)
		{
			Input::Mouse * mouse = inputMngr->GetMouse ();
			const Input::Keys * keys = inputMngr->GetKeys ();
			if (isFocused)
			{
				if (mouse->Left.IsJustOn () || keys->IsPressed (SDLK_RETURN))
				{
					wasPressed = true;
				}
			}
		}

		void Button::Render (Int2 offset, int passIdx, Gfx::Font * font)
		{
			const Palette * p = Palette_Get (isFocused);
			Int2 size (p->labelSize.y, p->labelSize.y);

			if (passIdx == 0)
			{
				if (isHighlighted)
				{
					int thickness = 4;
					Int2 fullWidth (-offset.x, tl.y - thickness);
					Int2 fullSize (2000, p->labelSize.y + thickness * 2);
					Draw_Box (offset + fullWidth, fullSize, p->highlight);
				}
				Draw_BoxOutline (offset + tl + p->centreLabelPos, p->labelSize, p->thickness, p->border, p->fill);
			}
			else
			{
				Int2 length ((p->labelSize.x - font->GetLength (label.Raw ())) / 2, p->textOffset.y);
				Draw_Text (offset + tl + p->centreLabelPos + length, label, font);
			}
		}

		bool Button::Executed ()
		{
			if (onExecuted != NULL)
			{
				if (!onExecuted (this, data))
				{
					wasPressed = false;
					return false;
				}
			}
			return true;
		}
	}
}
