#include "stdafx.h"

#include "Controls_ButtonCycle.h"
#include "Menus_Draw.h"
#include "Menus_Palette.h"

namespace Menus
{
	namespace Controls
	{
		ButtonCycle::ButtonCycle (Int2 tl, Text * options, int optionsCount, OnExecuted onExecuted, int * currentIdx)
			: Button (tl, "Draw Distance", onExecuted, NULL),
			options (options),
			optionsCount (optionsCount),
			currentIdx (currentIdx)
		{
			assert (options != NULL);
			assert (onExecuted != NULL);
			assert (currentIdx != NULL);
		}

		ButtonCycle::~ButtonCycle ()
		{
			delete [] options;
		}

		void ButtonCycle::Render (Int2 offset, int passIdx, Gfx::Font * font)
		{
			const Palette * p = Palette_Get (isFocused);

			if (passIdx == 0)
			{
				if (isHighlighted)
				{
					int thickness = 4;
					Int2 fullWidth (-offset.x, tl.y - thickness);
					Int2 fullSize (2000, p->labelSize.y + thickness * 2);
					Draw_Box (offset + fullWidth, fullSize, p->highlight);
				}
				Draw_BoxOutline (offset + tl, p->labelSize, p->thickness, p->border, p->fill);
				Draw_BoxOutline (offset + tl + p->labelControl, p->labelSize, p->thickness, p->border, p->fill);
			}
			else
			{
				Draw_Text (offset + tl + p->textOffset, label, font);
				Draw_Text (offset + tl + p->labelControl + p->textOffset, options [*currentIdx].Raw (), font);
			}
		}

		bool ButtonCycle::Executed ()
		{
			int idx = *currentIdx;
			if (onExecuted (this, currentIdx))
			{
				*currentIdx = *currentIdx % optionsCount;
			}
			else
			{
				*currentIdx = idx;
			}
			wasPressed = false;						

			return false;
		}
	}
}
