#include "stdafx.h"

#include "Input/Input_InputMngr.h"
#include "Input/Input_Keys.h"
#include "Input/Input_Mouse.h"

#include "Controls_Checkbox.h"
#include "Menus_Draw.h"
#include "Menus_Palette.h"

namespace Menus
{
	namespace Controls
	{
		Checkbox::Checkbox (Int2 tl, const char * label, bool * variable) 
			: Base (tl),
			label (label),
			variable (variable)
		{
		}

		Checkbox::~Checkbox () 
		{
		}

		bool Checkbox::IsHighlighted (Int2 cursor) const
		{
			const Palette * p = Palette_Get (isFocused);
			return (cursor.y >= tl.y && cursor.y <= tl.y + p->labelSize.y);
		}

		void Checkbox::Update (Input::InputMngr * inputMngr)
		{
			const Input::Mouse * mouse = inputMngr->GetMouse ();
			const Input::Keys * keys = inputMngr->GetKeys ();
			if (isFocused)
			{
				if (mouse->Left.IsJustOn () || keys->IsPressed (SDLK_RETURN))
				{
					*variable = !*variable;
				}
			}
		}

		void Checkbox::Render (Int2 offset, int passIdx, Gfx::Font * font)
		{
			const Palette * p = Palette_Get (isFocused);
			Int2 size (p->labelSize.y, p->labelSize.y);

			if (passIdx == 0)
			{
				if (isHighlighted)
				{
					int thickness = 4;
					Int2 fullWidth (-offset.x, tl.y - thickness);
					Int2 fullSize (2000, p->labelSize.y + thickness * 2);
					Draw_Box (offset + fullWidth, fullSize, p->highlight);
				}
				Draw_BoxOutline (offset + tl, p->labelSize, p->thickness, p->border, p->fill);
				Draw_BoxOutline (offset + tl + p->labelControl, size, p->thickness, p->border, p->fill);
			}
			else
			{
				Draw_Text (offset + tl + p->textOffset, label, font);
				Draw_Text (offset + tl + p->labelControl + p->textOffset, (*variable) ? "Y" : " ", font);
			}
		}
	}
}
