#include "stdafx.h"

#include "Gfx/Gfx_Font.h"
#include "Input/Input_InputMngr.h"
#include "Input/Input_Keys.h"
#include "Input/Input_Mouse.h"

#include "Controls_RecordKey.h"
#include "Menus_Draw.h"
#include "Menus_Palette.h"

namespace Menus
{
	namespace Controls
	{
		RecordKey::RecordKey (Int2 tl, const char * label, SDLKey * key, IsUniqueKey isUniqueKey) 
			: Base (tl),
			isUniqueKey (isUniqueKey),
			label (label),
			key (key),
			isAwaitingNewKey (false)
		{
		}

		RecordKey::~RecordKey () 
		{
		}

		bool RecordKey::IsHighlighted (Int2 cursor) const
		{
			const Palette * p = Palette_Get (isFocused);
			return (cursor.y >= tl.y && cursor.y <= tl.y + p->labelSize.y);
		}

		void RecordKey::SetIsFocused (bool isFocused)
		{
			if (this->isFocused != isFocused)
			{
				if (!isFocused)
				{
					isAwaitingNewKey = false;
				}
				Base::SetIsFocused (isFocused);
			}
		}

		void RecordKey::Update (Input::InputMngr * inputMngr)
		{
			blinker = (blinker + 1) % 30;

			const Input::Mouse * mouse = inputMngr->GetMouse ();
			Input::Keys * keys = inputMngr->GetKeys ();
			if (isFocused)
			{
				if (isAwaitingNewKey)
				{
					if (keys->IsDown (SDLK_ESCAPE))
					{
						isAwaitingNewKey = false;
					}
					else
					{
						for (int idx = 1; idx < SDLK_LAST; ++idx)
						{
							if (keys->IsPressed ((SDLKey) idx) && keys->isRegularInputKey ((SDLKey) idx))
							{
								*key = (SDLKey) idx;
								isAwaitingNewKey = false;
								break;
							}
						}
					}
				}
				else if (mouse->Left.IsJustOn () || keys->IsPressed (SDLK_RETURN))
				{
					isAwaitingNewKey = true;
				}
			}
		}

		void RecordKey::Render (Int2 offset, int passIdx, Gfx::Font * font)
		{
			const Palette * p = Palette_Get (isFocused);

			if (passIdx == 0)
			{
				if (isHighlighted)
				{
					int thickness = 4;
					Int2 fullWidth (-offset.x, tl.y - thickness);
					Int2 fullSize (2000, p->labelSize.y + thickness * 2);
					Draw_Box (offset + fullWidth, fullSize, p->highlight);
				}
				Draw_BoxOutline (offset + tl, p->labelSize, p->thickness, p->border, p->fill);
				Draw_BoxOutline (offset + tl + p->labelControl, p->labelSize, p->thickness, p->border, p->fill);
			}
			else
			{
				Draw_Text (offset + tl + p->textOffset, label, font);
		
				if (isAwaitingNewKey)
				{
					if (blinker < 15)
					{
						const char * text = "press new key or ESCAPE to cancel";
						Int2 length ((p->labelSize.x - font->GetLength (text)) / 2, p->textOffset.y);
						Draw_Text (offset + tl + p->labelControl + length, text, font);
					}
				}
				else
				{
					if (blinker < 15 && isUniqueKey != NULL && !isUniqueKey (*key))
					{
						const char * text = "CLASH";
						Int2 length ((p->labelSize.x - font->GetLength (text)) - p->textOffset.x, p->textOffset.y);
						Draw_Text (offset + tl + length, text, font);
					}
	
					char buffer [32];
					sprintf (buffer, "%s", SDL_GetKeyName (*key));
					Draw_Text (offset + tl + p->labelControl + p->textOffset, buffer, font);

					sprintf (buffer, "%d", *key);
					Int2 length ((p->labelSize.x - font->GetLength (buffer)) - p->textOffset.x, p->textOffset.y);
					Draw_Text (offset + tl + p->labelControl + length, buffer, font);
				}
			}
		}
	}
}
