#include "stdafx.h"

#include "Gfx/Gfx_Font.h"
#include "Input/Input_InputMngr.h"
#include "Input/Input_Keys.h"

#include "Controls_TextEntry.h"
#include "Menus_Draw.h"
#include "Menus_Palette.h"

namespace Menus
{
	namespace Controls
	{
		TextEntry::TextEntry (Text::VALIDATETYPES entryType, Int2 tl, const char * label, int maxLength, Text * text) 
			: Base (tl),
			label (label),
			maxLength (maxLength),
			text (text),
			entryType (entryType)
		{
		}

		TextEntry::~TextEntry () 
		{
		}

		bool TextEntry::IsHighlighted (Int2 cursor) const
		{
			const Palette * p = Palette_Get (isFocused);
			return (cursor.y >= tl.y && cursor.y <= tl.y + p->labelSize.y);
		}

		void TextEntry::SetIsFocused (bool isFocused)
		{
			if (this->isFocused != isFocused)
			{
				blinker = 0;
				Base::SetIsFocused (isFocused);
			}
		}

		void TextEntry::Update (Input::InputMngr * inputMngr)
		{
			if (isFocused)
			{
				blinker = (blinker + 1) % 30;
				Input::Keys * keys = inputMngr->GetKeys ();

				for (int key = keys->ConsumeTextEntry (); key != 0; key = keys->ConsumeTextEntry ())
				{
					switch (key)
					{
					case SDLK_RETURN: 
						break;

					case SDLK_BACKSPACE: 
						{
							int length = text->Length ();
							if (length > 0)
							{
								text->Truncate (length - 1);
							}
						}
						break;

					default:
						{
							if (Text::ValidateChar (entryType, key))
							{
								int length = text->Length ();
								if (length < maxLength)
								{
									*text += (char) key;
								}
							}
						}
						break;
					}
				}
			}
		}

		void TextEntry::Render (Int2 offset, int passIdx, Gfx::Font * font)
		{
			const Palette * p = Palette_Get (isFocused);

			if (passIdx == 0)
			{
				if (isHighlighted)
				{
					int thickness = 4;
					Int2 fullWidth (-offset.x, tl.y - thickness);
					Int2 fullSize (2000, p->labelSize.y + thickness * 2);
					Draw_Box (offset + fullWidth, fullSize, p->highlight);
				}
				Draw_BoxOutline (offset + tl, p->labelSize, p->thickness, p->border, p->fill);
				Draw_BoxOutline (offset + tl + p->labelControl, p->labelSize, p->thickness, p->border, p->fill);

				if (isFocused && blinker < 15)
				{
					int length = font->GetLength (text->Raw ());
					if (length > 0)
					{
						length += 2;
					}
					Draw_Box (offset + tl + p->labelControl + p->textOffset + Int2 (length, 0), Int2 (18, 18), Int4 (255, 255, 255, 255));
				}
			}
			else
			{
				Draw_Text (offset + tl + p->textOffset, label, font);
				Draw_Text (offset + tl + p->labelControl + p->textOffset, text->Raw (), font);
			}
		}
	}
}
