#pragma once

#include <vector>

namespace Gfx {class Font;}
namespace Input {class InputMngr;}
namespace Menus {namespace Controls {class Base;}}
namespace Menus {namespace Controls {class Button;}}
namespace Menus {namespace Notes {class NoteBase;}}

namespace Menus
{
	class Page
	{
	public:

		typedef void (*PageCallback) (Page * page, void * data);

		Page (Input::InputMngr * inputMngr, Gfx::Font * font, TextRef id);
		~Page ();

		Menus::Controls::Button * Add (Menus::Controls::Button * control);
		Menus::Controls::Base * Add (Menus::Controls::Base * control);		
		void MarkBottomRight (Int2 pageSize);

		void SetOnEntry (PageCallback callback, void * data);
		void SetOnExit (PageCallback callback, void * data);
		
		void RemoveControls ();

		void SetNote (Notes::NoteBase * note);

		void Init (bool resetPage);
		void SetActive (bool isActive);
		void Update ();
		void Render ();

		int GetControlCount () const;
		Menus::Controls::Button * GetButtonPressed () const;

		const Text Id;

	private:

		void ChangeHighlighted (int newIdx);
		void ChangeFocused (int newIdx);
		int FindNextHighlightableControl (int startIdx, int step) const;

		Input::InputMngr * inputMngr;
		Gfx::Font * font;

		bool isActive;

		Int2 pageSize;
		Int2 toCentre;
		int highlightedControl;
		int focusedControl;

		bool isMouseHighlighting;
		Int2 oldMouse;

		std::vector <Menus::Controls::Base *> controls;
		std::vector <Menus::Controls::Button *> buttons;

		Menus::Controls::Button * buttonPressed;
		int buttonPressedTimer;

		PageCallback onEntryCallback; 
		void * onEntryData;
		PageCallback onExitCallback; 
		void * onExitData;

		Notes::NoteBase * note;
	};
}
