#include "stdafx.h"

#include "Lua/Lua_LuaState.h"
#include "Utils/Utils_File.h"

#include "Session_Settings.h"

namespace Session
{
	Settings * Settings::instance = NULL;

	void Settings::Load (TextRef settingsFilename)
	{
		assert (instance == NULL);

		instance = aah_new Settings (settingsFilename);

		const char * raw = NULL;

		Lua::LuaState * lua = instance->clientSettingsLua;

		raw = lua->ReadString ("settings/network/client/connect_to_ip");
		Get ().connectToIp = raw;

		raw = lua->ReadString ("settings/network/client/connect_to_port");
		Get ().connectToPort = raw;

		raw = lua->ReadString ("settings/network/client/listen_to_port");
		Get ().listenToPort = raw;

		raw = lua->ReadString ("settings/menus/worlds_folder");
		Get ().worldsFolder = raw;
		Get ().worldsFolder = Utils::File_NeatenFolder (Get ().worldsFolder);

		raw = lua->ReadString ("settings/menus/default_new_world");
		Get ().defaultWorld = raw;

		raw = lua->ReadString ("settings/menus/default_random_seed");
		Get ().worldInitParams.randomSeed = raw;

		Get ().isCameraInverted = lua->ReadBool ("settings/player_account/controls/mouse_invert");

		raw = lua->ReadString ("settings/player_account/nickname");
		Get ().nickname = raw;
		Get ().nickname.Truncate (15);
		
		Get ().rgb = Vec3 (1.0f, 0.0f, 0.0f);//lua->ReadVec3 ("diorama/settings/client/rgb");

		Get ().up = (SDLKey) lua->ReadInt ("settings/player_account/controls/key_up");
		Get ().down = (SDLKey) lua->ReadInt ("settings/player_account/controls/key_down");
		Get ().left = (SDLKey) lua->ReadInt ("settings/player_account/controls/key_left");
		Get ().right = (SDLKey) lua->ReadInt ("settings/player_account/controls/key_right");
		Get ().jump = (SDLKey) lua->ReadInt ("settings/player_account/controls/key_jump");
		Get ().crouch = (SDLKey) lua->ReadInt ("settings/player_account/controls/key_crouch");
		Get ().chat = (SDLKey) lua->ReadInt ("settings/player_account/controls/key_chat");
		Get ().showScores = (SDLKey) lua->ReadInt ("settings/player_account/controls/key_scores");

		Get ().drawDistance = lua->ReadInt ("settings/graphics/drawDistance");
	}

	void Settings::Unload ()
	{
		assert (instance != NULL);
		delete instance;
		instance = NULL;
	}

	void Settings::Save ()
	{
		assert (instance != NULL);

		// copy over the awkward data back into lua

		Lua::LuaState * lua = instance->clientSettingsLua;

		lua->SetBool ("settings/player_account/controls/mouse_invert", Get ().isCameraInverted);

		lua->SetString ("settings/player_account/nickname", Get ().nickname);

		lua->SetString ("settings/network/client/connect_to_ip", Get ().connectToIp);
		lua->SetInt ("settings/network/client/connect_to_port", atoi (Get ().connectToPort.Raw ()));
		lua->SetInt ("settings/network/client/listen_to_port", atoi (Get ().listenToPort.Raw ()));

		lua->SetInt ("settings/player_account/controls/key_up", Get ().up);
		lua->SetInt ("settings/player_account/controls/key_down", Get ().down);
		lua->SetInt ("settings/player_account/controls/key_left", Get ().left);
		lua->SetInt ("settings/player_account/controls/key_right", Get ().right);
		lua->SetInt ("settings/player_account/controls/key_jump", Get ().jump);
		lua->SetInt ("settings/player_account/controls/key_crouch", Get ().crouch);
		lua->SetInt ("settings/player_account/controls/key_chat", Get ().chat);
		lua->SetInt ("settings/player_account/controls/key_scores", Get ().showScores);

		lua->SetInt ("settings/graphics/drawDistance", Get ().drawDistance);

		instance->clientSettingsLua->SaveText_GlobalTable (Get ().Get ().settingsFilename, "settings", Lua::LuaState::WL_SAVEDREPEATEDLY);
	}

	Settings & Settings::Get ()
	{
		assert (instance != NULL);
		return *instance;
	}

	Settings::Settings (TextRef settingsFilename)
		: gamemode (GM_NONE),
		worldsFolder ("../worlds/"),
		defaultWorld ("New World"),
		settingsFilename (settingsFilename)
	{
		clientSettingsLua = aah_new Lua::LuaState (settingsFilename);
		worldInitParams.name = defaultWorld;
	}

	Settings::~Settings ()
	{
		delete clientSettingsLua;
	}

	//unsigned int Settings::GetWorldRandomSeed ()
	//{
	//	return RakNet::RakString::ToInteger (randomSeed.Raw ());
	//}
}
