#include "stdafx.h"

#include "SceneSim/SceneSim_Column.h"
#include "SceneSim/SceneSim_ColumnCells.h"

#include "CellMesh_Defs.h"
#include "CellMesh_Mngr.h"

namespace CellMesh
{
	AddShapeData::AddShapeData 
	(
		int blockId,
		const SceneSim::AdjColumns * columns, 
		const CellMesh::Mngr * meshMngr,
		Gfx::VertexTypes::XyzUvRgb * verts,
		Gfx::VertexTypes::XyzUvRgb * extraVerts
	)
		: blockId (blockId),
		meshMngr (meshMngr),
		verts (verts),
		vertsUsed (0),
		extraVerts (extraVerts),
		extraVertsUsed (0),
		font (NULL),
		recreateBlock (NULL)
	{
		assert (columns != NULL);
		assert (columns->main != NULL);

		memset (&cells, 0, sizeof (cells));
		memset (&shapes, 0, sizeof (shapes));

		//for (int i = 0; i < (SceneSim::CellsPerColXz + 2) * 3; ++i)
		//{
		//	((SceneSim::Cell *) &cells) [i].lightSrc = 15;
		//}

		const SceneSim::ColumnCells * srcCells = columns->main->GetCells ();
		const int height = columns->main->GetSize ().y;

		for (int x = 0; x < SceneSim::CellsPerColXz; ++x)
		{
			for (int y = 0; y < height; ++y)
			{
				for (int z = 0; z < SceneSim::CellsPerColXz; ++z)
				{
					cells [x + 1][y + 1][z + 1] = *srcCells->GetCell (x, y, z);
				}
			}
		}
	
		if (columns->n != NULL && columns->n->flags != SceneSim::Column::SF_NO_CELLS)
		{
			srcCells = columns->n->GetCells ();
			for (int x = 0; x < SceneSim::CellsPerColXz; ++x)
			{
				for (int y = 0; y < height; ++y)
				{
					cells [x + 1][y + 1][SceneSim::CellsPerColXz + 1] = *srcCells->GetCell (x, y, 0);
				}
			}
		}

		if (columns->e != NULL && columns->e->flags != SceneSim::Column::SF_NO_CELLS)
		{
			srcCells = columns->e->GetCells ();
			for (int z = 0; z < SceneSim::CellsPerColXz; ++z)
			{
				for (int y = 0; y < height; ++y)
				{
					cells [SceneSim::CellsPerColXz + 1][y + 1][z + 1] = *srcCells->GetCell (0, y, z);
				}
			}
		}

		if (columns->s != NULL && columns->s->flags != SceneSim::Column::SF_NO_CELLS)
		{
			srcCells = columns->s->GetCells ();
			for (int x = 0; x < SceneSim::CellsPerColXz; ++x)
			{
				for (int y = 0; y < height; ++y)
				{
					cells [x + 1][y + 1][0] = *srcCells->GetCell (x, y, SceneSim::CellsPerColXz - 1);
				}
			}
		}

		if (columns->w != NULL && columns->w->flags != SceneSim::Column::SF_NO_CELLS)
		{
			srcCells = columns->w->GetCells ();
			for (int z = 0; z < SceneSim::CellsPerColXz; ++z)
			{
				for (int y = 0; y < height; ++y)
				{
					cells [0][y + 1][z + 1] = *srcCells->GetCell (SceneSim::CellsPerColXz - 1, y, z);
				}
			}
		}

		for (int x = 0; x < SceneSim::CellsPerColXz + 2; ++x)
		{
			for (int y = 0; y < height + 2; ++y)
			{
				for (int z = 0; z < SceneSim::CellsPerColXz + 2; ++z)
				{
					shapes [x][y][z] = meshMngr->GetShape (cells [x][y][z].type);
				}
			}
		}
	}	

	const SceneSim::Cell * AddShapeData::GetCell (Int3 cellId) const
	{
		return &cells [cellId.x + 1][cellId.y + 1][cellId.z + 1];
	}

	const CellMesh::ShapeBase * AddShapeData::GetShape (Int3 cellId) const
	{
		return shapes [cellId.x + 1][cellId.y + 1][cellId.z + 1];
	}
}
