#include "stdafx.h"

#include "SceneSim/Query_Cell.h"
#include "SceneSim/SceneSim_Column.h"
#include "SceneSim/SceneSim_ColumnCells.h"

#include "../Gfx/VertexTypes_XyzUvRgb.h"
#include "../SceneGfx/SceneGfx_Column.h"
#include "../SceneGfx/SceneGfx_Room.h"

#include "CellMesh_Defs.h"
#include "CellMesh_Mngr.h"
#include "CellMesh_ShapeBase.h"
#include "Shapes_InitData.h"

using namespace SceneSim;

namespace CellMesh
{

	SceneSim::Cell emptyLightCell =
	{	
		0, 
		0,
		15,
		0
	};


	//const FACES facesFromFlag [] =
	//{
	//	F_NONE,
	//	F_T,
	//	F_B,
	//	F_NONE,
	//	F_N,
	//	F_NONE, F_NONE, F_NONE,
	//	F_E,
	//	F_NONE, F_NONE, F_NONE, F_NONE, F_NONE, F_NONE, F_NONE,
	//	F_S,
	//	F_NONE, F_NONE, F_NONE, F_NONE, F_NONE, F_NONE, F_NONE, F_NONE, 
	//	F_NONE, F_NONE, F_NONE, F_NONE, F_NONE, F_NONE, F_NONE, 
	//	F_W,
	//};

	const SceneSim::FACES oppFaceFlags [] =
	{
		F_B,
		F_T,
		F_S,
		F_W,
		F_N,
		F_E,
	};

	const Int3 compassOffsets [] = 
	{
		Int3 (0, 1, 0),
		Int3 (0, -1, 0),
		Int3 (0, 0, 1),
		Int3 (1, 0, 0),
		Int3 (0, 0, -1),
		Int3 (-1, 0, 0),
		Int3 (0, 0, 0),
	};

	const FACES remapFaces [F_COUNT_CUBE][F_COUNT_ALL] =
	{
		{F_N, F_S, F_B, F_E, F_T, F_W, F_OTHER}, // F_B
		{F_S, F_N, F_T, F_E, F_B, F_W, F_OTHER}, // F_T - not done yet
		{F_T, F_B, F_N, F_E, F_S, F_W, F_OTHER}, // F_N - no change
		{F_T, F_B, F_E, F_S, F_W, F_N, F_OTHER}, // F_E
		{F_T, F_B, F_S, F_W, F_N, F_E, F_OTHER}, // F_S
		{F_T, F_B, F_W, F_N, F_E, F_S, F_OTHER}, // F_W
	};

	int defaultRemapTexId (int meshIdx, int texId, void * data)
	{
		return texId;
	}

	const SceneSim::Cell * GetCell (const SceneSim::ColumnCells * cells, Int3 cellId, Int3 offset)
	{
		Int3 next = (cellId + offset);
		if (next.x < 0 || next.x >= SceneSim::CellsPerColXz ||
			next.y < 0 || next.y >= SceneSim::CellsPerColY ||
			next.z < 0 || next.z >= SceneSim::CellsPerColXz)
		{
			return cells->GetCell (cellId);//&SceneSim::ColumnCells::dummyAirCell;
		}

		return cells->GetCell (next);
	}

	ShapeBase::ShapeBase ()
		: gfxBlocking (GB_ALL),
		isFullBright (false),
		meshes (NULL),
		meshTotal (0),
		meshIdxFromCellDataFuncPtr (NULL),
		awfulCalcDataFuncPtr (NULL),
		newAddExtraStuff (NULL),
		inventoryIconIdx (-1)
	{
	}

	ShapeBase::~ShapeBase ()
	{
		delete [] meshes;
	}

	bool ShapeBase::HasMesh () const
	{
		return (meshTotal > 0);
	}

	void ShapeBase::AddInstance (AddShapeData * data, Int3 cellId) const
	{
		const SceneSim::Cell * cell = data->GetCell (cellId);
		
		int meshToUseIdx = 0;
		if (meshIdxFromCellDataFuncPtr != NULL)
		{
			
			int blockData = cell->blockData;
			if (awfulCalcDataFuncPtr != NULL)
			{
				blockData = awfulCalcDataFuncPtr (data);
			}
			meshToUseIdx = meshIdxFromCellDataFuncPtr (cell->type, blockData);
			assert (meshToUseIdx >= 0 && meshToUseIdx < meshTotal);
		}

		const SingleMesh * singleMesh = &meshes [meshToUseIdx];

		int visibleFaces = 0;
		//if (data->recreateBlock == NULL)
		{
			for (int adj = 0; adj < F_COUNT_CUBE; ++adj)
			{
				if (singleMesh->hasFaces & (1 << adj))
				{
					const SceneSim::Cell * adjCell = data->GetCell (cellId + compassOffsets [adj]); 
					const CellMesh::ShapeBase * shape = data->GetShape (cellId + compassOffsets [adj]);
	
					assert (shape != NULL);

					const bool isBlockedBySameType = (shape->gfxBlocking == CellMesh::GB_SAMETYPE && adjCell->type == cell->type);

					//if (!isBlockedBySameType &&
					//	(shape->meshTotal == 0 || 
					//	(shape->meshes [0].canBlockAdjFaces & (1 << oppFaceFlags [adj])) == 0))
					//{
					//	visibleFaces |= (1 << adj);
					//}

					if (shape->gfxBlocking == CellMesh::GB_NONE ||
						(shape->gfxBlocking == CellMesh::GB_SAMETYPE && adjCell->type != cell->type))
					{
						visibleFaces |= (1 << adj);
					}
				}
			}
		}
		//else
		//{
		//	assert (false);
			//for (int adj = 0; adj < F_COUNT_CUBE; ++adj)
			//{
			//	Int3 offset = cellXyz + compassOffsets [adj];
			//	const Cell * cell = data->recreateBlock->GetCell (offset);
			//	assert (cell != NULL);

			//	if (data->recreateBlock->builderMngr->GetShape (cell->type)->canHideFaces != 0)
			//	//const CellGfx::Shape::ShapeDef * shapeDef = data->recreateBlock->uvSetMngr->GetUvSet (cell->type);
			//	//bool canBlock = (shapeDef != NULL) ? shapeDef->canHideFaces : true;
			//	//if (!canBlock)
			//	{
			//		isBlocked |= (1 << adj);
			//	}
			//}
		//}

		visibleFaces = singleMesh->hasFaces & (visibleFaces | FF_OTHER);

		int drawn = 0;
		int extraDrawn = 0;
		if (visibleFaces != 0)
		{
			for (int idx = 0; idx < singleMesh->vertsCount / 3; ++idx)
			{
				int faceBit = singleMesh->triFacings [idx] & visibleFaces;
				if (faceBit != 0)
				{
					memcpy (&data->verts [drawn * 3], &singleMesh->verts [idx * 3], sizeof (Gfx::VertexTypes::XyzUvRgb) * 3);

					int wibble = 0;
					for (wibble = 0; faceBit; faceBit >>= 1, ++wibble)
					{
					}
					FACES lightMapping = (FACES) (wibble - 1);

					//FACES lightMapping = singleMesh->triLightMappings [idx];
					//if (singleMesh->triFacings [idx] & FF_OTHER)
					//{
					//	if (!(isBlocked & 1 << lightMapping))
					//	{
					//		lightMapping = F_OTHER;
					//	}
					//}

					float light = 1.0f;
					if (!isFullBright)
					{
						const SceneSim::Cell * adjCell = data->GetCell (cellId + compassOffsets [lightMapping]); 

						//const Cell & adjCell = adjCells [lightMapping];
						//light = 15.0f;

						//if (adjCell->lightSky == 15)
						//{
						//	light = cellId.y * 0.75f / 127.0f + 0.25f;
						//}
						//else
						//{
						//	light = cellId.y * 0.75f / 127.0f + 0.1f;
						//}

						light = (float) (adjCell->lightSky > adjCell->lightSrc) ? adjCell->lightSky : adjCell->lightSrc;
						light /= 15.0f;
						light *= light * light;

						Vec4 rgba (255.0f / 255.0f * light, 255.0f / 255.0f * light, 255.0f / 255.0f * light, 1.0f);
						data->verts [drawn * 3 + 0].rgba *= rgba;
						data->verts [drawn * 3 + 1].rgba *= rgba;
						data->verts [drawn * 3 + 2].rgba *= rgba;
					}
					else
					{
						Vec4 rgba (1.0f, 1.0f, 1.0f, 1.0f);
						data->verts [drawn * 3 + 0].rgba = rgba;
						data->verts [drawn * 3 + 1].rgba = rgba;
						data->verts [drawn * 3 + 2].rgba = rgba;
					}

					++drawn;
				}
			}
		}

		if (newAddExtraStuff != NULL)
		{
			extraDrawn += newAddExtraStuff (data, extraDrawn);
		}

		data->verts += (drawn * 3);
		data->extraVerts += (extraDrawn * 3);
	}

	void ShapeBase::Init (InitData * data, int inventoryIdx)
	{
		inventoryIconIdx = inventoryIdx;

		if (meshes != NULL)
		{
			delete [] meshes;
			meshes = NULL;
		}

		meshTotal = data->meshTotal;
		meshIdxFromCellDataFuncPtr = data->meshIdxFromCellDataFuncPtr;
		awfulCalcDataFuncPtr = data->awfulCalcDataFuncPtr;
		newAddExtraStuff = data->newAddExtraStuff;

		meshes = aah_new SingleMesh [meshTotal];

		for (int idx = 0; idx < meshTotal; ++idx)
		{
			const InitData::PerMesh * perMesh = &data->perMesh [idx];

			meshes [idx].Init (perMesh->trisPerMeshTotal);
			meshes [idx].hasFaces = perMesh->hasFaces;
			meshes [idx].canBlockAdjFaces = perMesh->canBlockAdjFaces;
		}
	}

	void ShapeBase::Init_AddQuad (InitData * data, int meshIdx, FACES group, Vec3 origin, Vec3 w, Vec3 h, FACES texMapping, int texId)
	{
		Vec2 noUv (0.0f, 0.0f);
		Vec4 white (1.0f, 1.0f, 1.0f, 1.0f);

		Gfx::VertexTypes::XyzUvRgb * verts = &meshes [meshIdx].verts [data->perMesh [meshIdx].currentTriIdx * 3];
		InitData::PerMesh * perMesh = &data->perMesh [meshIdx];

		verts [0].Set (origin, noUv, white);
		verts [1].Set (origin + w, noUv, white);
		verts [2].Set (origin + w + h, noUv, white);
		perMesh->triFacings [perMesh->currentTriIdx] = group;
		perMesh->triLightMappings [perMesh->currentTriIdx] = texMapping;
		perMesh->triTexIds [perMesh->currentTriIdx] = texId;
		++perMesh->currentTriIdx;

		verts [3] = verts [2];
		verts [4].Set (origin + h, noUv, white);
		verts [5] = verts [0];
		perMesh->triFacings [perMesh->currentTriIdx] = group;
		perMesh->triLightMappings [perMesh->currentTriIdx] = texMapping;
		perMesh->triTexIds [perMesh->currentTriIdx] = texId;
		++perMesh->currentTriIdx;
	}

	void ShapeBase::Init_AddTri (InitData * data, int meshIdx, SceneSim::FACES group, Vec3 origin, Vec3 offset1, Vec3 offset2, SceneSim::FACES texMapping, int texId)
	{
		Vec2 noUv (0.0f, 0.0f);
		Vec4 white (1.0f, 1.0f, 1.0f, 1.0f);

		Gfx::VertexTypes::XyzUvRgb * verts = &meshes [meshIdx].verts [data->perMesh [meshIdx].currentTriIdx * 3];
		InitData::PerMesh * perMesh = &data->perMesh [meshIdx];

		verts [0].Set (origin, noUv, white);
		verts [1].Set (origin + offset1, noUv, white);
		verts [2].Set (origin + offset2, noUv, white);
		perMesh->triFacings [perMesh->currentTriIdx] = group;
		perMesh->triLightMappings [perMesh->currentTriIdx] = texMapping;
		perMesh->triTexIds [perMesh->currentTriIdx] = texId;
		++perMesh->currentTriIdx;
	}

	void ShapeBase::Init_AddBoxQuad (InitData * data, int meshIdx, SceneSim::FACES group, Vec3 bl, Vec3 tr, SceneSim::FACES texMapping, int texId)
	{
		Vec3 d (tr - bl);

		switch (texMapping)
		{
		case SceneSim::F_T: Init_AddQuad (data, meshIdx, group, Vec3 (bl.x, tr.y, tr.z), Vec3 (d.x, 0.0f, 0.0f), Vec3 (0.0f, 0.0f, -d.z), texMapping, texId); break;
		case SceneSim::F_B:	Init_AddQuad (data, meshIdx, group, Vec3 (bl.x, bl.y, bl.z), Vec3 (d.x, 0.0f, 0.0f), Vec3 (0.0f, 0.0f, d.z), texMapping, texId); break;
		case SceneSim::F_N:	Init_AddQuad (data, meshIdx, group, Vec3 (tr.x, tr.y, tr.z), Vec3 (-d.x, 0.0f, 0.0f), Vec3 (0.0f, -d.y, 0.0f), texMapping, texId); break;
		case SceneSim::F_E:	Init_AddQuad (data, meshIdx, group, Vec3 (tr.x, tr.y, bl.z), Vec3 (0.0f, 0.0f, d.z), Vec3 (0.0f, -d.y, 0.0f), texMapping, texId); break;
		case SceneSim::F_S:	Init_AddQuad (data, meshIdx, group, Vec3 (bl.x, tr.y, bl.z), Vec3 (d.x, 0.0f, 0.0f), Vec3 (0.0f, -d.y, 0.0f), texMapping, texId); break;
		case SceneSim::F_W:	Init_AddQuad (data, meshIdx, group, Vec3 (bl.x, tr.y, tr.z), Vec3 (0.0f, 0.0f, -d.z), Vec3 (0.0f, -d.y, 0.0f), texMapping, texId); break;
		default: assert (false); break;
		}
	}

	void ShapeBase::Init_Copy (InitData * data, int dstMeshIdx, int srcMeshIdx)
	{
		assert (data != NULL);
		assert (dstMeshIdx != srcMeshIdx);
		assert (dstMeshIdx >= 0 && dstMeshIdx < data->meshTotal);
		
		InitData::PerMesh * perMeshDest = &data->perMesh [dstMeshIdx];
		InitData::PerMesh * perMeshSrc = &data->perMesh [srcMeshIdx];

		memcpy (perMeshDest->triFacings, perMeshSrc->triFacings, sizeof (FACEFLAGS) * perMeshSrc->trisPerMeshTotal);
		memcpy (perMeshDest->triLightMappings, perMeshSrc->triLightMappings, sizeof (FACEFLAGS) * perMeshSrc->trisPerMeshTotal);
		memcpy (perMeshDest->triTexIds, perMeshSrc->triTexIds, sizeof (int) * perMeshSrc->trisPerMeshTotal);
		perMeshDest->hasFaces = perMeshSrc->hasFaces;
		perMeshDest->canBlockAdjFaces = perMeshSrc->canBlockAdjFaces; 
		perMeshDest->currentTriIdx = perMeshSrc->currentTriIdx;

		SingleMesh * dest = &meshes [dstMeshIdx];
		SingleMesh * src = &meshes [srcMeshIdx];

		memcpy (dest->verts, src->verts, sizeof (Gfx::VertexTypes::XyzUvRgb) * perMeshSrc->trisPerMeshTotal * 3);
	}

	void ShapeBase::Init_Rotate (InitData * data, int meshIdx, int rotateNorthTo)
	{
		assert (data != NULL);
		assert (meshIdx != 0 && meshIdx < data->meshTotal);
		
		// rotate all face information
		InitData::PerMesh * perMesh = &data->perMesh [meshIdx];

		const int hasFaces = perMesh->hasFaces;
		const int canBlockAdjFaces = perMesh->canBlockAdjFaces;
		
		perMesh->hasFaces = 0;
		perMesh->canBlockAdjFaces = 0;
		for (int idx = 0; idx < F_COUNT_ALL; ++idx)
		{
			if (hasFaces & (1 << idx))
			{
				perMesh->hasFaces |= 1 << remapFaces [rotateNorthTo][idx];
			}
			if (canBlockAdjFaces & (1 << idx))
			{
				perMesh->canBlockAdjFaces |= 1 << remapFaces [rotateNorthTo][idx];
			}
		}
		for (int idx = 0; idx < perMesh->trisPerMeshTotal; ++idx)
		{
			perMesh->triFacings [idx] = remapFaces [rotateNorthTo][perMesh->triFacings [idx]];
			perMesh->triLightMappings [idx] = remapFaces [rotateNorthTo][perMesh->triLightMappings [idx]];
		}

		// rotate the vertices
		SingleMesh * mesh = &meshes [meshIdx];

		for (int idx = 0; idx < perMesh->trisPerMeshTotal * 3; ++idx)
		{
			Vec3 srcPos = mesh->verts [idx].xyz;
			switch (rotateNorthTo)
			{
			case F_T: mesh->verts [idx].xyz = Vec3 (srcPos.x, 1.0f - srcPos.z, srcPos.y); break;
			case F_B: mesh->verts [idx].xyz = Vec3 (srcPos.x, srcPos.z, 1.0f - srcPos.y); break;
			case F_E: mesh->verts [idx].xyz = Vec3 (srcPos.z, srcPos.y, 1.0f - srcPos.x); break;
			case F_S: mesh->verts [idx].xyz = Vec3 (1.0f - srcPos.x, srcPos.y, 1.0f - srcPos.z); break;
			case F_W: mesh->verts [idx].xyz = Vec3 (1.0f - srcPos.z, srcPos.y, srcPos.x); break;
			}
		}
	}

	void ShapeBase::Init_Rotate (InitData * data, int meshIdx, float angle)
	{
		assert (data != NULL);
		assert (meshIdx >= 0 && meshIdx < data->meshTotal);

		SingleMesh * mesh = &meshes [meshIdx];

		Mat44 rotation = glm::rotate (Mat44 (), angle, Vec3 (0.0f, 1.0f, 0.0f));

		// update the vertices
		for (int idx = 0; idx < mesh->vertsCount; ++idx)
		{
			Vec4 srcPos = Vec4 (mesh->verts [idx].xyz.x - 0.5f, mesh->verts [idx].xyz.y, mesh->verts [idx].xyz.z - 0.5f, 0.0f);
			srcPos = rotation * srcPos;

			mesh->verts [idx] = mesh->verts [idx];

			mesh->verts [idx].xyz.x = srcPos.x + 0.5f;
			mesh->verts [idx].xyz.y = srcPos.y;
			mesh->verts [idx].xyz.z = srcPos.z + 0.5f;
		}
	}

	void ShapeBase::Init_MultiplyXyz (int meshIdx, Mat44 * mat)
	{
		assert (meshIdx >= 0 && meshIdx < meshTotal);
		assert (mat != NULL);
		
		SingleMesh * mesh = &meshes [meshIdx];
		Gfx::VertexTypes::XyzUvRgb * vert = mesh->verts;

		for (int idx = 0; idx < mesh->vertsCount; ++idx)
		{
			Vec4 rotate (vert->xyz.x, vert->xyz.y, vert->xyz.z, 1.0f);
			rotate = *mat * rotate;
			mesh->verts [idx].xyz = Vec3 (rotate.x, rotate.y, rotate.z);

			++vert;
		}
	}

	void ShapeBase::Init_Retexture (InitData * data, int meshIdx, RemapTexIdFuncPtr remapTexIdFuncPtr, void * remapTexIdData)
	{
		assert (data != NULL);
		assert (meshIdx >= 0 && meshIdx < data->meshTotal);
		
		InitData::PerMesh * perMesh = &data->perMesh [meshIdx];

		// copy the data
		for (int idx = 0; idx < perMesh->trisPerMeshTotal; ++idx)
		{
			perMesh->triTexIds [idx] = remapTexIdFuncPtr (meshIdx, perMesh->triTexIds [idx], remapTexIdData);
		}
	}

	void ShapeBase::Init_ApplyUvRgbs (InitData * data, int meshIdx, Vec4 rgba)
	{
		SingleMesh * mesh = &meshes [meshIdx];
		InitData::PerMesh * perMesh = &data->perMesh [meshIdx];

		const float pixel = 1.0f / 16.0f;
		Gfx::VertexTypes::XyzUvRgb * verts = mesh->verts;

		for (int triIdx = 0; triIdx < perMesh->trisPerMeshTotal; ++triIdx)
		{
			int texId = perMesh->triTexIds [triIdx];
			bool isTinted = (texId & addTint) == addTint;
			texId = (texId & noFlags);

			// temporary tinting
			Vec4 triRgba = rgba;
			if (isTinted)
			{
				triRgba = triRgba * Vec4 (144.0f / 255.0f, 224.0f / 255.0f, 097.0f / 255.0f, 1.0f);
			}
			switch (perMesh->triLightMappings [triIdx])
			{
			//case F_T:	rgba *= Vec4 (1.00f, 1.00f, 0.00f, 1.00f); break;
			//case F_B:	rgba *= Vec4 (0.00f, 0.00f, 0.00f, 1.00f); break;
			//case F_N:	rgba *= Vec4 (1.00f, 1.00f, 1.00f, 1.00f); break;
			//case F_E:	rgba *= Vec4 (1.00f, 0.00f, 0.00f, 1.00f); break;
			//case F_S:	rgba *= Vec4 (0.00f, 1.00f, 0.00f, 1.00f); break;
			//case F_W:	rgba *= Vec4 (0.00f, 0.00f, 1.00f, 1.00f); break;

			//case F_T:	rgba *= 1.00f; break;
			//case F_B:	rgba *= 0.75f; break;
			//case F_N:	rgba *= 0.95f; break;
			//case F_E:	rgba *= 0.90f; break;
			//case F_S:	rgba *= 0.80f; break;
			//case F_W:	rgba *= 0.85f; break;

			case F_T:	triRgba *= 1.00f; break;
			case F_B:	triRgba *= 0.5f; break;
			case F_N:	triRgba *= 0.90f; break;
			case F_E:	triRgba *= 0.80f; break;
			case F_S:	triRgba *= 0.60f; break;
			case F_W:	triRgba *= 0.70f; break;
			}

			Vec2 uvOffset (texId % 16, texId / 16);
			static const float epsilon = 0.001f;
			for (int idx = 0; idx < 3; ++idx)
			{
				// uv mapping
				Vec2 uv;
				switch (perMesh->triLightMappings [triIdx])
				{
				case F_T:	uv = Vec2 (1.0f - verts->xyz.x, 1.0f - verts->xyz.z); break;
				case F_B:	uv = Vec2 (verts->xyz.x, verts->xyz.z); break;
				case F_N:	uv = Vec2 (1.0f - verts->xyz.x, 1.0f - verts->xyz.y); break;
				case F_E:	uv = Vec2 (verts->xyz.z, 1.0f - verts->xyz.y); break;
				case F_S:	uv = Vec2 (verts->xyz.x, 1.0f - verts->xyz.y); break;
				case F_W:	uv = Vec2 (1.0f - verts->xyz.z, 1.0f - verts->xyz.y); break;
				case F_OTHER:	uv = Vec2 (1.0f - verts->xyz.x, 1.0f - verts->xyz.y); break; // same as F_N - had to pick one of the vertical ones
				}

				uv.x = (uv.x - 0.5f) * (1.0f - epsilon) + 0.5f;
				uv.y = (uv.y - 0.5f) * (1.0f - epsilon) + 0.5f;
				verts->uv = (uv + uvOffset) * pixel;
				//verts->uv = (uvOffset + Vec2 (0.25f, 0.25f)) * pixel;
				verts->rgba = triRgba;
				++verts;
			}
		}
	}

	void ShapeBase::Init_ApplyRgbs (InitData * data, int meshIdx, Vec4 rgba)
	{
		SingleMesh * mesh = &meshes [meshIdx];
		InitData::PerMesh * perMesh = &data->perMesh [meshIdx];

		Gfx::VertexTypes::XyzUvRgb * verts = mesh->verts;

		for (int triIdx = 0; triIdx < perMesh->trisPerMeshTotal; ++triIdx)
		{
			int texId = perMesh->triTexIds [triIdx];
			bool isTinted = (texId & addTint) == addTint;
			texId = (texId & noFlags);

			// temporary tinting
			Vec4 triRgba = rgba;
			if (isTinted)
			{
				triRgba = triRgba * Vec4 (144.0f / 255.0f, 224.0f / 255.0f, 097.0f / 255.0f, 1.0f);
			}
			switch (perMesh->triLightMappings [triIdx])
			{
			case F_T:	triRgba *= 1.00f; break;
			case F_B:	triRgba *= 0.5f; break;
			case F_N:	triRgba *= 0.90f; break;
			case F_E:	triRgba *= 0.80f; break;
			case F_S:	triRgba *= 0.60f; break;
			case F_W:	triRgba *= 0.70f; break;
			}

			Vec2 uvOffset (texId % 16, texId / 16);
			static const float epsilon = 0.001f;
			for (int idx = 0; idx < 3; ++idx)
			{
				verts->rgba = triRgba;
				++verts;
			}
		}
	}

	void ShapeBase::Init_Finish (InitData * data, int meshIdx)
	{
		SingleMesh * mesh = &meshes [meshIdx];
		InitData::PerMesh * perMesh = &data->perMesh [meshIdx];

		// recopy these in case they changed owing to rotations etc...
		mesh->hasFaces = perMesh->hasFaces;
		mesh->canBlockAdjFaces = perMesh->canBlockAdjFaces;
		for (int idx = 0; idx < perMesh->trisPerMeshTotal; ++idx)
		{
			mesh->triFacings [idx] = (FACEFLAGS) (1 << perMesh->triFacings [idx]);
			mesh->triLightMappings [idx] = perMesh->triLightMappings [idx];
		}
	}

	ShapeBase::SingleMesh::SingleMesh ()
		: hasFaces (0),
		canBlockAdjFaces (0),
		verts (NULL),
		triFacings (NULL),
		triLightMappings (NULL)
	{
	}

	ShapeBase::SingleMesh::~SingleMesh ()
	{
		delete [] verts;
		verts = NULL;

		delete [] triFacings;
		triFacings = NULL;

		delete [] triLightMappings;
		triLightMappings = NULL;
	}

	void ShapeBase::SingleMesh::Init (int triCount)
	{
		vertsCount = triCount * 3;
		verts = aah_new Gfx::VertexTypes::XyzUvRgb [vertsCount];
		triFacings = aah_new FACEFLAGS [triCount];
		triLightMappings = aah_new FACES [triCount];
	}

	int ShapeBase::GetInventoryIcon () const
	{
		return inventoryIconIdx;
	}
}