#pragma once

#include "SceneSim/SceneSim_Defs.h"

namespace CellMesh {struct InitData;}
namespace CellMesh {class AddShapeData;}
namespace Gfx {namespace VertexTypes {struct XyzUvRgb;}}

namespace CellMesh
{
	typedef int (*MeshIdxFromCellDataFuncPtr) (int type, int data);
	typedef int (*RemapTexIdFuncPtr) (int meshIdx, int texId, void * data);

	typedef int (*AwfulCalcDataFuncPtr) (AddShapeData * addData);

	typedef int (*NewAddExtraStuff) (AddShapeData * addData, int triIdx);

	static const int noFlags = 0xff;
	static const int addTint = 0x0100;

	enum GFXBLOCKINGS
	{
		GB_ALL,
		GB_NONE,
		GB_SAMETYPE,
	};

	class ShapeBase
	{
	public:

		ShapeBase ();
		~ShapeBase ();

		bool HasMesh () const;

		void AddInstance (CellMesh::AddShapeData * data, Int3 cellId) const;

		void Init (InitData * data, int inventoryIdx);
		void InitSeperate (InitData * data);

		void Init_AddQuad (InitData * data, int meshIdx, SceneSim::FACES group, Vec3 origin, Vec3 w, Vec3 h, SceneSim::FACES texMapping, int texId);
		void Init_AddTri (InitData * data, int meshIdx, SceneSim::FACES group, Vec3 v1, Vec3 offset1, Vec3 offset2, SceneSim::FACES texMapping, int texId);
		void Init_AddBoxQuad (InitData * data, int meshIdx, SceneSim::FACES group, Vec3 bl, Vec3 tr, SceneSim::FACES texMapping, int texId);
		void Init_Copy (InitData * data, int dstMeshIdx, int srcMeshIdx);
		void Init_Rotate (InitData * data, int meshIdx, int rotateNorthTo);
		void Init_Rotate (InitData * data, int meshIdx, float angle);
		void Init_MultiplyXyz (int meshIdx, Mat44 * mat);
		void Init_Retexture (InitData * data, int meshIdx, RemapTexIdFuncPtr remapTexIdFuncPtr, void * remapTexIdData);
		void Init_ApplyUvRgbs (InitData * data, int meshIdx, Vec4 rgba);
		void Init_ApplyRgbs (InitData * data, int meshIdx, Vec4 rgba);
		void Init_Finish (InitData * data, int meshIdx);

		int GetInventoryIcon () const;

		//SceneSim::ENTCOLLISIONS entCollisions; // TODO remove
		GFXBLOCKINGS gfxBlocking;
		bool isFullBright;

	protected:

		struct SingleMesh
		{
			SingleMesh ();
			~SingleMesh ();

			void Init (int triCount);

			int vertsCount;

			int hasFaces;
			int canBlockAdjFaces;

			Gfx::VertexTypes::XyzUvRgb * verts;
			SceneSim::FACEFLAGS * triFacings;
			SceneSim::FACES * triLightMappings;
		};

		SingleMesh * meshes;
		int meshTotal;

		MeshIdxFromCellDataFuncPtr meshIdxFromCellDataFuncPtr; 	
		AwfulCalcDataFuncPtr awfulCalcDataFuncPtr;

		NewAddExtraStuff newAddExtraStuff;

		// used for helping to draw the shape in the inventory
		int inventoryIconIdx;
	};
}
