#include "stdafx.h"

#include "Shapes_InitData.h"
#include "CellMesh_ShapeBase.h"

namespace CellMesh
{
	namespace Shapes
	{
		void InitBox (ShapeBase * shape, int t, int b, int n, int e, int s, int w, Int3 min, Int3 max, int canBlockFaces)
		{
			const float unit = 1.0f / 16.0f;

			const int triCount = 
				((t != -1) ? 2 : 0) +
				((b != -1) ? 2 : 0) +
				((n != -1) ? 2 : 0) +
				((e != -1) ? 2 : 0) +
				((s != -1) ? 2 : 0) +
				((w != -1) ? 2 : 0);

			InitData data (1, triCount, SceneSim::FF_OTHER, canBlockFaces);

			shape->Init (&data, n);

			//shape->entCollisions = SceneSim::EC_NONE;
			shape->gfxBlocking = CellMesh::GB_NONE;

			Vec3 minF = Vec3 (min.x, min.y, min.z) * unit;
			Vec3 maxF = Vec3 (max.x, max.y, max.z) * unit;

			if (t != -1)	shape->Init_AddQuad (&data, 0, SceneSim::F_OTHER, Vec3 (minF.x, maxF.y, maxF.z), Vec3 (maxF.x - minF.x, 0.0f, 0.0f), Vec3 (0.0f, 0.0f, minF.z - maxF.z), SceneSim::F_T, t);
			if (b != -1)	shape->Init_AddQuad (&data, 0, SceneSim::F_OTHER, Vec3 (minF.x, minF.y, minF.z), Vec3 (maxF.x - minF.x, 0.0f, 0.0f), Vec3 (0.0f, 0.0f, maxF.z - minF.z), SceneSim::F_B, b);
			if (n != -1)	shape->Init_AddQuad (&data, 0, SceneSim::F_OTHER, Vec3 (maxF.x, maxF.y, maxF.z), Vec3 (minF.x - maxF.x, 0.0f, 0.0f), Vec3 (0.0f, minF.y - maxF.y, 0.0f), SceneSim::F_N, n);
			if (e != -1)	shape->Init_AddQuad (&data, 0, SceneSim::F_OTHER, Vec3 (maxF.x, maxF.y, minF.z), Vec3 (0.0f, 0.0f, maxF.z - minF.z), Vec3 (0.0f, minF.y - maxF.y, 0.0f), SceneSim::F_E, e);
			if (s != -1)	shape->Init_AddQuad (&data, 0, SceneSim::F_OTHER, Vec3 (minF.x, maxF.y, minF.z), Vec3 (maxF.x - minF.x, 0.0f, 0.0f), Vec3 (0.0f, minF.y - maxF.y, 0.0f), SceneSim::F_S, s);
			if (w != -1)	shape->Init_AddQuad (&data, 0, SceneSim::F_OTHER, Vec3 (minF.x, maxF.y, maxF.z), Vec3 (0.0f, 0.0f, minF.z - maxF.z), Vec3 (0.0f, minF.y - maxF.y, 0.0f), SceneSim::F_W, w);

			shape->Init_ApplyUvRgbs (&data, 0, Vec4 (1.0f, 1.0f, 1.0f, 1.0f));
			shape->Init_Finish (&data, 0);
		}

		void InitBox_CompassRotations (ShapeBase * shape, int t, int b, int n, int e, int s, int w, Int3 min, Int3 max, MeshIdxFromCellDataFuncPtr meshIdxFuncPtr)
		{
			const float unit = 1.0f / 16.0f;

			const int triCount = 
				((t != -1) ? 2 : 0) +
				((b != -1) ? 2 : 0) +
				((n != -1) ? 2 : 0) +
				((e != -1) ? 2 : 0) +
				((s != -1) ? 2 : 0) +
				((w != -1) ? 2 : 0);

			InitData data (4, triCount, SceneSim::FF_OTHER, 0);
			data.meshIdxFromCellDataFuncPtr = meshIdxFuncPtr;

			shape->Init (&data, n);

			//shape->entCollisions = SceneSim::EC_NONE;
			shape->gfxBlocking = CellMesh::GB_NONE;

			Vec3 minF = Vec3 (min.x, min.y, min.z) * unit;
			Vec3 maxF = Vec3 (max.x, max.y, max.z) * unit;

			if (t != -1)	shape->Init_AddQuad (&data, 0, SceneSim::F_OTHER, Vec3 (minF.x, maxF.y, maxF.z), Vec3 (maxF.x - minF.x, 0.0f, 0.0f), Vec3 (0.0f, 0.0f, minF.z - maxF.z), SceneSim::F_T, t);
			if (b != -1)	shape->Init_AddQuad (&data, 0, SceneSim::F_OTHER, Vec3 (minF.x, minF.y, minF.z), Vec3 (maxF.x - minF.x, 0.0f, 0.0f), Vec3 (0.0f, 0.0f, maxF.z - minF.z), SceneSim::F_B, b);
			if (n != -1)	shape->Init_AddQuad (&data, 0, SceneSim::F_OTHER, Vec3 (maxF.x, maxF.y, maxF.z), Vec3 (minF.x - maxF.x, 0.0f, 0.0f), Vec3 (0.0f, minF.y - maxF.y, 0.0f), SceneSim::F_N, n);
			if (e != -1)	shape->Init_AddQuad (&data, 0, SceneSim::F_OTHER, Vec3 (maxF.x, maxF.y, minF.z), Vec3 (0.0f, 0.0f, maxF.z - minF.z), Vec3 (0.0f, minF.y - maxF.y, 0.0f), SceneSim::F_E, e);
			if (s != -1)	shape->Init_AddQuad (&data, 0, SceneSim::F_OTHER, Vec3 (minF.x, maxF.y, minF.z), Vec3 (maxF.x - minF.x, 0.0f, 0.0f), Vec3 (0.0f, minF.y - maxF.y, 0.0f), SceneSim::F_S, s);
			if (w != -1)	shape->Init_AddQuad (&data, 0, SceneSim::F_OTHER, Vec3 (minF.x, maxF.y, maxF.z), Vec3 (0.0f, 0.0f, minF.z - maxF.z), Vec3 (0.0f, minF.y - maxF.y, 0.0f), SceneSim::F_W, w);

			shape->Init_ApplyUvRgbs (&data, 0, Vec4 (1.0f, 1.0f, 1.0f, 1.0f));
			shape->Init_Finish (&data, 0);

			shape->Init_Copy (&data, 1, 0);
			shape->Init_Copy (&data, 2, 0);
			shape->Init_Copy (&data, 3, 0);

			shape->Init_Rotate (&data, 1, SceneSim::F_E);
			shape->Init_Rotate (&data, 2, SceneSim::F_S);
			shape->Init_Rotate (&data, 3, SceneSim::F_W);

			shape->Init_ApplyUvRgbs (&data, 0, Vec4 (1.0f, 1.0f, 1.0f, 1.0f));
			shape->Init_ApplyUvRgbs (&data, 1, Vec4 (1.0f, 1.0f, 1.0f, 1.0f));
			shape->Init_ApplyUvRgbs (&data, 2, Vec4 (1.0f, 1.0f, 1.0f, 1.0f));
			shape->Init_ApplyUvRgbs (&data, 3, Vec4 (1.0f, 1.0f, 1.0f, 1.0f));

			shape->Init_Finish (&data, 0);
			shape->Init_Finish (&data, 1);
			shape->Init_Finish (&data, 2);
			shape->Init_Finish (&data, 3);
		}

		void InitBox_CompassDownRotations (ShapeBase * shape, int t, int b, int n, int e, int s, int w, Int3 min, Int3 max, MeshIdxFromCellDataFuncPtr meshIdxFuncPtr)
		{
			const float unit = 1.0f / 16.0f;

			const int triCount = 
				((t != -1) ? 2 : 0) +
				((b != -1) ? 2 : 0) +
				((n != -1) ? 2 : 0) +
				((e != -1) ? 2 : 0) +
				((s != -1) ? 2 : 0) +
				((w != -1) ? 2 : 0);

			InitData data (5, triCount, SceneSim::FF_OTHER, 0);
			data.meshIdxFromCellDataFuncPtr = meshIdxFuncPtr;

			shape->Init (&data, n);

			/*shape->entCollisions = SceneSim::EC_NONE;
			*/shape->gfxBlocking = CellMesh::GB_NONE;

			Vec3 minF = Vec3 (min.x, min.y, min.z) * unit;
			Vec3 maxF = Vec3 (max.x, max.y, max.z) * unit;

			if (t != -1)	shape->Init_AddQuad (&data, 0, SceneSim::F_OTHER, Vec3 (minF.x, maxF.y, maxF.z), Vec3 (maxF.x - minF.x, 0.0f, 0.0f), Vec3 (0.0f, 0.0f, minF.z - maxF.z), SceneSim::F_T, t);
			if (b != -1)	shape->Init_AddQuad (&data, 0, SceneSim::F_OTHER, Vec3 (minF.x, minF.y, minF.z), Vec3 (maxF.x - minF.x, 0.0f, 0.0f), Vec3 (0.0f, 0.0f, maxF.z - minF.z), SceneSim::F_B, b);
			if (n != -1)	shape->Init_AddQuad (&data, 0, SceneSim::F_OTHER, Vec3 (maxF.x, maxF.y, maxF.z), Vec3 (minF.x - maxF.x, 0.0f, 0.0f), Vec3 (0.0f, minF.y - maxF.y, 0.0f), SceneSim::F_N, n);
			if (e != -1)	shape->Init_AddQuad (&data, 0, SceneSim::F_OTHER, Vec3 (maxF.x, maxF.y, minF.z), Vec3 (0.0f, 0.0f, maxF.z - minF.z), Vec3 (0.0f, minF.y - maxF.y, 0.0f), SceneSim::F_E, e);
			if (s != -1)	shape->Init_AddQuad (&data, 0, SceneSim::F_OTHER, Vec3 (minF.x, maxF.y, minF.z), Vec3 (maxF.x - minF.x, 0.0f, 0.0f), Vec3 (0.0f, minF.y - maxF.y, 0.0f), SceneSim::F_S, s);
			if (w != -1)	shape->Init_AddQuad (&data, 0, SceneSim::F_OTHER, Vec3 (minF.x, maxF.y, maxF.z), Vec3 (0.0f, 0.0f, minF.z - maxF.z), Vec3 (0.0f, minF.y - maxF.y, 0.0f), SceneSim::F_W, w);

			shape->Init_ApplyUvRgbs (&data, 0, Vec4 (1.0f, 1.0f, 1.0f, 1.0f));
			shape->Init_Finish (&data, 0);

			shape->Init_Copy (&data, 1, 0);
			shape->Init_Copy (&data, 2, 0);
			shape->Init_Copy (&data, 3, 0);
			shape->Init_Copy (&data, 4, 0);

			shape->Init_Rotate (&data, 1, SceneSim::F_E);
			shape->Init_Rotate (&data, 2, SceneSim::F_S);
			shape->Init_Rotate (&data, 3, SceneSim::F_W);
			shape->Init_Rotate (&data, 4, SceneSim::F_B);

			shape->Init_ApplyUvRgbs (&data, 0, Vec4 (1.0f, 1.0f, 1.0f, 1.0f));
			shape->Init_ApplyUvRgbs (&data, 1, Vec4 (1.0f, 1.0f, 1.0f, 1.0f));
			shape->Init_ApplyUvRgbs (&data, 2, Vec4 (1.0f, 1.0f, 1.0f, 1.0f));
			shape->Init_ApplyUvRgbs (&data, 3, Vec4 (1.0f, 1.0f, 1.0f, 1.0f));
			shape->Init_ApplyUvRgbs (&data, 4, Vec4 (1.0f, 1.0f, 1.0f, 1.0f));

			shape->Init_Finish (&data, 0);
			shape->Init_Finish (&data, 1);
			shape->Init_Finish (&data, 2);
			shape->Init_Finish (&data, 3);
			shape->Init_Finish (&data, 4);
		}

		void InitBox_Transparent (ShapeBase * shape, int t, int b, int n, int e, int s, int w, Int3 min, Int3 max)
		{
			const float unit = 1.0f / 16.0f;

			const int triCount = 
				((t != -1) ? 2 : 0) +
				((b != -1) ? 2 : 0) +
				((n != -1) ? 2 : 0) +
				((e != -1) ? 2 : 0) +
				((s != -1) ? 2 : 0) +
				((w != -1) ? 2 : 0);

			InitData data (1, triCount, SceneSim::FF_OTHER, 0);

			shape->Init (&data, n);

			//shape->entCollisions = SceneSim::EC_NONE;
			shape->gfxBlocking = CellMesh::GB_NONE;

			Vec3 minF = Vec3 (min.x, min.y, min.z) * unit;
			Vec3 maxF = Vec3 (max.x, max.y, max.z) * unit;

			if (t != -1)	shape->Init_AddQuad (&data, 0, SceneSim::F_OTHER, Vec3 (0.0f, maxF.y, 1.0f), Vec3 (1.0f, 0.0f, 0.0f), Vec3 (0.0f, 0.0f, -1.0f), SceneSim::F_T, t);
			if (b != -1)	shape->Init_AddQuad (&data, 0, SceneSim::F_OTHER, Vec3 (0.0f, minF.y, 0.0f), Vec3 (1.0f, 0.0f, 0.0f), Vec3 (0.0f, 0.0f, 1.0f), SceneSim::F_B, b);
			if (n != -1)	shape->Init_AddQuad (&data, 0, SceneSim::F_OTHER, Vec3 (1.0f, 1.0f, maxF.z), Vec3 (-1.0f, 0.0f, 0.0f), Vec3 (0.0f, -1.0f, 0.0f), SceneSim::F_N, n);
			if (e != -1)	shape->Init_AddQuad (&data, 0, SceneSim::F_OTHER, Vec3 (maxF.x, 1.0f, 0.0f), Vec3 (0.0f, 0.0f, 1.0f), Vec3 (0.0f, -1.0f, 0.0f), SceneSim::F_E, e);
			if (s != -1)	shape->Init_AddQuad (&data, 0, SceneSim::F_OTHER, Vec3 (0.0f, 1.0f, minF.z), Vec3 (1.0f, 0.0f, 0.0f), Vec3 (0.0f, -1.0f, 0.0f), SceneSim::F_S, s);
			if (w != -1)	shape->Init_AddQuad (&data, 0, SceneSim::F_OTHER, Vec3 (minF.x, 1.0f, 1.0f), Vec3 (0.0f, 0.0f, -1.0f), Vec3 (0.0f, -1.0f, 0.0f), SceneSim::F_W, w);

			shape->Init_ApplyUvRgbs (&data, 0, Vec4 (1.0f, 1.0f, 1.0f, 1.0f));
			shape->Init_Finish (&data, 0);
		}
	}
}
