#include "stdafx.h"

#include "Shapes_InitData.h"
#include "CellMesh_ShapeBase.h"

namespace CellMesh
{
	namespace Shapes
	{
		int CalcDoorRemapTexId (int meshId, int texId, void * data)
		{
			texId = (int) data;
			return texId;
		}

		int CalcDoorMeshIdx (int type, int data)
		{
			int meshIdx;
			if (data & 0x4)
			{
				meshIdx = (((data & 0x3) + 1) % 4) + ((data & 0x8) ? 4 : 0);
			}
			else
			{
				meshIdx = (data & 0x3) + ((data & 0x8) ? 4 : 0);
			}
			return meshIdx;
		}
	
		void InitDoor (ShapeBase * shape, int texT, int texB)
		{
			const float unit = 1.0f / 16.0f;

			InitData data (8, 12, SceneSim::FF_T | SceneSim::FF_OTHER, 0);
			data.meshIdxFromCellDataFuncPtr = CalcDoorMeshIdx;

			shape->Init (&data, texT);

			//shape->entCollisions = SceneSim::EC_NONE;
			shape->gfxBlocking = CellMesh::GB_NONE;

			Int3 min (0, 0, 14);
			Int3 max (16, 16, 16);
			Vec3 minF (min.x * unit, min.y * unit, min.z * unit);
			Vec3 maxF (max.x * unit, max.y * unit, max.z * unit);

			shape->Init_AddQuad (&data, 0, SceneSim::F_T, Vec3 (minF.x, maxF.y, maxF.z), Vec3 (maxF.x - minF.x, 0.0f, 0.0f), Vec3 (0.0f, 0.0f, minF.z - maxF.z), SceneSim::F_T, texB);
			shape->Init_AddQuad (&data, 0, SceneSim::F_OTHER, Vec3 (minF.x, minF.y, minF.z), Vec3 (maxF.x - minF.x, 0.0f, 0.0f), Vec3 (0.0f, 0.0f, maxF.z - minF.z), SceneSim::F_B, texB);
			shape->Init_AddQuad (&data, 0, SceneSim::F_OTHER, Vec3 (maxF.x, maxF.y, maxF.z), Vec3 (minF.x - maxF.x, 0.0f, 0.0f), Vec3 (0.0f, minF.y - maxF.y, 0.0f), SceneSim::F_N, texB);
			shape->Init_AddQuad (&data, 0, SceneSim::F_OTHER, Vec3 (maxF.x, maxF.y, minF.z), Vec3 (0.0f, 0.0f, maxF.z - minF.z), Vec3 (0.0f, minF.y - maxF.y, 0.0f), SceneSim::F_E, texB);
			shape->Init_AddQuad (&data, 0, SceneSim::F_OTHER, Vec3 (minF.x, maxF.y, minF.z), Vec3 (maxF.x - minF.x, 0.0f, 0.0f), Vec3 (0.0f, minF.y - maxF.y, 0.0f), SceneSim::F_S, texB);
			shape->Init_AddQuad (&data, 0, SceneSim::F_OTHER, Vec3 (minF.x, maxF.y, maxF.z), Vec3 (0.0f, 0.0f, minF.z - maxF.z), Vec3 (0.0f, minF.y - maxF.y, 0.0f), SceneSim::F_W, texB);

			shape->Init_Copy (&data, 1, 0);
			shape->Init_Copy (&data, 2, 0);
			shape->Init_Copy (&data, 3, 0);

			shape->Init_Rotate (&data, 1, SceneSim::F_E);
			shape->Init_Rotate (&data, 2, SceneSim::F_S);
			shape->Init_Rotate (&data, 3, SceneSim::F_W);

			for (int idx = 0; idx < 4; ++idx)
			{
				shape->Init_Copy (&data, 4 + idx, idx);
				shape->Init_Retexture (&data, 4 + idx, CalcDoorRemapTexId, (void *) texT);
			}

			for (int idx = 0; idx < 8; ++idx)
			{
				shape->Init_ApplyUvRgbs (&data, idx, Vec4 (1.0f, 1.0f, 1.0f, 1.0f));
				shape->Init_Finish (&data, idx);
			}
		}
	}
}
