#include "stdafx.h"

#include "Shapes_InitData.h"
#include "CellMesh_ShapeBase.h"

namespace CellMesh
{
	namespace Shapes
	{
		int CalcFenceMeshIdx (int type, int data)
		{
			return data;
		}

		const int triCounts [] =
		{
			12,
			12 + 1 * 10,
			12 + 1 * 10, 
			12 + 2 * 10, 
			12 + 1 * 10, 
			12 + 2 * 10, 
			12 + 2 * 10, 
			12 + 3 * 10, 
			12 + 1 * 10, 
			12 + 2 * 10, 
			12 + 2 * 10, 
			12 + 3 * 10, 
			12 + 2 * 10, 
			12 + 3 * 10, 
			12 + 3 * 10, 
			12 + 4 * 10, 
		};

		const int hasFaces [] =
		{
			SceneSim::FF_T | SceneSim::FF_B | SceneSim::FF_OTHER,
			SceneSim::FF_T | SceneSim::FF_B | SceneSim::FF_N | SceneSim::FF_OTHER, 
			SceneSim::FF_T | SceneSim::FF_B | SceneSim::FF_E | SceneSim::FF_OTHER, 
			SceneSim::FF_T | SceneSim::FF_B | SceneSim::FF_E | SceneSim::FF_N | SceneSim::FF_OTHER, 
			SceneSim::FF_T | SceneSim::FF_B | SceneSim::FF_S | SceneSim::FF_OTHER, 
			SceneSim::FF_T | SceneSim::FF_B | SceneSim::FF_S | SceneSim::FF_N | SceneSim::FF_OTHER, 
			SceneSim::FF_T | SceneSim::FF_B | SceneSim::FF_S | SceneSim::FF_E | SceneSim::FF_OTHER, 
			SceneSim::FF_T | SceneSim::FF_B | SceneSim::FF_S | SceneSim::FF_E | SceneSim::FF_N | SceneSim::FF_OTHER, 
			SceneSim::FF_T | SceneSim::FF_B | SceneSim::FF_W | SceneSim::FF_OTHER, 
			SceneSim::FF_T | SceneSim::FF_B | SceneSim::FF_W | SceneSim::FF_N | SceneSim::FF_OTHER, 
			SceneSim::FF_T | SceneSim::FF_B | SceneSim::FF_W | SceneSim::FF_E | SceneSim::FF_OTHER, 
			SceneSim::FF_T | SceneSim::FF_B | SceneSim::FF_W | SceneSim::FF_E | SceneSim::FF_N | SceneSim::FF_OTHER, 
			SceneSim::FF_T | SceneSim::FF_B | SceneSim::FF_W | SceneSim::FF_S | SceneSim::FF_OTHER, 
			SceneSim::FF_T | SceneSim::FF_B | SceneSim::FF_W | SceneSim::FF_S | SceneSim::FF_N | SceneSim::FF_OTHER, 
			SceneSim::FF_T | SceneSim::FF_B | SceneSim::FF_W | SceneSim::FF_S | SceneSim::FF_E | SceneSim::FF_OTHER, 
			SceneSim::FF_T | SceneSim::FF_B | SceneSim::FF_W | SceneSim::FF_S | SceneSim::FF_E | SceneSim::FF_N | SceneSim::FF_OTHER, 
		};

		void InitFence (ShapeBase * shape, int tex)
		{
			const float unit = 1.0f / 16.0f;

			InitData data (16, triCounts, hasFaces);
			data.meshIdxFromCellDataFuncPtr = CalcFenceMeshIdx;

			shape->Init (&data, tex);
			
			shape->gfxBlocking = CellMesh::GB_NONE;

			for (int idx = 0; idx < 16; ++idx)
			{
				{
					Vec3 bl (unit * 6.0f, unit * 0.0f, unit * 6.0f);
					Vec3 tr (unit * 10.0f, unit * 16.0f, unit * 10.0f);
					shape->Init_AddBoxQuad (&data, idx, SceneSim::F_T, bl, tr, SceneSim::F_T, tex);
					shape->Init_AddBoxQuad (&data, idx, SceneSim::F_B, bl, tr, SceneSim::F_B, tex);
					shape->Init_AddBoxQuad (&data, idx, SceneSim::F_OTHER, bl, tr, SceneSim::F_N, tex);
					shape->Init_AddBoxQuad (&data, idx, SceneSim::F_OTHER, bl, tr, SceneSim::F_E, tex);
					shape->Init_AddBoxQuad (&data, idx, SceneSim::F_OTHER, bl, tr, SceneSim::F_S, tex);
					shape->Init_AddBoxQuad (&data, idx, SceneSim::F_OTHER, bl, tr, SceneSim::F_W, tex);
				}

				if (idx & 1) // n
				{
					Vec3 bl (unit * 6.0f, unit * 12.0f, unit * 6.0f);
					Vec3 tr (unit * 10.0f, unit * 16.0f, unit * 16.0f);
					shape->Init_AddBoxQuad (&data, idx, SceneSim::F_T, bl, tr, SceneSim::F_T, tex);
					shape->Init_AddBoxQuad (&data, idx, SceneSim::F_B, bl, tr, SceneSim::F_B, tex);
					shape->Init_AddBoxQuad (&data, idx, SceneSim::F_OTHER, bl, tr, SceneSim::F_N, tex);
					shape->Init_AddBoxQuad (&data, idx, SceneSim::F_OTHER, bl, tr, SceneSim::F_E, tex);
					shape->Init_AddBoxQuad (&data, idx, SceneSim::F_OTHER, bl, tr, SceneSim::F_W, tex);
				}	

				if (idx & 2) // e
				{
					Vec3 bl (unit * 6.0f, unit * 12.0f, unit * 6.0f);
					Vec3 tr (unit * 16.0f, unit * 16.0f, unit * 10.0f);
					shape->Init_AddBoxQuad (&data, idx, SceneSim::F_T, bl, tr, SceneSim::F_T, tex);
					shape->Init_AddBoxQuad (&data, idx, SceneSim::F_B, bl, tr, SceneSim::F_B, tex);
					shape->Init_AddBoxQuad (&data, idx, SceneSim::F_OTHER, bl, tr, SceneSim::F_N, tex);
					shape->Init_AddBoxQuad (&data, idx, SceneSim::F_OTHER, bl, tr, SceneSim::F_E, tex);
					shape->Init_AddBoxQuad (&data, idx, SceneSim::F_OTHER, bl, tr, SceneSim::F_S, tex);
				}

				if (idx & 4) // s
				{
					Vec3 bl (unit * 6.0f, unit * 12.0f, unit * 0.0f);
					Vec3 tr (unit * 10.0f, unit * 16.0f, unit * 10.0f);
					shape->Init_AddBoxQuad (&data, idx, SceneSim::F_T, bl, tr, SceneSim::F_T, tex);
					shape->Init_AddBoxQuad (&data, idx, SceneSim::F_B, bl, tr, SceneSim::F_B, tex);
					shape->Init_AddBoxQuad (&data, idx, SceneSim::F_OTHER, bl, tr, SceneSim::F_E, tex);
					shape->Init_AddBoxQuad (&data, idx, SceneSim::F_OTHER, bl, tr, SceneSim::F_S, tex);
					shape->Init_AddBoxQuad (&data, idx, SceneSim::F_OTHER, bl, tr, SceneSim::F_W, tex);
				}

				if (idx & 8) // w
				{
					Vec3 bl (unit * 0.0f, unit * 12.0f, unit * 6.0f);
					Vec3 tr (unit * 10.0f, unit * 16.0f, unit * 10.0f);
					shape->Init_AddBoxQuad (&data, idx, SceneSim::F_T, bl, tr, SceneSim::F_T, tex);
					shape->Init_AddBoxQuad (&data, idx, SceneSim::F_B, bl, tr, SceneSim::F_B, tex);
					shape->Init_AddBoxQuad (&data, idx, SceneSim::F_OTHER, bl, tr, SceneSim::F_N, tex);
					shape->Init_AddBoxQuad (&data, idx, SceneSim::F_OTHER, bl, tr, SceneSim::F_S, tex);
					shape->Init_AddBoxQuad (&data, idx, SceneSim::F_OTHER, bl, tr, SceneSim::F_W, tex);
				}
			}

			for (int idx = 0; idx < 16; ++idx)
			{
				shape->Init_ApplyUvRgbs (&data, idx, Vec4 (1.0f, 1.0f, 1.0f, 1.0f));
				shape->Init_Finish (&data, idx);
			}
		}
	}
}
