#include "stdafx.h"

#include "SceneSim/BlockData_Sign.h"
#include "SceneSim/SceneSim_Cell.h"
#include "SceneSim/SceneSim_Column.h"

#include "../Gfx/Gfx_Font.h"
#include "../Gfx/VertexTypes_XyzUvRgb.h"
#include "../SceneGfx/SceneGfx_Column.h"

#include "Shapes_InitData.h"
#include "CellMesh_Defs.h"
#include "CellMesh_ShapeBase.h"

namespace CellMesh
{
	namespace Shapes
	{
		int CalcGateMeshIdx (int type, int data)
		{
			switch (data & 0x3)
			{
			case 2: 
			case 0: return 1;	// n + s
			case 1: 
			case 3: return 0;	// e + w
			}
			assert (false);
			return 0;
		}

		void InitGate (ShapeBase * shape, int tex)
		{
			const float unit = 1.0f / 16.0f;

			InitData data (2, 24, SceneSim::FF_T | SceneSim::FF_B | SceneSim::FF_N | SceneSim::FF_E | SceneSim::FF_S | SceneSim::FF_W | SceneSim::FF_OTHER, SceneSim::FF_E | SceneSim::FF_W);
			data.meshIdxFromCellDataFuncPtr = CalcGateMeshIdx;

			shape->Init (&data, tex);

			shape->gfxBlocking = CellMesh::GB_NONE;

			{
				Vec3 bl (unit * 0.0f, unit * 8.0f, unit * 0.0f);
				Vec3 tr (unit * 2.0f, unit * 16.0f, unit * 16.0f);
				shape->Init_AddBoxQuad (&data, 0, SceneSim::F_T, bl, tr, SceneSim::F_T, tex);
				shape->Init_AddBoxQuad (&data, 0, SceneSim::F_B, bl, tr, SceneSim::F_B, tex);
				shape->Init_AddBoxQuad (&data, 0, SceneSim::F_N, bl, tr, SceneSim::F_N, tex);
				shape->Init_AddBoxQuad (&data, 0, SceneSim::F_OTHER, bl, tr, SceneSim::F_E, tex);
				shape->Init_AddBoxQuad (&data, 0, SceneSim::F_S, bl, tr, SceneSim::F_S, tex);
				shape->Init_AddBoxQuad (&data, 0, SceneSim::F_W, bl, tr, SceneSim::F_W, tex);
			}

			{
				Vec3 bl (unit * 14.0f, unit * 8.0f, unit * 0.0f);
				Vec3 tr (unit * 16.0f, unit * 16.0f, unit * 16.0f);
				shape->Init_AddBoxQuad (&data, 0, SceneSim::F_T, bl, tr, SceneSim::F_T, tex);
				shape->Init_AddBoxQuad (&data, 0, SceneSim::F_B, bl, tr, SceneSim::F_B, tex);
				shape->Init_AddBoxQuad (&data, 0, SceneSim::F_N, bl, tr, SceneSim::F_N, tex);
				shape->Init_AddBoxQuad (&data, 0, SceneSim::F_E, bl, tr, SceneSim::F_E, tex);
				shape->Init_AddBoxQuad (&data, 0, SceneSim::F_S, bl, tr, SceneSim::F_S, tex);
				shape->Init_AddBoxQuad (&data, 0, SceneSim::F_OTHER, bl, tr, SceneSim::F_W, tex);
			}

			shape->Init_Copy (&data, 1, 0);

			shape->Init_Rotate (&data, 1, SceneSim::F_E);

			shape->Init_ApplyUvRgbs (&data, 0, Vec4 (1.0f, 1.0f, 1.0f, 1.0f));
			shape->Init_ApplyUvRgbs (&data, 1, Vec4 (1.0f, 1.0f, 1.0f, 1.0f));

			shape->Init_Finish (&data, 0);
			shape->Init_Finish (&data, 1);
		}
	}
}
