#include "stdafx.h"

#include "Shapes_InitData.h"
#include "CellMesh_ShapeBase.h"

namespace CellMesh
{
	namespace Shapes
	{
		int PistonBaseMeshIdxFromCellData (int type, int data) 
		{
			int meshIdx = 0;

			switch (data & 0x7)
			{
			case 0: meshIdx = 1; break;
			case 1: meshIdx = 0; break;
			case 2: meshIdx = 3; break;
			case 3: meshIdx = 5; break;
			case 4: meshIdx = 2; break;
			case 5: meshIdx = 4; break;
			default: assert (false);
			}
			
			if (data & 0x08)
			{
				meshIdx += 6;
			}

			return meshIdx;
		}

		void InitPistonBase (ShapeBase * shape, int t, int b, int sides, int mech)
		{
			float unit = 1.0f / 16.0f;

			InitData data (12, 6 * 2, SceneSim::FF_B | SceneSim::FF_N | SceneSim::FF_E | SceneSim::FF_S | SceneSim::FF_W | SceneSim::FF_OTHER, SceneSim::FF_B);
			data.meshIdxFromCellDataFuncPtr = PistonBaseMeshIdxFromCellData;

			shape->Init (&data, sides);

			shape->gfxBlocking = CellMesh::GB_NONE;

			shape->Init_AddQuad (&data, 0, SceneSim::F_B, Vec3 (0.0f, 0.0f, 0.0f), Vec3 (1.0f, 0.0f, 0.0f), Vec3 (0.0f, 0.0f, 1.0f), SceneSim::F_B, b);
			shape->Init_AddQuad (&data, 0, SceneSim::F_N, Vec3 (1.0f, 1.0f, 1.0f), Vec3 (-1.0f, 0.0f, 0.0f), Vec3 (0.0f, -1.0f, 0.0f), SceneSim::F_N, sides);
			shape->Init_AddQuad (&data, 0, SceneSim::F_E, Vec3 (1.0f, 1.0f, 0.0f), Vec3 (0.0f, 0.0f, 1.0f), Vec3 (0.0f, -1.0f, 0.0f), SceneSim::F_E, sides);
			shape->Init_AddQuad (&data, 0, SceneSim::F_S, Vec3 (0.0f, 1.0f, 0.0f), Vec3 (1.0f, 0.0f, 0.0f), Vec3 (0.0f, -1.0f, 0.0f), SceneSim::F_S, sides);
			shape->Init_AddQuad (&data, 0, SceneSim::F_W, Vec3 (0.0f, 1.0f, 1.0f), Vec3 (0.0f, 0.0f, -1.0f), Vec3 (0.0f, -1.0f, 0.0f), SceneSim::F_W, sides);
			shape->Init_AddQuad (&data, 0, SceneSim::F_OTHER, Vec3 (0.0f, 1.0f, 1.0f), Vec3 (1.0f, 0.0f, 0.0f), Vec3 (0.0f, 0.0f, -1.0f), SceneSim::F_T, t);

			shape->Init_Copy (&data, 1, 0);
			shape->Init_Copy (&data, 2, 0);
			shape->Init_Copy (&data, 3, 0);
			shape->Init_Copy (&data, 4, 0);
			shape->Init_Copy (&data, 5, 0);

			shape->Init_ApplyUvRgbs (&data, 0, Vec4 (1.0f, 1.0f, 1.0f, 1.0f));
			shape->Init_ApplyUvRgbs (&data, 1, Vec4 (1.0f, 1.0f, 1.0f, 1.0f));
			shape->Init_ApplyUvRgbs (&data, 2, Vec4 (1.0f, 1.0f, 1.0f, 1.0f));
			shape->Init_ApplyUvRgbs (&data, 3, Vec4 (1.0f, 1.0f, 1.0f, 1.0f));
			shape->Init_ApplyUvRgbs (&data, 4, Vec4 (1.0f, 1.0f, 1.0f, 1.0f));
			shape->Init_ApplyUvRgbs (&data, 5, Vec4 (1.0f, 1.0f, 1.0f, 1.0f));

			shape->Init_Rotate (&data, 1, SceneSim::F_B); shape->Init_Rotate (&data, 1, SceneSim::F_B);
			shape->Init_Rotate (&data, 2, SceneSim::F_T); 
			shape->Init_Rotate (&data, 3, SceneSim::F_T); shape->Init_Rotate (&data, 3, SceneSim::F_E);
			shape->Init_Rotate (&data, 4, SceneSim::F_T); shape->Init_Rotate (&data, 4, SceneSim::F_S);
			shape->Init_Rotate (&data, 5, SceneSim::F_T); shape->Init_Rotate (&data, 5, SceneSim::F_W);

			shape->Init_ApplyRgbs (&data, 0, Vec4 (1.0f, 1.0f, 1.0f, 1.0f));
			shape->Init_ApplyRgbs (&data, 1, Vec4 (1.0f, 1.0f, 1.0f, 1.0f));
			shape->Init_ApplyRgbs (&data, 2, Vec4 (1.0f, 1.0f, 1.0f, 1.0f));
			shape->Init_ApplyRgbs (&data, 3, Vec4 (1.0f, 1.0f, 1.0f, 1.0f));
			shape->Init_ApplyRgbs (&data, 4, Vec4 (1.0f, 1.0f, 1.0f, 1.0f));
			shape->Init_ApplyRgbs (&data, 5, Vec4 (1.0f, 1.0f, 1.0f, 1.0f));

			const float pixel = 16.0f / 16.0f;
			shape->Init_AddQuad (&data, 6, SceneSim::F_B, Vec3 (0.0f, 0.0f, 0.0f), Vec3 (1.0f, 0.0f, 0.0f), Vec3 (0.0f, 0.0f, 1.0f), SceneSim::F_B, b);
			shape->Init_AddQuad (&data, 6, SceneSim::F_N, Vec3 (1.0f, pixel, 1.0f), Vec3 (-1.0f, 0.0f, 0.0f), Vec3 (0.0f, -pixel, 0.0f), SceneSim::F_N, sides + 1);
			shape->Init_AddQuad (&data, 6, SceneSim::F_E, Vec3 (1.0f, pixel, 0.0f), Vec3 (0.0f, 0.0f, 1.0f), Vec3 (0.0f, -pixel, 0.0f), SceneSim::F_E, sides + 1);
			shape->Init_AddQuad (&data, 6, SceneSim::F_S, Vec3 (0.0f, pixel, 0.0f), Vec3 (1.0f, 0.0f, 0.0f), Vec3 (0.0f, -pixel, 0.0f), SceneSim::F_S, sides + 1);
			shape->Init_AddQuad (&data, 6, SceneSim::F_W, Vec3 (0.0f, pixel, 1.0f), Vec3 (0.0f, 0.0f, -1.0f), Vec3 (0.0f, -pixel, 0.0f), SceneSim::F_W, sides + 1);
			shape->Init_AddQuad (&data, 6, SceneSim::F_OTHER, Vec3 (0.0f, pixel, 1.0f), Vec3 (1.0f, 0.0f, 0.0f), Vec3 (0.0f, 0.0f, -1.0f), SceneSim::F_T, mech);

			shape->Init_Copy (&data, 7, 6);
			shape->Init_Copy (&data, 8, 6);
			shape->Init_Copy (&data, 9, 6);
			shape->Init_Copy (&data, 10, 6);
			shape->Init_Copy (&data, 11, 6);

			shape->Init_ApplyUvRgbs (&data, 6, Vec4 (1.0f, 1.0f, 1.0f, 1.0f));
			shape->Init_ApplyUvRgbs (&data, 7, Vec4 (1.0f, 1.0f, 1.0f, 1.0f));
			shape->Init_ApplyUvRgbs (&data, 8, Vec4 (1.0f, 1.0f, 1.0f, 1.0f));
			shape->Init_ApplyUvRgbs (&data, 9, Vec4 (1.0f, 1.0f, 1.0f, 1.0f));
			shape->Init_ApplyUvRgbs (&data, 10, Vec4 (1.0f, 1.0f, 1.0f, 1.0f));
			shape->Init_ApplyUvRgbs (&data, 11, Vec4 (1.0f, 1.0f, 1.0f, 1.0f));

			shape->Init_Rotate (&data, 7, SceneSim::F_B); shape->Init_Rotate (&data, 7, SceneSim::F_B);
			shape->Init_Rotate (&data, 8, SceneSim::F_T); 
			shape->Init_Rotate (&data, 9, SceneSim::F_T); shape->Init_Rotate (&data, 9, SceneSim::F_E);
			shape->Init_Rotate (&data, 10, SceneSim::F_T); shape->Init_Rotate (&data, 10, SceneSim::F_S);
			shape->Init_Rotate (&data, 11, SceneSim::F_T); shape->Init_Rotate (&data, 11, SceneSim::F_W);

			shape->Init_ApplyRgbs (&data, 6, Vec4 (1.0f, 1.0f, 1.0f, 1.0f));
			shape->Init_ApplyRgbs (&data, 7, Vec4 (1.0f, 1.0f, 1.0f, 1.0f));
			shape->Init_ApplyRgbs (&data, 8, Vec4 (1.0f, 1.0f, 1.0f, 1.0f));
			shape->Init_ApplyRgbs (&data, 9, Vec4 (1.0f, 1.0f, 1.0f, 1.0f));
			shape->Init_ApplyRgbs (&data, 10, Vec4 (1.0f, 1.0f, 1.0f, 1.0f));
			shape->Init_ApplyRgbs (&data, 11, Vec4 (1.0f, 1.0f, 1.0f, 1.0f));

			shape->Init_Finish (&data, 0);
			shape->Init_Finish (&data, 1);
			shape->Init_Finish (&data, 2);
			shape->Init_Finish (&data, 3);
			shape->Init_Finish (&data, 4);
			shape->Init_Finish (&data, 5);
			shape->Init_Finish (&data, 6);
			shape->Init_Finish (&data, 7);
			shape->Init_Finish (&data, 8);
			shape->Init_Finish (&data, 9);
			shape->Init_Finish (&data, 10);
			shape->Init_Finish (&data, 11);
		}
	}
}
