#include "stdafx.h"

#include "Shapes_InitData.h"
#include "CellMesh_ShapeBase.h"

namespace CellMesh
{
	namespace Shapes
	{
		int PistonHeadMeshIdxFromCellData (int type, int data) 
		{
			int meshIdx = 0;

			switch (data & 0x7)
			{
			case 0: meshIdx = 1; break;
			case 1: meshIdx = 0; break;
			case 2: meshIdx = 3; break;
			case 3: meshIdx = 5; break;
			case 4: meshIdx = 2; break;
			case 5: meshIdx = 4; break;
			default: assert (false);
			}
			
			if ((data & 0x08) == 0x00)
			{
				meshIdx += 6;
			}

			return meshIdx;
		}

		void InitPistonHead (ShapeBase * shape, int sticky, int normal, int sides, int stem)
		{
			float unit = 1.0f / 16.0f;

			InitData data (12, 11 * 2, SceneSim::FF_B | SceneSim::FF_T | SceneSim::FF_N | SceneSim::FF_E | SceneSim::FF_S | SceneSim::FF_W | SceneSim::FF_OTHER, SceneSim::FF_T);
			data.meshIdxFromCellDataFuncPtr = PistonHeadMeshIdxFromCellData;

			shape->Init (&data, sides);

			shape->gfxBlocking = CellMesh::GB_NONE;

			// head
			{
				Vec3 bl (unit * 0.0f, unit * 12.0f, unit * 0.0f);
				Vec3 tr (unit * 16.0f, unit * 16.0f, unit * 16.0f);
				shape->Init_AddBoxQuad (&data, 0, SceneSim::F_T, bl, tr, SceneSim::F_T, sticky);
				shape->Init_AddBoxQuad (&data, 0, SceneSim::F_B, bl, tr, SceneSim::F_B, normal);
				shape->Init_AddBoxQuad (&data, 0, SceneSim::F_N, bl, tr, SceneSim::F_N, sides);
				shape->Init_AddBoxQuad (&data, 0, SceneSim::F_E, bl, tr, SceneSim::F_E, sides);
				shape->Init_AddBoxQuad (&data, 0, SceneSim::F_S, bl, tr, SceneSim::F_S, sides);
				shape->Init_AddBoxQuad (&data, 0, SceneSim::F_W, bl, tr, SceneSim::F_W, sides);
			}

			// stem
			{
				Vec3 bl (unit * 6.0f, unit * 0.0f, unit * 6.0f);
				Vec3 tr (unit * 10.0f, unit * 12.0f, unit * 10.0f);
				shape->Init_AddBoxQuad (&data, 0, SceneSim::F_B, bl, tr, SceneSim::F_B, stem);
				shape->Init_AddBoxQuad (&data, 0, SceneSim::F_OTHER, bl, tr, SceneSim::F_N, stem);
				shape->Init_AddBoxQuad (&data, 0, SceneSim::F_OTHER, bl, tr, SceneSim::F_E, stem);
				shape->Init_AddBoxQuad (&data, 0, SceneSim::F_OTHER, bl, tr, SceneSim::F_S, stem);
				shape->Init_AddBoxQuad (&data, 0, SceneSim::F_OTHER, bl, tr, SceneSim::F_W, stem);			
			}

			shape->Init_Copy (&data, 1, 0);
			shape->Init_Copy (&data, 2, 0);
			shape->Init_Copy (&data, 3, 0);
			shape->Init_Copy (&data, 4, 0);
			shape->Init_Copy (&data, 5, 0);

			shape->Init_ApplyUvRgbs (&data, 0, Vec4 (1.0f, 1.0f, 1.0f, 1.0f));
			shape->Init_ApplyUvRgbs (&data, 1, Vec4 (1.0f, 1.0f, 1.0f, 1.0f));
			shape->Init_ApplyUvRgbs (&data, 2, Vec4 (1.0f, 1.0f, 1.0f, 1.0f));
			shape->Init_ApplyUvRgbs (&data, 3, Vec4 (1.0f, 1.0f, 1.0f, 1.0f));
			shape->Init_ApplyUvRgbs (&data, 4, Vec4 (1.0f, 1.0f, 1.0f, 1.0f));
			shape->Init_ApplyUvRgbs (&data, 5, Vec4 (1.0f, 1.0f, 1.0f, 1.0f));

			shape->Init_Rotate (&data, 1, SceneSim::F_B); shape->Init_Rotate (&data, 1, SceneSim::F_B);
			shape->Init_Rotate (&data, 2, SceneSim::F_T); 
			shape->Init_Rotate (&data, 3, SceneSim::F_T); shape->Init_Rotate (&data, 3, SceneSim::F_E);
			shape->Init_Rotate (&data, 4, SceneSim::F_T); shape->Init_Rotate (&data, 4, SceneSim::F_S);
			shape->Init_Rotate (&data, 5, SceneSim::F_T); shape->Init_Rotate (&data, 5, SceneSim::F_W);

			shape->Init_ApplyRgbs (&data, 0, Vec4 (1.0f, 1.0f, 1.0f, 1.0f));
			shape->Init_ApplyRgbs (&data, 1, Vec4 (1.0f, 1.0f, 1.0f, 1.0f));
			shape->Init_ApplyRgbs (&data, 2, Vec4 (1.0f, 1.0f, 1.0f, 1.0f));
			shape->Init_ApplyRgbs (&data, 3, Vec4 (1.0f, 1.0f, 1.0f, 1.0f));
			shape->Init_ApplyRgbs (&data, 4, Vec4 (1.0f, 1.0f, 1.0f, 1.0f));
			shape->Init_ApplyRgbs (&data, 5, Vec4 (1.0f, 1.0f, 1.0f, 1.0f));

			shape->Init_Finish (&data, 0);
			shape->Init_Finish (&data, 1);
			shape->Init_Finish (&data, 2);
			shape->Init_Finish (&data, 3);
			shape->Init_Finish (&data, 4);
			shape->Init_Finish (&data, 5);

			// head
			{
				Vec3 bl (unit * 0.0f, unit * 12.0f, unit * 0.0f);
				Vec3 tr (unit * 16.0f, unit * 16.0f, unit * 16.0f);
				shape->Init_AddBoxQuad (&data, 6, SceneSim::F_T, bl, tr, SceneSim::F_T, normal);
				shape->Init_AddBoxQuad (&data, 6, SceneSim::F_B, bl, tr, SceneSim::F_B, normal);
				shape->Init_AddBoxQuad (&data, 6, SceneSim::F_N, bl, tr, SceneSim::F_N, sides);
				shape->Init_AddBoxQuad (&data, 6, SceneSim::F_E, bl, tr, SceneSim::F_E, sides);
				shape->Init_AddBoxQuad (&data, 6, SceneSim::F_S, bl, tr, SceneSim::F_S, sides);
				shape->Init_AddBoxQuad (&data, 6, SceneSim::F_W, bl, tr, SceneSim::F_W, sides);
			}

			// stem
			{
				Vec3 bl (unit * 6.0f, unit * 0.0f, unit * 6.0f);
				Vec3 tr (unit * 10.0f, unit * 12.0f, unit * 10.0f);
				shape->Init_AddBoxQuad (&data, 6, SceneSim::F_B, bl, tr, SceneSim::F_B, stem);
				shape->Init_AddBoxQuad (&data, 6, SceneSim::F_OTHER, bl, tr, SceneSim::F_N, stem);
				shape->Init_AddBoxQuad (&data, 6, SceneSim::F_OTHER, bl, tr, SceneSim::F_E, stem);
				shape->Init_AddBoxQuad (&data, 6, SceneSim::F_OTHER, bl, tr, SceneSim::F_S, stem);
				shape->Init_AddBoxQuad (&data, 6, SceneSim::F_OTHER, bl, tr, SceneSim::F_W, stem);			
			}

			shape->Init_Copy (&data, 7, 0);
			shape->Init_Copy (&data, 8, 0);
			shape->Init_Copy (&data, 9, 0);
			shape->Init_Copy (&data, 10, 0);
			shape->Init_Copy (&data, 11, 0);

			shape->Init_ApplyUvRgbs (&data, 6, Vec4 (1.0f, 1.0f, 1.0f, 1.0f));
			shape->Init_ApplyUvRgbs (&data, 7, Vec4 (1.0f, 1.0f, 1.0f, 1.0f));
			shape->Init_ApplyUvRgbs (&data, 8, Vec4 (1.0f, 1.0f, 1.0f, 1.0f));
			shape->Init_ApplyUvRgbs (&data, 9, Vec4 (1.0f, 1.0f, 1.0f, 1.0f));
			shape->Init_ApplyUvRgbs (&data, 10, Vec4 (1.0f, 1.0f, 1.0f, 1.0f));
			shape->Init_ApplyUvRgbs (&data, 11, Vec4 (1.0f, 1.0f, 1.0f, 1.0f));

			shape->Init_Rotate (&data, 7, SceneSim::F_B); shape->Init_Rotate (&data, 7, SceneSim::F_B);
			shape->Init_Rotate (&data, 8, SceneSim::F_T); 
			shape->Init_Rotate (&data, 9, SceneSim::F_T); shape->Init_Rotate (&data, 9, SceneSim::F_E);
			shape->Init_Rotate (&data, 10, SceneSim::F_T); shape->Init_Rotate (&data, 10, SceneSim::F_S);
			shape->Init_Rotate (&data, 11, SceneSim::F_T); shape->Init_Rotate (&data, 11, SceneSim::F_W);

			shape->Init_ApplyRgbs (&data, 6, Vec4 (1.0f, 1.0f, 1.0f, 1.0f));
			shape->Init_ApplyRgbs (&data, 7, Vec4 (1.0f, 1.0f, 1.0f, 1.0f));
			shape->Init_ApplyRgbs (&data, 8, Vec4 (1.0f, 1.0f, 1.0f, 1.0f));
			shape->Init_ApplyRgbs (&data, 9, Vec4 (1.0f, 1.0f, 1.0f, 1.0f));
			shape->Init_ApplyRgbs (&data, 10, Vec4 (1.0f, 1.0f, 1.0f, 1.0f));
			shape->Init_ApplyRgbs (&data, 11, Vec4 (1.0f, 1.0f, 1.0f, 1.0f));

			shape->Init_Finish (&data, 6);
			shape->Init_Finish (&data, 7);
			shape->Init_Finish (&data, 8);
			shape->Init_Finish (&data, 9);
			shape->Init_Finish (&data, 10);
			shape->Init_Finish (&data, 11);
		}
	}
}
