#include "stdafx.h"

#include "Shapes_InitData.h"
#include "CellMesh_ShapeBase.h"

namespace CellMesh
{
	namespace Shapes
	{
		int RailwayMeshIdxFromCellData (int type, int data) 
		{
			switch (data)
			{
			case 0: return 1;	// flat ew
			case 1: return 0;	// flat ns
			case 2: return 4;	// ascend s
			case 3: return 2;	// ascend n
			case 4: return 3;	// ascend e
			case 5: return 5;	// ascend w
			case 6: return 6;	// flat s and w
			case 7: return 7;	// flat n and w
			case 8: return 8;	// flat n and e
			case 9: return 9;	// flat s and e
			}
			assert (false);
			return 0;
		}

		int PoweredRailwayMeshIdxFromCellData (int type, int data) 
		{
			int idx = 0;

			idx += (data & 0x08) ? 2 : 0; // powered or not
			idx += (data & 0x01) ? 0 : 1; // NS or EW

			return idx;
		}		

		int DetectorRailwayMeshIdxFromCellData (int type, int data) 
		{
			int idx = 0;

			idx += (data & 0x01) ? 0 : 1; // NS or EW

			return idx;
		}		

		void InitRailway (ShapeBase * shape, int texStraight, int texCorner)
		{
			const float unit = 1.0f / 16.0f;

			InitData data (10, 2, SceneSim::FF_OTHER, 0);
			data.meshIdxFromCellDataFuncPtr = RailwayMeshIdxFromCellData;

			shape->Init (&data, texStraight);

			//shape->entCollisions = SceneSim::EC_NONE;
			shape->gfxBlocking = CellMesh::GB_NONE;

			// straights
			shape->Init_AddQuad (&data, 0, SceneSim::F_OTHER, Vec3 (0.0f, unit * 1.0f, 1.0f), Vec3 (1.0f, 0.0f, 0.0f), Vec3 (0.0f, 0.0f, -1.0f), SceneSim::F_T, texStraight);
			shape->Init_ApplyUvRgbs (&data, 0, Vec4 (1.0f, 1.0f, 1.0f, 1.0f));
			shape->Init_Finish (&data, 0);

			shape->Init_Copy (&data, 1, 0);
			shape->Init_Rotate (&data, 1, SceneSim::F_E);
			shape->Init_Finish (&data, 1);

			// slopes
			shape->Init_AddQuad (&data, 2, SceneSim::F_OTHER, Vec3 (0.0f, unit * 17.0f, 1.0f), Vec3 (1.0f, 0.0f, 0.0f), Vec3 (0.0f, -1.0f, -1.0f), SceneSim::F_T, texStraight);
			shape->Init_ApplyUvRgbs (&data, 2, Vec4 (1.0f, 1.0f, 1.0f, 1.0f));
			shape->Init_Finish (&data, 2);

			shape->Init_Copy (&data, 3, 2);
			shape->Init_Rotate (&data, 3, SceneSim::F_E);
			shape->Init_Finish (&data, 3);

			shape->Init_Copy (&data, 4, 2);
			shape->Init_Rotate (&data, 4, SceneSim::F_S);
			shape->Init_Finish (&data, 4);


			shape->Init_Copy (&data, 5, 2);
			shape->Init_Rotate (&data, 5, SceneSim::F_W);
			shape->Init_Finish (&data, 5);

			// corners
			shape->Init_AddQuad (&data, 6, SceneSim::F_OTHER, Vec3 (0.0f, unit * 1.0f, 1.0f), Vec3 (1.0f, 0.0f, 0.0f), Vec3 (0.0f, 0.0f, -1.0f), SceneSim::F_T, texCorner);
			shape->Init_ApplyUvRgbs (&data, 6, Vec4 (1.0f, 1.0f, 1.0f, 1.0f));
			shape->Init_Finish (&data, 6);

			shape->Init_Copy (&data, 7, 6);
			shape->Init_Rotate (&data, 7, SceneSim::F_E);
			shape->Init_Finish (&data, 7);

			shape->Init_Copy (&data, 8, 6);
			shape->Init_Rotate (&data, 8, SceneSim::F_S);
			shape->Init_Finish (&data, 8);

			shape->Init_Copy (&data, 9, 6);
			shape->Init_Rotate (&data, 9, SceneSim::F_W);
			shape->Init_Finish (&data, 9);
		}

		void InitPoweredRailway (ShapeBase * shape, int texPowered, int texUnpowered)
		{
			const float unit = 1.0f / 16.0f;

			InitData data (4, 2, SceneSim::FF_OTHER, 0);
			data.meshIdxFromCellDataFuncPtr = PoweredRailwayMeshIdxFromCellData;

			shape->Init (&data, texPowered);

			//shape->entCollisions = SceneSim::EC_NONE;
			shape->gfxBlocking = CellMesh::GB_NONE;

			// unpowered straights
			shape->Init_AddQuad (&data, 0, SceneSim::F_OTHER, Vec3 (0.0f, unit * 1.0f, 1.0f), Vec3 (1.0f, 0.0f, 0.0f), Vec3 (0.0f, 0.0f, -1.0f), SceneSim::F_T, texUnpowered);
			shape->Init_ApplyUvRgbs (&data, 0, Vec4 (1.0f, 1.0f, 1.0f, 1.0f));
			shape->Init_Finish (&data, 0);

			shape->Init_Copy (&data, 1, 0);
			shape->Init_Rotate (&data, 1, SceneSim::F_E);
			shape->Init_Finish (&data, 1);

			// powered straights
			shape->Init_AddQuad (&data, 2, SceneSim::F_OTHER, Vec3 (0.0f, unit * 1.0f, 1.0f), Vec3 (1.0f, 0.0f, 0.0f), Vec3 (0.0f, 0.0f, -1.0f), SceneSim::F_T, texPowered);
			shape->Init_ApplyUvRgbs (&data, 2, Vec4 (1.0f, 1.0f, 1.0f, 1.0f));
			shape->Init_Finish (&data, 2);

			shape->Init_Copy (&data, 3, 2);
			shape->Init_Rotate (&data, 3, SceneSim::F_E);
			shape->Init_Finish (&data, 3);
		}

		void InitDetectorRailway (ShapeBase * shape, int texStraight)
		{
			const float unit = 1.0f / 16.0f;

			InitData data (2, 2, SceneSim::FF_OTHER, 0);
			data.meshIdxFromCellDataFuncPtr = DetectorRailwayMeshIdxFromCellData;

			shape->Init (&data, texStraight);

			//shape->entCollisions = SceneSim::EC_NONE;
			shape->gfxBlocking = CellMesh::GB_NONE;
	
			// straights
			shape->Init_AddQuad (&data, 0, SceneSim::F_OTHER, Vec3 (0.0f, unit * 1.0f, 1.0f), Vec3 (1.0f, 0.0f, 0.0f), Vec3 (0.0f, 0.0f, -1.0f), SceneSim::F_T, texStraight);
			shape->Init_ApplyUvRgbs (&data, 0, Vec4 (1.0f, 1.0f, 1.0f, 1.0f));
			shape->Init_Finish (&data, 0);

			shape->Init_Copy (&data, 1, 0);
			shape->Init_Rotate (&data, 1, SceneSim::F_E);
			shape->Init_Finish (&data, 1);
		}
	}
}
