#include "stdafx.h"

#include "Shapes_InitData.h"
#include "CellMesh_ShapeBase.h"

namespace CellMesh
{
	namespace Shapes
	{
		int CalcTorchMeshIdx (int type, int data)
		{
			switch (data)
			{
			case 5: return 0;	// floor
			case 1: return 1;	// n
			case 3: return 2;	// e
			case 2: return 3;	// s
			case 4: return 4;	// w
			}
			assert (false);
			return 0;
		}

		void InitTorch (ShapeBase * shape, int tex)
		{
			const float unit = 1.0f / 16.0f;

			InitData data (5, 10, SceneSim::FF_OTHER, 0);
			data.meshIdxFromCellDataFuncPtr = CalcTorchMeshIdx;

			shape->Init (&data, tex);

			//shape->entCollisions = SceneSim::EC_NONE;
			shape->gfxBlocking = CellMesh::GB_NONE;
			shape->isFullBright = true;

			shape->Init_AddQuad (&data, 0, SceneSim::F_OTHER, Vec3 (unit * 7.0f, unit * 10.0f, unit * 9.0f), Vec3 (unit * 2.0f, 0.0f, 0.0f), Vec3 (0.0f, 0.0f, -unit * 2.0f), SceneSim::F_T, 64);
			shape->Init_AddQuad (&data, 0, SceneSim::F_OTHER, Vec3 (1.0f, 1.0f, unit * 9.0f), Vec3 (-1.0f, 0.0f, 0.0f), Vec3 (0.0f, -1.0f, 0.0f), SceneSim::F_N, tex);
			shape->Init_AddQuad (&data, 0, SceneSim::F_OTHER, Vec3 (unit * 9.0f, 1.0f, 0.0f), Vec3 (0.0f, 0.0f, 1.0f), Vec3 (0.0f, -1.0f, 0.0f), SceneSim::F_E, tex);
			shape->Init_AddQuad (&data, 0, SceneSim::F_OTHER, Vec3 (0.0f, 1.0f, unit * 7.0f), Vec3 (1.0f, 0.0f, 0.0f), Vec3 (0.0f, -1.0f, 0.0f), SceneSim::F_S, tex);
			shape->Init_AddQuad (&data, 0, SceneSim::F_OTHER, Vec3 (unit * 7.0f, 1.0f, 1.0f), Vec3 (0.0f, 0.0f, -1.0f), Vec3 (0.0f, -1.0f, 0.0f), SceneSim::F_W, tex);

			shape->Init_Copy (&data, 1, 0);
			shape->Init_Copy (&data, 2, 0);
			shape->Init_Copy (&data, 3, 0);
			shape->Init_Copy (&data, 4, 0);

			shape->Init_ApplyUvRgbs (&data, 0, Vec4 (1.0f, 1.0f, 1.0f, 1.0f));
			shape->Init_ApplyUvRgbs (&data, 1, Vec4 (1.0f, 1.0f, 1.0f, 1.0f));
			shape->Init_ApplyUvRgbs (&data, 2, Vec4 (1.0f, 1.0f, 1.0f, 1.0f));
			shape->Init_ApplyUvRgbs (&data, 3, Vec4 (1.0f, 1.0f, 1.0f, 1.0f));
			shape->Init_ApplyUvRgbs (&data, 4, Vec4 (1.0f, 1.0f, 1.0f, 1.0f));

			Mat44 mat = glm::translate (Mat44 (), Vec3 (0.0f, unit * 4.0f, unit * 8.0f));
			mat [1] = Vec4 (0.0f, 1.0f, -0.5f, 0.0f);
			shape->Init_MultiplyXyz (1, &mat);

			mat = glm::translate (Mat44 (), Vec3 (unit * 8.0f, unit * 4.0f, 0.0f));
			mat [1] = Vec4 (-0.5f, 1.0f, 0.0f, 0.0f);
			shape->Init_MultiplyXyz (2, &mat);

			mat = glm::translate (Mat44 (), Vec3 (0.0f, unit * 4.0f, -unit * 8.0f));
			mat [1] = Vec4 (0.0f, 1.0f, 0.5f, 0.0f);
			shape->Init_MultiplyXyz (3, &mat);

			mat = glm::translate (Mat44 (), Vec3 (-unit * 8.0f,unit * 4.0f, 0.0f));
			mat [1] = Vec4 (0.5f, 1.0f, 0.0f, 0.0f);
			shape->Init_MultiplyXyz (4, &mat);

			shape->Init_Finish (&data, 0);
			shape->Init_Finish (&data, 1);
			shape->Init_Finish (&data, 2);
			shape->Init_Finish (&data, 3);
			shape->Init_Finish (&data, 4);
		}
	}
}
