#include "stdafx.h"

#include "SceneSim/Query_Cell.h"
#include "SceneSim/SceneSim_Cell.h"
#include "SceneSim/SceneSim_Column.h"
#include "SceneSim/SceneSim_ColumnCells.h"

#include "../SceneGfx/SceneGfx_Column.h"

#include "Shapes_InitData.h"
#include "CellMesh_Defs.h"
#include "CellMesh_ShapeBase.h"

namespace CellMesh
{
	namespace Shapes
	{
		int WireMeshIdxFromCellData (int type, int data) 
		{
			int res = (data & 0x10) ? 11 : 0;
			switch (data & 0x0f)
			{
			case 0: return 0 + res;	// 0 none (so use cross)
			case 1: return 2 + res;	// 1 N
			case 2: return 1 + res;	// 2 E
			case 3: return 7 + res;	// 3 NE
			case 4: return 2 + res;	// 4 S
			case 5: return 2 + res;	// 5 NS
			case 6: return 8 + res;	// 6 SE
			case 7: return 3 + res;	// 7 NES
			case 8: return 1 + res;	// 8 W
			case 9: return 10 + res;	// 9 NW
			case 10: return 1 + res;	// 10 EW
			case 11: return 6 + res;	// 11 NEW
			case 12: return 9 + res;	// 12 SW
			case 13: return 5 + res;	// 13 NSW
			case 14: return 4 + res;	// 14 SEW
			case 15: return 0 + res;	// 15 NESW
			}
			assert (false);
			return 0;
		}

		int RemapWireTex (int meshIdx, int texId, void * data)
		{
			return texId - 16;
		}

		bool IsAdjCellWire (const SceneSim::ColumnCells * cellsSrc, Int3 cellId, Int3 cellOffset)
		{
			const SceneSim::Cell * cell = cellsSrc->GetCell (cellId + cellOffset);
			return (cell->type == 55 || cell->type == 75 ||cell->type == 76);
		}

		int WireAwfulCalcData (AddShapeData * addData)
		{
			int value = 0;

			//const SceneSim::ColumnCells * cellsSrc = addData->columnSrc->GetCells ();

			//if (IsAdjCellWire (cellsSrc, addData->cellXyz, Int3 (0, 0, 1)))
			//{
			//	value |= 1;
			//}
			//if (IsAdjCellWire (cellsSrc, addData->cellXyz, Int3 (1, 0, 0)))
			//{
			//	value |= 2;
			//}
			//if (IsAdjCellWire (cellsSrc, addData->cellXyz, Int3 (0, 0, -1)))
			//{
			//	value |= 4;
			//}
			//if (IsAdjCellWire (cellsSrc, addData->cellXyz, Int3 (-1, 0, 0)))
			//{
			//	value |= 8;
			//}
			//if (addData->cell->blockData > 0)
			//{
			//	value |= 16;
			//}

			return value;
		}

		void InitWire (ShapeBase * shape, InitData * data, int fromIdx, int texLine, int texCross, Vec4 rgba)
		{
			const float unit = 1.0f / 16.0f;

			// 0 = cross
			shape->Init_AddQuad (data, fromIdx + 0, SceneSim::F_OTHER, Vec3 (0.0f, unit * 1.0f, 1.0f), Vec3 (1.0f, 0.0f, 0.0f), Vec3 (0.0f, 0.0f, -1.0f), SceneSim::F_T, texCross);
			shape->Init_ApplyUvRgbs (data, fromIdx + 0, rgba);
			shape->Init_Finish (data, fromIdx + 0);

			// 1 = EW
			shape->Init_AddQuad (data, fromIdx + 1, SceneSim::F_OTHER, Vec3 (0.0f, unit * 1.0f, 1.0f), Vec3 (1.0f, 0.0f, 0.0f), Vec3 (0.0f, 0.0f, -1.0f), SceneSim::F_T, texLine);
			shape->Init_ApplyUvRgbs (data, fromIdx + 1, rgba);
			shape->Init_Finish (data, fromIdx + 1);

			// 2 = NW
			shape->Init_Copy (data, fromIdx + 2, fromIdx + 1);
			shape->Init_Rotate (data, fromIdx + 2, SceneSim::F_E);
			shape->Init_Finish (data, fromIdx + 2);

			// 3 = NES
			shape->Init_AddQuad (data, fromIdx + 3, SceneSim::F_OTHER, Vec3 (5.0f * unit, unit * 1.0f, 1.0f), Vec3 (11 * unit, 0.0f, 0.0f), Vec3 (0.0f, 0.0f, -1.0f), SceneSim::F_T, texCross);
			shape->Init_ApplyUvRgbs (data, fromIdx + 3, rgba);
			shape->Init_Finish (data, fromIdx + 3);

			// 4 = ESW
			shape->Init_Copy (data, fromIdx + 4, fromIdx + 3);
			shape->Init_Rotate (data, fromIdx + 4, SceneSim::F_E);
			//shape->Init_ApplyUvRgbs (data, fromIdx + 4);
			shape->Init_Finish (data, fromIdx + 4);

			// 5 = NSW
			shape->Init_Copy (data, fromIdx + 5, fromIdx + 3);
			shape->Init_Rotate (data, fromIdx + 5, SceneSim::F_S);
			//shape->Init_ApplyUvRgbs (data, fromIdx + 5);
			shape->Init_Finish (data, fromIdx + 5);

			// 6 = NEW
			shape->Init_Copy (data, fromIdx + 6, fromIdx + 3);
			shape->Init_Rotate (data, fromIdx + 6, SceneSim::F_W);
			//shape->Init_ApplyUvRgbs (data, fromIdx + 6);
			shape->Init_Finish (data, fromIdx + 6);

			// 7 = NE
			shape->Init_AddQuad (data, fromIdx + 7, SceneSim::F_OTHER, Vec3 (5.0f * unit, unit * 1.0f, 1.0f), Vec3 (11 * unit, 0.0f, 0.0f), Vec3 (0.0f, 0.0f, -11.0f * unit), SceneSim::F_T, texCross);
			shape->Init_ApplyUvRgbs (data, fromIdx + 7, rgba);
			shape->Init_Finish (data, fromIdx + 7);

			// 8 = ES
			shape->Init_Copy (data, fromIdx + 8, fromIdx + 7);
			shape->Init_Rotate (data, fromIdx + 8, SceneSim::F_E);
			//shape->Init_ApplyUvRgbs (data, fromIdx + 8);
			shape->Init_Finish (data, fromIdx + 8);

			// 9 = SW
			shape->Init_Copy (data, fromIdx + 9, fromIdx + 7);
			shape->Init_Rotate (data, fromIdx + 9, SceneSim::F_S);
			//shape->Init_ApplyUvRgbs (data, fromIdx + 9);
			shape->Init_Finish (data, fromIdx + 9);

			// 10 = NW
			shape->Init_Copy (data, fromIdx + 10, fromIdx + 7);
			shape->Init_Rotate (data, fromIdx + 10, SceneSim::F_W);
			//shape->Init_ApplyUvRgbs (data, fromIdx + 10);
			shape->Init_Finish (data, fromIdx + 10);
		}

		void InitWire (ShapeBase * shape, int texLine, int texCross)
		{
			InitData data (22, 2, SceneSim::FF_OTHER, 0);
			data.meshIdxFromCellDataFuncPtr = WireMeshIdxFromCellData;
			data.awfulCalcDataFuncPtr = WireAwfulCalcData;

			shape->Init (&data, texCross);

			//shape->entCollisions = SceneSim::EC_NONE;
			shape->gfxBlocking = CellMesh::GB_NONE;

			InitWire (shape, &data, 0, texLine, texCross, Vec4 (0.25f, 0.0f, 0.0f, 0.0f));
			InitWire (shape, &data, 11, texLine, texCross, Vec4 (1.0f, 0.0f, 0.0f, 0.0f));

		}
	}
}
