#include "../../stdafx.h"

//*********************************************************
//GLFONT.CPP -- glFont routines
//Copyright (c) 1998 Brad Fish
//Copyright (c) 2002 Henri Kyrki
//See glFont.txt for terms of use
//10.5 2002
//*********************************************************

#include <stdio.h>
#include "glfont.h"

#include "../../Gfx/VertexTypes_XyzUvRgb.h"
#include "../../Gfx/Gfx_Funcs.h"

//*********************************************************
//GLFontBase
//*********************************************************

GLFontBase::GLFontBase() : ok(FALSE)
{
}

void GLFontBase::CreateImpl(const char * Filename, GLuint Tex, bool PixelPerfect)
{
	Font.Char = NULL;
	FreeResources();

	FILE *Input;

	//Open font file
	if ((Input = fopen(Filename, "rb")) == NULL)
		throw GLFontError::InvalidFile();

	//Read glFont structure
	fread(&Font, sizeof(GLFONT), 1, Input);

	//Save texture number
	Font.Tex = Tex;

	//Get number of characters
	int Num = Font.IntEnd - Font.IntStart + 1;

	//Allocate memory for characters
	//if ((Font.Char = (GLFONTCHAR *)malloc(sizeof(GLFONTCHAR) * Num)) == NULL)
	Font.Char = aah_new GLFONTCHAR[Num];

	//Read glFont characters
	fread(Font.Char, sizeof(GLFONTCHAR), Num, Input);

	//Get texture size
	Num = Font.TexWidth * Font.TexHeight * 2;

	//Allocate memory for texture data
	//TexBytes = (char *)malloc(Num)
	char *TexBytes = aah_new char[Num];

	//Read texture data
	fread(TexBytes, sizeof(char), Num, Input);

	//Set texture attributes
	glBindTexture(GL_TEXTURE_2D, Font.Tex);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
	if(PixelPerfect)
	{
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		//glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
		//glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
	}
	else
	{
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	}
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);

	//Create texture
	glTexImage2D(GL_TEXTURE_2D, 0, 2, Font.TexWidth, Font.TexHeight, 0, GL_LUMINANCE_ALPHA, GL_UNSIGNED_BYTE, (void *)TexBytes);

	// make mipmaps?
	{
		glBindTexture( GL_TEXTURE_2D, Font.Tex );
		glEnable(GL_TEXTURE_2D); // bind then enable to avoid ATI bug
		//Gfx::glGenerateMipmapARB (GL_TEXTURE_2D);
	}

	//Clean up

	delete []TexBytes;
	fclose(Input);

	ok = TRUE;
}
//*********************************************************
void GLFontBase::FreeResources ()
{
	//Free character memory
	if (Font.Char != NULL) delete []Font.Char;
	ok = FALSE;
}
//*********************************************************
void GLFontBase::Begin ()
{
	if (!ok)
	{
		throw GLFontError::InvalidFont();
	}

	glBindTexture(GL_TEXTURE_2D, Font.Tex);
}
//*********************************************************
GLFontBase::~GLFontBase ()
{
	FreeResources();
}

//*********************************************************
//PixelPerfectGLFont
//*********************************************************

PixelPerfectGLFont::PixelPerfectGLFont()
{
}
//*********************************************************
void PixelPerfectGLFont::Create(const char * Filename, GLuint Tex)
{
	GLFontBase::CreateImpl(Filename, Tex, TRUE);
	for (int i = 0; i < Font.IntEnd - Font.IntStart + 1; i++)
	{
		Font.Char[i].width = (int)((Font.Char[i].tx2 - Font.Char[i].tx1)*Font.TexWidth);
		Font.Char[i].height = (int)((Font.Char[i].ty2 - Font.Char[i].ty1)*Font.TexHeight);
	}
}

//*********************************************************
int PixelPerfectGLFont::CharLength (char c)
{
	assert (c != '\0');

	GLFONTCHAR *Char = &Font.Char[c - Font.IntStart];
	return Char->width;
}

//*********************************************************
int PixelPerfectGLFont::TextLength (const char * String)
{
	//Return if we don't have a valid glFont
	if (!ok)
	{
		throw GLFontError::InvalidFont();
	}

	int pixLength = 0;

	//Get length of std::string
	int Length = strlen (String);

	//Loop through characters
	for (int i = 0; i < Length; i++)
	{
		GLFONTCHAR *Char = &Font.Char[(int)String[i] - Font.IntStart];
		pixLength += Char->width;
	}

	return pixLength;
}

//*********************************************************
int PixelPerfectGLFont::TextOut (const char * String, int x, int y, int z)
{
	//Return if we don't have a valid glFont
	if (!ok)
	{
		throw GLFontError::InvalidFont();
	}

	int startX = x;

	//Get length of std::string
	int Length = strlen (String);

	//Begin rendering quads
	glBegin(GL_QUADS);

	//Loop through characters
	for (int i = 0; i < Length; i++)
	{
		//Get pointer to glFont character
		GLFONTCHAR *Char = &Font.Char[(int)String[i] - Font.IntStart];

		//Specify vertices and texture coordinates
		glTexCoord2f(Char->tx1, Char->ty1);
		glVertex3i(x, y, z);
		glTexCoord2f(Char->tx1, Char->ty2);
		glVertex3i(x, y + Char->height, z);
		glTexCoord2f(Char->tx2, Char->ty2);
		glVertex3i(x + Char->width, y + Char->height, z);
		glTexCoord2f(Char->tx2, Char->ty1);
		glVertex3i(x + Char->width, y, z);

		//Move to next character
		x += Char->width;
	}

	//Stop rendering quads
	glEnd();

	return x - startX;
}

//*********************************************************
int PixelPerfectGLFont::TextOut (const char * String, int charStart, int charEnd, int x, int y, int z)
{
	//Return if we don't have a valid glFont
	if (!ok)
	{
		throw GLFontError::InvalidFont();
	}

	int startX = x;

	//Get length of std::string
	assert (charStart >= 0 && charEnd >= 0);
	assert (charStart <= strlen (String) && charEnd <= strlen (String));

	//Begin rendering quads
	glBegin(GL_QUADS);

	//Loop through characters
	for (int i = charStart; i < charEnd; i++)
	{
		//Get pointer to glFont character
		GLFONTCHAR *Char = &Font.Char[(int)String[i] - Font.IntStart];

		//Specify vertices and texture coordinates
		glTexCoord2f(Char->tx1, Char->ty1);
		glVertex3i(x, y, z);
		glTexCoord2f(Char->tx1, Char->ty2);
		glVertex3i(x, y + Char->height, z);
		glTexCoord2f(Char->tx2, Char->ty2);
		glVertex3i(x + Char->width, y + Char->height, z);
		glTexCoord2f(Char->tx2, Char->ty1);
		glVertex3i(x + Char->width, y, z);

		//Move to next character
		x += Char->width;
	}

	//Stop rendering quads
	glEnd();

	return x - startX;
}

//*********************************************************
//GLFont
//*********************************************************

GLFont::GLFont()
{
}
//*********************************************************
void GLFont::Create(const char * Filename, GLuint Tex)
{
	GLFontBase::CreateImpl(Filename, Tex, FALSE);
}
//*********************************************************
int GLFont::TextLength (const char * String)
{
	//Return if we don't have a valid glFont
	if (!ok)
	{
		throw GLFontError::InvalidFont();
	}

	int pixLength = 0;

	//Get length of std::string
	int Length = strlen (String);

	//Loop through characters
	for (int i = 0; i < Length; i++)
	{
		GLFONTCHAR *Char = &Font.Char[(int)String[i] - Font.IntStart];
		pixLength += Char->dx;
	}

	return pixLength;
}

//*********************************************************
int GLFont::TextOut (const char * String, float x, float y, float z)
{
	//Return if we don't have a valid glFont
	if (!ok)
	{
		throw GLFontError::InvalidFont();
	}

	int startX = (int) x;

	//Get length of std::string
	int Length = strlen (String);

	//Begin rendering quads
	glBegin(GL_QUADS);

	//Loop through characters
	for (int i = 0; i < Length; i++)
	{
		//Get pointer to glFont character
		GLFONTCHAR *Char = &Font.Char[(int)String[i] - Font.IntStart];

		//Specify vertices and texture coordinates
		//glTexCoord2f(Char->tx1, Char->ty1);
		//glVertex3f(x, y, z);
		//glTexCoord2f(Char->tx1, Char->ty2);
		//glVertex3f(x, y + Char->dy, z);
		//glTexCoord2f(Char->tx2, Char->ty2);
		//glVertex3f(x + Char->dx, y + Char->dy, z);
		//glTexCoord2f(Char->tx2, Char->ty1);
		//glVertex3f(x + Char->dx, y, z);

		glTexCoord2f(Char->tx1, Char->ty2);
		glVertex3f(x, y, z);
		glTexCoord2f(Char->tx1, Char->ty1);
		glVertex3f(x, y + Char->dy, z);
		glTexCoord2f(Char->tx2, Char->ty1);
		glVertex3f(x + Char->dx, y + Char->dy, z);
		glTexCoord2f(Char->tx2, Char->ty2);
		glVertex3f(x + Char->dx, y, z);

		//Move to next character
		x += Char->dx;
	}

	//Stop rendering quads
	glEnd();

	return (int)  (x - startX);
}

int GLFont::AddTris (const char * text, Gfx::VertexTypes::XyzUvRgb * verts, Vec3 origin, Vec3 dx, Vec3 dy) const
{
	assert (false);
	return 0;
}

int PixelPerfectGLFont::AddTris (const char * text, Gfx::VertexTypes::XyzUvRgb * verts, Vec3 origin, Vec3 dx, Vec3 dy) const
{
	int len = strlen (text);

	Vec3 pos (origin);
	Vec4 rgba (0.0f, 0.0f, 0.0f, 1.0f);

	int drawn = 0;

	float width = 0.0f;
	for (int i = 0; i < len; ++i)
	{
		GLFONTCHAR *Char = &Font.Char[text[i] - Font.IntStart];
		width += (float) Char->width;
	}

	width *= 0.5f;
	pos -= dx * width;

	for (int i = 0; i < len; ++i)
	{
		//Get pointer to glFont character
		GLFONTCHAR *Char = &Font.Char[text[i] - Font.IntStart];

		Vec3 x = dx * (float) Char->width;

		if (text [i] != ' ')
		{
			Vec3 y = dy * (float) Char->height;

			verts [0].Set (pos + y, Vec2 (Char->tx1, Char->ty1), rgba);
			verts [1].Set (pos + x + y, Vec2 (Char->tx2, Char->ty1), rgba);
			verts [2].Set (pos + x, Vec2 (Char->tx2, Char->ty2), rgba);
			++drawn;
			verts += 3;

			verts [0].Set (pos + x, Vec2 (Char->tx2, Char->ty2), rgba);
			verts [1].Set (pos, Vec2 (Char->tx1, Char->ty2), rgba);
			verts [2].Set (pos + y, Vec2 (Char->tx1, Char->ty1), rgba);
			++drawn;
			verts += 3;
		}

		pos += x;
	}

	// return tri count
	return drawn;
}



//End of file



