#include "stdafx.h"

#include "Gfx_Funcs.h"
#include "VertexTypes_Xyz.h"
#include "Buffers_PointList.h"

namespace Gfx
{
	namespace Buffers
	{	
		PointList::PointList ()
			: vboId (NULL),
			vertsCount (0)
		{
				
		}

		PointList::~PointList ()
		{
			if (vboId != NULL)
			{
				glDeleteBuffersARB (1, &vboId);
				vboId = NULL;	
			}
		}

		void PointList::SetVerts (const VertexTypes::Xyz * const verts, int vertsCount)
		{
			assert (vboId == NULL);

			if (vertsCount == 0)
			{
				return;
			}
			this->vertsCount = vertsCount;

			glGenBuffersARB (1, &vboId);
			glBindBufferARB (GL_ARRAY_BUFFER_ARB, vboId);

			const int stride = sizeof (VertexTypes::Xyz);

			glBufferDataARB (GL_ARRAY_BUFFER, stride * vertsCount, verts, GL_STATIC_DRAW);
		}

		void PointList::Render ()
		{
			if (vertsCount != 0)
			{
				assert (vboId != NULL);

				// enable 
				glEnableClientState (GL_VERTEX_ARRAY);
				glEnableClientState (GL_COLOR_ARRAY);
				//glEnableClientState (GL_TEXTURE_COORD_ARRAY);

				glBindBufferARB (GL_ARRAY_BUFFER_ARB, vboId);

				// before draw, specify vertex and index arrays with their offsets
				const int stride = sizeof (VertexTypes::Xyz);

#if 0

				float attens [4] = {0.01f, 0.01f, 0.01f, 0.0f};
				glPointParameterfARB (GL_POINT_SIZE_MIN, 10.0f);
				glPointParameterfARB (GL_POINT_SIZE_MAX, 10.0f);
				glPointParameterfvARB (GL_POINT_DISTANCE_ATTENUATION_ARB, attens);
				glPointParameterfARB (GL_POINT_FADE_THRESHOLD_SIZE_ARB, 60.0f);
				//glEnable (GL_POINT_SMOOTH);

#else

				//glEnable (GL_POINT_SMOOTH);

				glPointParameterfARB (GL_POINT_SIZE_MIN, 1.0f);
				glPointParameterfARB (GL_POINT_SIZE_MAX, 32.0f);

				int viewport [4];
				float mat [16];
				glGetFloatv (GL_PROJECTION_MATRIX, mat);
				glGetIntegerv (GL_VIEWPORT, viewport);
				float H = (float) viewport [2];
				float h = 2.0f / mat [0];
				float D0 = glm::sqrt (2.0f) * H / h;

				float temp = 1.0f / mat [0];
				float k = 1.0f / (1.0f + 2.0f * temp * temp);

				temp = 1.0f / D0;
				float atten [4] = {0.0f, 0.0f, temp * temp * k, 0.0f};
				
				glPointParameterfvARB (GL_POINT_DISTANCE_ATTENUATION_ARB, atten);
				glPointSize (1.0f); 
				//glPointSize (0.87f); // sqrt (3) * 0.5f
				//glPointSize (0.71f); // sqrt (2) * 0.5f
				//glPointSize (0.55f); // sqrt (2) * 0.5f
				glPointSize (1.0f);

#endif

				glVertexPointer (3, GL_FLOAT, stride, MEMBER_OFFSET (VertexTypes::Xyz, xyz));
				//glTexCoordPointer (2, GL_FLOAT, stride, MEMBER_OFFSET (VertexTypes::Xyz, uv));
				glColorPointer (3, GL_FLOAT, stride, MEMBER_OFFSET (VertexTypes::Xyz, rgba));

				glDrawArrays (GL_POINTS, 0, vertsCount);

				// cleanup

				glDisableClientState (GL_VERTEX_ARRAY);
				glDisableClientState (GL_COLOR_ARRAY);
				//glDisableClientState (GL_TEXTURE_COORD_ARRAY);

				glBindBufferARB (GL_ARRAY_BUFFER_ARB, 0);
			}
			else
			{
				assert (vboId == NULL);
			}
		}
	}
}