#include "stdafx.h"

#include "Gfx_Funcs.h"
#include "VertexTypes_XyzUvRgb.h"
#include "Buffers_TriList.h"

namespace Gfx
{
	namespace Buffers
	{
		TriList::TriList ()
			: vboId (NULL),
			vertsCount (0)
		{
			
		}

		TriList::~TriList ()
		{
			if (vboId != NULL)
			{
				glDeleteBuffersARB (1, &vboId);
				vboId = NULL;	
			}
		}

		void TriList::SetVerts (const VertexTypes::XyzUvRgb * const verts, int vertsCount)
		{
			assert (vertsCount % 3 == 0);
			assert (vboId == NULL);

			if (vertsCount == 0)
			{
				return;
			}
			this->vertsCount = vertsCount;

			glGenBuffersARB (1, &vboId);
			assert (vboId != NULL);

			glBindBufferARB (GL_ARRAY_BUFFER_ARB, vboId);

			const int stride = sizeof (VertexTypes::XyzUvRgb);

			glBufferDataARB (GL_ARRAY_BUFFER, stride * vertsCount, verts, GL_STATIC_DRAW_ARB);
			//glBufferDataARB (GL_ARRAY_BUFFER, stride * vertsCount, verts, GL_DYNAMIC_DRAW_ARB);

			//error = glGetError ();
			//if (error != GL_NO_ERROR)
			//{
			//	char temp [128];
			//	sprintf (temp, "OpenGl glBufferDataARB = %d", error);
			//	Debug_WriteLine (0, temp);
			//	vboId = NULL;
			//	return;
			//}
		}

		void TriList::Render ()
		{
			if (vertsCount != 0 && vboId != NULL)
			{
				GLenum result = glGetError (); 
				assert (result == GL_NO_ERROR);

				// enable 
				glEnableClientState (GL_VERTEX_ARRAY);
				glEnableClientState (GL_COLOR_ARRAY);
				glEnableClientState (GL_TEXTURE_COORD_ARRAY);

				glBindBufferARB (GL_ARRAY_BUFFER_ARB, vboId);

				// before draw, specify vertex and index arrays with their offsets
				const int stride = sizeof (VertexTypes::XyzUvRgb);

				glVertexPointer (3, GL_FLOAT, stride, MEMBER_OFFSET (VertexTypes::XyzUvRgb, xyz));
				glTexCoordPointer (2, GL_FLOAT, stride, MEMBER_OFFSET (VertexTypes::XyzUvRgb, uv));
				glColorPointer (3, GL_FLOAT, stride, MEMBER_OFFSET (VertexTypes::XyzUvRgb, rgba));

				glDrawArrays (GL_TRIANGLES, 0, vertsCount);

				// cleanup

				glDisableClientState (GL_VERTEX_ARRAY);
				glDisableClientState (GL_COLOR_ARRAY);
				glDisableClientState (GL_TEXTURE_COORD_ARRAY);

				glBindBufferARB (GL_ARRAY_BUFFER_ARB, 0);

				result = glGetError (); 
				assert (result == GL_NO_ERROR);
			}
		}
	}
}