#include "stdafx.h"

#include "../ExternalLibs/glfont/glfont.h"
#include "../Gfx/VertexTypes_XyzUvRgb.h"

#include "Gfx_Font.h"

namespace Gfx
{
	Font::Font (const char * filename)
	{
		glGenTextures(1, &texture);
		font = aah_new PixelPerfectGLFont ();
		//font = aah_new GLFont ();
		font->Create (filename, texture);

		hasBegun = false;
	}

	Font::~Font ()
	{
		delete font;
		font = NULL;

		glDeleteTextures (1, &texture);
	}

	int Font::GetLength (char c)
	{
		return (int) font->CharLength (c);
	}

	int Font::GetLength (const char * text)
	{
		return (int) font->TextLength (text);
	}

	int Font::DoMaxLengthTestReverse (const char * text, int maxLength, int * outStartAtCharIdx)
	{
		assert (text != NULL);
		assert (outStartAtCharIdx != NULL);

		int charIdx = strlen (text) - 1;
		int length = 0;

		while (charIdx >= 0)
		{
			int charLength = font->CharLength (text [charIdx]);
			if (length + charLength > maxLength)
			{
				break;
			}
			length += charLength;
			--charIdx;
		}

		*outStartAtCharIdx = charIdx + 1;

		assert (*outStartAtCharIdx >= 0 && *outStartAtCharIdx <= (signed) strlen (text));
		assert (length == GetLength (text + *outStartAtCharIdx));
		
		return length;
	}

	void Font::Begin ()
	{
		assert (!hasBegun);
		hasBegun = true;
		font->Begin ();

		glDisable (GL_DEPTH_TEST);
		glDisable (GL_CULL_FACE); 
	}

	int Font::DrawString (const char * text, int x, int y)
	{
		assert (hasBegun);
		return (int) font->TextOut (text, x, y, 0);
	}

	int Font::DrawString (const char * text, int x, int y, int z)
	{
		assert (hasBegun);
		return (int) font->TextOut (text, x, y, z);
	}

	int Font::DrawString (const char * text, int charStart, int charEnd, int x, int y)
	{
		assert (hasBegun);
		return (int) font->TextOut (text, charStart, charEnd, x, y, 0);
	}

	void Font::End ()
	{
		assert (hasBegun);
		hasBegun = false;
	}

	int Font::AddTris (const char * text, Gfx::VertexTypes::XyzUvRgb * verts, Vec3 origin, Vec3 dx, Vec3 dy) const
	{
		if (text == NULL)
		{
			return 0;
		}
		return font->AddTris (text, verts, origin, dx, dy);
	}
}
