#include "stdafx.h"

#include "Gfx_Font.h"
#include "Gfx_FontSentance.h"

namespace Gfx
{
	FontSentance::FontSentance ()
		: hasFitted (false),
		linesUsed (0)
	{
	}

	void FontSentance::Set (Gfx::Font * font, const char * text, int maxPixels)
	{
		assert (font != NULL);
		assert (text != NULL);

		hasFitted = true;
		linesUsed = 0;
		pixelLongest = 0;

		int currentLength = 0;
		int charIdx = 0;
		int lastSpaceCharIdx = -1;
		int lastSpaceLength = -1;

		charStart [linesUsed] = 0;
		while (true)
		{
			if (text [charIdx] == '\0')
			{
				charEnd [linesUsed] = charIdx;
				pixelLength [linesUsed] = currentLength;
				pixelLongest = glm::max (pixelLength [linesUsed], pixelLongest);
				++linesUsed;
				break;
			}

			int charLength = font->GetLength (text [charIdx]);
			if (currentLength + charLength > maxPixels)
			{
				if (lastSpaceCharIdx != -1)
				{
					charEnd [linesUsed] = lastSpaceCharIdx;
					pixelLength [linesUsed] = lastSpaceLength;
				}
				else
				{
					charEnd [linesUsed] = charIdx;
					pixelLength [linesUsed] = currentLength;
				}

				pixelLongest = glm::max (pixelLength [linesUsed], pixelLongest);
				currentLength = 0;
				++linesUsed;

				if (linesUsed > linesMax)
				{
					hasFitted = false;
					break;
				}

				if (lastSpaceCharIdx != -1)
				{
					charStart [linesUsed] = lastSpaceCharIdx + 1;
					charIdx = lastSpaceCharIdx + 1;
					lastSpaceCharIdx = -1;
					lastSpaceLength = -1;
				}
				else
				{
					charStart [linesUsed] = charIdx;
				}
			}
			else
			{
				if (text [charIdx] == ' ')
				{
					lastSpaceCharIdx = charIdx;
					lastSpaceLength = currentLength;
				}
				++charIdx;
				currentLength += charLength;
			}
		}
	}

	void FontSentance::Draw (Gfx::Font * font, const char * text, Int2 pos)
	{
		for (int lineIdx = 0; lineIdx < linesUsed; ++lineIdx)
		{
			font->DrawString (text, charStart [lineIdx], charEnd [lineIdx], pos.x, pos.y + lineIdx * 20);
		}
	}
}