#include "stdafx.h"

#include <iostream>
#include "ExternalLibs/libpng/png.h"
#include "ExternalLibs/libpng/pnginfo.h"

#include "../Gfx/Gfx_Funcs.h"

#include "Gfx_Tex.h"

namespace Gfx
{
	bool loadPngImage(const char *name, int &outWidth, int &outHeight, bool &outHasAlpha, GLubyte **outData) 
	{
		png_structp png_ptr;
		png_infop info_ptr;
		unsigned int sig_read = 0;
		//int color_type, interlace_type;
		FILE *fp;

		if ((fp = fopen(name, "rb")) == NULL)
		 return false;

		/* Create and initialize the png_struct
		* with the desired error handler
		* functions.  If you want to use the
		* default stderr and longjump method,
		* you can supply NULL for the last
		* three parameters.  We also supply the
		* the compiler header file version, so
		* that we know if the application
		* was compiled with a compatible version
		* of the library.  REQUIRED
		*/
		png_ptr = png_create_read_struct(PNG_LIBPNG_VER_STRING,
			 NULL, NULL, NULL);

		if (png_ptr == NULL) {
		 fclose(fp);
		 return false;
		}

		/* Allocate/initialize the memory
		* for image information.  REQUIRED. */
		info_ptr = png_create_info_struct(png_ptr);
		if (info_ptr == NULL) {
		 fclose(fp);
		 //png_destroy_read_struct(&png_ptr, png_infopp_NULL, png_infopp_NULL);
		 png_destroy_read_struct(&png_ptr, NULL, NULL);
		 return false;
		}

		/* Set error handling if you are
		* using the setjmp/longjmp method
		* (this is the normal method of
		* doing things with libpng).
		* REQUIRED unless you  set up
		* your own error handlers in
		* the png_create_read_struct()
		* earlier.
		*/
		if (setjmp(png_jmpbuf(png_ptr))) {
		 /* Free all of the memory associated
		  * with the png_ptr and info_ptr */
		 //png_destroy_read_struct(&png_ptr, &info_ptr, png_infopp_NULL);
		 png_destroy_read_struct(&png_ptr, &info_ptr, NULL);
		 fclose(fp);
		 /* If we get here, we had a
		  * problem reading the file */
		 return false;
		}

		/* Set up the output control if
		* you are using standard C streams */
		png_init_io(png_ptr, fp);

		/* If we have already
		* read some of the signature */
		png_set_sig_bytes(png_ptr, sig_read);

		/*
		* If you have enough memory to read
		* in the entire image at once, and
		* you need to specify only
		* transforms that can be controlled
		* with one of the PNG_TRANSFORM_*
		* bits (this presently excludes
		* dithering, filling, setting
		* background, and doing gamma
		* adjustment), then you can read the
		* entire image (including pixels)
		* into the info structure with this
		* call
		*
		* PNG_TRANSFORM_STRIP_16 |
		* PNG_TRANSFORM_PACKING  forces 8 bit
		* PNG_TRANSFORM_EXPAND forces to
		*  expand a palette into RGB
		*/
		//png_read_png(png_ptr, info_ptr, PNG_TRANSFORM_STRIP_16 | PNG_TRANSFORM_PACKING | PNG_TRANSFORM_EXPAND, png_voidp_NULL);
		png_read_png(png_ptr, info_ptr, PNG_TRANSFORM_STRIP_16 | PNG_TRANSFORM_PACKING | PNG_TRANSFORM_EXPAND, NULL);

		outWidth = info_ptr->width;
		outHeight = info_ptr->height;
		switch (info_ptr->color_type) {
		 case PNG_COLOR_TYPE_RGBA:
			 outHasAlpha = true;
			 break;
		 case PNG_COLOR_TYPE_RGB:
			 outHasAlpha = false;
			 break;
		 default:
			 std::cout << "Color type " << info_ptr->color_type << " not supported" << std::endl;
			 png_destroy_read_struct(&png_ptr, &info_ptr, NULL);
			 fclose(fp);
			 return false;
		}
		unsigned int row_bytes = png_get_rowbytes(png_ptr, info_ptr);
		*outData = (unsigned char*) malloc(row_bytes * outHeight);

		png_bytepp row_pointers = png_get_rows(png_ptr, info_ptr);

		for (int i = 0; i < outHeight; i++) {
			// note that png is ordered top to
			// bottom, but OpenGL expect it bottom to top
			// so the order or swapped
			// memcpy(*outData+(row_bytes * (outHeight-1-i)), row_pointers[i], row_bytes);
			memcpy(*outData+(row_bytes * (i)), row_pointers[i], row_bytes);
		}

		/* Clean up after the read,
		* and free any memory allocated */
		//png_destroy_read_struct(&png_ptr, &info_ptr, png_infopp_NULL);
		png_destroy_read_struct(&png_ptr, &info_ptr, NULL);

		/* Close the file */
		fclose(fp);

		/* That's it */
		return true;
	}
	

	GfxTex::GfxTex (TextRef filename)
		: rawData (NULL),
		wh (-1, -1)
	{
		GLenum error = glGetError ();
		assert (error == GL_NO_ERROR);

		LoadTexture (filename);

		error = glGetError ();
		assert (error == GL_NO_ERROR);
	}

	GfxTex::~GfxTex ()
	{
		delete [] rawData;
		rawData = NULL;

		glDeleteTextures (1, &texture);
	}

	Int2 GfxTex::GetWh () const
	{
		return wh;
	}

	void GfxTex::Bind ()
	{
		glBindTexture (GL_TEXTURE_2D, texture);
	}

	void GfxTex::Unbind ()
	{
		glBindTexture (GL_TEXTURE_2D, NULL);
	}

	void GfxTex::UpdateRegion (unsigned * pixels, int x, int y, int w, int h)
	{
		//void glTexSubImage2D( GLenum target, GLint level, GLint xoffset, GLint yoffset, GLsizei width, GLsizei height, GLenum format, GLenum type, void *data);
		glBindTexture (GL_TEXTURE_2D, texture);
		glTexSubImage2D (GL_TEXTURE_2D, 0, x, y, w, h, GL_RGBA, GL_UNSIGNED_BYTE, pixels);
	}

	bool GfxTex::LoadTexture (TextRef filename)
	{
		assert (rawData == NULL);

		bool hasAlpha = false;

		bool isOk = loadPngImage (filename.Raw (), wh.x, wh.y, hasAlpha, &rawData);
		assert (isOk);

		UploadToGfxCard ();
		return true;
	}

	void GfxTex::UploadToGfxCard ()
	{
		assert (rawData != NULL);

		GLenum error = glGetError ();
		assert (error == GL_NO_ERROR);

		// Have OpenGL generate a texture object handle for us
		glGenTextures( 1, &texture );

		// Bind the texture object
		glBindTexture( GL_TEXTURE_2D, texture );

		// Set the texture's stretching and wrapping properties
		glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
		glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
		glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);

		// Edit the texture object's image data using the information SDL_Surface gives us
		//glTexImage2D( GL_TEXTURE_2D, 0, nOfColors, w, h, 0, texture_format, GL_UNSIGNED_BYTE, outData );
		glTexImage2D( GL_TEXTURE_2D, 0, GL_RGBA, wh.x, wh.y, 0, GL_RGBA, GL_UNSIGNED_BYTE, rawData);
		error = glGetError ();
		assert (error == GL_NO_ERROR);


		//glBindTexture( GL_TEXTURE_2D, texture );
		//glEnable(GL_TEXTURE_2D); // bind then enable to avoid ATI bug
		//Gfx::glGenerateMipmapARB (GL_TEXTURE_2D);
	}
}
