#include "stdafx.h"

#include "SceneSim/SceneSim_Room.h"
#include "Thread/Thread_JobMngr.h"

#include "../SceneGfx/SceneGfx_BlockChangeList.h"
#include "../SceneGfx/SceneGfx_Defs.h"
#include "../SceneGfx/SceneGfx_Column.h"
#include "../SceneGfx/SceneGfx_Room.h"
#include "../SceneGfx/SceneGfx_World.h"

#include "GfxFactory_GfxFactory.h"
#include "GfxFactory_JobApplyCsgSim.h"
#include "GfxFactory_JobCreateBlockMeshes.h"

namespace GfxFactory
{
	GfxFactory::GfxFactory (SceneSim::World * simWorld, SceneGfx::World * gfxWorld)
		: simWorld (simWorld), 
		gfxWorld (gfxWorld)
	{
		assert (simWorld != NULL);
		assert (gfxWorld != NULL);

		jobMngr = aah_new Thread::JobMngr ("GfxFactory", false);
	}

	GfxFactory::~GfxFactory ()
	{
		jobMngr->FinishCurrentJobAndDeleteAll ();
		delete jobMngr;
	}

	int GfxFactory::GetJobCount () const
	{
		return jobMngr->GetJobCount ();
	}

	void GfxFactory::Update ()
	{
		jobMngr->MainThread_Update ();
	}

	void GfxFactory::RequestGfxColumnMeshRefresh (Int3 columnId)
	{
		JobCreateBlockMeshes * job = aah_new JobCreateBlockMeshes (jobMngr, gfxWorld->GetPrimaryRoom (), columnId);
		job->AddToQueue (Thread::JB_NORMAL);
	}

	void GfxFactory::OnSimWorldAdded (SceneSim::World * simWorld)
	{
		assert (this->simWorld == simWorld);

		gfxWorld->OnLoadCompleted (simWorld);
	}

	void GfxFactory::OnSimRoomAdded (SceneSim::Room * room)
	{
		assert (room != NULL);
		assert (gfxWorld->GetPrimaryRoom () == NULL);

		SceneGfx::Room * gfxRoom = aah_new SceneGfx::Room (room);
		gfxWorld->SetPrimaryRoom (gfxRoom);
	}

	void GfxFactory::OnSimColumnAdded (SceneSim::Column * column)
	{
		assert (column != NULL);
		
		SceneGfx::Room * gfxRoom = gfxWorld->GetPrimaryRoom ();
		SceneGfx::Column * gfxColumn = aah_new SceneGfx::Column (gfxRoom, column);
		gfxRoom->AddColumn (gfxColumn);
	}
}
