#include "stdafx.h"

#include "SceneSim/SceneSim_Column.h"
#include "SceneSim/SceneSim_ColumnCells.h"
#include "SceneSim/SceneSim_Room.h"
#include "SceneSim/SceneSim_World.h"

#include "../SceneChange/SceneChange_CreateBlockMeshes.h"
#include "../SceneGfx/SceneGfx_Column.h"
#include "../SceneGfx/SceneGfx_Room.h"
#include "../SceneGfx/SceneGfx_World.h"

#include "GfxFactory_GfxFactory.h"
#include "GfxFactory_JobApplyCsgGfx.h"

namespace GfxFactory
{
	JobApplyCsgGfx::JobApplyCsgGfx (GfxFactory * gfxFactory, RakNet::Time * applied, TextRef roomId, SceneCollections::LocalColumns * columns, SceneCollections::LocalColumns * columnsEdge)
		: Job (gfxFactory->jobMngr, false),
		gfxFactory (gfxFactory),
		applied (*applied),
		roomId (roomId),
		columns (*columns),
		columnsEdge (*columnsEdge),
		changedColumnsCount (0),
		blockCreaters (NULL)
	{
		assert (gfxFactory != NULL);
		assert (applied != NULL);
		assert (columns != NULL);
		assert (columnsEdge != NULL);
		assert (columns != columnsEdge);
	}

	JobApplyCsgGfx::~JobApplyCsgGfx ()
	{
		for (int idx = 0; idx < changedColumnsCount; ++idx)
		{
			delete blockCreaters [idx];
			blockCreaters [idx] = NULL;
		}

		delete [] blockCreaters;
		blockCreaters = NULL;

		columns.DeleteAll ();
		columnsEdge.DeleteAll ();
	}

	Thread::JOBSTATES JobApplyCsgGfx::JobDoWork ()
	{
		const int maxColumnsCount = columns.GetMaxColumnsCount ();
		blockCreaters = aah_new SceneChange::CreateBlockMeshes * [maxColumnsCount];

		SceneGfx::Room * gfxRoom = gfxFactory->gfxWorld->GetPrimaryRoom ();

		for (int columnIdx = 0; columnIdx < maxColumnsCount; ++columnIdx)
		{
			SceneSim::Column * simColumn = columns.GetNthColumn (columnIdx);
			if (simColumn != NULL && (simColumn->flags & SceneSim::Column::SF_NO_CELLS) == 0)
			{
				SceneSim::AdjColumns adjColumns;
				adjColumns.main = simColumn;

				adjColumns.n = columns.GetColumn (simColumn->columnId + Int3 (0, 0, 1));
				adjColumns.e = columns.GetColumn (simColumn->columnId + Int3 (1, 0, 0));
				adjColumns.s = columns.GetColumn (simColumn->columnId + Int3 (0, 0, -1));
				adjColumns.w = columns.GetColumn (simColumn->columnId + Int3 (-1, 0, 0));

				if (adjColumns.n == NULL) adjColumns.n = columnsEdge.GetColumn (simColumn->columnId + Int3 (0, 0, 1));
				if (adjColumns.e == NULL) adjColumns.e = columnsEdge.GetColumn (simColumn->columnId + Int3 (1, 0, 0));
				if (adjColumns.s == NULL) adjColumns.s = columnsEdge.GetColumn (simColumn->columnId + Int3 (0, 0, -1));
				if (adjColumns.w == NULL) adjColumns.w = columnsEdge.GetColumn (simColumn->columnId + Int3 (-1, 0, 0));

				blockCreaters [changedColumnsCount] = aah_new SceneChange::CreateBlockMeshes (gfxRoom, &adjColumns);
				++changedColumnsCount;
			}
		}

		columnsEdge.DeleteAll ();

		return Thread::JS_WORKDONE;
	}

	Thread::JOBSTATES JobApplyCsgGfx::JobOnCompleted ()
	{
		SceneGfx::Room * gfxRoom = gfxFactory->gfxWorld->GetPrimaryRoom ();
		assert (gfxRoom != NULL);

		SceneSim::Room * simRoom = gfxRoom->GetSimRoom ();
		assert (simRoom != NULL);


		for (int idx = 0; idx < changedColumnsCount; ++idx)
		{
			const SceneSim::Column * simColumnSrc = blockCreaters [idx]->GetColumnSrc ();
			SceneSim::Column * simColumnDst = simRoom->GetColumn (simColumnSrc->columnId);

			if (simColumnDst != NULL)
			{
				SceneGfx::Column * gfxColumn = gfxRoom->GetColumn (simColumnDst->columnId);
				const bool wasExisting = (gfxColumn != NULL);

				if (gfxColumn == NULL)
				{
					gfxColumn = aah_new SceneGfx::Column (gfxRoom, simColumnDst);
				}

				int blockCount = -1;
				SceneGfx::Block * * blocks = blockCreaters [idx]->GetOutput (&blockCount);
				{
					for (int idx = 0; idx < blockCount; ++idx)
					{
						assert (blocks [idx] != NULL);
						gfxColumn->AddOrReplaceBlock (blocks [idx]);
						blocks [idx] = NULL;
					}
				}				

				if (!wasExisting)
				{
					gfxRoom->AddColumn (gfxColumn);
				}

				simColumnDst->GetCells ()->CopyFrom (simColumnSrc->GetCells ());
				simColumnDst->SetLastModifiedTime (applied);
			}

			delete blockCreaters [idx];
			blockCreaters [idx] = NULL;
		}

		changedColumnsCount = 0;
		delete [] blockCreaters;
		blockCreaters = NULL;

		columns.DeleteAll ();

		return Thread::JS_COMPLETED;
	}
}
