#include "stdafx.h"

#include "Profile/Profile_ProfileScope.h"
#include "SceneSim/SceneSim_Column.h"
#include "SceneSim/SceneSim_ColumnCells.h"
#include "SceneSim/SceneSim_Room.h"
#include "SceneSim/SceneSim_World.h"

#include "../SceneChange/SceneChange_ColumnDataCollection.h"
#include "../SceneChange/SceneChange_CreateBlockMeshes.h"
#include "../SceneChange/SceneChange_SceneChangeMngr.h"
#include "../SceneGfx/SceneGfx_Block.h"
#include "../SceneGfx/SceneGfx_Column.h"
#include "../SceneGfx/SceneGfx_Room.h"
#include "../SceneGfx/SceneGfx_World.h"

#include "GfxFactory_JobCreateBlockMeshes.h"

namespace GfxFactory
{
	JobCreateBlockMeshes::JobCreateBlockMeshes (Thread::JobMngr * jobMngr, SceneGfx::Room * gfxRoom, Int3 columnId)
		: Thread::Job (jobMngr, true),
		gfxRoom (gfxRoom),
		columnId (columnId),
		blockCreater (NULL)
	{
		assert (gfxRoom != NULL);

		//CHANGECMG_PRINT (("col gfxCreate CONST %d,%d\n", columnId.x, columnId.z));
		
		SceneSim::Room * simRoom = gfxRoom->GetSimRoom ();
		assert (simRoom != NULL);

		columns.main = simRoom->GetColumn (columnId);
		columns.n = simRoom->GetColumn (columnId + Int3 (0, 0, 1));
		columns.e = simRoom->GetColumn (columnId + Int3 (1, 0, 0));
		columns.s = simRoom->GetColumn (columnId + Int3 (0, 0, -1));
		columns.w = simRoom->GetColumn (columnId + Int3 (-1, 0, 0));

		assert (columns.main != NULL);

		// duplicate adj so they can be modified

		columns.main = aah_new SceneSim::Column (*columns.main);
		if (columns.n != NULL) columns.n = aah_new SceneSim::Column (*columns.n);
		if (columns.e != NULL) columns.e = aah_new SceneSim::Column (*columns.e);
		if (columns.s != NULL) columns.s = aah_new SceneSim::Column (*columns.s);
		if (columns.w != NULL) columns.w = aah_new SceneSim::Column (*columns.w);
	}

	JobCreateBlockMeshes::~JobCreateBlockMeshes ()
	{
		columns.DeleteAll ();
		delete blockCreater;
	}

	Thread::JOBSTATES JobCreateBlockMeshes::JobDoWork ()
	{
		blockCreater = aah_new SceneChange::CreateBlockMeshes (gfxRoom, &columns);

		return Thread::JS_WORKDONE;
	}

	Thread::JOBSTATES JobCreateBlockMeshes::JobOnCompleted ()
	{
		// check to see if we still need this column gfx

		SceneSim::Room * simRoom = gfxRoom->GetSimRoom ();
		assert (simRoom != NULL);

		SceneSim::Column * simColumn = simRoom->GetColumn (columnId);

		if (simColumn != NULL)
		{
			SceneGfx::Column * gfxColumn = gfxRoom->GetColumn (columnId);
			assert (gfxColumn != NULL);

			int blockCount = -1;
			SceneGfx::Block * * blocks = blockCreater->GetOutput (&blockCount);
			{
				for (int idx = 0; idx < blockCount; ++idx)
				{
					assert (blocks [idx] != NULL);
					gfxColumn->AddOrReplaceBlock (blocks [idx]);
					blocks [idx] = NULL;
				}
			}
		}

		delete blockCreater;
		blockCreater = NULL;

		columns.DeleteAll ();

		//CHANGECMG_PRINT (("col gfxCreate END %d,%d\n", columnId.x, columnId.z));

		return Thread::JS_COMPLETED;
	}
}
