#include "stdafx.h"

#include "SceneSim/SceneSim_Defs.h"

#include "Particles_Billboard.h"

namespace Particles
{
	Billboard::Billboard ()
		: inUse (false)
	{
	}

	void Billboard::Update (float timeDelta)
	{
		age += timeDelta;
		if (age >= lifespan)
		{
			inUse = false;
		}
		columnPos += timeDelta * velocity;
		rotation += timeDelta * rotationDelta;
	}

	void Billboard::Render (Int3 fromColumnId, Vec3 right, Vec3 up, Vec3 in)
	{
		if (age < 0.0f)
		{
			return;
		}

		fromColumnId = columnId - fromColumnId;
		Vec3 pos (
			fromColumnId.x * SceneSim::CellsPerColXz, 
			0, //fromColumnId.y * SceneSim::CellsPerColY, 
			fromColumnId.z * SceneSim::CellsPerColXz);

		pos += columnPos;

		Vec4 r (right.x, right.y, right.z, 0.0f);
		Vec4 u (up.x, up.y, up.z, 0.0f);

		Mat44 rot = glm::rotate (Mat44 (), rotation, in);

		r = r * rot;
		u = u * rot;

		float s;
		const float change = lifespan * 0.1f;
		if (age < change)
		{
			s = size * ((age / change) * 0.5f + 0.5f);
		}
		else
		{
			const float c2 = lifespan * 0.9f;
			s = size * (1.0f - (age - change) / c2);
		}

		right = Vec3 (r.x, r.y, r.z) * s;
		up = Vec3 (u.x, u.y, u.z) * s;

		float epsilon = 0.001f;

		int blend = (int) (511.0f * age / lifespan);
		Int4 col (rgba.r, rgba.g, rgba.b, 255);
		if (blend > 255)
		{
			blend -= 256;
			col.r = (col.r * (255 - blend) + 255 * blend) / 255;
			col.g = (col.g * (255 - blend) + 255 * blend) / 255;
			col.b = (col.b * (255 - blend) + 255 * blend) / 255;
		}

		glColor4ub ((GLubyte) col.r, (GLubyte) col.g, (GLubyte) col.b, (GLubyte) col.a);
		glTexCoord2f (epsilon, epsilon);
		glVertex3f (pos.x - right.x - up.x, pos.y - right.y - up.y, pos.z - right.z - up.z);

		glColor4ub ((GLubyte) col.r, (GLubyte) col.g, (GLubyte) col.b, (GLubyte) col.a);
		glTexCoord2f (1.0f - epsilon, epsilon);
		glVertex3f (pos.x + right.x - up.x, pos.y + right.y - up.y, pos.z + right.z - up.z);

		glColor4ub ((GLubyte) col.r, (GLubyte) col.g, (GLubyte) col.b, (GLubyte) col.a);
		glTexCoord2f (epsilon, 1.0f - epsilon);
		glVertex3f (pos.x - right.x + up.x, pos.y - right.y + up.y, pos.z - right.z + up.z);

		glColor4ub ((GLubyte) col.r, (GLubyte) col.g, (GLubyte) col.b, (GLubyte) col.a);
		glTexCoord2f (epsilon, 1.0f - epsilon);
		glVertex3f (pos.x - right.x + up.x, pos.y - right.y + up.y, pos.z - right.z + up.z);

		glColor4ub ((GLubyte) col.r, (GLubyte) col.g, (GLubyte) col.b, (GLubyte) col.a);
		glTexCoord2f (1.0f - epsilon, epsilon);
		glVertex3f (pos.x + right.x - up.x, pos.y + right.y - up.y, pos.z + right.z - up.z);

		glColor4ub ((GLubyte) col.r, (GLubyte) col.g, (GLubyte) col.b, (GLubyte) col.a);
		glTexCoord2f (1.0f - epsilon, 1.0f - epsilon);
		glVertex3f (pos.x + right.x + up.x, pos.y + right.y + up.y, pos.z + right.z + up.z);
	}

	void Billboard::Set (Int3 columnId, Vec3 columnPos, Vec3 velocity, float size, float rotation, float rotationDelta, float age, float lifespan, Int4 rgba)
	{
		this->inUse = true;
		this->texId = 0;
		this->columnId = columnId;
		this->columnPos = columnPos;
		this->velocity = velocity;
		this->size = size;
		this->rotation = rotation;
		this->rotationDelta = rotationDelta;
		this->age = age;
		this->lifespan = lifespan;
		this->rgba = rgba;
	}
}
