#include "stdafx.h"

#include "Noise/Noise_Random.h"

#include "../Gfx/Gfx_Tex.h"

#include "Particles_Billboard.h"
#include "Particles_BillboardMngr.h"

namespace Particles
{
	BillboardMngr::BillboardMngr (int instancesMax)
		: instancesMax (instancesMax)
	{
		assert (instancesMax > 0);

		instances = aah_new Billboard [instancesMax];

		tex = aah_new Gfx::GfxTex ("gfx/explosion.png");
	}

	BillboardMngr::~BillboardMngr ()
	{
		delete tex;

		delete [] instances;
	}

	Billboard * BillboardMngr::GetFree (int texId)
	{
		Billboard * oldest = NULL;
		for (int idx = 0; idx < instancesMax; ++idx)
		{
			Billboard * current = &instances [idx];
			if (oldest == NULL || current->age > oldest->age)
			{
				oldest = current;
			}
			if (!current->inUse)
			{
				return current;
			}
		}
		return oldest;
	}

	void BillboardMngr::Update (float timeDelta)
	{
		for (int idx = 0; idx < instancesMax; ++idx)
		{
			Billboard * billboard = &instances [idx];
			if (billboard->inUse)
			{
				billboard->Update (timeDelta);
			}
		}
	}

	void BillboardMngr::Render (Int3 fromColumnId, Vec3 right, Vec3 up, Vec3 in)
	{
		tex->Bind ();

		glEnable (GL_ALPHA_TEST);
		glEnable (GL_BLEND);
		glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		glEnable (GL_DEPTH_TEST);
		glDisable (GL_CULL_FACE); 

		glBegin (GL_TRIANGLES);

		for (int idx = 0; idx < instancesMax; ++idx)
		{
			Billboard * billboard = &instances [idx];
			if (billboard->inUse)
			{
				billboard->Render (fromColumnId, right, up, in);
			}
		}

		glEnd ();

		tex->Unbind ();
	}

	void BillboardMngr::CreateExplosion (int seed, Int3 columnId, Vec3 columnPos, float radius, Int4 rgba)
	{
		float size = radius * 0.65f;

		// create mass in the middle
		for (int i = 0; i < 25; ++i)
		{
			seed += i;

			Mat44 mat;
			mat = glm::rotate (mat, Noise::Random (seed, 0x48905185) * 360.0f, Vec3 (0.0f, 0.0f, 1.0f));
			mat = glm::rotate (mat, Noise::Random (seed, 0x032895c0) * 360.0f, Vec3 (0.0f, 1.0f, 0.0f));
			Vec4 xyz = Vec4 (0.0f, size, 0.0f, 0.0f) * mat;
			Vec3 xyz3 (xyz.x, xyz.y * 2.0f, xyz.z);

			float rotationDelta = 100.0f * Noise::Random (seed, 2573);
			GetFree (0)->Set (columnId, columnPos + xyz3, xyz3 * 0.2f + Vec3 (0.0f, 1.0f, 0.0f), size * 0.75f, Noise::Random (seed, 778) * 180.0f, rotationDelta, Noise::Random (seed, -654) * 0.25f, Noise::Random (seed, 124) + 3.0f, rgba);
		}

		// create outer ring
		for (int i = 0; i < 50; ++i)
		{
			seed += i;
			const float angle = Noise::Random (seed, 6547 + 534) * 180.0f;
			Vec3 offset (glm::sin (angle) * size, 0.0f, glm::cos (angle) * size);
			float rotationDelta = 100.0f * Noise::Random (seed, 2573);
			GetFree (0)->Set (columnId, columnPos + offset, offset * 6.0f, size * 0.25f, Noise::Random (seed, 778) * 180.0f, rotationDelta, Noise::Random (seed, -654) * 0.1f, Noise::Random (seed, 124) * 0.1f + 0.6f, Int4 (255, 0, 0, 255));
		}

		// create sparks
	}
}
