#include "stdafx.h"

#include "Input/Input_InputMngr.h"

#include "../Gfx/Gfx_Tex.h"
#include "../Gfx/Gfx_Viewport.h"

#include "Plot_Prims.h"

namespace Plot
{
	struct PrimGlobals
	{
		PrimGlobals ()
			: inputMngr (NULL),
			type (PT_NONE)
		{
		}

		Input::InputMngr * inputMngr;
		PRIMTYPES type;
	};

	PrimGlobals g_prims;

	void Prims_Init (Input::InputMngr * inputMngr)
	{
		assert (inputMngr != NULL);
		g_prims.inputMngr = inputMngr;
	}

	void Prims_BatchBegin (PRIMTYPES primType)
	{
		glEnable (GL_BLEND);
		glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		glDisable (GL_DEPTH_TEST);
		if (primType == PT_XYUVRGB)
		{
			glEnable (GL_TEXTURE_2D);
		}
		else
		{
			glDisable (GL_TEXTURE_2D);
		}
		glDisable (GL_CULL_FACE); 
		glDisable (GL_ALPHA_TEST);
		
		glColor3f (1.0f, 0.0f, 0.0f);

		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity( );
		
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		glOrtho(0.0f, (GLsizei) g_prims.inputMngr->GetViewport ()->w, (GLsizei) g_prims.inputMngr->GetViewport ()->h, 0.0f, 0.0f, 1.0f);

		glBegin (GL_TRIANGLES);

		assert (g_prims.type == PT_NONE);
		g_prims.type = primType;
	}

	void Prims_BatchEnd ()
	{
		assert (g_prims.type != PT_NONE);

		glEnd ();

		g_prims.type = PT_NONE;

		glEnable (GL_DEPTH_TEST);
		glEnable (GL_TEXTURE_2D);
		glEnable (GL_CULL_FACE); 
		glEnable (GL_ALPHA_TEST);
	}

	void Prims_Quad (Int2 tl, Int2 br, Int4 rgba)
	{
		assert (g_prims.type == PT_XYRGB);

		glColor4ub ((GLubyte) rgba.r, (GLubyte) rgba.g, (GLubyte) rgba.b, (GLubyte) rgba.a);
		glVertex2i (tl.x, tl.y);
		glColor4ub ((GLubyte) rgba.r, (GLubyte) rgba.g, (GLubyte) rgba.b, (GLubyte) rgba.a);
		glVertex2i (br.x, tl.y);
		glColor4ub ((GLubyte) rgba.r, (GLubyte) rgba.g, (GLubyte) rgba.b, (GLubyte) rgba.a);
		glVertex2i (br.x, br.y);

		glColor4ub ((GLubyte) rgba.r, (GLubyte) rgba.g, (GLubyte) rgba.b, (GLubyte) rgba.a);
		glVertex2i (br.x, br.y);
		glColor4ub ((GLubyte) rgba.r, (GLubyte) rgba.g, (GLubyte) rgba.b, (GLubyte) rgba.a);
		glVertex2i (tl.x, br.y);
		glColor4ub ((GLubyte) rgba.r, (GLubyte) rgba.g, (GLubyte) rgba.b, (GLubyte) rgba.a);
		glVertex2i (tl.x, tl.y);
	}

	void Prims_QuadTex (Int2 tl, Gfx::GfxTex * tex)
	{
		Prims_QuadTex (tl, tl + tex->GetWh (), Vec2 (0.0f, 0.0f), Vec2 (1.0f, 1.0f), Int4 (255, 255, 255, 255));
	}

	void Prims_QuadTex (Int2 tl, Gfx::GfxTex * tex, int scale)
	{
		Prims_QuadTex (tl, tl + tex->GetWh () * scale, Vec2 (0.0f, 0.0f), Vec2 (1.0f, 1.0f), Int4 (255, 255, 255, 255));
	}

	void Prims_QuadTex (Int2 tl, Int2 br, Vec2 tlUv, Vec2 brUv, Int4 rgba)
	{
		assert (g_prims.type == PT_XYUVRGB);

		glColor4ub ((GLubyte) rgba.r, (GLubyte) rgba.g, (GLubyte) rgba.b, (GLubyte) rgba.a);
		glTexCoord2f (tlUv.x, tlUv.y);
		glVertex2i (tl.x, tl.y);
		glColor4ub ((GLubyte) rgba.r, (GLubyte) rgba.g, (GLubyte) rgba.b, (GLubyte) rgba.a);
		glTexCoord2f (brUv.x, tlUv.y);
		glVertex2i (br.x, tl.y);
		glColor4ub ((GLubyte) rgba.r, (GLubyte) rgba.g, (GLubyte) rgba.b, (GLubyte) rgba.a);
		glTexCoord2f (brUv.x, brUv.y);
		glVertex2i (br.x, br.y);

		glColor4ub ((GLubyte) rgba.r, (GLubyte) rgba.g, (GLubyte) rgba.b, (GLubyte) rgba.a);
		glTexCoord2f (brUv.x, brUv.y);
		glVertex2i (br.x, br.y);
		glColor4ub ((GLubyte) rgba.r, (GLubyte) rgba.g, (GLubyte) rgba.b, (GLubyte) rgba.a);
		glTexCoord2f (tlUv.x, brUv.y);
		glVertex2i (tl.x, br.y);
		glColor4ub ((GLubyte) rgba.r, (GLubyte) rgba.g, (GLubyte) rgba.b, (GLubyte) rgba.a);
		glTexCoord2f (tlUv.x, tlUv.y);
		glVertex2i (tl.x, tl.y);
	}
}
