#include "stdafx.h"

#include "SceneSim/SceneSim_Column.h"
#include "SceneSim/SceneSim_Room.h"

#include "../SceneGfx/SceneGfx_Block.h"
#include "../SceneGfx/SceneGfx_Defs.h"
#include "../SceneGfx/SceneGfx_Room.h"

#include "SceneChange_CreateBlockMeshes.h"

namespace SceneChange
{
	CreateBlockMeshes::CreateBlockMeshes (const SceneGfx::Room * gfxRoom, const SceneSim::AdjColumns * adjColumns)
		//: blockIdsAsBits (0),
		: blockIdsAsBits ((1 << SceneGfx::BlocksPerColY) - 1),
		columns (*adjColumns),
		blocks (NULL)
	{
		assert (blockIdsAsBits != 0);
		assert (gfxRoom != NULL);
		assert (columns.main != NULL);

		blockChangesCount = Utils::CountBitsSet (blockIdsAsBits);
		assert (blockChangesCount <= SceneGfx::BlocksPerColY);

		blocks = aah_new SceneGfx::Block * [blockChangesCount];

		int nextBlockIdx = 0;
		for (int blockId = 0; blockId < SceneGfx::BlocksPerColY; ++blockId)
		{
			if (blockIdsAsBits & (1 << blockId))
			{
				blocks [nextBlockIdx] = aah_new SceneGfx::Block (blockId, &columns, gfxRoom->GetCellMesh (), gfxRoom->GetCellBuilder ());
				++nextBlockIdx;
			}
		}
	}

	CreateBlockMeshes::~CreateBlockMeshes ()
	{
		for (int idx = 0; idx < blockChangesCount; ++idx)
		{
			delete blocks [idx];
		}
		delete [] blocks;
	}

	const SceneSim::Column * CreateBlockMeshes::GetColumnSrc () const
	{
		return columns.main;
	}

	SceneGfx::Block * * CreateBlockMeshes::GetOutput (int * outputCount) const
	{
		*outputCount = blockChangesCount;
		return blocks;
	}
}
