#include "stdafx.h"

#include "Ents/Comps_Coord.h"
#include "SceneSim/SceneSim_AreaLow.h"
#include "SceneSim/SceneSim_Defs.h"

#include "../Gfx/Buffers_PointList.h"
#include "../Gfx/VertexTypes_Xyz.h"

#include "SceneGfx_AreaLow.h"

namespace SceneGfx
{
	AreaLow::AreaLow (Room * room, SceneSim::AreaLow * areaLow)
		: room (room),
		simAreaLow (areaLow),
		copyToGfxCardIfNeeded (true),
		verts (NULL),
		buffer (NULL),
		vertsTotal (0)
	{
		static const int maxPerY = 4;
		vertsTotal = simAreaLow->x * simAreaLow->y * maxPerY;
		verts = new Gfx::VertexTypes::Xyz [vertsTotal];

		Gfx::VertexTypes::Xyz * currentXyz = verts;

		for (int x = 0; x < simAreaLow->x; ++x)
		{
			for (int z = 0; z < simAreaLow->z; ++z)
			{
				int found = 0;
				for (int y = simAreaLow->y - 1; y > 0; --y)
				{
					if (simAreaLow->GetCellBit (x, y, z))
					{

#if 0

						const bool draw = (!simAreaLow->GetCellBitNoBounds (x + 1, y, z) ||
							!simAreaLow->GetCellBitNoBounds (x - 1, y, z) ||
							!simAreaLow->GetCellBitNoBounds (x, y + 1, z) ||
							!simAreaLow->GetCellBitNoBounds (x, y - 1, z) ||
							!simAreaLow->GetCellBitNoBounds (x, y, z + 1) ||
							!simAreaLow->GetCellBitNoBounds (x, y, z - 1));

#else

						bool draw = false;
						for (int xx = x - 1; xx <= x + 1; ++xx)
						{
							for (int yy = y - 1; yy <= y + 1; ++yy)
							{
								for (int zz = z - 1; zz <= z + 1; ++zz)
								{
									if (xx != 0 && yy != 0 && zz != 0)
									{
										if (!simAreaLow->GetCellBitNoBounds (xx, yy, zz))
										{
											draw = true;
											goto bad_goto;
										}
									}
								}
							}
						}
bad_goto:

#endif

						if (draw)
						{
							//float col = ((x + y + z) % 16) / 15.0f;
							//float col = ((y) % 128) / 127.0f;
							//float col = 1.0f;
							float col = 0.0f;

							//currentXyz->Set (Vec3 (x + 0.5f, y + 0.5f, z + 0.5f), Vec4 (144.0f / 255.0f, 224.0f / 255.0f, 097.0f / 255.0f, 0.5f));
							//currentXyz->Set (Vec3 (x + 0.5f, y + 0.5f, z + 0.5f), Vec4 (1.0f, col, 1.0f - col, 0.5f));
							currentXyz->Set (Vec3 (x + 0.5f, y + 0.5f, z + 0.5f), Vec4 (col, col, col, 1.0f));

							++currentXyz;

							assert (currentXyz <= verts + vertsTotal);

							++found;
							if (found == maxPerY)
							{
								break;
							}
						}
					}
				}
			}
		}

		assert (currentXyz <= verts + vertsTotal);
		vertsTotal = currentXyz - verts;
	}

	AreaLow::~AreaLow ()
	{
		delete [] verts;
		verts = NULL;

		delete buffer;
		buffer = NULL;
	}

	void AreaLow::CopyToGfxCardIfNeeded ()
	{
		if (!copyToGfxCardIfNeeded)
		{
			return;
		}

		assert (buffer == NULL);
		buffer = new Gfx::Buffers::PointList ();
		buffer->SetVerts (verts, vertsTotal);

		delete [] verts;
		verts = NULL;

		copyToGfxCardIfNeeded = false;
	}

	void AreaLow::Update ()
	{
		CopyToGfxCardIfNeeded ();
	}

	void AreaLow::Render (int passIdx, Int3 columnId, Vec3 colRelPos)
	{
		assert (passIdx == 0);

		if (copyToGfxCardIfNeeded)
		{
			return;
		}

		if (buffer == NULL)
		{
			return;
		}

		Int3 areaToColumnId (
			simAreaLow->areaId.x * (simAreaLow->x * simAreaLow->scale) / SceneSim::CellsPerBlock,
			0,											   
			simAreaLow->areaId.z * (simAreaLow->z * simAreaLow->scale) / SceneSim::CellsPerBlock);

		Ents::Comps::Coord coord (areaToColumnId, Vec3 (0.0f, 0.0f, 0.0f));
		Vec3 offset = coord.GetColumnRelPos (columnId);

		glPushMatrix ();
		glTranslatef (offset.x, offset.y, offset.z);
		glScalef (simAreaLow->scale, simAreaLow->scale, simAreaLow->scale);

		buffer->Render ();

		glPopMatrix ();
	}

}