#include "stdafx.h"

#include "CellBuilder/CellBuilder_Mngr.h"
#include "CellBuilder/CellBuilder_ShapeBase.h"
#include "SceneSim/SceneSim_Cell.h"
#include "SceneSim/SceneSim_Column.h"
#include "SceneSim/SceneSim_ColumnCells.h"

#include "../CellMesh/CellMesh_Defs.h"
#include "../CellMesh/CellMesh_Mngr.h"
#include "../CellMesh/CellMesh_ShapeBase.h"
#include "../Gfx/Buffers_TriList.h"
#include "../Gfx/Gfx_Funcs.h"
#include "../Gfx/VertexTypes_XyzUvRgb.h"

#include "SceneGfx_Block.h"
#include "SceneGfx_Column.h"
#include "SceneGfx_Room.h"

static int recreateCount = 0;

namespace SceneGfx
{
	void TranslateXyz (Gfx::VertexTypes::XyzUvRgb * vert, int count, Int3 xyz)
	{
		while (count)
		{
			vert->xyz.x += xyz.x;
			vert->xyz.y += xyz.y;
			vert->xyz.z += xyz.z;

			--count;
			++vert;
		}
	}

	Block::Block (Vec3 rgb)
		: blockId (-1),
		copyToGfxCardIfNeeded (true),
		verts (NULL),
		buffer (NULL),
		vertsTotal (0),
		extraVerts (NULL),
		bufferExtra (NULL),
		extraVertsTotal (0)
	{
		//SceneSim

		const float size = 1.0f;

		vertsTotal = 24;
		verts = aah_new Gfx::VertexTypes::XyzUvRgb [vertsTotal];

		Vec4 rgba (rgb.r, rgb.g, rgb.b, 0.0f);

		verts [0].Set (Vec3 (size * 0.7f, 0.0f, -size * 0.7f), Vec2 (0.0f, 0.0f), rgba);
		verts [1].Set (Vec3 (-size * 0.7f, 0.0f, -size * 0.7f), Vec2 (0.0f, 0.0f), rgba);
		verts [2].Set (Vec3 (0.0f, 0.0f, size), Vec2 (0.0f, 0.0f), Vec4 (1.0f, 1.0f, 1.0f, 0.0f));

		verts [3] = verts [2]; 
		verts [4] = verts [1];
		verts [5] = verts [0];

		verts [6].Set (Vec3 (0.0f, size * 0.7f, -size * 0.7f), Vec2 (0.0f, 0.0f), rgba);
		verts [7].Set (Vec3 (0.0f, -size * 0.7f, -size * 0.7f), Vec2 (0.0f, 0.0f), rgba);
		verts [8].Set (Vec3 (0.0f, 0.0f, size), Vec2 (0.0f, 0.0f), Vec4 (1.0f, 1.0f, 1.0f, 0.0f));

		verts [9] = verts [8]; 
		verts [10] = verts [7];
		verts [11] = verts [6];

		verts [12] = verts [0];
		verts [13] = verts [6];
		verts [14] = verts [7];

		verts [15] = verts [14]; 
		verts [16] = verts [13];
		verts [17] = verts [12];

		verts [18] = verts [1];
		verts [19] = verts [6];
		verts [20] = verts [7];

		verts [21] = verts [20]; 
		verts [22] = verts [19];
		verts [23] = verts [18];
	}

#if 0
	Block::Block (Column * column, int blockId)
		: blockId (blockId),
		copyToGfxCardIfNeeded (true),
		verts (NULL),
		buffer (NULL),
		vertsTotal (0),
		extraVerts (NULL),
		bufferExtra (NULL),
		extraVertsTotal (0)
	{
		//copyToGfxCardIfNeeded = false;
		//return;

		assert (column != NULL);
		assert (blockId >= 0 && blockId < BlocksPerColY);

		const int CellsPerColXz = SceneSim::CellsPerColXz;
		const int maxTris = CellsPerColXz * CellsPerColXz * CellsPerColXz * 6 * 6 * 2;

		CellBuilder::Mngr * builderMngr = column->GetRoom ()->GetCellBuilder ();
		CellMesh::Mngr * meshMngr = column->GetRoom ()->GetCellMesh ();

		verts = aah_new Gfx::VertexTypes::XyzUvRgb [maxTris];

		const int maxExtraTris = 200 * 200;
		extraVerts = aah_new Gfx::VertexTypes::XyzUvRgb [maxExtraTris];
		bufferExtra = NULL;

		CellMesh::AddShapeData addShapeData;
		addShapeData.verts = verts;
		addShapeData.vertsUsed = 0;
		addShapeData.extraVerts = extraVerts;
		addShapeData.extraVertsUsed = 0;
		addShapeData.cellXyz = Int3 (0, 0, 0);
		addShapeData.column = column;
		addShapeData.blockId = blockId;

		//addShapeData.font = builderMngr->GetFont ();
		addShapeData.recreateBlock = NULL;

		for (addShapeData.cellXyz.x = 0; addShapeData.cellXyz.x < CellsPerColXz; ++addShapeData.cellXyz.x)
		{
			for (addShapeData.cellXyz.y = blockId * CellsPerBlockXz; addShapeData.cellXyz.y < (blockId * CellsPerBlockXz + CellsPerColXz); ++addShapeData.cellXyz.y)
			{
				for (addShapeData.cellXyz.z = 0; addShapeData.cellXyz.z < CellsPerColXz; ++addShapeData.cellXyz.z)
				{
					addShapeData.cell = column->GetSim ()->GetCells ()->GetCell (addShapeData.cellXyz);

					if (addShapeData.cell->type != 0)
					{
						//const CellBuilder::ShapeBase * shape = builderMngr->GetShape (addShapeData.cell->type);
						const CellMesh::ShapeBase * shape = meshMngr->GetShape (addShapeData.cell->type);
						if (shape != NULL)
						{
							Gfx::VertexTypes::XyzUvRgb * oldVerts = addShapeData.verts;
							Gfx::VertexTypes::XyzUvRgb * oldExtraVerts = addShapeData.extraVerts;

							shape->AddInstance (&addShapeData);

							const int vertsAdded = addShapeData.verts - oldVerts;
							const int extraVertsAdded = addShapeData.extraVerts - oldExtraVerts;

							TranslateXyz (oldVerts, vertsAdded, addShapeData.cellXyz);
							TranslateXyz (oldExtraVerts, extraVertsAdded, addShapeData.cellXyz);
						}
					}
				}
			}
		}

		vertsTotal = addShapeData.verts - verts;
		extraVertsTotal = addShapeData.extraVerts - extraVerts;
	}
/*
	Block::Block (SceneFile::Command_BlockRecreate * cmd)
		: copyToGfxCardIfNeeded (true),
		sim (cmd->block)
	{
		const int CellsPerColXz = SceneSim::CellsPerColXz;
		const int maxTris = CellsPerColXz * CellsPerColXz * CellsPerColXz * 6 * 6;

		if (sim->blockId == Int3 (0, 4, 0) && sim->column->columnId == Int3 (-27, 0, -220))
		{
			++recreateCount;
		}

		verts = aah_new Gfx::VertexTypes::XyzUvRgb [maxTris];
		buffer = NULL;

		const int maxExtraTris = 200 * 200;
		extraVerts = aah_new Gfx::VertexTypes::XyzUvRgb [maxExtraTris];
		bufferExtra = NULL;

		CellBuilder::AddShapeData addShapeData;
		addShapeData.verts = verts;
		addShapeData.vertsUsed = 0;
		addShapeData.extraVerts = extraVerts;
		addShapeData.extraVertsUsed = 0;
		addShapeData.cellXyz = Int3 (0, 0, 0);
		addShapeData.block = sim;
		addShapeData.font = cmd->builderMngr->GetFont ();
		addShapeData.recreateBlock = cmd;

		for (addShapeData.cellXyz.x = 0; addShapeData.cellXyz.x < CellsPerColXz; ++addShapeData.cellXyz.x)
		{
			for (addShapeData.cellXyz.y = 0; addShapeData.cellXyz.y < CellsPerColXz; ++addShapeData.cellXyz.y)
			{
				for (addShapeData.cellXyz.z = 0; addShapeData.cellXyz.z < CellsPerColXz; ++addShapeData.cellXyz.z)
				{
					addShapeData.cell = cmd->GetCell (addShapeData.cellXyz);

					if (addShapeData.cell->type != SceneSim::AIR)
					{
						const CellBuilder::ShapeBase * shape = cmd->builderMngr->GetShape (addShapeData.cell->type);
						if (shape != NULL)
						{
							Gfx::VertexTypes::XyzUvRgb * oldVerts = addShapeData.verts;
							Gfx::VertexTypes::XyzUvRgb * oldExtraVerts = addShapeData.extraVerts;

							shape->AddInstance (&addShapeData);

							const int vertsAdded = addShapeData.verts - oldVerts;
							const int extraVertsAdded = addShapeData.extraVerts - oldExtraVerts;

							TranslateXyz (oldVerts, vertsAdded, addShapeData.cellXyz);
							TranslateXyz (oldExtraVerts, extraVertsAdded, addShapeData.cellXyz);
						}
					}
				}
			}
		}

		vertsTotal = addShapeData.verts - verts;
		extraVertsTotal = addShapeData.extraVerts - extraVerts;
	}
*/
#endif

	Block::Block (int blockId, const SceneSim::AdjColumns * columns, const CellMesh::Mngr * meshMngr, const CellBuilder::Mngr * builderMngr)
		: columnId (columns->main->columnId),
		blockId (blockId),
		copyToGfxCardIfNeeded (true),
		verts (NULL),
		buffer (NULL),
		vertsTotal (0),
		extraVerts (NULL),
		bufferExtra (NULL),
		extraVertsTotal (0)
	{
		assert (columns != NULL);
		assert (blockId >= 0 && blockId < BlocksPerColY);

		if (columns->main->flags == SceneSim::Column::SF_NO_CELLS)
		{
			copyToGfxCardIfNeeded = false;
			return;
		}

		const int CellsPerColXz = SceneSim::CellsPerColXz;
		const int maxTris = CellsPerColXz * CellsPerColXz * CellsPerColXz * 6 * 6 * 2;

		verts = aah_new Gfx::VertexTypes::XyzUvRgb [maxTris];

		const int maxExtraTris = 200 * 200;
		extraVerts = aah_new Gfx::VertexTypes::XyzUvRgb [maxExtraTris];
		bufferExtra = NULL;

		CellMesh::AddShapeData addShapeData (blockId, columns, meshMngr, verts, extraVerts);

		//addShapeData.verts = verts;
		//addShapeData.vertsUsed = 0;
		//addShapeData.extraVerts = extraVerts;
		//addShapeData.extraVertsUsed = 0;
		//addShapeData.cellXyz = Int3 (0, 0, 0);
		//addShapeData.columnSrc = columnSrc;
		//addShapeData.meshMngr = meshMngr;
		//addShapeData.blockId = blockId;

		//addShapeData.font = builderMngr->GetFont ();
		//addShapeData.recreateBlock = NULL;

		//const SceneSim::ColumnCells * cells = columnSrc->GetCells ();

		const int height = columns->main->GetSize ().y;
		Int3 cellId (0, 0, 0);

		for (cellId.x = 0; cellId.x < SceneSim::CellsPerColXz; ++cellId.x)
		{
			for (cellId.y = 0; cellId.y < height; ++cellId.y)
			{
				for (cellId.z = 0; cellId.z < SceneSim::CellsPerColXz; ++cellId.z)
				{
					const SceneSim::Cell * cell = addShapeData.GetCell (cellId);
					const CellMesh::ShapeBase * shape = addShapeData.GetShape (cellId);
					if (cell->type != 0 && shape->HasMesh ())
					{
						const CellMesh::ShapeBase * shape = addShapeData.GetShape (cellId);

						Gfx::VertexTypes::XyzUvRgb * oldVerts = addShapeData.verts;
						Gfx::VertexTypes::XyzUvRgb * oldExtraVerts = addShapeData.extraVerts;

						shape->AddInstance (&addShapeData, cellId);

						const int vertsAdded = addShapeData.verts - oldVerts;
						const int extraVertsAdded = addShapeData.extraVerts - oldExtraVerts;

						TranslateXyz (oldVerts, vertsAdded, cellId);
						TranslateXyz (oldExtraVerts, extraVertsAdded, cellId);
					}
					else
					{
						int i = 0;
					}
				}
			}
		}

		//for (addShapeData.cellXyz.x = 0; addShapeData.cellXyz.x < CellsPerColXz; ++addShapeData.cellXyz.x)
		//{
		//	for (addShapeData.cellXyz.y = blockId * CellsPerBlockY; addShapeData.cellXyz.y < (blockId * CellsPerBlockY + CellsPerBlockY); ++addShapeData.cellXyz.y)
		//	{
		//		for (addShapeData.cellXyz.z = 0; addShapeData.cellXyz.z < CellsPerColXz; ++addShapeData.cellXyz.z)
		//		{
		//			addShapeData.cell = cells->GetCell (addShapeData.cellXyz);

		//			if (addShapeData.cellXyz.y == 16 && addShapeData.cell->type == 0)
		//			{
		//				int i = 0;
		//				 cells->GetCell (addShapeData.cellXyz);
		//			}

		//			if (addShapeData.cell->type != 0)
		//			{
		//				//const CellBuilder::ShapeBase * shape = builderMngr->GetShape (addShapeData.cell->type);
		//				const CellMesh::ShapeBase * shape = meshMngr->GetShape (addShapeData.cell->type);
		//				assert (shape != NULL);
		//				if (shape != NULL)
		//				{
		//					Gfx::VertexTypes::XyzUvRgb * oldVerts = addShapeData.verts;
		//					Gfx::VertexTypes::XyzUvRgb * oldExtraVerts = addShapeData.extraVerts;

		//					shape->AddInstance (&addShapeData);

		//					const int vertsAdded = addShapeData.verts - oldVerts;
		//					const int extraVertsAdded = addShapeData.extraVerts - oldExtraVerts;

		//					TranslateXyz (oldVerts, vertsAdded, addShapeData.cellXyz);
		//					TranslateXyz (oldExtraVerts, extraVertsAdded, addShapeData.cellXyz);
		//				}
		//			}
		//		}
		//	}
		//}

		vertsTotal = addShapeData.verts - verts;
		extraVertsTotal = addShapeData.extraVerts - extraVerts;
	}

	Block::~Block ()
	{
		delete [] extraVerts;
		extraVerts = NULL;

		delete [] verts;
		verts = NULL;

		delete buffer;
		buffer = NULL;

		delete bufferExtra;
		bufferExtra = NULL;

		//if (vboId != ;ULL)
		//{
		//	Gfx::glDeleteBuffersARB (1, &vboId);
		//	vboId = NULL;
		//}
	}

	void Block::CopyToGfxCardIfNeeded ()
	{
		if (!copyToGfxCardIfNeeded)
		{
			return;
		}

		assert (buffer == NULL);
		buffer = aah_new Gfx::Buffers::TriList ();
		buffer->SetVerts (verts, vertsTotal);

		delete [] verts;
		verts = NULL;

		assert (bufferExtra == NULL);
		bufferExtra = aah_new Gfx::Buffers::TriList ();
		bufferExtra->SetVerts (extraVerts, extraVertsTotal);

		delete [] extraVerts;
		extraVerts = NULL;

		copyToGfxCardIfNeeded = false;
	}

	void Block::Render (int passIdx)
	{
		if (copyToGfxCardIfNeeded)
		{
			return;
		}

		if (passIdx == 1)
		{
			if (bufferExtra != NULL)
			{
				bufferExtra->Render ();
			}
		}
		else
		{
			if (buffer != NULL)
			{
				buffer->Render ();
			}
		}
	}
}
