#include "stdafx.h"

#include "Ents/Comps_Coord.h"
#include "SceneSim/SceneSim_Column.h"
#include "SceneSim/SceneSim_Defs.h"
#include "Thread/Thread_Job.h"
#include "Thread/Thread_JobMngr.h"

#include "SceneGfx_Block.h"
#include "SceneGfx_Column.h"
#include "SceneGfx_Room.h"

namespace SceneGfx
{
	struct ColumnGlobals
	{
		ColumnGlobals () 
			: blocksDrawn (0),
			blocksNotDrawn (0)
		{
		}

		int blocksDrawn;
		int blocksNotDrawn;
	};

	ColumnGlobals g_column;

	void Column_DrawDebug ()
	{
		//Debug_DrawNewLine ();
		//Debug_WriteLine (0, "Blocks rendered =");
		//Debug_DrawInt (g_column.blocksDrawn);
		//Debug_DrawNewLine ();
		//Debug_WriteLine (0, "Blocks not rendered =");
		//Debug_DrawInt (g_column.blocksNotDrawn);

		g_column.blocksDrawn = g_column.blocksNotDrawn = 0;
	}

	Column::Column (Room * room, SceneSim::Column * column)
		: room (room),
		sim (column)
	{
		assert (room != NULL);
		assert (sim != NULL);

		memset (cellsSolids, 0, sizeof (cellsSolids));
		memset (blocks, 0, sizeof (blocks));

		//for (int idx = 0; idx < SceneSim::F_COUNT_CUBE; ++idx)
		//{
		//	SceneSim::Column * adjSim = sim->GetAdjacentColumn ((SceneSim::FACES) idx);
		//	if (adjSim != NULL)
		//	{
		//		Column * adj = room->FindColumn (adjSim);
		//		assert (adj);
		//		adj->MarkDirty ();
		//	}
		//}
	}

	Column::~Column ()
	{
		for (int blockId = 0; blockId < BlocksPerColY; ++blockId)
		{
			delete blocks [blockId];
			blocks [blockId] = NULL;
		}
	}

	Room * Column::GetRoom () const
	{
		return room;
	}

	SceneSim::Column * Column::GetSim () const
	{
		return sim;
	}
	
	void Column::Update ()
	{
		for (int blockId = 0; blockId < BlocksPerColY; ++blockId)
		{
			if (blocks [blockId] != NULL)
			{
				++g_column.blocksDrawn;
			}
			else
			{
				++g_column.blocksNotDrawn;
			}
		}
	}

	void Column::Render (int passIdx, Int3 columnId, Vec3 colRelPos, int drawDistance, Int3 offset)
	{
		const SceneSim::Column * column = GetSim ();

		for (int blockId = 0; blockId < BlocksPerColY; ++blockId)
		{
			if (blocks [blockId] != NULL)
			{
				Ents::Comps::Coord coord (column->columnId, Vec3 (0.0f, 0.0f, 0.0f));
				Vec3 offset2 = coord.GetColumnRelPos (columnId + offset);

				//if ((offset2.x * offset2.x + offset2.z * offset.z) < (drawDistance * drawDistance) + 1)
				{
					glPushMatrix ();
					glTranslatef (offset2.x, offset2.y, offset2.z);

					blocks [blockId]->Render (passIdx);

					glPopMatrix ();
				}
			}
		}
	}

	void Column::AddOrReplaceBlock (Block * block)
	{
		assert (block != NULL);

		delete blocks [block->blockId];
		blocks [block->blockId] = block;
		//isBlockDirty [block->blockId] = false;

		block->CopyToGfxCardIfNeeded ();
	}

	void Column::SwapWith (Column * other)
	{
		GetSim ()->SwapCells (other->GetSim ());

		for (int i = 0; i < BlocksPerColY; ++i)
		{
			Block * tempBlock = blocks [i];
			blocks [i] = other->blocks [i];
			other->blocks [i] = tempBlock;
		}
	}
}