#include "stdafx.h"

#include "SceneHelper/SceneHelper_Helper.h"
#include "SceneSim/SceneSim_Column.h"

#include "SceneGfx_Block.h"
#include "SceneGfx_BlockChangeList.h"
#include "SceneGfx_Column.h"
#include "SceneGfx_JobCreateBlockMeshes.h"
#include "SceneGfx_Room.h"

namespace SceneGfx
{
	JobCreateBlockMeshes::JobCreateBlockMeshes 
	(
		Thread::JobMngr * jobMngr, 
		Room * room,
		BlockChangeList * changes
	)
		: Job (jobMngr),
		room (room),
		changes (changes),
		blockChangesCount (0),
		blocks (NULL)
	{
		assert (room != NULL);
		assert (changes != NULL);

		const BlockChangeList::EntryColl & entries = changes->GetEntries ();

		columnSrcs = aah_new SceneSim::Column * [entries.size ()];

		for (int idx = 0; idx < entries.size (); ++idx)
		{
			Column * column = room->GetColumn (entries [idx].columnId);
			assert (column != NULL);
			SceneSim::Column * simColumn = column->GetSim ();
			assert (simColumn != NULL);

			columnSrcs [idx] = aah_new SceneSim::Column (*simColumn);

			blockChangesCount += Utils::CountBitsSet (entries [idx].blockIdsAsBits);
		}

		blocks = aah_new SceneGfx::Block * [blockChangesCount];
		memset (blocks, 0, sizeof (SceneGfx::Block *) * blockChangesCount);
	}

	JobCreateBlockMeshes::~JobCreateBlockMeshes ()
	{
	}

	Thread::JOBSTATES JobCreateBlockMeshes::JobDoWork ()
	{
		assert (false);
		//const BlockChangeList::EntryColl & entries = changes->GetEntries ();

		//int nextBlockIdx = 0;
		//for (int idx = 0; idx < entries.size (); ++idx)
		//{
		//	for (int blockId = 0; blockId < BlocksPerColY; ++blockId)
		//	{
		//		if (entries [idx].blockIdsAsBits & (1 << blockId))
		//		{
		//			blocks [nextBlockIdx] = aah_new Block (blockId, dataForBlockGeneration [idx]);
		//			++nextBlockIdx;
		//		}
		//	}
		//}
		return Thread::JS_WORKDONE;
	}

	Thread::JOBSTATES JobCreateBlockMeshes::JobOnCompleted ()
	{
		Job::JobOnCompleted ();

		const BlockChangeList::EntryColl & entries = changes->GetEntries ();

		for (int blockIdx = 0; blockIdx < blockChangesCount; ++blockIdx)
		{
			Column * column = room->GetColumn (blocks [blockIdx]->columnId);
			if (column != NULL)
			{
				column->AddOrReplaceBlock (blocks [blockIdx]);
			}
			else
			{
				delete blocks [blockIdx];
			}
			blocks [blockIdx] = NULL;
		}

		for (int idx = 0; idx < entries.size (); ++idx)
		{
			delete columnSrcs [idx];
		}
		delete [] columnSrcs;
		columnSrcs = NULL;

		delete [] blocks;
		blocks = NULL;

		delete changes;

		return Thread::JS_COMPLETED;
	}
}
