#include "stdafx.h"

#include <algorithm>

#include "CellBuilder/CellBuilder_Mngr.h"
#include "Ld/Ld_ColumnMask.h"
#include "SceneSim/SceneSim_Column.h"
#include "SceneSim/SceneSim_ColumnCells.h"
#include "SceneSim/SceneSim_Room.h"
#include "SceneSim/SceneSim_World.h"

#include "../CellMesh/CellMesh_Mngr.h"
#include "../Gfx/Gfx_Font.h"

#include "SceneGfx_BlockChangeList.h"
#include "SceneGfx_Column.h"
#include "SceneGfx_ColumnBlockBits.h"
#include "SceneGfx_JobCreateBlockMeshes.h"
#include "SceneGfx_Room.h"
#include "SceneGfx_TexMngr.h"

namespace SceneGfx
{
	struct ColumnUpdateFunctor
	{
		ColumnUpdateFunctor & operator () (Column * column)
		{
			column->Update ();
			return *this;
		}
	};

	struct ColumnRenderFunctor
	{
		ColumnRenderFunctor (int passIdx, Ld::ColumnMask * mask, Int3 columnId, Vec3 colRelPos, int drawDistance, Int3 offset) : passIdx (passIdx), mask (mask), columnId (columnId), colRelPos (colRelPos), drawDistance (drawDistance), renderCount (0), offset (offset) {}
		ColumnRenderFunctor & operator () (Column * column)
		{
			if (mask->use [column->GetSim ()->columnId.x][column->GetSim ()->columnId.z])
			{
				column->Render (passIdx, columnId, colRelPos, drawDistance, offset);
				++renderCount;
			}
			else
			{
				int i = 0;
			}
			return *this;
		}

		const int passIdx;
		const Ld::ColumnMask * mask;
		const Int3 columnId;
		const Vec3 colRelPos;
		int drawDistance;
		const Int3 offset;

		int renderCount;
	};

	Room::Room (SceneSim::Room * simRoom)
		: simWorld (NULL),
		simRoom (simRoom)
	{
		assert (simRoom != NULL);

		texMngr = aah_new TexMngr (NULL, "world/graphics_sets/default/");
		meshMngr = aah_new CellMesh::Mngr ();
		builderMngr = aah_new CellBuilder::Mngr ();
	}

	Room::~Room ()
	{
		while (!columns.empty ())
		{
			Column * column = columns.front ();
			columns.pop_front ();
			delete column;
		}

		delete builderMngr;
		builderMngr = NULL;

		delete meshMngr;
		meshMngr = NULL;

		delete texMngr;
		texMngr = NULL;
	}

#if 0

	void Room::SetColumn (Int3 columnId, unsigned int columnFlags, int compressedSize, unsigned char * compressedData)
	{
		assert (false);
	/*	assert (GetColumn (columnId) == NULL);

		SceneSim::Column * simColumn = aah_new SceneSim::Column (columnId, columnFlags);

		if (!(columnFlags & SceneSim::Column::SF_NO_CELLS))
		{
			assert (compressedSize > 0 && compressedData != NULL);

			static const int sizeOfTest = sizeof (SceneSim::ColumnCells);
			int upTo = Utils::Zlib_Uncompress (compressedData, compressedSize, (unsigned char *) simColumn->GetCells (), sizeOfTest);
			assert (upTo == sizeOfTest);
		}
		else
		{
			assert (compressedSize == 0 && compressedData == NULL);
		}

		simColumn->CalcSkyLightMap (simRoom->GetCellBuilder ());

		simRoom->AddColumn (simColumn);

		Column * column = aah_new Column (this, simColumn);
		columns.push_back (column);


		{
			BlockChangeList * changes = aah_new BlockChangeList ();
			changes->AddEntry (columnId, (1 << BlocksPerColY) - 1);

			SceneSim::Column * adjColumn = NULL;
			for (int face = SceneSim::F_N; face <= SceneSim::F_W; ++face)
			{
				adjColumn = simColumn->GetAdjacentColumn ((SceneSim::FACES) face);
				if (adjColumn != NULL)
				{
					changes->AddEntry (adjColumn->columnId, (1 << BlocksPerColY) - 1);
				}
			}
			
			JobCreateBlockMeshes * job = aah_new JobCreateBlockMeshes (jobMngr, this, changes);
			job->AddToQueue (Thread::JB_DEFAULT);
		}*/
	}

#endif 

	SceneSim::Room * Room::GetSimRoom () const
	{
		return simRoom;
	}

	void Room::AddColumn (Column * column)
	{
		assert (column != NULL);
		assert (GetColumn (column->GetSim ()->columnId) == NULL);

		columns.push_back (column);
	}

	void Room::RemoveColumn (Column * column)
	{
		assert (column != NULL);
		assert (GetColumn (column->GetSim ()->columnId) != NULL);

		columns.remove (column);
	}

	void Room::Update ()
	{
		std::for_each (columns.begin (), columns.end (), ColumnUpdateFunctor ());
	}

	void Room::Render_Begin (int passIdx)
	{
		if (passIdx == 0)
		{
			glEnable (GL_DEPTH_TEST);

			//glEnable (GL_CULL_FACE); 
			//glFrontFace (GL_CW);
			//glCullFace (GL_BACK);

			glDisable (GL_BLEND);
			//glEnable (GL_ALPHA_TEST);
			//glAlphaFunc (GL_NOTEQUAL, 0.0f);
			//glAlphaFunc (GL_GEQUAL, 1.0f);
			//glAlphaFunc (GL_GREATER, 0.5f);
			//(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA)

		}

		Start (passIdx);
	}

	void Room::Render_End (int passIdx)
	{
		Stop (passIdx);
	}

	void Room::Render (int passIdx, Ld::ColumnMask * mask, Int3 columnId, Vec3 colRelPos, int drawDistance, Int3 offset)
	{
		Render_Begin (passIdx);
		ColumnRenderFunctor result = std::for_each (columns.begin (), columns.end (), ColumnRenderFunctor (passIdx, mask, columnId, colRelPos, drawDistance, offset));
		Render_End (passIdx);

		//Debug_DrawNewLine ();
		//Debug_DrawNewLine ();
		//Debug_WriteLine (0, "Columns rendered =");	
		//Debug_DrawInt (result.renderCount);
		//Column_DrawDebug ();
	}

	void Room::Start (int texIdx)
	{
		switch (texIdx)
		{
		case 0: texMngr->Bind (0); break;
		case 1: assert (false); /*builderMngr->GetFont ()->Begin ();*/ break;
		}

		glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
		glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
	}

	void Room::Stop (int texIdx)
	{
		switch (texIdx)
		{
		case 0: texMngr->Unbind (0); break;
		case 1: assert (false); /*builderMngr->GetFont ()->End ();*/ break;
		}
	}

	const TexMngr * Room::GetTexMngr () const
	{
		return texMngr;
	}

	TexMngr * Room::GetTexMngr ()
	{
		return texMngr;
	}

	const CellMesh::Mngr * Room::GetCellMesh () const
	{
		return meshMngr;
	}

	CellMesh::Mngr * Room::GetCellMesh ()
	{
		return meshMngr;
	}

	const CellBuilder::Mngr * Room::GetCellBuilder () const
	{
		return builderMngr;
	}

	CellBuilder::Mngr * Room::GetCellBuilder ()
	{
		return builderMngr;
	}

	Column * Room::GetColumn (Int3 columnId)
	{
		ColumnCollIt current = columns.begin ();
		while (current != columns.end ())
		{
			Column * column = *current;
			if (column->GetSim ()->columnId == columnId)
			{
				return column;
			}
			++current;
		}
		return NULL;
	}

	Column * Room::FindColumn (SceneSim::Column * simColumn)
	{
		ColumnCollIt current = columns.begin ();
		while (current != columns.end ())
		{
			Column * column = *current;
			if (column->GetSim () == simColumn)
			{
				return column;
			}
			++current;
		}
		return NULL;
	}
}
