#pragma once

#include <list>

#include "Thread/Thread_SafeList.h"

namespace CellBuilder {class Mngr;}
namespace CellMesh {class Mngr;}
namespace Ld {class ColumnMask;}
namespace SceneGfx {class JobCreateBlockMesh;}
namespace SceneGfx {class Column;}
namespace SceneGfx {class TexMngr;}
namespace SceneGfx {class World;}
namespace SceneSim {class Column;}
namespace SceneSim {class Room;}
namespace SceneSim {class World;}
namespace Thread {class JobMngr;}

namespace SceneGfx
{
	class Room
	{
	public:

		Room (SceneSim::Room * simRoom);
		~Room ();

		void OnAddedToWorld (World * world);
		void OnRemovedFromWorld ();

		//void SetColumn (Int3 columnId, unsigned int columnFlags, int compressedSize, unsigned char * compressedData);
		SceneSim::Room * GetSimRoom () const;

		void AddColumn (Column * column);
		void RemoveColumn (Column * column);

		void Update ();
		void Render_Begin (int passIdx);
		void Render_End (int passIdx);
		void Render (int passIdx, Ld::ColumnMask * mask, Int3 columnId, Vec3 colRelPos, int drawDistance, Int3 offset);

		void Start (int texIdx);
		void Stop (int texIdx);

		const TexMngr * GetTexMngr () const;
		TexMngr * GetTexMngr ();

		const CellMesh::Mngr * GetCellMesh () const;
		CellMesh::Mngr * GetCellMesh ();

		const CellBuilder::Mngr * GetCellBuilder () const;
		CellBuilder::Mngr * GetCellBuilder ();

		Column * GetColumn (Int3 columnId);
		Column * FindColumn (SceneSim::Column * simColumn);

		//virtual void OnRoomAdded (SceneSim::Room * room);
		//virtual void OnRoomRemoved (SceneSim::Room * room);
		//virtual void OnColumnAdded (SceneSim::Column * column);
		//virtual void OnColumnRemoved (SceneSim::Column * column);
		//virtual void OnTerrainCsgApplied (SceneSim::Column * column, Int3 cellId, float radius, int cellType);
		//virtual void OnTerrainCellsChanged (const SceneSim::CellChangeData * changes, int changesCount);

	private:

		typedef std::list <Column *> ColumnColl;
		typedef ColumnColl::iterator ColumnCollIt;

		SceneSim::World * simWorld;
		SceneSim::Room * simRoom;
		ColumnColl columns;

		TexMngr * texMngr;
		CellMesh::Mngr * meshMngr;
		CellBuilder::Mngr * builderMngr;

		Thread::SafeList <JobCreateBlockMesh *> blockMeshCreates;	
	};
}
