#include "stdafx.h"

#include <algorithm>

#include "SceneFile/SceneFile_Lighting.h"
#include "SceneSim/SceneSim_CellChangeData.h"
#include "SceneSim/SceneSim_Column.h"
#include "SceneSim/SceneSim_ColumnCells.h"
#include "SceneSim/SceneSim_Defs.h"
#include "SceneSim/SceneSim_Room.h"
#include "SceneSim/SceneSim_World.h"

#include "SceneGfx_Column.h"
#include "SceneGfx_ColumnBlockBits.h"
#include "SceneGfx_BlockChangeList.h"
#include "SceneGfx_JobCreateBlockMesh.h"
#include "SceneGfx_JobCreateBlockMeshes.h"
#include "SceneGfx_Room.h"
#include "SceneGfx_World.h"

namespace SceneGfx
{
	World::World ()
		: roomPrimary (NULL),
		simWorld (NULL)
	{
		//loadState = SceneSim::LS_NONE;
	}

	World::~World ()
	{
		//assert (simWorld == NULL);

		delete roomPrimary;
	}

	void World::Update ()
	{
		if (roomPrimary != NULL)
		{
			roomPrimary->Update ();
		}
	}

	void World::Render (int passIdx, Ld::ColumnMask * mask, Int3 columnId, Vec3 colRelPos, int drawDistance, Int3 offset)
	{
		if (roomPrimary != NULL)
		{
			roomPrimary->Render (passIdx, mask, columnId, colRelPos, drawDistance, offset);
		}
	}

	void World::OnLoadCompleted (SceneSim::World * simWorld)
	{
		assert (simWorld != NULL);
		assert (this->simWorld == NULL);

		this->simWorld = simWorld;
	}

	void World::SetPrimaryRoom (Room * room)
	{
		assert (room != NULL);
		assert (roomPrimary == NULL);

		roomPrimary = room;
	}
	
	Room * World::GetPrimaryRoom () const
	{
		return roomPrimary;
	}

	//void World::ClearWorld ()
	//{
	//	assert (false);
	//}

	void World::ClearPrimaryRoom ()
	{
		assert (false);
	}


	
//
//	void World::OnRoomRemoved (SceneSim::Room * room)
//	{
//	}
//
//	void World::OnColumnAdded (SceneSim::Column * simColumn)
//	{
//		Room * room = rooms [0];
//		Column * column = room->FindColumn (simColumn);
//		if (column == NULL)
//		{
//			column = room->AddColumn (simColumn);
//		}
//
//		//Debug_WriteLine (0, "Column loaded (%d, %d)", simColumn->columnId.x, simColumn->columnId.z);
//
//		// this mode only generates a column once all 4 of its neighbours are loaded - this is best for finite levels
//
//#if 1
//
//		for (int adjIdx = 0; adjIdx < 5; ++adjIdx)
//		{
//			SceneSim::Column * columnToCheck = (adjIdx == 0) ? simColumn : simColumn->GetAdjacentColumn ((SceneSim::FACES) (adjIdx - 1 + SceneSim::F_N));
//			if (columnToCheck != NULL && !(columnToCheck->flags & SceneSim::Column::SF_NO_CELLS))
//			{
//				int adjCount = 0;
//				for (int adjIdx2 = 0; adjIdx2 < SceneSim::F_COUNT_CUBE; ++adjIdx2)
//				{
//					SceneSim::Column * simAdjColumn = columnToCheck->GetAdjacentColumn ((SceneSim::FACES) adjIdx2);
//					if (simAdjColumn != NULL)
//					{
//						++adjCount;
//					}
//				}
//
//				//Debug_WriteLine (
//				//	0, 
//				//	"     adj (%d, %d), neighboursLoaded = %d", 
//				//	simAdjColumn->columnId.x, 
//				//	simAdjColumn->columnId.z, 
//				//	adjCount);
//
//				if (adjCount == 4) 
//				{
//					assert (room->GetColumn (columnToCheck->columnId) != NULL);
//
//					BlockChangeList * changes = aah_new BlockChangeList ();
//					changes->AddEntry (columnToCheck->columnId, (1 << BlocksPerColY) - 1);
//
//					JobCreateBlockMeshes * job = aah_new JobCreateBlockMeshes (jobMngr, room, changes);
//					job->AddToQueue (Thread::JB_DEFAULT);
//				}
//			}
//		}
//
//#else
//
//		Int3 columnId = column->GetSim ()->columnId;
//		
//		for (int blockId = 0; blockId < BlocksPerColY; ++blockId)
//		{
//			CreateBlockMeshData * validChanges = aah_new CreateBlockMeshData [1 * 5]; // itself and up to 4 neighbours with identical blockId
//
//			int validChangeCount = 1;
//			validChanges [0].blockId = blockId;
//			validChanges [0].column = column;
//
//			SceneSim::Column * adjSimColumn = simColumn->GetAdjacentColumn (SceneSim::F_N);
//			if (adjSimColumn != NULL && adjSimColumn->loadState == SceneSim::LS_LOADCOMPLETE)
//			{
//				assert (adjSimColumn->loadState == SceneSim::LS_LOADCOMPLETE);
//				validChanges [validChangeCount].blockId = blockId;
//				validChanges [validChangeCount].column = room->FindColumn (adjSimColumn);
//				assert (validChanges [validChangeCount].column != NULL);
//				++validChangeCount;
//			}
//
//			adjSimColumn = simColumn->GetAdjacentColumn (SceneSim::F_E);
//			if (adjSimColumn != NULL && adjSimColumn->loadState == SceneSim::LS_LOADCOMPLETE)
//			{
//				assert (adjSimColumn->loadState == SceneSim::LS_LOADCOMPLETE);
//				validChanges [validChangeCount].blockId = blockId;
//				validChanges [validChangeCount].column = room->FindColumn (adjSimColumn);
//				assert (validChanges [validChangeCount].column != NULL);
//				++validChangeCount;
//			}
//
//			adjSimColumn = simColumn->GetAdjacentColumn (SceneSim::F_S);
//			if (adjSimColumn != NULL && adjSimColumn->loadState == SceneSim::LS_LOADCOMPLETE)
//			{
//				assert (adjSimColumn->loadState == SceneSim::LS_LOADCOMPLETE);
//				validChanges [validChangeCount].blockId = blockId;
//				validChanges [validChangeCount].column = room->FindColumn (adjSimColumn);
//				assert (validChanges [validChangeCount].column != NULL);
//				++validChangeCount;
//			}
//
//			adjSimColumn = simColumn->GetAdjacentColumn (SceneSim::F_W);
//			if (adjSimColumn != NULL && adjSimColumn->loadState == SceneSim::LS_LOADCOMPLETE)
//			{
//				assert (adjSimColumn->loadState == SceneSim::LS_LOADCOMPLETE);
//				validChanges [validChangeCount].blockId = blockId;
//				validChanges [validChangeCount].column = room->FindColumn (adjSimColumn);
//				assert (validChanges [validChangeCount].column != NULL);
//				++validChangeCount;
//			}
//
//			JobCreateBlockMeshes * job = aah_new JobCreateBlockMeshes (jobMngr, validChanges, validChangeCount);
//			job->AddToQueue (Thread::JB_DEFAULT);
//		}
//
//#endif 
//
//	}
//
//	void World::OnColumnRemoved (SceneSim::Column * simColumn)
//	{
//		Room * room = rooms [0];
//
//		//Column * column = room->GetColumn (simColumn->columnId);
//		//column->markAsDead = true;
//
//		Column * column = room->FindColumn (simColumn);
//		assert (column != NULL);
//
//		while (column->jobsOutstandingCount > 0)
//		{
//			Sleep (1);
//		}
//		
//		room->RemoveColumn (simColumn);
//	}
//
//	void World::OnTerrainCsgApplied (SceneSim::Column * simColumn, Int3 cellId, float radius, int cellType)
//	{
//	}
//
//	void World::OnTerrainCellsChanged (const SceneSim::CellChangeData * changes, int changesCount)
//	{	
//		if (changesCount == 0)
//		{
//			return;
//		}
//
//		BlockChangeList allChangeList;
//
//		Room * room = rooms [0];
//
//		// work out non duplicate list of blocks to recreate meshes for
//		for (int idx = 0; idx < changesCount; ++idx)
//		{
//			const Int3 columnId (changes [idx].column->columnId);
//			const Int3 cellId (changes [idx].cellId);
//			const int blockId = cellId.y / SceneGfx::CellsPerBlockXyz;
//
//			allChangeList.AddEntry (columnId, 1 << blockId);
//
//			// "grow" up
//			if (cellId.y < SceneSim::CellsPerColY - 1 && (cellId.y % SceneGfx::CellsPerBlockXyz) == (SceneGfx::CellsPerBlockXyz - 1))
//			{
//				allChangeList.AddEntry (columnId,1 << (blockId + 1));
//			}
//
//			// "grow" down
//			if (cellId.y > 0 && (cellId.y % SceneGfx::CellsPerBlockXyz) == 0)
//			{
//				allChangeList.AddEntry (columnId, 1 << (blockId - 1));
//			}
//
//			// "grow" n
//			if (cellId.z == SceneSim::CellsPerColXz - 1)
//			{
//				allChangeList.AddEntry (columnId + Int3 (0, 0, 1), 1 << (blockId));
//			}
//
//			// "grow" e
//			if (cellId.x == SceneSim::CellsPerColXz - 1)
//			{
//				allChangeList.AddEntry (columnId + Int3 (1, 0, 0), 1 << (blockId));
//			}
//
//			// "grow" s
//			if (cellId.z == 0)
//			{
//				allChangeList.AddEntry (columnId + Int3 (0, 0, -1), 1 << (blockId));
//			}
//
//			// "grow" w
//			if (cellId.x == 0)
//			{
//				allChangeList.AddEntry (columnId + Int3 (-1, 0, 0), 1 << (blockId));
//			}
//		}
//
//		// TODO
//		// now add blocks below those in the list - because changes will / may affect the shadows
//
//		// validate and copy the list
//
//		BlockChangeList * validChangeList = aah_new BlockChangeList ();
//
//		const BlockChangeList::EntryColl & checkAgainst = allChangeList.GetEntries ();
//		for (int idx = 0; idx < checkAgainst.size (); ++idx)
//		{
//			Column * column = room->GetColumn (checkAgainst [idx].columnId);
//			if (column != NULL && (column->GetSim ()->flags & SceneSim::Column::SF_NO_CELLS) == 0)
//			{
//				// extend updates to all blocks below those affected - so shadows are correctly applied
//
//				unsigned int blockIdsAsBits = checkAgainst [idx].blockIdsAsBits;
//				int isSet = 0;
//				for (int blockId = BlocksPerColY - 1; blockId >= 0; --blockId)
//				{
//					if (blockIdsAsBits & (1 << blockId))
//					{
//						isSet = 1;
//					}
//					blockIdsAsBits |= isSet << blockId;
//				}
//				validChangeList->AddEntry (checkAgainst [idx].columnId, blockIdsAsBits);
//			}
//		}
//
//		// with the final list of what needs changing - apply it
//		if (validChangeList->GetEntries ().size () > 0)
//		{
//			JobCreateBlockMeshes * job = aah_new JobCreateBlockMeshes (jobMngr, room, validChangeList);
//			job->AddToQueue (Thread::JB_HIGH);
//		}
//		else
//		{
//			delete [] validChangeList;
//		}
//	}
}
