#include "stdafx.h"

#include "../CellBuilder/CellBuilder_Mngr.h"
#include "../CellBuilder/CellBuilder_ShapeBase.h"
#include "../SceneSim/SceneSim_Column.h"
#include "../SceneSim/SceneSim_ColumnCells.h"
#include "../SceneSim/SceneSim_Room.h"

#include "Collision_BoxSwept_Box.h"
#include "Collision_BoxSwept_Column.h"
#include "Collision_Result.h"

namespace Collision
{
	bool BoxSwept_Recursive (Vec3 bl, Vec3 tr, Vec3 delta, Int3 bl2, Int3 tr2, SceneSim::Column * column, Result * result)
	{
		const float pixel = 1.0f / 16.0f;

		const bool isSingleCell = (tr2 - bl2 == Int3 (1, 1, 1));

		Result tempResult;
		Vec3 bl2Vec = Vec3 (bl2);
		Vec3 tr2Vec = Vec3 (tr2);

		if (isSingleCell)
		{
			const CellBuilder::Mngr * cellBuilder = column->GetRoom ()->GetCellBuilder ();
			SceneSim::Cell * cell = column->GetCells ()->GetCell (bl2);
			SceneSim::ENTCOLLISIONS entCollision = cellBuilder->GetShape (cell->type)->entCollisions;
			switch (entCollision)
			{
			case SceneSim::EC_NONE: return false;

			case SceneSim::EC_UNITCUBE: 
				if (!BoxSwept_Box (bl, tr, delta, bl2Vec, tr2Vec, &tempResult))
				{
					return false;
				}
				break;

			case SceneSim::EC_HALFSTEP: 
				if (!BoxSwept_Box (bl, tr, delta, bl2Vec, tr2Vec + Vec3 (0.0f, -0.5f, 0.0f), &tempResult))
				{
					return false;
				}	
				break;

			case SceneSim::EC_STAIR: 
				{
					bool anyCollision = false;
					anyCollision |= BoxSwept_Box (bl, tr, delta, bl2Vec, tr2Vec + Vec3 (0.0f, -0.5f, 0.0f), &tempResult);
					switch (cell->blockData)
					{
					case 0: anyCollision |= BoxSwept_Box (bl, tr, delta, bl2Vec + Vec3 (0.0f, 0.5f, 0.0f), tr2Vec + Vec3 (0.0f, 0.0f, -0.5f), &tempResult); break; // south
					case 1: anyCollision |= BoxSwept_Box (bl, tr, delta, bl2Vec + Vec3 (0.0f, 0.5f, 0.5f), tr2Vec, &tempResult); break; // north
					case 2: anyCollision |= BoxSwept_Box (bl, tr, delta, bl2Vec + Vec3 (0.0f, 0.5f, 0.0f), tr2Vec + Vec3 (-0.5f, 0.0f, 0.0f), &tempResult); break; // west
					case 3: anyCollision |= BoxSwept_Box (bl, tr, delta, bl2Vec + Vec3 (0.5f, 0.5f, 0.0f), tr2Vec, &tempResult); break; // east
					default: assert (false); break;
					}
					
					if (!anyCollision)
					{
						return false;
					}
				}
				break;

			case SceneSim::EC_FENCEPOST:
				{
					bool anyCollision = false;
					anyCollision |= BoxSwept_Box (bl, tr, delta, bl2Vec + Vec3 (pixel * 6.0f, 0.0f, pixel * 6.0f), tr2Vec + Vec3 (pixel * -6.0f, 0.0f, pixel * -6.0f), &tempResult);
					switch (cell->blockData & 0x05)
					{
					case 0x00: break;
					case 0x01: anyCollision |= BoxSwept_Box (bl, tr, delta, bl2Vec + Vec3 (pixel * 6.0f, 0.0f, pixel * 6.0f), tr2Vec + Vec3 (pixel * -6.0f, 0.0f, 0.0f), &tempResult); break;
					case 0x04: anyCollision |= BoxSwept_Box (bl, tr, delta, bl2Vec + Vec3 (pixel * 6.0f, 0.0f, 0.0f), tr2Vec + Vec3 (pixel * -6.0f, 0.0f, pixel * -6.0f), &tempResult); break;
					case 0x05: anyCollision |= BoxSwept_Box (bl, tr, delta, bl2Vec + Vec3 (pixel * 6.0f, 0.0f, 0.0f), tr2Vec + Vec3 (pixel * -6.0f, 0.0f, 0.0f), &tempResult); break;
					}
					switch (cell->blockData & 0x0a)
					{
					case 0x00: break;
					case 0x02: anyCollision |= BoxSwept_Box (bl, tr, delta, bl2Vec + Vec3 (pixel * 6.0f, 0.0f, pixel * 6.0f), tr2Vec + Vec3 (0.0f, 0.0f, pixel * -6.0f), &tempResult); break;
					case 0x08: anyCollision |= BoxSwept_Box (bl, tr, delta, bl2Vec + Vec3 (0.0f, 0.0f, pixel * 6.0f), tr2Vec + Vec3 (pixel * -6.0f, 0.0f, pixel * -6.0f), &tempResult); break;
					case 0x0a: anyCollision |= BoxSwept_Box (bl, tr, delta, bl2Vec + Vec3 (0.0f, 0.0f, pixel * 6.0f), tr2Vec + Vec3 (0.0f, 0.0f, pixel * -6.0f), &tempResult); break;
					}

					if (!anyCollision)
					{
						return false;
					}
				}
				break;

			default: assert (false);
			}
		}
		else if (!BoxSwept_Box (bl, tr, delta, bl2Vec, tr2Vec, &tempResult))
		{
			return false;
		}

		if (result->resultType != RT_NONE && result->entryTime < tempResult.entryTime)
		{
			return false;
		}

		if (isSingleCell)
		{
			*result = tempResult;
			result->resultFlags |= RF_ENTRYCOLUMN | RF_ENTRYPOS;
			result->entryPos.MoveTo (column->columnId, bl + delta * result->entryTime);
			return true;
		}

		Int3 size2 = tr2 - bl2;
		Int3 split;
		if (size2.y >= size2.x && size2.y >= size2.z) 
		{
			split = Int3 (0, size2.y / 2, 0);
		}
		else if (size2.x >= size2.z) 
		{
			split = Int3 (size2.x / 2, 0, 0);
		}
		else
		{
			split = Int3 (0, 0, size2.z / 2);
		}
		bool hasCollided = BoxSwept_Recursive (bl, tr, delta, bl2, tr2 - split, column, result);
		hasCollided |= BoxSwept_Recursive (bl, tr, delta, bl2 + split, tr2, column, result);

		return hasCollided;
	}

	bool BoxSwept_Column (Vec3 bl, Vec3 tr, Vec3 delta, SceneSim::Column * column, Result * result)
	{
		bool hasCollided = false;
		if (column->flags & SceneSim::Column::SF_NO_CELLS)
		{
			Vec3 bl2 (0.0f, bl.y, 0.0f);
			Vec3 tr2 (SceneSim::CellsPerColXz, tr.y, SceneSim::CellsPerColXz);
			if (delta.y > 0.0f)
			{
				tr2.y += delta.y;
			}
			else
			{
				bl2.y += delta.y;
			}
			//hasCollided = BoxSwept_Box (bl, tr, delta, Vec3 (0, 0, 0), Vec3 (SceneSim::CellsPerColXz, SceneSim::CellsPerColY, SceneSim::CellsPerColXz), result);
			hasCollided = BoxSwept_Box (bl, tr, delta, bl2, tr2, result);
			if (hasCollided)
			{
				result->resultFlags |= RF_ENTRYCOLUMN | RF_ENTRYPOS;
				result->entryPos.MoveTo (column->columnId, bl + delta * result->entryTime);
			}
		}
		else
		{
			hasCollided = BoxSwept_Recursive (bl, tr, delta, Int3 (0, 0, 0), column->GetSize (), column, result);
		}
		return hasCollided;
	}
}
