#include "stdafx.h"

#include "../Ld/Ld_ColumnMask.h"
#include "../SceneHelper/SceneHelper_Helper.h"
#include "../SceneSim/SceneSim_Column.h"
#include "../SceneSim/SceneSim_Room.h"

#include "Collision_BoxSwept_Box.h"
#include "Collision_BoxSwept_Column.h"
#include "Collision_BoxSwept_Room.h"
#include "Collision_Result.h"

namespace Collision
{
	const int adjColumnOffsetCounts [3] = {2, 4, 2};

	const Int3 adjColumnOffsets [][4] =
	{
		{Int3 (0, 0, 0), Int3 (0, 0, 1)}, // x
		{Int3 (0, 0, 0), Int3 (1, 0, 0), Int3 (0, 0, 1), Int3 (1, 0, 1)}, // y
		{Int3 (0, 0, 0), Int3 (1, 0, 0)}, // z
	};

	bool TestColumn (Vec3 bl, Vec3 tr, Vec3 delta, SceneSim::Room * room, Int3 columnId, Int3 startColumnId, Ld::ColumnMask * mask, Result * result)
	{
		assert (room != NULL);
		assert (result != NULL);

		const Int3 columnOffset = startColumnId - columnId;
		const Vec3 offset (
			columnOffset.x * SceneSim::CellsPerColXz,
			0,//columnOffset.y * SceneSim::CellsPerColY,
			columnOffset.z * SceneSim::CellsPerColXz);


		const Vec3 colRelBl = bl + offset;
		const Vec3 colRelTr = tr + offset;

		const Vec3 bl2 (0.0f, 0.0f, 0.0f);

		bool wasInColumnVolume = false;

		SceneSim::Column * column = room->GetColumn (columnId);
		
		if (column != NULL && mask->use [columnId.x][columnId.z])
		{
			wasInColumnVolume = BoxSwept_Column (colRelBl, colRelTr, delta, column, result);
			if (!wasInColumnVolume)
			{
				const Vec3 tr2 (SceneSim::CellsPerColXz, column->GetSize ().y, SceneSim::CellsPerColXz);
				wasInColumnVolume = BoxSwept_Box (colRelBl, colRelTr, delta, bl2, tr2, NULL);
			}
		}
		else
		{
			const Vec3 tr2 (SceneSim::CellsPerColXz, room->GetNewColumnHeight (), SceneSim::CellsPerColXz);

			if (BoxSwept_Box (colRelBl, colRelTr, delta, bl2 + Vec3 (0.0f, -128.0f, 0.0f), tr2 + Vec3 (0.0f, 128.0f, 0.0f), result))
			{
				wasInColumnVolume = true;
				result->resultFlags |= RF_ENTRYCOLUMN | RF_ENTRYPOS;
				result->entryPos.MoveTo (columnId, colRelBl + result->entryTime * delta);
			}
		}

		return wasInColumnVolume;


		//-----------------------------------------
#if 0

		SceneSim::Column * adjColumn = columnFrom;
		if (face != SceneSim::F_NONE)
		{
			adjColumn = columnFrom->GetAdjacentColumn (face);

			// if no adjacent column, treat the column as if it is entirely empty (and infinitely tall for now)
			if (adjColumn == NULL)
			{
				//return false;

				Int3 columnOffset = columnFrom->columnId + adjDir [face];
				Vec3 offset (
					columnOffset.x * SceneSim::CellsPerColXz,
					columnOffset.y * SceneSim::CellsPerColY,
					columnOffset.z * SceneSim::CellsPerColXz);

				Vec3 colRelBl = bl - offset;
				Vec3 colRelTr = tr - offset;

				Vec3 bl2 (0.0f, 0.0f, 0.0f);
				Vec3 tr2 (SceneSim::CellsPerColXz, SceneSim::CellsPerColY, SceneSim::CellsPerColXz);

				return BoxSwept_Box (colRelBl, colRelTr, delta, bl2, tr2, result);
			}
		}

		Int3 columnOffset = startColumn->columnId - columnFrom->columnId;
		if (face != SceneSim::F_NONE)
		{
			columnOffset -= adjDir [face];
		}
		Vec3 offset (
			columnOffset.x * SceneSim::CellsPerColXz,
			columnOffset.y * SceneSim::CellsPerColY,
			columnOffset.z * SceneSim::CellsPerColXz);

		Vec3 colRelBl = bl - offset;
		Vec3 colRelTr = tr - offset;

		bool collided;
		//if (adjColumn->flags & SceneSim::Column::SF_NO_CELLS)
		//{
		//	Vec3 bl2 (offset);
		//	Vec3 tr2 (offset + Vec3 (SceneSim::CellsPerColXz, SceneSim::CellsPerColY, SceneSim::CellsPerColXz));

		//	collided = BoxSwept_Box (colRelBl, colRelTr, delta, bl2, tr2, result);
		//}
		//else
		{
			collided = BoxSwept_Column (colRelBl, colRelTr, delta, adjColumn, result);
		}

		return collided;

#endif 

	}

	bool BoxSwept_Room (Vec3 bl, Vec3 tr, Vec3 delta, SceneSim::Room * room, Int3 startColumnId, Ld::ColumnMask * mask, Result * result)
	{
		// for now this only works in one axis at a time.
		assert (
			(delta.x == 0.0f && delta.y == 0.0f) ||
			(delta.x == 0.0f && delta.z == 0.0f) ||
			(delta.y == 0.0f && delta.z == 0.0f));

		if (delta.x != 0.0f || delta.z != 0.0f)
		{
			int i = 0;
		}

		SceneSim::FACES nextFace = SceneSim::F_NONE;
		nextFace = (delta.x < 0) ? SceneSim::F_W : ((delta.x > 0) ? SceneSim::F_E : nextFace);
		nextFace = (delta.y < 0) ? SceneSim::F_B : ((delta.y > 0) ? SceneSim::F_T : nextFace);
		nextFace = (delta.z < 0) ? SceneSim::F_S : ((delta.z > 0) ? SceneSim::F_N : nextFace);

		Int3 advanceBy (0, 0, 0);
		int adjColumnOffsetIdx = 0;

		switch (nextFace)
		{
		case SceneSim::F_T: advanceBy = Int3 (0, 1, 0); adjColumnOffsetIdx = 1; break;
		case SceneSim::F_B: advanceBy = Int3 (0, -1, 0); adjColumnOffsetIdx = 1; break;
		case SceneSim::F_N: advanceBy = Int3 (0, 0, 1); adjColumnOffsetIdx = 2; break;
		case SceneSim::F_E: advanceBy = Int3 (1, 0, 0); adjColumnOffsetIdx = 0; break;
		case SceneSim::F_S:	advanceBy = Int3 (0, 0, -1); adjColumnOffsetIdx = 2; break;
		case SceneSim::F_W: advanceBy = Int3 (-1, 0, 0); adjColumnOffsetIdx = 0; break;
		default: assert (false); break;
		}

		Int3 currentColumnId = startColumnId;

		while (true)
		{
			bool wasValidCheck = false;
			for (int checkIdx = 0; checkIdx < adjColumnOffsetCounts [adjColumnOffsetIdx]; ++checkIdx)
			{
				wasValidCheck |= TestColumn (bl, tr, delta, room, currentColumnId + adjColumnOffsets [adjColumnOffsetIdx][checkIdx], startColumnId, mask, result);
			}
			
			if (result->resultType != RT_NONE || !wasValidCheck)
			{
				break;
			}
			else if (advanceBy.y == 0)
			{
				currentColumnId += advanceBy;
			}
			else
			{
				break;
			}
		}

		assert (
			result->resultType == RT_NONE || 
			(result->resultFlags & RF_ENTRYPOS | RF_ENTRYCOLUMN | RF_ENTRYTIME) == (RF_ENTRYPOS | RF_ENTRYCOLUMN | RF_ENTRYTIME));

		return (result->resultType != RT_NONE);
	}
}
