#include "stdafx.h"

#include "../CellBuilder/CellBuilder_Mngr.h"
#include "../CellBuilder/CellBuilder_ShapeBase.h"
#include "../SceneSim/SceneSim_Column.h"
#include "../SceneSim/SceneSim_ColumnCells.h"
#include "../SceneSim/SceneSim_Room.h"

#include "Collision_Result.h"
#include "Collision_Segment_Cube.h"
#include "Collision_Segment_Column.h"

namespace Collision
{
	bool Segment_Column (Vec3 segPos, Vec3 segDir, SceneSim::Column * column, Result * result)
	{
		assert (column != NULL);

		Result tempResult;

		// high level test
		Int3 size = column->GetSize ();

		Vec3 bl (0.0f, 0.0f, 0.0f);
		Vec3 tr (size.x, size.y, size.z);
		bool hasCollided = Segment_Cube (segPos, segDir, bl, tr, &tempResult);

		if (column->flags & SceneSim::Column::SF_NO_CELLS)
		{
			assert (false);
			//if (hasCollided)
			//{
			//	result->resultFlags |= RF_ENTRYCOLUMN | RF_ENTRYPOS;
			//	result->entryPos.MoveTo (column->columnId, segPos + segDir * result->entryTime);
			//}
		}
		else if (hasCollided)
		{
			// detailed cube test

			const CellBuilder::Mngr * cellBuilder = column->GetRoom ()->GetCellBuilder ();
			const float pixel = 1.0f / 16.0f;

			const SceneSim::ColumnCells * cells = column->GetCells ();

			int xMin = glm::floor (glm::min (segPos.x, segPos.x + segDir.x));
			xMin = glm::max (xMin, 0);
			int xMax = glm::ceil (glm::max (segPos.x, segPos.x + segDir.x));
			xMax = glm::min (xMax, SceneSim::CellsPerColXz);

			int yMin = glm::floor (glm::min (segPos.y, segPos.y + segDir.y));
			yMin = glm::max (yMin, 0);
			int yMax = glm::ceil (glm::max (segPos.y, segPos.y + segDir.y));
			yMax = glm::min (yMax, size.y);

			int zMin = glm::floor (glm::min (segPos.z, segPos.z + segDir.z));
			zMin = glm::max (zMin, 0);
			int zMax = glm::ceil (glm::max (segPos.z, segPos.z + segDir.z));
			zMax = glm::min (zMax, SceneSim::CellsPerColXz);
			
			for (int x = xMin; x < xMax; ++x)
			{
				for (int y = yMin; y < yMax; ++y)
				{
					for (int z = zMin; z < zMax; ++z)
					{
						const SceneSim::Cell * cell = cells->GetCell (x, y, z);

						if (cell->type != 0)
						{
							SceneSim::ENTCOLLISIONS entCollision = cellBuilder->GetShape (cell->type)->entCollisions;
							Vec3 bl (x, y, z);

							bool hasCollided = false;
							switch (entCollision)
							{
							case SceneSim::EC_NONE: 
								break;

							case SceneSim::EC_UNITCUBE:
								hasCollided = Segment_Cube (segPos, segDir, bl, bl + Vec3 (1.0f, 1.0f, 1.0f), &tempResult);
								break;

							case SceneSim::EC_HALFSTEP: 
								hasCollided = Segment_Cube (segPos, segDir, bl, bl + Vec3 (1.0f, 0.5f, 1.0f), &tempResult); 
								break;

							case SceneSim::EC_STAIR:
								hasCollided = Segment_Cube (segPos, segDir, bl, bl + Vec3 (1.0f, 0.5f, 1.0f), &tempResult); 
								switch (cell->blockData)
								{
								case 0: hasCollided |= Segment_Cube (segPos, segDir, bl + Vec3 (0.0f, 0.5f, 0.0f), bl + Vec3 (1.0f, 1.0f, 0.5f), &tempResult); break; // south
								case 1: hasCollided |= Segment_Cube (segPos, segDir, bl + Vec3 (0.0f, 0.5f, 0.5f), bl + Vec3 (1.0f, 1.0f, 1.0f), &tempResult); break; // north
								case 2: hasCollided |= Segment_Cube (segPos, segDir, bl + Vec3 (0.0f, 0.5f, 0.0f), bl + Vec3 (0.5f, 1.0f, 1.0f), &tempResult); break; // west
								case 3: hasCollided |= Segment_Cube (segPos, segDir, bl + Vec3 (0.5f, 0.5f, 0.0f), bl + Vec3 (1.0f, 1.0f, 1.0f), &tempResult); break; // east
								default: assert (false); break;
								}
								break;

							case SceneSim::EC_FENCEPOST:
								hasCollided = Segment_Cube (segPos, segDir, bl + Vec3 (6.0f * pixel, 0.0f, 6.0f * pixel), bl + Vec3 (10.0f * pixel, 1.0f, 10.0f * pixel), &tempResult); 
								break;

							default: 
								assert (false);
								break;
							}

							if (hasCollided)
							{
								if (result->resultType == RT_NONE || tempResult.entryTime < result->entryTime)
								{
									*result = tempResult;
									result->resultFlags |= RF_ENTRYCELL | RF_ENTRYCOLUMN;
									result->entryCell = Int3 (x, y, z);
									result->entryPos.MoveTo (column->columnId, result->entryPos.GetColumnRelPos ());
								}
							}
						}
					}
				}
			}
		}

		return (result->resultType != RT_NONE);
	}
}
