#include "stdafx.h"

#include <limits>

#include "Collision_Segment_Cube.h"
#include "Collision_Result.h"

namespace Collision
{
	bool Segment_Cube (Vec3 segPos, Vec3 segDir, Vec3 bl, Vec3 tr, Result * result)
	{
		result->Clear ();

		float Tnear = -std::numeric_limits <float>::infinity ();
        float Tfar = +std::numeric_limits <float>::infinity ();

        // test in X
        if (segDir.x == 0.0f)
        {
            if (segPos.x < bl.x || segPos.x >= tr.x)
            {
                return false;
            }
        }
        else
        {
            bool swapped = false;
            float T1 = (bl.x - segPos.x) / segDir.x;
            float T2 = (tr.x - segPos.x) / segDir.x;
            if (T1 > T2) { float temp = T1; T1 = T2; T2 = temp; swapped = true; }
			if (T1 > Tnear) { Tnear = T1; result->entryFace = (swapped) ? SceneSim::F_E : SceneSim::F_W; }
            if (T2 < Tfar) { Tfar = T2; result->exitFace = (swapped) ? SceneSim::F_W : SceneSim::F_E; }
            if (Tnear > Tfar) { return false; }
            if (Tfar < 0.0f) { return false; }
        }

        // test in Y
        if (segDir.y == 0.0f)
        {
            if (segPos.y < bl.y || segPos.y >= tr.y)
            {
                return false;
            }
        }
        else
        {
            bool swapped = false;
            float T1 = (bl.y - segPos.y) / segDir.y;
            float T2 = (tr.y - segPos.y) / segDir.y;
            if (T1 > T2) { float temp = T1; T1 = T2; T2 = temp; swapped = true; }
            if (T1 > Tnear) { Tnear = T1; result->entryFace = (swapped) ? SceneSim::F_T : SceneSim::F_B; }
            if (T2 < Tfar) { Tfar = T2; result->exitFace = (swapped) ? SceneSim::F_B : SceneSim::F_T; }
            if (Tnear > Tfar) { return false; }
            if (Tfar < 0.0f) { return false; }
        }

        // test in Z
        if (segDir.z == 0.0f)
        {
            if (segPos.z < bl.z || segPos.z >= tr.z)
            {
                return false;
            }
        }
        else
        {
            bool swapped = false;
            float T1 = (bl.z - segPos.z) / segDir.z;
            float T2 = (tr.z - segPos.z) / segDir.z;
            if (T1 > T2) { float temp = T1; T1 = T2; T2 = temp; swapped = true; }
            if (T1 > Tnear) { Tnear = T1; result->entryFace = (swapped) ? SceneSim::F_N : SceneSim::F_S; }
            if (T2 < Tfar) { Tfar = T2; result->exitFace = (swapped) ? SceneSim::F_S : SceneSim::F_N; }
            if (Tnear > Tfar) { return false; }
            if (Tfar < 0.0f) { return false; }
        }

        assert (result->entryFace != SceneSim::F_NONE);
		
		if (Tnear < 0.0f)
		{
			Tnear = 0.0f;
		}
		else if (Tnear > 1.0f)
		{
			return false;
		}

		result->resultType = RT_COLLISION;
		result->resultFlags = RF_ENTRYPOS | RF_ENTRYTIME | RF_ENTRYDISTSQR | RF_ENTRYFACE;

		if (Tfar <= 1.0f)
		{
			result->resultFlags |= RF_EXITPOS | RF_EXITTIME | RF_EXITDISTSQR | RF_EXITFACE;

			Vec3 exitPos = segDir * Tfar;
			result->exitTime = Tfar;
			result->exitDistanceSqr = LengthSqr (exitPos);
			exitPos += segPos;
			result->exitPos.SetColRelPos (exitPos);
		}

		assert (Tnear <= Tfar);
		assert (Tnear >= 0.0f && Tnear <= 1.0f);
             
		Vec3 entryPos = segDir * Tnear;
        result->entryTime = Tnear;
		result->entryDistanceSqr = LengthSqr (entryPos);
        entryPos += segPos;
		result->entryPos.SetColRelPos (entryPos);

        return true;
    }
}
