#include "stdafx.h"

#include "../Ld/Ld_ColumnMask.h"
#include "../SceneHelper/SceneHelper_Helper.h"
#include "../SceneSim/SceneSim_Column.h"
#include "../SceneSim/SceneSim_Room.h"

#include "Collision_Result.h"
#include "Collision_Segment_Column.h"
#include "Collision_Segment_Cube.h"
#include "Collision_Segment_Room.h"

namespace Collision
{
	bool Segment_Room (Vec3 segPos, Vec3 segDir, SceneSim::Room * room, Int3 startColumnId, Ld::ColumnMask * mask, Result * result)
	{
		assert (segPos.x >= 0.0f && segPos.x < SceneSim::CellsPerColXz &&
			segPos.z >= 0.0f && segPos.z < SceneSim::CellsPerColXz);

		const float segLengthSqr = glm::length2 (segDir);

		SceneSim::Column * column = SceneHelper::GetColumnFromRoom (room, startColumnId);
		if (column == NULL || column->flags & SceneSim::Column::SF_NO_CELLS || !mask->use [startColumnId.x][startColumnId.z])
		{
			result->Clear ();
			return false;
		}

		Int3 columnSize = column->GetSize ();

		Result tempResult;

		while (true)
		{
			Vec3 offset (
				(column->columnId.x - startColumnId.x) * columnSize.x,
				(column->columnId.y - startColumnId.y) * columnSize.y,
				(column->columnId.z - startColumnId.z) * columnSize.z);

			Vec3 colRelSegPos = segPos - offset;

			if (Segment_Column (colRelSegPos, segDir, column, result))
			{
				break;
			}
			else
			{
				// column cells exist between 0 and SceneSim::CellsPerColY but we add a buffer above and 
				// below this to allow for players falling out the bottom or jumping out the top of a column

				const float lowerBound = -256.0f;
				const float upperBound = columnSize.y + 256.0f;

				assert (segPos.y >= lowerBound && segPos.y <= upperBound); // starting point is already outside the bounds?

				const Vec3 bl (0.0f, lowerBound, 0.0f);
				const Vec3 tr (SceneSim::CellsPerColXz, upperBound, SceneSim::CellsPerColXz);

				bool hasCollided = Segment_Cube (colRelSegPos, segDir, bl, tr, &tempResult);
				assert (hasCollided);

				if (tempResult.resultFlags & RF_EXITFACE)
				{
					// stop testing - has left world boundary vertically
					if (tempResult.exitFace == SceneSim::F_T || tempResult.exitFace == SceneSim::F_B)
					{
						result->resultType = RT_BOUNDARYCOLLISION;
						break;
					}

					// stop testing - has left world boundary horizontally
					column = column->GetAdjacentColumn (tempResult.exitFace);
					if (column == NULL || column->flags & SceneSim::Column::SF_NO_CELLS || !mask->use [column->columnId.x][column->columnId.z])
					{
						result->resultType = RT_BOUNDARYCOLLISION;
						break;
					}
				}
				else
				{
					break;
				}
			}
		}

		return (result->resultType != RT_NONE);
	}
}