#include "stdafx.h"

#include "../SceneFile/SceneFile_Lighting.h"
#include "../SceneSim/SceneSim_Column.h"
#include "../SceneSim/SceneSim_ColumnCells.h"
#include "../SceneSim/SceneSim_Defs.h"

#include "CsgEffect_ColumnChanges.h"
#include "CsgEffect_Sphere.h"

namespace CsgEffect
{
	Sphere::Sphere (Int3 columnId, Int3 cellId, float radius)
		: columnId (columnId),
		cellId (cellId),
		radius (radius),
		affectedColumnCount (0)
	{
		Vec3 csgCentre (cellId.x + 0.5f, cellId.y + 0.5f, cellId.z + 0.5f);

		const int colRadius = affectColumnRadiusMax;

		for (int colX = -colRadius; colX <= colRadius; ++colX)
		{
			for (int colZ = -colRadius; colZ <= colRadius; ++colZ)
			{
				// circle -> square intersection test
				//const Vec3 colCorner (colX * SceneSim::CellsPerColXz, 0.0f, colZ * SceneSim::CellsPerColXz);
				//const Vec3 colSize (SceneSim::CellsPerColXz, 0.0f, SceneSim::CellsPerColXz);

				//bool floatRes = Utils::CircleSquareIntersects (csgCentre, radius, colCorner, colSize);
				//if (floatRes)
				//{
				//	//assert (affectedColumnCount < affectColumnCountMax);

				//	//affectedColumnIds [affectedColumnCount] = columnId;
				//	//affectedColumnIds [affectedColumnCount].x += colX;
				//	//affectedColumnIds [affectedColumnCount].z += colZ;

				//	//++affectedColumnCount;
				//}

				const Int3 colCornerInt (colX * SceneSim::CellsPerColXz, 0, colZ * SceneSim::CellsPerColXz);
				const Int3 colSizeInt (SceneSim::CellsPerColXz - 1, 0, SceneSim::CellsPerColXz - 1);
				bool intRes = Utils::CircleSquareIntersectsInt (cellId, (int) radius, colCornerInt, colSizeInt);
				if (intRes)
				{
					assert (affectedColumnCount < affectColumnCountMax);

					affectedColumnIds [affectedColumnCount] = columnId;
					affectedColumnIds [affectedColumnCount].x += colX;
					affectedColumnIds [affectedColumnCount].z += colZ;

					++affectedColumnCount;
				}

				//if (intRes != floatRes)
				//{
				//	int i = 0;
				//}
			}	
		}
	}

	Sphere::~Sphere ()
	{
	}

	int Sphere::GetColumnAffectedCount () const
	{
		return affectedColumnCount;
	}

	Int3 Sphere::GetColumnIdAffected (int nth) const
	{
		assert (nth < affectedColumnCount);

		return affectedColumnIds [nth];
	}

	void Sphere::ApplyChanges (int nth, SceneSim::Column * column, const CellBuilder::Mngr * cellBuilder, ColumnChanges * changes)
	{
		assert (nth < affectedColumnCount);
		assert (column != NULL);
		assert (changes != NULL);
		assert ((column->flags & SceneSim::Column::SF_NO_CELLS) == 0);
		assert (column->columnId == affectedColumnIds [nth]);

		SceneSim::ColumnCells * cells = column->GetCells ();
		const int height = column->GetSize ().y;

		const Int3 offset (
			((columnId.x - affectedColumnIds [nth].x) * SceneSim::CellsPerColXz),
			0,
			((columnId.z - affectedColumnIds [nth].z) * SceneSim::CellsPerColXz));

		const Int3 offsetCellId (offset + cellId);
		const int radiusInt = (int) (radius);

		const int minX = glm::max (offsetCellId.x - radiusInt, 0);
		const int maxX = glm::min (offsetCellId.x + radiusInt, SceneSim::CellsPerColXz - 1);

		const int minY = glm::max (offsetCellId.y - radiusInt, 1);
		const int maxY = glm::min (offsetCellId.y + radiusInt, height - 1);

		const int minZ = glm::max (offsetCellId.z - radiusInt, 0);
		const int maxZ = glm::min (offsetCellId.z + radiusInt, SceneSim::CellsPerColXz - 1);

		const float radiusSqr = radius * radius;

		for (int x = minX; x <= maxX; ++x)
		{
			int distSqrX = (x - offsetCellId.x);
			distSqrX *= distSqrX;

			for (int z = minZ; z <= maxZ; ++z)
			{
				int distSqrZ = (z - offsetCellId.z);
				distSqrZ *= distSqrZ;

				bool touched = false;

				for (int y = minY; y <= maxY; ++y)
				{
					int distSqr = (y - offsetCellId.y);
					distSqr *= distSqr;
					distSqr += distSqrX + distSqrZ;

					if (distSqr <= radiusSqr)
					{
						SceneSim::Cell * cell = cells->GetCell (x, y, z);
						if (cell->type != changes->newCellType || (cell->type == changes->newCellType && cell->blockData != changes->newCellData))
						{
							touched = true;
							cell->type = changes->newCellType;
							cell->blockData = changes->newCellData;
							cell->lightSky = 0;
							cell->lightSrc = 0;
						}
					}
				}

				if (touched)
				{
					changes->stripsTouched [z] |= 1 << x;
				}
			}
		}

		// apply new lighting scheme
		for (int x = 0; x < SceneSim::CellsPerColXz; ++x)
		{
			for (int z = 0; z < SceneSim::CellsPerColXz; ++z)
			{
				Int2 stripId (x, z);
				SceneFile::Lighting::CalcStripSky (column, stripId, cellBuilder);
			}
		}
	}
}