#include "stdafx.h"

#include "../SceneSim/SceneSim_Column.h"

#include "Comps_Coord.h"

namespace Ents
{
	namespace Comps
	{
		Coord::Coord ()
			: columnId (0, 0, 0),
			colRelPos (0.0f, 0.0f, 0.0f)
		{
		}

		Coord::Coord (Int3 columnId, Vec3 colRelPos)
			: columnId (columnId),
			colRelPos (colRelPos)
		{
		}

		Coord::Coord (const Coord & other)
			: columnId (other.columnId),
			colRelPos (other.colRelPos)
		{
		}

		Coord::~Coord ()
		{
		}

		Int3 Coord::GetColumnId () const
		{
			return columnId;
		}

		Vec3 Coord::GetColumnRelPos () const
		{
			return colRelPos;
		}

		void Coord::Fix ()
		{
			MoveBy (Vec3 (0.0f, 0.0f, 0.0f));
		}

		Vec3 Coord::GetColumnRelPos (Int3 other) const
		{
			Vec3 offset (
				(columnId.x - other.x) * SceneSim::CellsPerColXz,
				0, //(columnId.y - other.y) * SceneSim::CellsPerColY,
				(columnId.z - other.z) * SceneSim::CellsPerColXz);

			return colRelPos + offset;
		}
		
		void Coord::SetColumnId (Int3 columnId)
		{
			this->columnId = columnId;
		}

		void Coord::SetColRelPos (Vec3 colRelPos)
		{
			this->colRelPos = colRelPos;
		}

		bool Coord::MoveTo (Int3 columnId, Vec3 colRelPos)
		{
			assert (columnId.y == 0);

			this->columnId = columnId;
			this->colRelPos = colRelPos;
			return true;
		}

		bool Coord::MoveTo (Vec3 colRelPos)
		{
			while (colRelPos.x < 0.0f)
			{
 				colRelPos.x += SceneSim::CellsPerColXz;
				--columnId.x;
			}
			while (colRelPos.x >= SceneSim::CellsPerColXz)
			{
				colRelPos.x -= SceneSim::CellsPerColXz;
				++columnId.x;
			}
			while (colRelPos.z < 0.0f)
			{
				colRelPos.z += SceneSim::CellsPerColXz;
				--columnId.z;
			}
			while (colRelPos.z >= SceneSim::CellsPerColXz)
			{
				colRelPos.z -= SceneSim::CellsPerColXz;
				++columnId.z;
			}

			this->colRelPos = colRelPos;
			return true;
		}

		bool Coord::MoveBy (Vec3 delta)
		{
			Vec3 newColRelPos = colRelPos + delta;
			return MoveTo (newColRelPos);
		}

		Coord & Coord::operator += (const Vec3 & delta)
		{
			bool result = MoveBy (delta);
			assert (result);
			return *this;
		}

		Coord & Coord::operator = (const Coord & rhs)
		{
			columnId = rhs.columnId;
			colRelPos = rhs.colRelPos;
			return *this;
		}

		Vec3 Coord::DeltaTo (const Coord * to) const
		{
			Vec3 offset (
				(columnId.x - to->columnId.x) * SceneSim::CellsPerColXz,
				0, // (columnId.y - to->columnId.y) * SceneSim::CellsPerColY,
				(columnId.z - to->columnId.z) * SceneSim::CellsPerColXz);

			Vec3 delta = offset;
			delta += colRelPos - to->colRelPos;

			return delta;
		}

		float Coord::DistSqr (const Coord * to) const
		{
			Vec3 deltaTo = DeltaTo (to);
			return (deltaTo.x * deltaTo.x + deltaTo.y * deltaTo.y + deltaTo.z * deltaTo.z);
		}
	}
}
