#include "stdafx.h"

#include "../Collision/Collision_BoxSwept_Room.h"
#include "../Collision/Collision_Result.h"
#include "../SceneHelper/SceneHelper_Helper.h"
#include "../SceneSim/SceneSim_Room.h"

#include "Comps_Coord.h"
#include "Comps_Motion.h"
#include "Comps_Orient.h"

namespace Ents
{
	namespace Comps
	{
		Motion::Motion (Coord * coord)
			: coord (coord),
			orient (NULL),
			velocity (0.0f, 0.0f, 0.0f),
			size (0.0f, 0.0f, 0.0f),
			intersectionTimes (-1.0f, -1.0f, -1.0f)
		{
		}

		Motion::Motion (Coord * coord, Orient * orient)
			: coord (coord),
			orient (orient),
			velocity (0.0f, 0.0f, 0.0f),
			size (0.0f, 0.0f, 0.0f),
			intersectionTimes (-1.0f, -1.0f, -1.0f)
		{
		}

		Motion::~Motion ()
		{
		}

		Vec3 Motion::GetVelocity () const
		{
			return velocity;
		}

		void Motion::SetVelocity (Vec3 velocity)
		{
			this->velocity = velocity;
		}

		void Motion::SetSize (Vec3 size)
		{
			this->size = size;
		}

		Vec3 Motion::GetSize () const
		{
			return size;
		}

		int Motion::AttemptMotion (float delta, SceneSim::Room * room, int directions)
		{
			assert (false);

			int results = 0;
			if (directions & MD_Y)
			{
				intersectionTimes.y = AttemptSimpleMotion (room, Vec3 (0.0f, velocity.y * delta, 0.0f), NULL);
				if (intersectionTimes.y != -1.0f)
				{
					intersectionTimes.y /= delta;
					velocity.y = 0.0f;
					results |= MR_BLOCKEDY;
				}
				//Vec3 columnPos = coord->GetColumnRelPos ();
				//if (columnPos.y < -128.0f)
				//{
				//	columnPos.y = 128.0f;
				//	coord->SetColRelPos (columnPos);
				//}
			}

			Vec4 xzMotion (velocity.x * delta, 0.0f, velocity.z * delta, 0.0f);
			if (orient != NULL)
			{
				Mat44 rotation;
				orient->GetYawMat (&rotation);
				xzMotion = xzMotion * rotation;
			}

			if (directions & MD_X)
			{
				intersectionTimes.x = AttemptSimpleMotion (room, Vec3 (xzMotion.x, 0.0f, 0.0f), NULL);
				if (intersectionTimes.x != -1.0f)
				{
					intersectionTimes.x /= delta;
					velocity.x = 0.0f;
					results |= MR_BLOCKEDX;
				}
			}
            if (directions & MD_Z)
            {
				intersectionTimes.z = AttemptSimpleMotion (room, Vec3 (0.0f, 0.0f, xzMotion.z), NULL);
				if (intersectionTimes.z != -1.0f)
				{
					intersectionTimes.z /= delta;
					velocity.z = 0.0f;
					results |= MR_BLOCKEDZ;
				}
            }

			return results;
		}

		int Motion::AttemptMotionWithClimbing (float delta, SceneSim::Room * room, int directions, float maxClimbHeight, Ld::ColumnMask * mask)
		{	
			assert (directions == (MD_X | MD_Y | MD_Z)); // does this work when not including all 3 directions?

			// find earliest collision in all 3 directions.
			Comps::Coord tempCoord (*coord);
			Comps::Motion tempMotion (&tempCoord, orient);
			tempMotion.SetVelocity (velocity);
			tempMotion.SetSize (GetSize ());

			Vec3 timesRemaining (1.0f, 1.0f, 1.0f);
			Vec3 timesRecent (0.0f, 0.0f, 0.0f);

			// do Y velocity once (nasty but good enough)
			int results = 0;
			int prevResults = 0;
			timesRecent.y = tempMotion.AttemptSimpleMotion (room, Vec3 (0.0f, velocity.y * delta, 0.0f), mask);
			if (timesRecent.y != -1.0f)
			{
				velocity.y = 0.0f;
				tempMotion.SetVelocity (velocity);
				results |= MR_BLOCKEDY;
			}

			Vec4 targetXzMotion (velocity.x * delta, 0.0f, velocity.z * delta, 0.0f);
			if (orient != NULL)
			{
				Mat44 rotation;
				orient->GetYawMat (&rotation);
				targetXzMotion = targetXzMotion * rotation;
			}

			Vec4 initialXzMotion = targetXzMotion;
			
			bool isAttemptingStep = false;
			while (true)
			{
				results = results & MR_BLOCKEDY; // clear xz flags

				// test x
				if (timesRecent.x != -1.0f)
				{
					timesRecent.x = tempMotion.AttemptSimpleMotion (room, Vec3 (targetXzMotion.x, 0.0f, 0.0f), mask);
					if (timesRecent.x != -1.0f)
					{
						timesRemaining.x -= timesRecent.x;
						targetXzMotion.x = initialXzMotion.x * timesRemaining.x;
						results |= MR_BLOCKEDX;
					}
					else 
					{
						timesRemaining.x = 0.0f;
					}
				}

				// test z
				if (timesRecent.z != -1.0f)
				{
					timesRecent.z = tempMotion.AttemptSimpleMotion (room, Vec3 (0.0f, 0.0f, targetXzMotion.z), mask);
					if (timesRecent.z != -1.0f)
					{
						timesRemaining.z -= timesRecent.z;
						targetXzMotion.z = initialXzMotion.z * timesRemaining.z;
						results |= MR_BLOCKEDZ;
					}
					else 
					{
						timesRemaining.z = 0.0f;
					}
				}

				// if both are satisifed then use last known good position
				if (timesRecent.x == -1.0f && timesRecent.z == -1.0f)
				{
					coord->MoveTo (tempCoord.GetColumnId (), tempCoord.GetColumnRelPos ());
					SetVelocity (Vec3 (0.0f, tempMotion.GetVelocity ().y, 0.0f));
					break;
				}

				if (timesRecent.x != -1.0f && timesRecent.z != -1.0f)
				{
					if (isAttemptingStep)
					{
						if (prevResults == results)
						{
							break;
						}
					}
					else
					{
						coord->MoveTo (tempCoord.GetColumnId (), tempCoord.GetColumnRelPos ());
						SetVelocity (Vec3 (0.0f, tempMotion.GetVelocity ().y, 0.0f));
						tempCoord.MoveBy (Vec3 (0.0f, maxClimbHeight, 0.0f));
						isAttemptingStep = true;
					}
				}

				if ((timesRecent.x == -1.0f && timesRecent.z != -1.0f) || 
					(timesRecent.z == -1.0f && timesRecent.x != -1.0f))
				{
					if (isAttemptingStep)
					{
						if (prevResults == results)
						{
							break;
						}
					}

					coord->MoveTo (tempCoord.GetColumnId (), tempCoord.GetColumnRelPos ());
					SetVelocity (Vec3 (0.0f, tempMotion.GetVelocity ().y, 0.0f));
					if (prevResults == results)
					{
						tempCoord.MoveBy (Vec3 (0.0f, maxClimbHeight, 0.0f));
						isAttemptingStep = true;
					}
				}

				prevResults = results;
			}

			return results;
		}

		Vec3 Motion::GetAttemptMotionIntersectTimes () const
		{
			assert (false); // are these ever used?
			return intersectionTimes;
		}

        float Motion::AttemptSimpleMotion (SceneSim::Room * room, Vec3 simpleVel, Ld::ColumnMask * mask) const
        {
			if (coord->GetColumnRelPos ().y < 63.0f &&
				(coord->GetColumnRelPos ().y + simpleVel.y) >= 63.0f)
			{
				int i = 0;
			}

			float length = glm::length2 (simpleVel);
			if (length == 0.0f)
			{
                return -1.0f;
            }

			Collision::Result result;
			Vec3 xyz = coord->GetColumnRelPos ();

			if (Collision::BoxSwept_Room (xyz, xyz + size, simpleVel, room, coord->GetColumnId (), mask, &result))
            {
				if (simpleVel.y != 0.0f)
				{
					int i = 0;
				}

                float epsilon = 0.0001f;
				
				if (result.entryTime > 0.0f)
				{
					assert ((result.resultFlags & Collision::RF_ENTRYCOLUMN) != 0);

					simpleVel = glm::normalize (simpleVel);
					coord->MoveTo (result.entryPos.GetColumnId (), result.entryPos.GetColumnRelPos () + simpleVel * -epsilon);
					coord->Fix ();
				}
				else
				{
					assert (result.entryTime == 0.0f);
				}
                return result.entryTime;
            }
            else
            {
				if (simpleVel.y != 0.0f)
				{
					int i = 0;
				}

				coord->MoveBy (simpleVel);
                return -1.0f;
            }
        }
	}
}
