#include "stdafx.h"

#include "../Collision/Collision_Segment_Room.h"
#include "../Collision/Collision_Result.h"
#include "../SceneHelper/SceneHelper_Helper.h"
#include "../SceneSim/SceneSim_Room.h"

#include "Comps_Coord.h"
#include "Comps_MotionRay.h"
#include "Comps_Orient.h"

namespace Ents
{
	namespace Comps
	{
		MotionRay::MotionRay (Coord * coord)
			: coord (coord),
			orient (NULL),
			velocity (0.0f, 0.0f, 0.0f)
		{
			result = aah_new Collision::Result ();
		}

		MotionRay::MotionRay (Coord * coord, Orient * orient)
			: coord (coord),
			orient (orient),
			velocity (0.0f, 0.0f, 0.0f)
		{
			result = aah_new Collision::Result ();
			assert (false); // what does adding the orient mean when motion has velocity
		}

		MotionRay::~MotionRay ()
		{
			delete result;
		}

		Vec3 MotionRay::GetVelocity () const
		{
			return velocity;
		}

		void MotionRay::SetVelocity (Vec3 velocity)
		{
			this->velocity = velocity;
		}

		bool MotionRay::AttemptMotion (float timeStep, SceneSim::Room * room, Ld::ColumnMask * mask)
		{
			result->Clear ();

			SceneSim::Column * column = SceneHelper::GetColumnFromRoom (room, coord->GetColumnId ());
			if (column == NULL)
			{
				return true;
			}

			if (Collision::Segment_Room (coord->GetColumnRelPos (), velocity * timeStep, room, coord->GetColumnId (), mask, result))
			{
				assert (result->entryTime >= 0.0f && result->entryTime <= 1.0f);

				*coord = result->entryPos;
				result->entryTime /= timeStep;
				if (result->resultFlags & Collision::RF_EXITTIME)
				{
					result->exitTime /= timeStep;
				}

                float epsilon = 0.0001f;
				Vec3 pushBack = glm::normalize (velocity) * epsilon;
				coord->MoveBy (-pushBack);
			}
			else
			{
				coord->MoveBy (velocity * timeStep);
			}

			return (result->resultType != Collision::RT_NONE);
		}

		const Collision::Result * MotionRay::GetMotionResult () const
		{
			return result;
		}
	}
}
