#include "stdafx.h"

#include "../Gfx/Gfx_Camera.h"

#include "Comps_Coord.h"
#include "Comps_Orient.h"

namespace Ents
{
	namespace Comps
	{
		Orient::Orient (Coord * coord, Vec3 ypr)
			: coord ( coord),
			ypr (ypr)
		{
		}

		Orient::~Orient ()
		{
		}

		void Orient::TurnYawBy (float yawBy)
		{
			ypr.y += yawBy;
		}

		void Orient::TurnPitchBy (float pitchBy)
		{
			ypr.x += pitchBy;
		}

		void Orient::ClampPitch ()
		{
			ypr.x = glm::clamp (ypr.x, -90.0f, 90.0f);
		}

		void Orient::SetYpr (Vec3 ypr)
		{
			this->ypr = ypr;
		}

		Vec3 Orient::GetYpr () const
		{
			return ypr;
		}

		void Orient::GetYawMat (Mat44 * result) const
		{
			*result = glm::rotate (Mat44 (), ypr.y, Vec3 (0.0f, 1.0f, 0.0f));
		}

		void Orient::GetYprMat (Mat44 * result) const
		{
			Mat44 mat;
			mat = glm::rotate (mat, ypr.x, Vec3 (1.0f, 0.0f, 0.0f));
			mat = glm::rotate (mat, ypr.y, Vec3 (0.0f, 1.0f, 0.0f));
			*result = mat;
		}

		void Orient::SetCamera (Gfx::Camera * camera, Vec3 offset) const
		{
			Comps::Coord tempCoord (*coord);
			tempCoord.MoveBy (offset);

			camera->xyz = tempCoord.GetColumnRelPos ();
			camera->ypr = ypr;
			camera->Apply ();
		}
	}
}
