#include "stdafx.h"

#include "Input_Keys.h"

namespace Input
{
	bool Keys::isRegularInputKey (SDLKey key)
	{
		return (
			key != SDLK_CAPSLOCK && 
			key != SDLK_MENU && 
			key != SDLK_LSUPER && 
			key != SDLK_RSUPER &&
			key != SDLK_PRINT && 
			key != SDLK_PAUSE && 
			key != SDLK_BREAK && 
			key != SDLK_NUMLOCK &&
			key != SDLK_RMETA &&
			key != SDLK_LMETA &&
			key != SDLK_MODE &&
			key != SDLK_COMPOSE &&
			key != SDLK_SYSREQ &&
			key != SDLK_POWER);
	}

	Keys::Keys ()
	{
		keys [0][0] = keys [0][1] = 0;
		keysRecorded [0] = keysRecorded [1] = 0;
		keysConsumed [0] = keysConsumed [1] = 0;
		keysRecordBufferIdx = 0;

		Uint8 * buffer = SDL_GetKeyState (&stateLength);
		down = aah_new char [stateLength];
		pressed = aah_new char [stateLength];
		released = aah_new char [stateLength];
		timeDown = aah_new int [stateLength];

		memset (down, 0, stateLength);
		memset (pressed, 0, stateLength);
		memset (released, 0, stateLength);
		memset (timeDown, 0, sizeof (int) * stateLength);
	}

	Keys::~Keys ()
	{
		delete [] timeDown;
		delete [] released;
		delete [] pressed;
		delete [] down;
	}

	bool Keys::IsDown (SDLKey sym) const
	{
		return (down [sym] == 1);
	}

	bool Keys::IsPressed (SDLKey sym) const
	{
		return (pressed [sym] > 0);
	}

	bool Keys::IsPressedOrRepeated (SDLKey sym) const
	{
		if (pressed [sym])
		{
			return true;
		}
		else if (down [sym])
		{
			if (timeDown [sym] - 60 > 0)
			{
				if (((timeDown [sym] - 60) % 30) == 0)
				{
					return true;
				}
			}
		}
		return false;
	}

	bool Keys::IsRelease (SDLKey sym) const
	{
		return (released [sym] > 0);
	}

	Uint16 Keys::ConsumeTextEntry ()
	{
		if (keysConsumed [1 - keysRecordBufferIdx] == keysRecorded [1 - keysRecordBufferIdx])
		{
			return 0;
		}
		++keysConsumed [1 - keysRecordBufferIdx];
		return keys [1 - keysRecordBufferIdx][keysConsumed [1 - keysRecordBufferIdx] - 1];
	}

	void Keys::Update ()
	{
		keysRecordBufferIdx = 1 - keysRecordBufferIdx;
		keysRecorded [keysRecordBufferIdx] = 0;
		keysConsumed [keysRecordBufferIdx] = 0;

		for (int idx = 0; idx < stateLength; ++idx)
		{
			if (down [idx])
			{
				++timeDown [idx];
			}

			if (pressed [idx] > 0)
			{
				--pressed [idx];
			}
			if (released [idx] > 0)
			{
				--released [idx];
			}
		}
 	}

	void Keys::RecordKeyPressed (SDLKey sym, SDLMod mod, Uint16 unicode)
	{
		if (sym != SDLK_UNKNOWN && sym != SDLK_LSHIFT)
		{
			if (unicode == SDLK_RETURN || unicode == SDLK_BACKSPACE || (unicode >= SDLK_SPACE && unicode <= SDLK_z))
			{
				if (keysRecorded [keysRecordBufferIdx] < keysLength + 1)
				{
					keys [keysRecordBufferIdx][keysRecorded [keysRecordBufferIdx]] = unicode;
					++keysRecorded [keysRecordBufferIdx];
				}
			}
		}

		if (down [sym] == 0)
		{
			down [sym] = 1;
			pressed [sym] = 2;
		}
	}

	void Keys::RecordKeyReleased (SDLKey sym, SDLMod mod, Uint16 unicode)
	{
		down [sym] = 0;
		released [sym] = 2;
		timeDown [sym] = 0;
	}
}