#include "stdafx.h"

#include "SDL/SDL-1.2.14 sdk/include/SDL.h" // main SDL header
#include "SDL/SDL-1.2.14 sdk/include/SDL_syswm.h"

#include "Input_Mouse.h"

 extern "C" {

	Uint32 SDL_eventstate;
	Uint8 SDL_ProcessEvents[SDL_NUMEVENTS];

	extern int SDL_PrivateMouseMotion(Uint8 buttonstate, int relative, Sint16 x, Sint16 y);
 }

namespace Input
{
	Mouse::Mouse ()
		: WheelDelta (0),
		pos (0, 0),
		delta (0, 0),
		isAbsPositionValid (false),
		ignoreNextInput (false),
		nextWheelDelta (0)
	{
		SDL_WM_GrabInput (SDL_GRAB_ON);

		isAppFocused = true;
		isFpsMouse = false;
	}

	void Mouse::SetInputFocus (bool isAppFocused)
	{
		if (!this->isAppFocused && isAppFocused)
		{
			Left.SetIsOn (false, 0.0f);
			Middle.SetIsOn (false, 0.0f);
			Right.SetIsOn (false, 0.0f);
			ignoreNextInput = true;
			isAbsPositionValid = false;
		}

		this->isAppFocused = isAppFocused;

		ChangeMouse ();
	}

	void Mouse::SetIsFpsMouse (bool useFpsMouse)
	{
		if (useFpsMouse && !isFpsMouse)
		{
			Left.SetIsOn (false, 0.0f);
			Middle.SetIsOn (false, 0.0f);
			Right.SetIsOn (false, 0.0f);
			ignoreNextInput = true;
			isAbsPositionValid = false;
		}

		isFpsMouse = useFpsMouse;

		ChangeMouse ();
	}

	bool Mouse::GetIsFpsMouse () const
	{
		return isFpsMouse;
	}

	void Mouse::ReportMouseButton (int idx, bool isDown)
	{
		if (!isAppFocused || ignoreNextInput)
		{
			return;
		}

		switch (idx)
		{
		case 0: Left.SetIsOn (isDown, 0.0f); break;
		case 1: Right.SetIsOn (isDown, 0.0f); break;
		case 2: Middle.SetIsOn (isDown, 0.0f); break;
		default: assert (false); break;
		}
	}

	void Mouse::ReportWheel (bool isUpClick)
	{
		nextWheelDelta += (isUpClick) ? -1 : +1;
	}

	bool Mouse::GetIsAbsoluteValid () const
	{
		return isAbsPositionValid;
	}

	Int2 Mouse::GetAbsolute () const
	{
		return pos;
	}

	Int2 Mouse::GetDelta () const
	{
		return delta;
	}

	void Mouse::Update ()
	{
		Left.Update ();
		Middle.Update ();
		Right.Update ();

		POINT p;

		SDL_SysWMinfo info;
		SDL_VERSION (&info.version);
		int result = SDL_GetWMInfo (&info);
		assert (result == 1);

		if (ignoreNextInput)
		{
			WheelDelta = 0;
		}
		else
		{
			WheelDelta = nextWheelDelta;
		}
		nextWheelDelta = 0;

		if (isFpsMouse && isAppFocused)
		{
			RECT rect;
			GetWindowRect (info.window, &rect);
			Int2 windowCentre ((rect.left + rect.right) / 2, (rect.top + rect.bottom) / 2);

			if (ignoreNextInput)
			{
				ignoreNextInput = false;
				delta.x = delta.y = 0;
			}
			else
			{
				GetCursorPos (&p);
				pos.x = windowCentre.x - p.x;
				pos.y = windowCentre.y - p.y;

				delta.x = pos.x;
				delta.y = pos.y;
				pos.x = pos.y = 0;
			}
	
			SetCursorPos (windowCentre.x, windowCentre.y);
		}
		else
		{
			GetCursorPos (&p);

			ScreenToClient (info.window, &p); 

			pos.x = p.x;
			pos.y = p.y;
			delta.x = delta.y = 0;

			RECT rect;
			GetClientRect (info.window, &rect);
			isAbsPositionValid = (isAppFocused && (p.x >= 0 && p.x < rect.right && p.y >= 0 && p.y < rect.bottom));

			if (ignoreNextInput)
			{
				ignoreNextInput = false;
				isAbsPositionValid = false;
			}
		}
	}

	void Mouse::ChangeMouse ()
	{
		if (isAppFocused)
		{
			SDL_ShowCursor (!isFpsMouse);
		}
		else
		{
			SDL_ShowCursor (true);
		}
	}
}
