#include "stdafx.h"

#include "Noise_Perlin.h"
#include "Noise_Random.h"

namespace Noise
{
	float Perlin2d (const Seed & seed, float x, float y, const PerlinParams * params)
	{
		assert (params != NULL);
		assert (params->scale != 0.0f);

		float value = 0.0f;
		float frequency = 1.0f / params->scale;
		float amplitude = 0.5f;

		for (int octave = 0; octave < params->octavesMax; ++octave)
		{
			value += RandomInterpolated (seed, x * frequency, y * frequency) * amplitude;

			frequency *= params->frequencyPerOctave;
			amplitude *= params->amplitudePerOctave;
		}

		return value;
	}

	float Perlin2d (const Seed & seed, float x, float y, float * octaveWeights)
	{
		assert (octaveWeights);

		float value = 0.0f;
		float frequency = 1.0f;
		int weightIdx = 0;
 
		while (octaveWeights [weightIdx] != -1.0f)
		{
			value += RandomInterpolated (seed, x * frequency, y * frequency) * octaveWeights [weightIdx];

			frequency *= 2.0f;
			++weightIdx;
		}

		return value;
	}

	float Perlin3d (const Seed & seed, float x, float y, float z, const PerlinParams * params)
	{
		assert (params != NULL);
		assert (params->scale != 0.0f);

		float value = 0.0f;
		float frequency = 1.0f / params->scale;
		float amplitude = 0.5f;

		for (int octave = 0; octave < params->octavesMax; ++octave)
		{
			value += RandomInterpolated (seed, x * frequency, y * frequency, z * frequency) * amplitude;

			frequency *= params->frequencyPerOctave;
			amplitude *= params->amplitudePerOctave;
		}

		return value;
	}
}