#include "stdafx.h"

#include "Noise_Random.h"

namespace Noise
{
	//static int seed = 0xa478ba321;

	int RandomInt (const Seed & seed, int x)
	{
		x ^= seed.s;
		x = (x << 13) ^ x;
		x = (x * (x * x * 15371 + 789221) + 1376312589) & 0x7fffffff;
		return x;
	}

	float Random (const Seed & seed, int x)
	{
		x ^= seed.s;
		x = (x << 13) ^ x;
		x = (x * (x * x * 15371 + 789221) + 1376312589) & 0x7fffffff;
		float val = (1.0f - ((float) x / 1073741824.0f));
		assert (val >= -1.0f && val <= 1.0f);
		return val;
	}

	float Random (const Seed & seed, int x, int y)
	{
		x ^= seed.s;
		y ^= seed.s;
		int n = x * 13 + y * 57;
		n = (n << 13) ^ n;
		n = (n * (n * n * 15371 + 789221) + 1376312589) & 0x7fffffff;
		return (1.0f - ((float) n / 1073741824.0f));
	}

	float Random (const Seed & seed, int x, int y, int z)
	{
		x ^= seed.s;
		y ^= seed.s;
		z ^= seed.s;
		int n = x * 13 + y * 57 + z * 7;
		n = (n << 13) ^ n;
		n = (n * (n * n * 15371 + 789221) + 1376312589) & 0x7fffffff;
		return (1.0f - ((float) n / 1073741824.0f));
	}

	float RandomSmoothed (const Seed & seed, int x, int y)
	{
		float corners = (Random (seed, x - 1, y - 1) + Random (seed, x + 1, y - 1) + Random (seed, x - 1, y + 1) + Random (seed, x + 1, y + 1)) / 16.0f;
		float sides = (Random (seed, x - 1, y) + Random (seed, x + 1, y) + Random (seed, x, y - 1) + Random (seed, x, y + 1)) / 8.0f;
		float centre = Random (seed, x, y) / 4.0f;
		return corners + sides + centre;
	}

	float RandomSmoothed (const Seed & seed, int x, int y, int z)
	{
		float a = RandomSmoothed (seed, x, y);
		float b = RandomSmoothed (seed, x, z);
		float c = RandomSmoothed (seed, y, z);

		return (a + b + c) / 3.0f;
	}

	float Interpolate (float a, float b, float t)
	{
		return a * (1.0f - t) + b * t;
	}

	float RandomInterpolated (const Seed & seed, float x, float y)
	{
		int intX = (int) glm::floor (x);
		float fracX = x - intX;
		assert (fracX >= 0.0f);

		int intY = (int) glm::floor (y);
		float fracY = y - intY;
		assert (fracY >= 0.0f);

		float v1 = RandomSmoothed (seed, intX, intY);
		float v2 = RandomSmoothed (seed, intX + 1, intY);
		float v3 = RandomSmoothed (seed, intX, intY + 1);
		float v4 = RandomSmoothed (seed, intX + 1, intY + 1);

		float i1 = Interpolate (v1, v2, fracX);
		float i2 = Interpolate (v3, v4, fracX);

		return Interpolate (i1, i2, fracY);
	}

	float RandomInterpolated (const Seed & seed, float x, float y, float z)
	{
		int intX = (int) glm::floor (x);
		float fracX = x - intX;
		assert (fracX >= 0.0f);

		int intY = (int) glm::floor (y);
		float fracY = y - intY;
		assert (fracY >= 0.0f);

		int intZ = (int) glm::floor (z);
		float fracZ = z - intZ;
		assert (fracZ >= 0.0f);

		float v1 = Random (seed, intX, intY, intZ);
		float v2 = Random (seed, intX + 1, intY, intZ);
		float v3 = Random (seed, intX, intY + 1, intZ);
		float v4 = Random (seed, intX + 1, intY + 1, intZ);

		float v5 = Random (seed, intX, intY, intZ + 1);
		float v6 = Random (seed, intX + 1, intY, intZ + 1);
		float v7 = Random (seed, intX, intY + 1, intZ + 1);
		float v8 = Random (seed, intX + 1, intY + 1, intZ + 1);

		float i1 = Interpolate (v1, v2, fracX);
		float i2 = Interpolate (v3, v4, fracX);

		float i3 = Interpolate (v5, v6, fracX);
		float i4 = Interpolate (v7, v8, fracX);

		float i5 = Interpolate (i1, i2, fracY);
		float i6 = Interpolate (i3, i4, fracY);

		return Interpolate (i5, i6, fracZ);
	}

	float EaseInEaseOut (float in)
	{
		assert (in >= 0.0f && in <= 1.0f);
		return 3 * in * in - 2 * in * in * in;
	}
}
