#include "stdafx.h"

#include "../SceneSim/SceneSim_Column.h"

#include "SceneCollections_LocalColumns.h"

namespace SceneCollections
{
	LocalColumns::LocalColumns (Int3 centreColumnId, int radius)
		: topLeftColumnId (centreColumnId - Int3 (radius, 0, radius)),
		dims (radius * 2 + 1),
		columns (NULL)
	{
		columns = aah_new SceneSim::Column * [dims * dims];
		memset (columns, 0, sizeof (SceneSim::Column *) * dims * dims);
	}

	LocalColumns::LocalColumns (const LocalColumns & copy)
		: topLeftColumnId (copy.topLeftColumnId),
		dims (copy.dims),
		columns (NULL)
	{
		columns = aah_new SceneSim::Column * [dims * dims];
		memcpy (columns, copy.columns, sizeof (SceneSim::Column *) * dims * dims);
	}

	LocalColumns::~LocalColumns ()
	{
		delete [] columns;
	}

	void LocalColumns::DeleteAll ()
	{
		for (int idx = 0; idx < (dims * dims); ++idx)
		{
			delete columns [idx];
			columns [idx] = NULL;
		}
	}

	void LocalColumns::AddColumn (SceneSim::Column * column)
	{
		AAH_ASSERT ((column != NULL));
		
		columns [CalcArrayIndex (column->columnId)] = column;
	}

	const SceneSim::Column * LocalColumns::GetColumn (Int3 columnId) const
	{
		return columns [CalcArrayIndex (columnId)];
	}

	SceneSim::Column * LocalColumns::GetColumn (Int3 columnId)
	{
		return columns [CalcArrayIndex (columnId)];
	}

	int LocalColumns::GetMaxColumnsCount () const
	{
		return dims * dims;
	}

	SceneSim::Column * LocalColumns::GetNthColumn (int idx)
	{
		AAH_ASSERT ((idx >= 0 && idx < GetMaxColumnsCount ()));
		return columns [idx];
	}

	int LocalColumns::CalcArrayIndex (Int3 columnId) const
	{
		AAH_ASSERT ((columnId.x - topLeftColumnId.x >= 0 && columnId.x - topLeftColumnId.x < dims));
		AAH_ASSERT ((columnId.z - topLeftColumnId.z >= 0 && columnId.z - topLeftColumnId.z < dims));

		int idx = (columnId.x - topLeftColumnId.x) + (columnId.z - topLeftColumnId.z) * dims;
		return idx;
	}
}
