#include "stdafx.h"

#include "File/File_PakFileCompressed.h"
#include "Utils/Utils_Compress.h"

#include "../SceneSim/SceneSim_Column.h"
#include "../SceneSim/SceneSim_ColumnCells.h"

#include "Area_AreaInstance.h"

namespace SceneFile
{
	namespace Area
	{
		AreaInstance::AreaInstance (TextRef fileFolder, AreaId areaId)
			: areaId (areaId)
		{
			pakFile = aah_new File::PakFileCompressed (fileFolder + FileLeafFromAreaId (areaId), ColumnsPerAreaXz * ColumnsPerAreaXz);
		}

		AreaInstance::~AreaInstance ()
		{
			delete pakFile;
		}

		AreaId AreaInstance::GetId () const
		{
			return areaId;
		}

		bool AreaInstance::IsColumnLoadable (Int3 columnId) const
		{
			return pakFile->DoesRecordExist (RecordIdxFromColumnId (columnId));
		}

		void AreaInstance::LoadColumn_BaseData (SceneSim::Column * column)
		{
			assert (column != NULL);
			assert (IsColumnLoadable (column->columnId));

			int saveSize;
			void * saveData = column->GetCells ()->GetRawDataToSave (&saveSize);

			RakNet::Time lastModified;
			const int size = pakFile->LoadRecord (RecordIdxFromColumnId (column->columnId), &lastModified, saveData, saveSize);
			assert (size == saveSize);

			column->SetLastModifiedTime (lastModified);
		}

		void AreaInstance::SaveColumn (SceneSim::Column * column)
		{
			assert (column != NULL);

			int saveSize;
			void * saveData = column->GetCells ()->GetRawDataToSave (&saveSize);

			const int compressionLevel = 9;
			pakFile->SaveRecord (RecordIdxFromColumnId (column->columnId), &column->GetLastModifiedTime (), saveData, saveSize, compressionLevel);
		}

		Text AreaInstance::FileLeafFromAreaId (AreaId areaId)
		{
			Text fileLeaf = Text::Format ("area_%+08d%+08d.bin", areaId.x, areaId.z);
			return fileLeaf;
		}

		int AreaInstance::RecordIdxFromColumnId (Int3 columnId) const
		{
			assert (columnId.y == 0);

			columnId.x -= areaId.x * ColumnsPerAreaXz;
			columnId.z -= areaId.z * ColumnsPerAreaXz;
			assert (columnId.x >= 0 && columnId.x < ColumnsPerAreaXz && columnId.z >= 0 && columnId.z < ColumnsPerAreaXz);

			return (columnId.x + columnId.z * ColumnsPerAreaXz);
		}
	}
}