#include "stdafx.h"

#include "Lua/Lua_LuaState.h"

#include "../SceneGen/Gen00_Room.h"
#include "../SceneGen/Ld00_Room.h"
#include "../SceneHelper/SceneHelper_Collections.h"
#include "../SceneHelper/SceneHelper_SimEvents.h"
#include "../SceneHelper/SceneHelper_SimMngr.h"
#include "../SceneSim/SceneSim_Room.h"
#include "../SceneSim/SceneSim_World.h"
#include "../SceneSim/SceneSim_WorldInit.h"

#include "Factory_RoomLoadJob.h"
#include "SceneFile_OperationsRoom.h"

namespace SceneFile
{
	namespace Factory
	{
		RoomLoadJob::RoomLoadJob (
			Thread::JobMngr * jobMngr, 
			SceneSim::WorldInit * worldInit,
			TextRef roomId,
			SceneHelper::Collections::RoomList * roomsLoading, 
			OperationsRoom * * roomOps, 
			SceneSim::World * world,
			SceneHelper::SimEvents * events)

		: Thread::Job (jobMngr, true),
			worldInit (worldInit),
			room (NULL),
			jobMngr (jobMngr),
			roomsLoading (roomsLoading),
			roomOps (roomOps),
			world (world),
			events (events)
		{
			assert (worldInit != NULL);
			assert (roomsLoading != NULL);
			assert (roomOps != NULL);
			assert (*roomOps == NULL);
			assert (world != NULL);
			assert (events != NULL);

			room = aah_new SceneSim::Room (roomId);

			roomsLoading->Add (room);
		}

		RoomLoadJob::~RoomLoadJob ()
		{
		}

		Thread::JOBSTATES RoomLoadJob::JobDoWork ()
		{
			assert (room->GetWorld () == NULL);

			if (worldInit->isNew)
			{
				*roomOps = aah_new SceneGen::Ld_00::Ld_00_Room (room, jobMngr, world, events);
				SceneFile::CreateResults createResults = (*roomOps)->CreateRoom (worldInit);
				assert (createResults == SceneFile::CR_SUCCESS);
			}
			else
			{
				room->LoadLua (world->GetGameFolder () + "data/rooms/" + room->roomId + "/room.lua");
				Lua::LuaState * lua = room->GetLua ();

				Text generationType = lua->ReadString ("room/terrain/primary_generation/type");
				*roomOps = aah_new SceneGen::Ld_00::Ld_00_Room (room, jobMngr, world, events);
				SceneFile::LoadResults loadResults = (*roomOps)->LoadRoom ();
				assert (loadResults == SceneFile::LR_SUCCESS);
			}


			//// try to load the world lua
			//Text luaFilename (gameFolder + "room.lua");
			//if (Utils::File_Exists (luaFilename))
			//{
			//	room->LoadLua (luaFilename);
			//	Lua::LuaState * lua = room->GetLua ();

			//	Text dataFormat = lua->ReadString ("room/data_format");
			//	assert (dataFormat == "room_diorama_00");

			//	Text terrainGen = lua->ReadString ("room/terrain/primary_generation/type");
			//	if (terrainGen == "terrain_gen_00")
			//	{
			//		*roomOps = aah_new SceneGen::Gen00::Gen00_Room (room, jobMngr, world);
			//		//*roomOps = aah_new Gen01::Gen01_Room (room, jobMngr, world);
			//	}
			//	else if (terrainGen == "terrain_import_00_18")
			//	{
			//		//assert (false); // needs fixing with load / save
			//		*roomOps = aah_new SceneGen::Mc18::Mc18_Room (room, jobMngr, world);
			//	}
			//	else
			//	{
			//		assert (false);
			//	}
			//}
			//else
			//{
			//	// if there isn't one - then create the latest version
			//	*roomOps = aah_new SceneGen::Gen00::Gen00_Room (room, jobMngr, world);
			//	SceneFile::CreateResults createResults = (*roomOps)->CreateRoom ();
			//	assert (createResults == SceneFile::CR_SUCCESS);
			//	room->LoadLua (luaFilename);
			//}

			//SceneFile::LoadResults loadResults = (*roomOps)->LoadRoom ();
			//assert (loadResults == SceneFile::LR_SUCCESS);

			return Thread::JS_WORKDONE;
		}

		Thread::JOBSTATES RoomLoadJob::JobOnCompleted ()
		{
			assert (room->GetWorld () == NULL);

			Job::JobOnCompleted ();

			roomsLoading->Erase (room);

			world->AddRoom (room);

			events->OnRoomAdded (room);

			return Thread::JS_COMPLETED;
		}
	}
}