#include "stdafx.h"

#include "Lua/Lua_LuaState.h"

#include "../SceneGen/Gen00_World.h"
#include "../SceneHelper/SceneHelper_SimEvents.h"
#include "../SceneSim/SceneSim_World.h"
#include "../SceneSim/SceneSim_WorldInit.h"

#include "Factory_WorldLoadJob.h"
#include "SceneFile_OperationsWorld.h"

namespace SceneFile
{
	namespace Factory
	{
		WorldLoadJob::WorldLoadJob (Thread::JobMngr * jobMngr, SceneSim::WorldInit * worldInit, SceneSim::World * world, OperationsWorld * * ops, SceneHelper::SimEvents * events)
			: Thread::Job (jobMngr, true),
			worldInit (worldInit),
			world (world),
			ops (ops),
			events (events),
			jobMngrForChildren (jobMngr)
		{
			assert (worldInit != NULL);
			assert (world != NULL);
			assert (ops != NULL);
			assert (*ops == NULL);
			assert (events != NULL);

			assert (world->loadState == SceneSim::LS_NONE);
			world->loadState = SceneSim::LS_LOADREQUESTED;
		}

		WorldLoadJob::~WorldLoadJob ()
		{
		}

		Thread::JOBSTATES WorldLoadJob::JobDoWork ()
		{
			assert (world->loadState == SceneSim::LS_LOADREQUESTED);

			world->loadState = SceneSim::LS_LOADING;

			worldInit->wasSuccessful = false;

			world->AsyncInit (worldInit);

			*ops = aah_new SceneGen::Gen00::Gen00_World (jobMngrForChildren, worldInit, world, events);

			if (worldInit->isNew)
			{
				SceneFile::CreateResults createResults = (*ops)->CreateWorld ();
				assert (createResults == SceneFile::CR_SUCCESS);
			}
			else
			{
				SceneFile::LoadResults loadResults = (*ops)->LoadWorld ();
				assert (loadResults == SceneFile::LR_SUCCESS);
			}

			worldInit->wasSuccessful = true;

			world->loadState = SceneSim::LS_JUSTLOADED;

			return Thread::JS_WORKDONE;
		}

		Thread::JOBSTATES WorldLoadJob::JobOnCompleted ()
		{
			assert (world->loadState == SceneSim::LS_JUSTLOADED);

			Job::JobOnCompleted ();

			world->OnLoadCompleted ();

			events->OnWorldAdded (world);

			return Thread::JS_COMPLETED;
		}
	}
}