#include "stdafx.h"

#include "Lua/Lua_LuaState.h"
#include "Thread/Thread_JobMngr.h"

#include "../SceneSim/SceneSim_Column.h"
#include "../SceneSim/SceneSim_Room.h"
#include "../SceneSim/SceneSim_World.h"
#include "../SceneSim/SceneSim_WorldInit.h"

#include "../SceneHelper/SceneHelper_Helper.h"
#include "../SceneHelper/SceneHelper_SimMngr.h"

//#include "Factory_RoomLoadJob.h"
#include "Factory_WorldLoadJob.h"
#include "SceneFile_OperationsRoom.h"
#include "SceneFile_OperationsWorld.h"
#include "SceneFile_SimFactory.h"

namespace SceneFile
{
	SimFactory::SimFactory (SceneSim::WorldInit * worldInit, SceneSim::World * world, SceneHelper::SimEvents * events, bool isEditing)
		: worldInit (worldInit),
		world (world),
		events (events),
		worldOps (NULL),
		isEditing (isEditing)
	{
		assert (world != NULL);

		jobMngr = aah_new Thread::JobMngr ("SimFactory", false);

		worldInit->wasSuccessful = false;
	}

	SimFactory::~SimFactory () 
	{
		jobMngr->FinishCurrentJobAndDeleteAll ();

		delete worldOps;

		delete jobMngr;
	}

	int SimFactory::GetJobCount ()
	{
		return jobMngr->GetJobCount ();
	}

	void SimFactory::Update ()
	{
		jobMngr->MainThread_Update ();
	}

	void SimFactory::AbortAsyncAndSave ()
	{
		//jobMngr->FinishCurrentJobAndDeleteAll ();

		// queue up all columns to be saved
		for (int x = 0; x < SceneSim::FixedSize; ++x)
		{
			for (int z = 0; z < SceneSim::FixedSize; ++z)
			{
				SceneSim::Column * column = world->GetRoom ("hub")->GetColumn (Int3 (x, 0, z));
				if (column != NULL)
				{
					RequestColumnUnload (column);
				}
			}
		}

		while (jobMngr->GetJobCount () > 0)
		{
			jobMngr->MainThread_Update ();
			Sleep (100);
		}
	}

	void SimFactory::RequestWorldLoad ()
	{
		Factory::WorldLoadJob * job = aah_new Factory::WorldLoadJob (jobMngr, worldInit, world, &worldOps, events);

		// worlds are always added to queue
		job->AddToQueue (Thread::JB_NORMAL);
	}

	void SimFactory::RequestRoomLoad (TextRef roomId)
	{
		worldOps->NewRoomLoad (roomId);
	}

	void SimFactory::RequestColumnLoad (SceneSim::Room * room, Int3 columnId)
	{
		assert (room != NULL);
		assert (room->GetWorld () != NULL);
		assert (!IsColumnRequested (room, columnId));

		OperationsRoom * roomOps = worldOps->GetRoomOps (room->roomId);
		assert (roomOps != NULL);

		roomOps->NewColumnLoad (columnId, isEditing);
	}

	bool SimFactory::IsColumnRequested (const SceneSim::Room * room, Int3 columnId) const
	{
		OperationsRoom * roomOps = worldOps->GetRoomOps (room->roomId);
		assert (roomOps != NULL);

		return roomOps->NewIsColumnRequested (room, columnId);
	}
	
	void SimFactory::RequestWorldUnload ()
	{
		assert (false);
	}

	void SimFactory::RequestRoomUnload (TextRef roomId)
	{
		assert (false);
	}

	void SimFactory::RequestColumnUnload (SceneSim::Column * column)
	{
		assert (column != NULL);

		OperationsRoom * roomOps = worldOps->GetRoomOps (column->GetRoom ()->roomId);
		assert (roomOps != NULL);

		roomOps->NewColumnUnload (column);
		//roomOps->UnloadColumn (column);
	}
}