#pragma once

#include "../Noise/Noise_Random.h"
#include "../Noise/Noise_Perlin.h"

#include "../SceneFile/SceneFile_OperationsRoom.h"
#include "../SceneSim/SceneSim_Defs.h"

namespace SceneSim {class Room;}

namespace SceneGen
{
	namespace Gen00
	{
		class Gen00_Room : public SceneFile::OperationsRoom
		{
		public:
			
			Gen00_Room (SceneSim::Room * room, Thread::JobMngr * const jobMngr, SceneSim::World * world, SceneHelper::SimEvents * events);
			virtual ~Gen00_Room ();

			virtual void NewColumnLoad (Int3 columnId, bool isEditing);
			virtual void NewColumnUnload (SceneSim::Column * column);
			virtual void NewOnColumnLoadCompleted (SceneSim::Room * room, SceneSim::Column * column);
			virtual void NewOnColumnUnloadCompleted (SceneSim::Column * column);
			virtual bool NewIsColumnRequested (const SceneSim::Room * room, Int3 columnId) const;
			virtual SceneFile::LoadResults NewAsyncLoadColumn (SceneSim::Column * column, bool isEditing);
			virtual SceneFile::LoadResults NewAsyncUnloadColumn (SceneSim::Column * column);

			virtual SceneFile::CreateResults CreateRoom (SceneSim::WorldInit * worldInit);
			virtual SceneFile::LoadResults LoadRoom ();

			virtual SceneFile::CreateResults CreateColumn (SceneSim::Column * column);

			virtual void UnloadColumn (SceneSim::Column * column);

		private:

			void LoadDataFromLua ();

			// column funcs
			void CalcWeights (SceneSim::Column * column);
			float GetLandTypeWeight (int x, int z);
			float GetTerrainWeight (int x, int y, int z);

			float SmoothWeight (float a, float b, float weight) const;
			float SmoothWeight2d (float a1, float b1, float a2, float b2, float weight1, float weight2) const;
			void AddTree (SceneSim::Column * column, int waterLevel, unsigned int mask);

			// room data

			Noise::Seed randomSeed;
			int newColumnHeight;

			Noise::PerlinParams landscapeParams;
			Noise::PerlinParams landTypeParams;
			float landscapePerlinScale;
			float landscapePerlinTranslate;
			float landscapeSolidScale;
			float landscapeSolidTranslate;

			// column working data
			//SceneSim::Column * column;

			static const int smoothRes = 4;
			static const int smoothX = SceneSim::CellsPerColXz / smoothRes + 1;
			static const int smoothY = SceneSim::CellsPerColY / smoothRes + 1;
			static const int smoothZ = SceneSim::CellsPerColXz / smoothRes + 1;

			float landTypeWeights [smoothX][smoothZ];
			float terrainWeights [smoothX][smoothY][smoothZ];
		};
	}
}
