#include "stdafx.h"

#include "../SceneSim/SceneSim_Column.h"
#include "../SceneSim/SceneSim_Room.h"

#include "SceneHelper_Collections.h"

namespace SceneHelper
{
	namespace Collections
	{
		ColumnList::~ColumnList ()
		{
			assert (Size () == 0);
		}

		void ColumnList::Add (SceneSim::Column * column)
		{
			assert (column != NULL);
			assert (FindById (column->columnId) == NULL);

			push_back (column);
		}

		int ColumnList::Size () const
		{
			return size ();
		}

		SceneSim::Column * ColumnList::FindById (Int3 columnId)
		{
			for (iterator it = begin (); it != end (); ++it)
			{
				SceneSim::Column * column = *it;
				if (column->columnId == columnId)
				{
					return column;
				}
			}
			return NULL;
		}

		void ColumnList::Erase (SceneSim::Column * column)
		{
			assert (column != NULL);
			
			for (iterator it = begin (); it != end (); ++it)
			{
				if (*it == column)
				{
					erase (it);
					break;
				}
			}
		}

		void ColumnList::DeleteAll ()
		{
			iterator it = begin ();
			while (it != end ())
			{
				SceneSim::Column * column = *it;
				delete column;
				it = erase (it);
			}
		}

		RoomList::~RoomList ()
		{
			assert (Size () == 0);
		}

		void RoomList::Add (SceneSim::Room * room)
		{
			assert (room != NULL);
			assert (FindById (room->roomId) == NULL);

			push_back (room);
		}

		int RoomList::Size () const
		{
			return size ();
		}

		SceneSim::Room * RoomList::FindById (TextRef roomId)
		{
			for (iterator it = begin (); it != end (); ++it)
			{
				SceneSim::Room * room = *it;
				if (room->roomId == roomId)
				{
					return room;
				}
			}
			return NULL;
		}

		SceneSim::Room * RoomList::GetNth (int nth)
		{
			for (iterator it = begin (); it != end () && nth >= 0; ++it, --nth)
			{
				if (nth == 0)
				{
					return *it;
				}
			}
			return NULL;
		}

		void RoomList::Erase (SceneSim::Room * room)
		{
			assert (room != NULL);
			
			for (iterator it = begin (); it != end (); ++it)
			{
				if (*it == room)
				{
					erase (it);
					break;
				}
			}
		}

		void RoomList::DeleteAll ()
		{
			iterator it = begin ();
			while (it != end ())
			{
				SceneSim::Room * room = *it;
				delete room;
				it = erase (it);
			}
		}
	}
}
