#include "stdafx.h"

#include "../CellBuilder/CellBuilder_Mngr.h"
#include "../CellBuilder/CellBuilder_ShapeBase.h"
#include "../Ents/Ents_Player.h"
#include "../SceneFile/SceneFile_Lighting.h"
#include "../SceneHelper/SceneHelper_Helper.h"

#include "BlockData_Sign.h"
#include "Query_Cell.h"
#include "SceneSim_Column.h"
#include "SceneSim_ColumnCells.h"
#include "SceneSim_Room.h"

namespace SceneSim
{
	Column::Column (Int3 columnId)
		: columnId (columnId),
		flags (SF_NOTSET),
		lastModified (0),
		room (NULL),
		cells (NULL)
	{
	}

	Column::Column (Column & copyFrom)
		: columnId (copyFrom.columnId),
		cellsHeight (copyFrom.cellsHeight),
		flags (copyFrom.flags),
		lastModified (copyFrom.lastModified),
		room (NULL),
		cells (NULL)
	{
		if (copyFrom.cells)
		{
			cells = aah_new ColumnCells (*copyFrom.cells);
		}
	}

	Column::~Column ()
	{
		delete cells;
	}

	void Column::Init (int cellsHeight, int flags)
	{
		this->cellsHeight = cellsHeight;
		this->flags = flags;

		if (!(flags & SF_NO_CELLS))
		{
			cells = aah_new ColumnCells (cellsHeight);
		}
	}

	void Column::OnAddedToRoom (Room * room)
	{
		assert (flags != SF_NOTSET);
		assert (room != NULL);
		assert (this->room == NULL);

		this->room = room;

		//adjacent [F_T] = NULL; // cant stack columns
		//adjacent [F_B] = NULL; // cant stack columns
		//CreateAdjacent (F_N, true);
		//CreateAdjacent (F_E, true);
		//CreateAdjacent (F_S, true);
		//CreateAdjacent (F_W, true);
	}

	void Column::OnRemovedFromRoom ()
 	{
		assert (room != NULL);

		//for (SignListIt current = signs.begin (); current != signs.end (); ++current)
		//{
		//	delete *current;
		//}
		//signs.clear ();

		//CreateAdjacent (F_N, false);
		//CreateAdjacent (F_E, false);
		//CreateAdjacent (F_S, false);
		//CreateAdjacent (F_W, false);

		room = NULL;
	}

	void Column::SetLastModifiedTime (RakNet::Time & time)
	{
		lastModified = time;
	}

	const RakNet::Time & Column::GetLastModifiedTime () const
	{
		return lastModified;
	}
	
	Room * Column::GetRoom () const
	{
		return room;
	}

	Int3 Column::GetSize () const
	{
		return Int3 (CellsPerColXz, cellsHeight, CellsPerColXz);
	}

	void Column::Update (Ents::Player * player)
	{
		assert (flags != SF_NOTSET);
		assert (room != NULL);
	}

	Column * Column::GetAdjacentColumn (FACES face) const
	{
		assert (room != NULL);

		switch (face)
		{
		case F_T: break;
		case F_B: break;
		case F_N: return room->GetColumn (columnId + Int3 (0, 0, 1));
		case F_E: return room->GetColumn (columnId + Int3 (1, 0, 0));
		case F_S: return room->GetColumn (columnId + Int3 (0, 0, -1));
		case F_W: return room->GetColumn (columnId + Int3 (-1, 0, 0));
		default: assert (false);
		}
			
		return NULL;

		//return adjacent [face];
	}

	ColumnCells * Column::GetCells ()
	{
		assert (flags != SF_NOTSET);
		assert (cells != NULL);
		return cells;
	}

	const ColumnCells * Column::GetCells () const
	{
		assert (flags != SF_NOTSET);
		assert (cells != NULL);
		return cells;
	}

	//bool Column::GetAdjacentCell (Int3 colRelPos, FACES face, Query::CellByColumn * query)
	//{
	//	assert (room != NULL);

	//	Column * column = this;
	//	switch (face)
	//	{
	//	case F_T: 
	//		++colRelPos.y; 
	//		if (colRelPos.y >= CellsPerColY)
	//		{
	//			column = column->adjacent [F_T];
	//			colRelPos.y -= CellsPerColY;
	//		}
	//		break;

	//	case F_B: 
	//		--colRelPos.y; 
	//		if (colRelPos.y < 0)
	//		{
	//			column = column->adjacent [F_B];
	//			colRelPos.y += CellsPerColY;
	//		}
	//		break;

	//	case F_N: 
	//		++colRelPos.z; 
	//		if (colRelPos.z >= CellsPerColXz)
	//		{
	//			column = column->adjacent [F_N];
	//			colRelPos.z -= CellsPerColXz;
	//		}
	//		break;

	//	case F_E: 
	//		++colRelPos.x; 
	//		if (colRelPos.x >= CellsPerColXz)
	//		{
	//			column = column->adjacent [F_E];
	//			colRelPos.x -= CellsPerColXz;
	//		}
	//		break;

	//	case F_S: 
	//		--colRelPos.z; 
	//		if (colRelPos.z < 0)
	//		{
	//			column = column->adjacent [F_S];
	//			colRelPos.z += CellsPerColXz;
	//		}
	//		break;

	//	case F_W: 
	//		--colRelPos.x; 
	//		if (colRelPos.x < 0)
	//		{
	//			column = column->adjacent [F_W];
	//			colRelPos.x += CellsPerColXz;
	//		}
	//		break;
	//	}

	//	if (column != NULL && !(column->flags & SceneSim::Column::SF_NO_CELLS))
	//	{
	//		query->colRelPos = colRelPos;
	//		query->column = column;
	//		query->cell = column->GetCells ()->GetCell (colRelPos);
	//		return true;
	//	}

	//	return false;
	//}

	void Column::SetCell (Int3 cellId, int cellType, int cellData, int cellLightSky, int cellLightSrc)
	{
		Cell * cell = GetCells ()->GetCell (cellId);
		cell->type = cellType;
		cell->blockData = cellData;
		cell->lightSky = cellLightSky;
		cell->lightSrc = cellLightSrc;

		SceneFile::Lighting::CalcStripSky (this, cellId, room->GetCellBuilder ());
	}

	void Column::SwapCells (Column * other)
	{
		SceneSim::ColumnCells * tempCells = cells;
		cells = other->cells;
		other->cells = tempCells;
	}

	//void Column::RecordBlockData_Sign (BlockData::Sign * sign)
	//{
	//	assert (room != NULL);

	//	signs.push_back (sign);
	//}

	//BlockData::Sign * Column::BlockData_GetSign (Int3 pos)
	//{
	//	assert (room != NULL);

	//	for (SignListIt current = signs.begin (); current != signs.end (); ++current)
	//	{
	//		if ((*current)->pos == pos)
	//		{
	//			return *current;
	//		}
	//	}
	//	return NULL;
	//}

	void Column::CalcSkyLightMap (const CellBuilder::Mngr * builder)
	{
		//assert (builder != NULL);
		//
		//if (cells == NULL)
		//{
		//	return;
		//}

		//Int2 stripId;
		//for (stripId.x = 0; stripId.x < CellsPerColXz; ++stripId.x)
		//{
		//	for (stripId.y = 0; stripId.y < CellsPerColXz; ++stripId.y)
		//	{
		//		for (int y = SceneSim::CellsPerColY - 1; y >= 0; --y)
		//		{
		//			const Cell * cell = cells->GetCell (stripId.x, y, stripId.y);
		//			const CellBuilder::ShapeBase * shape = builder->GetShape (cell->type);

		//			if (shape->blocksLight)
		//			{
		//				cells->SetSkyLightLevel (stripId, y);
		//				break;
		//			}
		//		}
		//	}
		//}
	}

	//void Column::CreateAdjacent (FACES face, bool isCreating)
	//{
	//	assert (room != NULL);

	//	Int3 adjColId = columnId;

	//	switch (face)
	//	{
	//	case F_N: ++adjColId.z; break;
	//	case F_E: ++adjColId.x; break;
	//	case F_S: --adjColId.z; break;
	//	case F_W: --adjColId.x; break;
	//	default: assert (false); break;
	//	}
	//	
	//	Column * adjColumn = room->GetColumn (adjColId);
	//	if (adjColumn != NULL)
	//	{
	//		FACES oppositeFace = GetOppositeFace (face);
	//		if (isCreating)
	//		{
	//			adjacent [face] = adjColumn;
	//			adjColumn->adjacent [oppositeFace] = this;

	//			//CopyAdjCellsFrom (face, adjColumn);
	//			//adjColumn->CopyAdjCellsFrom (oppositeFace, this);
	//		}
	//		else
	//		{
	//			adjacent [face] = NULL;
	//			adjColumn->adjacent [oppositeFace] = NULL;
	//		}
	//		//adjColumn->MarkDirty ();
	//	}
	//	else
	//	{
	//		adjacent [face] = NULL;
	//	}
	//}

	//void Column::CopyAdjCellsFrom (FACES face, const Column * adjacent)
	//{
	//	const ColumnCells * adjCells = adjacent->GetCells ();

	//	switch (face)
	//	{
	//	case F_N: 
	//		for (int x = 0; x < CellsPerColXz; ++x)
	//		{
	//			for (int y = 0; y < CellsPerColY; ++y)
	//			{
	//				*cells->GetCell (x, y, CellsPerColXz) = *adjCells->GetCell (x, y, 0);
	//			}
	//		}
	//		break;

	//	case F_E: 
	//		for (int z = 0; z < CellsPerColXz; ++z)
	//		{
	//			for (int y = 0; y < CellsPerColY; ++y)
	//			{
	//				*cells->GetCell (CellsPerColXz, y, z) = *adjCells->GetCell (0, y, z);
	//			}
	//		}
	//		break;

	//	case F_S: 
	//		for (int x = 0; x < CellsPerColXz; ++x)
	//		{
	//			for (int y = 0; y < CellsPerColY; ++y)
	//			{
	//				*cells->GetCell (x, y, -1) = *adjCells->GetCell (x, y, CellsPerColXz - 1);
	//			}
	//		}
	//		break;

	//	case F_W: 
	//		for (int z = 0; z < CellsPerColXz; ++z)
	//		{
	//			for (int y = 0; y < CellsPerColY; ++y)
	//			{
	//				*cells->GetCell (-1, y, z) = *adjCells->GetCell (CellsPerColXz - 1, y, z);
	//			}
	//		}
	//		break;

	//	default: assert (false); break;
	//	}
	//}
}
