#pragma once

#include <list>

#include "SceneSim_Defs.h"

namespace CellBuilder {class Mngr;}
namespace Ents {class Player;}
namespace SceneSim {class ColumnCells;}
namespace SceneSim {class Room;}
namespace SceneSim {namespace Query {struct CellByColumn;}}
namespace SceneSim {namespace BlockData {class Sign;}}

namespace SceneSim
{
	class Column
	{
	public:

		enum SPECIALFLAGS
		{
			SF_NONE					= 0x00,
			SF_PREVENT_DESTROY		= 0x01,
			SF_PREVENT_BUILD		= 0x02,
			SF_PREVENT_RENDER		= 0x04,
			SF_SOLID_TO_ENTS		= 0x08,
			SF_SOLID_TO_PROJECTILES	= 0x10,
			SF_NO_CELLS				= 0x20, // very lightweight column

			SF_NOTSET				= 0xff,
		};

		Column (Int3 columnId);
		Column (Column & copyFrom);
		~Column ();

		void Init (int cellsHeight, int flags);

		void OnAddedToRoom (Room * room);
		void OnRemovedFromRoom ();

		void SetLastModifiedTime (RakNet::Time & time);
		const RakNet::Time & GetLastModifiedTime () const;

		Room * GetRoom () const; // this is definitive test of whether it is ready or not
		Int3 GetSize () const;

		void Update (Ents::Player * player);

		Column * GetAdjacentColumn (FACES face) const;
		ColumnCells * GetCells ();
		const ColumnCells * GetCells () const;
		//bool GetAdjacentCell (Int3 colRelPos, FACES face, Query::CellByColumn * query);

		void SetCell (Int3 cellId, int cellType, int cellData, int cellLightSky, int cellLightSrc);
		void SwapCells (Column * other);

		//void RecordBlockData_Sign (BlockData::Sign * sign);
		//BlockData::Sign * BlockData_GetSign (Int3 pos);

		void CalcSkyLightMap (const CellBuilder::Mngr * builder);

		const Int3 columnId;
		int cellsHeight;
		int flags;

	private:

		//void CreateAdjacent (FACES face, bool isCreating);
		//void CopyAdjCellsFrom (FACES face, const Column * adjacent);

		RakNet::Time lastModified;

		Room * room;
		
		//Column * adjacent [F_COUNT_CUBE];
		ColumnCells * cells;

		//typedef std::list <BlockData::Sign *> SignList;
		//typedef SignList::iterator SignListIt;
		//SignList signs;
	};
}
