#include "stdafx.h"

#include "SceneSim_Column.h"
#include "SceneSim_ColumnCells.h"

namespace SceneSim
{
	const SceneSim::Cell ColumnCells::dummyAirCell = {0, 0, 15, 15};

	ColumnCells::ColumnCells (int cellsPerColY)
		: rawData (NULL),
		cellsPerColY (cellsPerColY)
	{
		assert (cellsPerColY == 128);

		rawData = aah_new Cell [cellsPerColY * CellsPerColXz * CellsPerColXz];

		memset (rawData, 0, sizeof (Cell) * cellsPerColY * CellsPerColXz * CellsPerColXz);
	}
	
	ColumnCells::ColumnCells (const ColumnCells & copyFrom)
		: rawData (NULL),
		cellsPerColY (copyFrom.cellsPerColY)
	{
		rawData = aah_new Cell [cellsPerColY * CellsPerColXz * CellsPerColXz];

		memcpy (rawData, copyFrom.rawData, sizeof (Cell) * cellsPerColY * CellsPerColXz * CellsPerColXz);
	}

	ColumnCells::~ColumnCells ()
	{
		delete [] rawData;
	}

	const Cell * ColumnCells::GetCell (int colRelX, int colRelY, int colRelZ) const
	{
		// all access to "cells" must be Y X Y, not X Y Z
		return GetCell (Int3 (colRelX, colRelY, colRelZ));
	}

	const Cell * ColumnCells::GetCell (Int3 colRelPos) const
	{
		assert (
			colRelPos.x >= 0 && colRelPos.y >= 0 && colRelPos.z >= 0 && 
			colRelPos.x < SceneSim::CellsPerColXz && colRelPos.y < cellsPerColY && colRelPos.z < SceneSim::CellsPerColXz);
/*
		if (colRelPos.x == -1)
		{
			Column * adj = column->GetAdjacentColumn (F_W);
			if (adj != NULL)
			{
				return adj->GetCells ()->GetCell (SceneSim::CellsPerColXz - 1, colRelPos.y, colRelPos.z);
			}
			//return &rawData [colRelPos.y][colRelPos.x + 1][colRelPos.z];
			return &dummyAirCell;
			//return &w [colRelPos.y][colRelPos.z];
		}
		else if (colRelPos.x == SceneSim::CellsPerColXz)
		{
			Column * adj = column->GetAdjacentColumn (F_E);
			if (adj != NULL)
			{
				return adj->GetCells ()->GetCell (0, colRelPos.y, colRelPos.z);
			}
			//return &rawData [colRelPos.y][colRelPos.x - 1][colRelPos.z];
			return &dummyAirCell;
			//return &e [colRelPos.y][colRelPos.z];
		}
		else if (colRelPos.y == -1)
		{
			//return column->GetAdjacentColumn (F_W)->GetCells ()->GetCell (colRelPos.x, colRelPos.y, colRelPos.z);
			//return &rawData [colRelPos.y + 1][colRelPos.x][colRelPos.z];
			return &dummyAirCell;
			//return &t [colRelPos.x][colRelPos.z];
		}
		else if (colRelPos.y == SceneSim::CellsPerColY)
		{
			//return column->GetAdjacentColumn (F_W)->GetCells ()->GetCell (colRelPos.x, colRelPos.y, colRelPos.z);
			//return &rawData [colRelPos.y - 1][colRelPos.x][colRelPos.z];
			return &dummyAirCell;
			//return &b [colRelPos.x][colRelPos.z];
		}
		else if (colRelPos.z == -1)
		{
			Column * adj = column->GetAdjacentColumn (F_S);
			if (adj != NULL)
			{
				return adj->GetCells ()->GetCell (colRelPos.x, colRelPos.y, SceneSim::CellsPerColXz - 1);
			}
			//return &rawData [colRelPos.y][colRelPos.x][colRelPos.z + 1];
			return &dummyAirCell;
			//return &s [colRelPos.x][colRelPos.y];
		}
		else if (colRelPos.z == SceneSim::CellsPerColXz)
		{
			Column * adj = column->GetAdjacentColumn (F_N);
			if (adj != NULL)
			{
				return adj->GetCells ()->GetCell (colRelPos.x, colRelPos.y, 0);
			}
			//return &rawData [colRelPos.y][colRelPos.x][colRelPos.z - 1];
			return &dummyAirCell;
			//return &n [colRelPos.x][colRelPos.y];
		}
		else*/
		{
			// NOTE: all direct access to "cells" array must be Y X Y, not X Y Z
			return rawData + (colRelPos.y * CellsPerColXz * CellsPerColXz + colRelPos.x * CellsPerColXz + colRelPos.z);
			//return &rawData [colRelPos.y][colRelPos.x][colRelPos.z];
		}
	}

	Cell * ColumnCells::GetCell (int colRelX, int colRelY, int colRelZ)
	{
		// NOTE: all direct access to "cells" array must be Y X Y, not X Y Z
		return GetCell (Int3 (colRelX, colRelY, colRelZ));
	}

	Cell * ColumnCells::GetCell (Int3 colRelPos)
	{
		assert (
			colRelPos.x >= 0 && colRelPos.y >= 0 && colRelPos.z >= 0 && 
			colRelPos.x < SceneSim::CellsPerColXz && colRelPos.y < cellsPerColY && colRelPos.z < SceneSim::CellsPerColXz);

/*		if (colRelPos.x == -1)
		{
			Column * adj = column->GetAdjacentColumn (F_W);
			if (adj != NULL)
			{
				return adj->GetCells ()->GetCell (SceneSim::CellsPerColXz - 1, colRelPos.y, colRelPos.z);
			}
			//return &rawData [colRelPos.y][colRelPos.x + 1][colRelPos.z];
			return &dummyAirCell;
			//return &w [colRelPos.y][colRelPos.z];
		}
		else if (colRelPos.x == SceneSim::CellsPerColXz)
		{
			Column * adj = column->GetAdjacentColumn (F_E);
			if (adj != NULL)
			{
				return adj->GetCells ()->GetCell (0, colRelPos.y, colRelPos.z);
			}
			//return &rawData [colRelPos.y][colRelPos.x - 1][colRelPos.z];
			return &dummyAirCell;
			//return &e [colRelPos.y][colRelPos.z];
		}
		else if (colRelPos.y == -1)
		{
			//return column->GetAdjacentColumn (F_W)->GetCells ()->GetCell (colRelPos.x, colRelPos.y, colRelPos.z);
			//return &rawData [colRelPos.y + 1][colRelPos.x][colRelPos.z];
			return &dummyAirCell;
			//return &t [colRelPos.x][colRelPos.z];
		}
		else if (colRelPos.y == SceneSim::CellsPerColY)
		{
			//return column->GetAdjacentColumn (F_W)->GetCells ()->GetCell (colRelPos.x, colRelPos.y, colRelPos.z);
			//return &rawData [colRelPos.y - 1][colRelPos.x][colRelPos.z];
			return &dummyAirCell;
			//return &b [colRelPos.x][colRelPos.z];
		}
		else if (colRelPos.z == -1)
		{
			Column * adj = column->GetAdjacentColumn (F_S);
			if (adj != NULL)
			{
				return adj->GetCells ()->GetCell (colRelPos.x, colRelPos.y, SceneSim::CellsPerColXz - 1);
			}
			//return &rawData [colRelPos.y][colRelPos.x][colRelPos.z + 1];
			return &dummyAirCell;
			//return &s [colRelPos.x][colRelPos.y];
		}
		else if (colRelPos.z == SceneSim::CellsPerColXz)
		{
			Column * adj = column->GetAdjacentColumn (F_N);
			if (adj != NULL)
			{
				return adj->GetCells ()->GetCell (colRelPos.x, colRelPos.y, 0);
			}
			//return &rawData [colRelPos.y][colRelPos.x][colRelPos.z - 1];
			return &dummyAirCell;
			//return &n [colRelPos.x][colRelPos.y];
		}
		else*/
		{
			// NOTE: all direct access to "cells" array must be Y X Y, not X Y Z

			return rawData + (colRelPos.y * CellsPerColXz * CellsPerColXz + colRelPos.x * CellsPerColXz + colRelPos.z);

			//return &rawData [colRelPos.y][colRelPos.x][colRelPos.z];
		}
	}

	//unsigned char ColumnCells::GetSkyLightLevel (Int2 stripId)
	//{
	//	assert (stripId.x >= 0 && stripId.y >= 0 && stripId.y >= 0 && stripId.y < CellsPerColXz);
	//	return skyLightLevels [stripId.x][stripId.y];
	//}

	//void ColumnCells::SetSkyLightLevel (Int2 stripId, unsigned char level)
	//{
	//	assert (stripId.x >= 0 && stripId.y >= 0 && stripId.y >= 0 && stripId.y < CellsPerColXz);
	//	skyLightLevels [stripId.x][stripId.y] = level;
	//}

	void ColumnCells::CopyFrom (const ColumnCells * copyFrom)
	{
		assert (copyFrom != NULL);
		memcpy (rawData, copyFrom->rawData, sizeof (Cell) * cellsPerColY * CellsPerColXz * CellsPerColXz);
	}

	void * ColumnCells::GetRawDataToSave (int * length)
	{
		assert (length != NULL);
		*length = sizeof (Cell) * cellsPerColY * CellsPerColXz * CellsPerColXz;
		return rawData;
	}
}

