#include "stdafx.h"

#include "SceneSim_Column.h"

namespace SceneSim
{
	FACES GetOppositeFace (FACES face)
	{
		switch (face)
		{
		case F_T : return F_B;
		case F_B : return F_T;
		case F_N : return F_S;
		case F_E : return F_W;
		case F_S : return F_N;
		case F_W : return F_E;
		default: assert (false);
		}
		return F_NONE;
	}

	Int3 GetDeltaFromFace (FACES face)
	{
		switch (face)
		{
		case F_T : return Int3 (0, 1, 0);
		case F_B : return Int3 (0, -1, 0);
		case F_N : return Int3 (0, 0, 1);
		case F_E : return Int3 (1, 0, 0);
		case F_S : return Int3 (0, 0, -1);
		case F_W : return Int3 (-1, 0, 0);
		default: assert (false);
		}
		return Int3 (0, 0, 0);
	}

	void FixCell (Int3 * columnId, Int3 * cellId)
	{
		assert (columnId != NULL);
		assert (cellId != NULL);

		for (; cellId->x < 0; cellId->x += CellsPerColXz, --columnId->x);
		for (; cellId->x >= CellsPerColXz; cellId->x -= CellsPerColXz, ++columnId->x);
		for (; cellId->y < 0; cellId->y += CellsPerColY, --columnId->y);
		for (; cellId->y >= CellsPerColY; cellId->y -= CellsPerColY, ++columnId->y);
		for (; cellId->z < 0; cellId->z += CellsPerColXz, --columnId->z);
		for (; cellId->z >= CellsPerColXz; cellId->z -= CellsPerColXz, ++columnId->z);
	}

	AdjColumns::AdjColumns ()
		: main (NULL), n (NULL), e (NULL), s (NULL), w (NULL)
	{
	}

	AdjColumns::AdjColumns (Column * main, Column * n, Column * e, Column * s, Column * w)
		: main (main), n (n), e (e), s (s), w (w)
	{
	}

	bool AdjColumns::operator == (const AdjColumns & other) const
	{
		const void * a = main, * b = other.main; assert (a == b);
		a = n, b = other.n; assert (a == b);
		a = e, b = other.e; assert (a == b);
		a = s, b = other.s; assert (a == b);
		a = w, b = other.w; assert (a == b);

		return (main == other.main &&
			n == other.n &&
			e == other.e &&
			s == other.s &&
			w == other.w);
	}

	void AdjColumns::DeleteAll ()
	{
		delete main;	main = NULL;
		delete n;		n = NULL;
		delete e;		e = NULL;
		delete s;		s = NULL;
		delete w;		w = NULL;
	}
}
