2 (
#include "stdafx.h"

#include <algorithm>

#include "Lua/Lua_LuaState.h"

#include "../CellBuilder/CellBuilder_Mngr.h"
#include "../Ents/Comps_Coord.h"
#include "../Ents/Ents_Player.h"
#include "../Input/Input_InputMngr.h"
#include "../SceneHelper/SceneHelper_Helper.h"

#include "SceneSim_Cell.h"
#include "SceneSim_Column.h"
#include "SceneSim_ColumnCells.h"
#include "SceneSim_ColumnCollection.h"
#include "SceneSim_Room.h"
#include "SceneSim_World.h"

namespace SceneSim
{
	Room::Room (Text roomId)
		: roomId (roomId),
		lua (NULL),
		randomSeed (0x0),
		newColumnHeight (0),
		world (NULL),
		//columns (NULL),
		cellBuilder (NULL)
	{
		//columns = aah_new SceneCollections::SparseGridCell <Column *> ();
		cellBuilder = aah_new CellBuilder::Mngr ();

		memset (columns, 0, sizeof (columns));
		columnsLoadedCount = 0;
	}

	Room::~Room ()
	{
		delete lua;
		delete cellBuilder;
	}

	void Room::LoadLua (TextRef filename)
	{
		assert (lua == NULL);

		lua = aah_new Lua::LuaState (filename);

		newColumnHeight = lua->ReadInt ("room/terrain/column_height");
		
		Text seedAsText = lua->ReadString ("room/terrain/primary_generation/random_seed");
		randomSeed = RakNet::RakString::ToInteger (seedAsText.Raw ());
	}

	void Room::OnAddedToWorld (World * world)
	{
		assert (this->world == NULL);
		assert (world != NULL);

		this->world = world;
	}

	void Room::OnRemovedFromWorld ()
	{
		assert (world != NULL);

		for (int x = 0; x < FixedSize; ++x)
		{
			for (int z = 0; z < FixedSize; ++z)
			{
				Column * column = columns [x][z];
				if (column != NULL)
				{
					RemoveColumn (column);
					delete column;
				}
			}
		}

		world = NULL;

		assert (columnsLoadedCount == 0); // check has no columns attached
	}

	World * Room::GetWorld () 
	{
		return world;
	}

	Lua::LuaState * Room::GetLua ()
	{
		assert (lua != NULL);

		return lua;
	}

	const CellBuilder::Mngr * Room::GetCellBuilder () const
	{
		assert (world != NULL);

		return cellBuilder;
	}

	int Room::GetRandomSeed () const
	{
		return randomSeed;
	}

	int Room::GetNewColumnHeight () const
	{
		return newColumnHeight;
	}

	void Room::Update (Input::InputMngr * mngr, Ents::Player * player)
	{
		assert (world != NULL);

		// TODO: readd animating textures
		//roomGfx->GetTexMngr ()->UpdateAnims ();

		assert (false); // changing what updates what i guess...
	}

	void Room::AddColumn (Column * column)
	{
		assert (world != NULL);
		assert (column != NULL);
		assert (GetColumn (column->columnId) == NULL);

		++columnsLoadedCount;
		columns [column->columnId.x][column->columnId.z] = column;

		column->OnAddedToRoom (this);
	}

	void Room::RemoveColumn (Column * column)
	{
		assert (world != NULL);
		assert (column != NULL);
		assert (GetColumn (column->columnId) != NULL);

		column->OnRemovedFromRoom ();
		--columnsLoadedCount;
		columns [column->columnId.x][column->columnId.z] = NULL;
	}

	Column * Room::GetColumn (Int3 columnId) const
	{
		assert (world != NULL);

		// ludum dare
		if (columnId.x < 0 || columnId.x >= FixedSize || columnId.z < 0 || columnId.z >= FixedSize)
		{
			return NULL;
		}

		Column * column = columns [columnId.x][columnId.z];
		return column;
	}

	Column * Room::GetNthLoadedColumn (int nth) const
	{
		assert (false);

		//assert (world != NULL);

		//assert (nth >= 0);
		//if (nth >= columnsLoaded.size ())
		//{
		//	return NULL;
		//}

		//for (std::list <Column *>::const_iterator it = columnsLoaded.begin (); ; ++it, --nth)
		//{
		//	if (nth == 0)
		//	{
		//		return *it;
		//	}
		//}

		//return NULL;
		//return columns->GetNth (nth);

		return NULL;
	}
}
