#include "stdafx.h"

#include <GL/gl.h>	/* Header File For The OpenGL Library */
#include <GL/glu.h>	/* Header File For The GLU Library */

#include <algorithm>

#include "Lua/Lua_LuaState.h"

#include "../Ents/Ents_Player.h"
#include "../Gfx/Gfx_Camera.h"
#include "../Gfx/Gfx_Viewport.h"
#include "../SceneHelper/SceneHelper_Collections.h"

#include "SceneSim_Column.h"
#include "SceneSim_Room.h"
#include "SceneSim_World.h"
#include "SceneSim_WorldInit.h"

namespace SceneSim
{
	World::World ()
		: gameFolder ("uninitialised"),
		lua (NULL),
		loadState (LS_NONE)
	{
		roomsLoaded = aah_new SceneHelper::Collections::RoomList ();
	}

	//World::World (WorldInit * init)
	//{
	//	assert (init != NULL);
	//}

	//World::World (TextRef gameFolder)
	//	: gameFolder (gameFolder),
	//	lua (NULL),
	//	loadState (LS_NONE)
	//{
	//	roomsLoaded = aah_new SceneHelper::Collections::RoomList ();
	//}

	World::~World ()
	{
		Room * room = NULL;
		while ((room = roomsLoaded->GetNth (0)) != NULL)
		{
			RemoveRoom (room);
			delete room;
		}

		delete roomsLoaded;

		delete lua;
	}

	TextRef World::GetGameFolder () const
	{
		return gameFolder;
	}

	void World::AsyncInit (WorldInit * init)
	{
		assert (init != NULL);

		gameFolder = Utils::File_NeatenFolder (init->folder + init->name);
	}

	void World::OnLoadCompleted ()
	{
		loadState = SceneSim::LS_LOADCOMPLETE;
	}

	void World::AddRoom (Room * room)
	{
		assert (room != NULL);
		assert (!IsRoomLoaded (room->roomId));

		roomsLoaded->Add (room);
		room->OnAddedToWorld (this);
	}

	void World::RemoveRoom (Room * room)
	{
		assert (room != NULL);
		assert (IsRoomLoaded (room->roomId));

		room->OnRemovedFromWorld ();
		roomsLoaded->Erase (room);
	}

	bool World::IsRoomLoaded (Text roomId) const
	{
		return (roomsLoaded->FindById (roomId) != NULL);
	}

	Room * World::GetRoom (Text roomId)
	{
		assert (loadState == LS_LOADCOMPLETE);
		assert (IsRoomLoaded (roomId));

		return roomsLoaded->FindById (roomId);
	}

	Room * World::GetRoomNth (int nth)
	{
		assert (loadState == LS_LOADCOMPLETE);

		return roomsLoaded->GetNth (nth);
	}
}
