#pragma once

#include "File_PakFileHeader.h"

namespace File {struct PakFileRecord;}
namespace Utils {struct Time;}

namespace File
{
	class PakFileBase
	{
	public:

		PakFileBase (TextRef filename, int recordCount);
		virtual ~PakFileBase ();

		bool DoesRecordExist (int recordIdx) const;

		int LoadRecord (int recordIdx, RakNet::Time * lastModified, unsigned char * out, int maxOut);

		void SaveRecord (int recordIdx, const RakNet::Time * lastModified, unsigned char * in, int size);

		unsigned int Debug_GetChecksum (int recordIdx, unsigned char * data, int length) const;

	protected:

		Text filename;
		PakFileHeader header;
		PakFileRecord * records;

	private:

		static const int blockSize = 1024 * 4; // any benefit to matching cluster size?

		void InitNewHeader (int recordCount);
		void SaveHeader (bool addPadding);

		int GetMinBlockCount (int size) const;
		void PadToNextBlock (std::fstream * file);
		int FindFirstFreeBlocks (int blockCount) const;
		
		static unsigned char padBuffer [blockSize];

		int * sortedRecordIds;
	};
}
