#include "stdafx.h"
//
//#include <algorithm>
//#include <iostream>
//#include <fstream>

#include "File_PakFileCompressed.h"
#include "File_PakFileRecord.h"

namespace File
{
	PakFileCompressed::PakFileCompressed (TextRef filename, int recordCount)
		: PakFileBase (filename, recordCount),
		compressionBufferSize (1024 * 1024) // 1 MB
	{
		compressionBuffer = aah_new unsigned char [compressionBufferSize];
	}

	PakFileCompressed::~PakFileCompressed ()
	{
		delete [] compressionBuffer;
	}

	bool PakFileCompressed::DoesRecordExist (int recordIdx) const
	{
		return PakFileBase::DoesRecordExist (recordIdx);
	}

	int PakFileCompressed::LoadRecord (int recordIdx, RakNet::Time * lastModified, void * out, int maxOut)
	{
		assert (recordIdx >= 0 && recordIdx < header.recordCount);

		const int compressedSize = PakFileBase::LoadRecord (recordIdx, lastModified, compressionBuffer, compressionBufferSize);
		const int decompressedSize = Utils::Zlib_Uncompress (compressionBuffer, compressedSize, out, maxOut);

		return decompressedSize;
	}

	void PakFileCompressed::SaveRecord (int recordIdx, const RakNet::Time * lastModified, void * in, int size, int compressionLevel)
	{
		assert (recordIdx >= 0 && recordIdx < header.recordCount);
		PakFileRecord * record = &records [recordIdx];

		// only save if data to be saved is newer than what is in the record
		if (*lastModified > record->lastModified)
		{
			const int compressedSize = Utils::Zlib_Compress (in, size, compressionBuffer, compressionBufferSize, compressionLevel);
	
			PakFileBase::SaveRecord (recordIdx, lastModified, compressionBuffer, compressedSize);
		}
	}
}
