#include "stdafx.h"

#include <fstream>

extern "C"
{
	#include "../ExternalLibs/lua-5.1.4/src/lua.h"
	#include "../ExternalLibs/lua-5.1.4/src/lauxlib.h"
}

#include "Lua_LuaState.h"

namespace Lua
{
	void RecurseTable (std::ofstream & file, lua_State * state, int indentCount)
	{
		char * tab = new char [indentCount + 1];
		memset (tab, '\t', indentCount);
		tab [indentCount] = '\0';
		
		lua_pushnil (state);
		while (true)
		{
			int result = lua_next (state, -2);
			if (result == 0)
			{
				break;
			}

			//printf("%s - %s\n", lua_typename (state, lua_type (state, -2)), lua_typename (state, lua_type (state, -1)));
			if (lua_isnumber (state, -2))
			{
				file << tab;
			}
			else if (lua_isstring (state, -2))
			{
				file << tab << lua_tolstring (state, -2, NULL) << " = ";
				//printf ("%s%s = ", tab, ;
			}

			if (lua_isnumber (state, -1))
			{
				float number = (float) lua_tonumber (state, -1);
				if (number == (int) number)
				{
					file << (int) number << "," << std::endl;
					//printf ("%d,\n", (int) number);
				}
				else
				{
					file << number << "," << std::endl;
					//printf ("%f,\n", number);
				}
			}
			else if (lua_isboolean (state, -1))
			{
				file << ((lua_toboolean (state, -1)) ? "true" : "false") << "," << std::endl;
				//printf ("%s,\n", lua_toboolean (state, -1) ? "true" : "false");
			}
			else if (lua_isstring (state, -1))
			{
				file << "\"" << lua_tolstring (state, -1, NULL) << "\"," << std::endl;
				//printf ("\"%s\",\n", lua_tolstring (state, -1, NULL));
			}
			else if (lua_istable (state, -1))
			{
				file << std::endl << tab << "{" << std::endl;
				//printf ("\n%s{\n", tab);
				RecurseTable (file, state, indentCount + 1);
				file << tab << "}," << std::endl;
				//printf ("%s},\n", tab);
			}
			lua_pop (state, 1);
		}

		delete [] tab;
	}


	int AtPanic (lua_State * state)
	{
		const char * out = lua_tostring (state, -1);
		printf ("LUA ERROR %s", out);
		assert (false);
		return 0;
	}

	LuaState::LuaState (TextRef filename)
		: filename (filename),
		state (NULL)
	{
		state = luaL_newstate ();
		assert (state != NULL);

		lua_atpanic (state, AtPanic);

		int ret = luaL_dofile (state, filename.Raw ());
		assert (ret == 0);
	}

	LuaState::~LuaState ()
	{
		assert (state != NULL);

		lua_close (state);
		state = NULL;
	}

	void LuaState::Save ()
	{
		assert (false);
	}

	void LuaState::SaveAs (TextRef filename)
	{
		assert (false);
	}

	int LuaState::ReadInt (TextRef path) const
	{
		int out = 0;
		if (LoadPath (path, true))
		{
			out = lua_tointeger (state, -1);
		}
		lua_settop (state, 0);

		return out;
	}

	bool LuaState::ReadBool (TextRef path) const
	{
		bool out = 0;
		if (LoadPath (path, true))
		{
			out = (lua_toboolean (state, -1) != 0);
		}
		lua_settop (state, 0);

		return out;
	}

	float LuaState::ReadFloat (TextRef path) const
	{
		float out = 0;
		if (LoadPath (path, true))
		{
			out = (float) lua_tonumber (state, -1);
		}
		lua_settop (state, 0);

		return out;
	}

	const char * LuaState::ReadString (TextRef path) const
	{
		const char * out = NULL;
		if (LoadPath (path, true))
		{
			out = lua_tostring (state, -1);
		}
		lua_settop (state, 0);

		return out;
	}

	Int3 LuaState::ReadInt3 (TextRef path) const
	{
		Int3 out;
		out.x = ReadInt (path + "/[1]");
		out.y = ReadInt (path + "/[2]");
		out.z = ReadInt (path + "/[3]");
		return out;
	}

	Vec3 LuaState::ReadVec3 (TextRef path) const
	{
		Vec3 out;
		out.x = ReadFloat (path + "/[1]");
		out.y = ReadFloat (path + "/[2]");
		out.z = ReadFloat (path + "/[3]");
		return out;
	}

	bool LuaState::SetInt (TextRef path, int value)
	{
		LoadPath (path, false);

		lua_pushnumber (state, value);
		lua_settable (state, -3);
		lua_settop (state, 0);

		return true;
	}

	bool LuaState::SetBool (TextRef path, bool value)
	{
		LoadPath (path, false);

		lua_pushboolean (state, value);
		lua_settable (state, -3);
		lua_settop (state, 0);

		return true;
	}

	bool LuaState::SetFloat (TextRef path, float value)
	{
		assert (false);
		return false;
	}

	bool LuaState::SetString (TextRef path, TextRef value)
	{
		LoadPath (path, false);

		lua_pushstring (state, value.Raw ());
		lua_settable (state, -3);
		lua_settop (state, 0);

		return true;
	}

	bool LuaState::SetInt3 (TextRef path, Int3 value)
	{
		assert (false);
		return false;
	}

	bool LuaState::SetVec3 (TextRef path, Vec3 value)
	{
		assert (false);
		return false;
	}

	bool LuaState::SaveText_GlobalTable (TextRef filename, TextRef globalTable, WARNINGLEVELS warningLevel)
	{
		std::ofstream file;
		file.open (filename.Raw (), 0);
		
		lua_getglobal (state, globalTable.Raw ());

		switch (warningLevel)
		{
		case WL_SAVEDONCEONLY: file << "-- auto generated file. safe to hand edit" << std::endl; break;
		case WL_SAVEDREPEATEDLY: file << "-- repeatedly generated file. not safe to hand edit" << std::endl; break;
		default: break;
		}


		file << globalTable.Raw () << " =" << std::endl;
		file << "{" << std::endl;

		RecurseTable (file, state, 1);

		file << "}" << std::endl;

		file.close ();

		lua_settop (state, 0);

		return true;
	}

	bool LuaState::LoadPath (TextRef path, bool getFinalValue) const
	{
		assert (lua_gettop (state) == 0); // stack not empty

		int tableIdx = LUA_GLOBALSINDEX;

		const char * folder = path.Raw ();
		while (true)
		{
			int debugStackSize = lua_gettop (state);

			const char * delim = strchr (folder, '/');
			if (delim == NULL)
			{
				// is final 
				if (folder [0] == '[')
				{
					const char * end = strchr (folder + 1, ']');
					assert (end != NULL);
					strncpy (buffer, folder + 1, end - folder - 1);
					buffer [end - folder - 1] = '\0';
					int idx = atoi (buffer);
					lua_rawgeti (state, -1, idx);
				}
				else
				{
					lua_pushstring (state, folder);
					if (getFinalValue)
					{
						lua_gettable (state, tableIdx);
					}
					//lua_getfield (state, tableIdx, folder);
				}
				break;
			}
			else
			{
				if (folder [0] == '[')
				{
					const char * end = strchr (folder + 1, ']');
					assert (end != NULL);
					strncpy (buffer, folder + 1, end - folder - 1);
					buffer [end - folder - 1] = '\0';
					int idx = atoi (buffer);
					lua_rawgeti (state, -1, idx);
				}
				else
				{
					strncpy (buffer, folder, delim - folder);
					buffer [delim - folder] = '\0';

					lua_pushstring (state, buffer);
					lua_gettable (state, tableIdx);
					//lua_getfield (state, tableIdx, buffer);
				}

				folder = delim + 1;
				tableIdx = -2;
				//tableIdx = -1;
			}
		}

		int debugStackSize = lua_gettop (state);

		return true;
	}
}
