#include "stdafx.h"

#include "Mem_Context.h"
#include "Mem_Record.h"
#include "Mem_Sentinels.h"

namespace Mem
{
	Context::Context (TextRef title, bool isThreadSafe)
		: title (title),
		lock (NULL),
		allocCount (0),
		allocSize (0),
		records (NULL),
		trackingCurrent (0)
	{
		if (isThreadSafe)
		{
			lock = SDL_CreateMutex ();
		}

		records = aah_new Record [recordsMax];

		memset (tracking, 0, sizeof (tracking));
	}

	Context::~Context ()
	{
		delete [] records;
		records = NULL;

		assert (allocCount == 0);
		assert (allocSize == 0);

		if (lock != NULL)
		{
			SDL_DestroyMutex (lock);
		}
	}

	void Context::AddRecord (SentinelPre * sentinel)
	{
		assert (sentinel != NULL);

		for (int idx = 0; ; ++idx)
		{
			assert (idx < recordsMax);
			if (records [idx].sentinel == NULL)
			{
				records [idx].Set (sentinel);

				for (int trackIdx = 0; trackIdx < trackingCurrent; ++trackIdx)
				{
					assert (tracking [trackIdx] != records [idx].adr);
				}

				break;
			}
		}
		
		++allocCount;
		allocSize += sentinel->size;
	}

	void Context::RemoveRecord (SentinelPre * sentinel)
	{
		assert (sentinel != NULL);

		int idx = 0;
		for (; records [idx].sentinel != sentinel; ++idx)
		{
			assert (idx < recordsMax);
		}

		records [idx].Reset ();

		--allocCount;
		allocSize -= sentinel->size;
	}

	void Context::AssertOnAlloc (void * adr)
	{
		assert (adr != NULL);
		assert (trackingCurrent < trackingMax);

		tracking [trackingCurrent] = adr;
		++trackingCurrent;
	}
}
