#include "stdafx.h"

#include "Profile_ProfileThread.h"
#include "Profile_Record.h"

namespace Profile
{
	ProfileThread::ProfileThread (const char * title)
		: title (strdup (title)),
		recordsMax (500),
		recordsUsedCount (0),
		historyIdx (0)
	{
		records = aah_new Record [recordsMax];
		LARGE_INTEGER frequency;
		QueryPerformanceFrequency (&frequency);
		queryPerformanceFrequency = (double) frequency.QuadPart;
	}

	ProfileThread::~ProfileThread ()
	{
		delete [] records;
		delete [] title;
	}

	void ProfileThread::Update ()
	{
		for (int idx = 0; idx < recordsUsedCount; ++idx)
		{
			Record & record = records [idx];
			record.history [historyIdx] = record.mostRecent;
			record.mostRecent = 0.0f;
		}
		historyIdx = (historyIdx + 1) % Record::historyLength;
	}

	int ProfileThread::CreateRecordId (const char * name, int r, int g, int b)
	{
		const int recordId = recordsUsedCount;
		++recordsUsedCount;
		records [recordId].Reset (name, r, g, b);
		return recordId;
	}

	Record * ProfileThread::GetRecord (int recordId)
	{
		if (recordId >= 0 && recordId < recordsUsedCount)
		{
			return &records [recordId];
		}
		return NULL;
	}

	void ProfileThread::RecordScope (int recordId)
	{
		assert (recordId >= 0 && recordId < recordsUsedCount);

		Record & record = records [recordId];
		assert (record.start.QuadPart == -1);

		QueryPerformanceCounter (&record.start);
	}

	void ProfileThread::StopScope (int recordId)
	{
		assert (recordId >= 0 && recordId < recordsUsedCount);

		Record & record = records [recordId];
		assert (record.start.QuadPart != -1);

		LARGE_INTEGER time;
		QueryPerformanceCounter (&time);
		time.QuadPart -= record.start.QuadPart;
		record.start.QuadPart = -1;
		++record.calledCount;

		record.mostRecent = (float) ((double) time.QuadPart / queryPerformanceFrequency);
		record.cumulative += record.mostRecent;
		if (record.mostRecent > record.longest)
		{
			record.longest = record.mostRecent;
		}
	}

	void ProfileThread::ReportToScreen ()
	{
		printf ("\n");
		printf ("  THREAD -- %s\n\n", title);
		printf ("  %-20s %5s %7s %7s\n", 
			"title",
			"count",
			"avg",
			"worst");

		for (int idx = 0; idx < recordsUsedCount; ++idx)
		{
			Record & record = records [idx];
			printf ("  %-20s %5d % 2.4f % 2.4f\n", 
				record.name,
				record.calledCount,
				record.cumulative / record.calledCount,
				record.longest);
		}
	}
}
