#pragma once

#include "Thread_SafeList.h"
#include "Thread_SafeQueue.h"
#include "Thread_Job.h"

namespace Thread
{
	class JobMngr
	{
	public:
		JobMngr (const char * debugName, bool isDebugging);
		~JobMngr ();

		int GetJobCount () const;

		void DeleteJob (Job * job);

		// main thread only calls this - calls JobOnCompleted on completed jobs
		void MainThread_Update (); 

		// block until current job complete, and then remove everything else from the queue and return
		void FinishCurrentJobAndDeleteAll ();

	private:

		friend Job;

		enum STATES
		{
			S_NONE,
			S_RUNNING,
			S_QUITTING,
			S_QUIT,
		};

		void AddJob (Job * job, JOBPRIORITIES priority);
		static int RunThreadStatic (void * data);
		int RunThread ();

		const char * debugName;
		bool isDebugging;

		STATES state;

		SDL_Thread * thread;

		typedef SafeList <Job *> JobList;
		typedef SafeQueue <Job *> JobQueue;

		JobList jobsPending;
		JobQueue jobsCompleted;
		JobList jobsAwaitingDelete;

		int jobsPendingAndCompletedTotal;
	};
}
