#pragma once

#include <list>

namespace Thread
{
	template <typename T>
	class SafeList : private std::list <T>
	{
	public:

		SafeList ();
		~SafeList ();

		void Lock () const;
		void Unlock () const;
		int Size () const;

		void PushFront (T t);
		void PushBack (T t);
		T PopFront ();
		void Remove (T t);

		template <typename F> // put this inline cos I don't know nomenclature for definition in "Thread_SafeList.cpp" !
		F & FindFirst (F & f) const
		{
			Lock ();
			std::list <T>::const_iterator currentIt = begin ();
			std::list <T>::const_iterator endIt = end ();
			while (currentIt != endIt)
			{
				if (f (*currentIt))
				{
					Unlock ();
					return f;
				}
				++currentIt;
			}
			Unlock ();
			return f;
		}

	private:

		mutable SDL_mutex * lock;
	};
}

#include "Thread_SafeList.impl"
