#include "stdafx.h"

//#include "../Gfx/Gfx_Font.h"

#include "Utils_Debug.h"

namespace Utils
{

	DebugStream::~DebugStream ()
	{
	}

	void DebugStream::Flush ()
	{
	}

	void DebugStream_Console::WriteLine (const char * str)
	{
		std::cout << str << std::endl;
	}
	
	struct DebugGlobals
	{
		DebugGlobals ()
		{
			memset (stream, 0, sizeof (stream));
		}

		static const int channelsMax = 10;
		DebugStream * stream [channelsMax];
	};

	DebugGlobals g_debug;

	void Debug_Init (int channel, DebugStream * debugStream)
	{
		assert (debugStream != NULL);

		g_debug.stream [channel] = debugStream;
	}

	void Debug_Exit ()
	{
		for (int idx = 0; idx < g_debug.channelsMax; ++idx)
		{
			delete g_debug.stream [idx];
			g_debug.stream [idx] = NULL;
		}
	}

	void Debug_Clear (int channel)
	{
		g_debug.stream [channel]->Clear ();
	}

	void Debug_WriteLine (int channel, const char * fmt, ...)
	{
		assert (g_debug.stream [channel] != NULL);
		assert (fmt != NULL);

		static char buffer [2048];

		va_list argptr;
		va_start (argptr, fmt);
		vsprintf (buffer, fmt, argptr);
		va_end (argptr);

		g_debug.stream [channel]->WriteLine (buffer);
	}

	void Debug_Flush (int channel)
	{
		assert (g_debug.stream [channel] != NULL);

		g_debug.stream [channel]->Flush ();
	}

/*
	struct DebugGlobals
	{
		DebugGlobals ()
			: useConsole (true),
			file (NULL),
			stringCount (0),
			briefStringOffsetY (0),
			briefStringCount (0)
		{
		}

		bool useConsole;

		FILE * file;

		static const int stringMax = 100;
		static const int stringLengthMax = 100;
		
		int stringCount;
		int xy [stringMax][2];
		char strings [stringMax][stringLengthMax];

		int briefStringOffsetY;
		int briefStringCount;
		int briefAges [stringMax];
		char briefStrings [stringMax][stringLengthMax];

		int nextX;
		int nextY;
	};

	DebugGlobals g_debug;

	void Debug_Init (bool useConsole)
	{
		g_debug.useConsole = useConsole;
		//g_debug.file = fopen ("log.txt", "wt");
	}

	FILE * Debug_GetLog ()
	{
		//return g_debug.file;
		return NULL;
	}

	void Debug_SetDrawCursor (int x, int y)
	{
		assert (x >= 0);
		assert (y >= 0);

		g_debug.nextX = x;
		g_debug.nextY = y;
	}

	void Debug_DrawNewLine ()
	{
		g_debug.nextX = -1;
		--g_debug.nextY;
	}

	void Debug_DrawInt (int in)
	{
		g_debug.xy [g_debug.stringCount][0] = g_debug.nextX;
		g_debug.xy [g_debug.stringCount][1] = g_debug.nextY;
		sprintf (&g_debug.strings [g_debug.stringCount][0], "%d", in);
		++g_debug.stringCount;
		g_debug.nextX = -1;
		g_debug.nextY = 0;
	}

	void Debug_WriteLine (0, const char * string)
	{
		assert (string != NULL);

		g_debug.xy [g_debug.stringCount][0] = g_debug.nextX;
		g_debug.xy [g_debug.stringCount][1] = g_debug.nextY;
		strncpy (&g_debug.strings [g_debug.stringCount][0], string, g_debug.stringLengthMax - 1);
		++g_debug.stringCount;
		g_debug.nextX = -1;
		g_debug.nextY = 0;
	}

	void Debug_WriteLine (0, const char * fmt, ... )
	{
		static char buffer [2048];

		va_list argptr;
		va_start (argptr, fmt);
		vsprintf (buffer, fmt, argptr);
		va_end (argptr);

		if (g_debug.useConsole)
		{
			std::cout << buffer << std::endl;
		}
		else
		{
			if (g_debug.briefStringCount < g_debug.stringMax - 1)
			{
				g_debug.briefAges [g_debug.briefStringCount] = 60 * 10;
				strncpy (&g_debug.briefStrings [g_debug.briefStringCount][0], buffer, g_debug.stringLengthMax - 1);	
				++g_debug.briefStringCount;
			}
		}
	}

	void Debug_RenderText (Gfx::Font * font)
	{
		// only uncomment these when not wanting to render any text
		//g_debug.briefStringCount = 0;
		//g_debug.stringCount = 0;
		//return;

		int x = 0;
		int y = 0;
		int currentX = 0;
		int lineHeight = 20;

		// render fixed messages
		for (int idx = 0; idx < g_debug.stringCount; ++idx)
		{
			if (g_debug.xy [idx][0] != -1)
			{
				x = currentX = g_debug.xy [idx][0];
				y = g_debug.xy [idx][1];
			}
			else if (g_debug.xy [idx][1] < 0)
			{
				currentX = x;
				y += g_debug.xy [idx][1] * -lineHeight;
			}

			int width = font->DrawString (g_debug.strings [idx], currentX, y); 
			currentX += width + 15;
		}

		g_debug.stringCount = 0;

		// strip dead brief messages
		for (int idx = 0; idx < g_debug.briefStringCount; )
		{
			--g_debug.briefAges [idx];
			if (g_debug.briefAges [idx] == 0)
			{
				for (int copyIdx = idx; copyIdx < g_debug.briefStringCount - 1; ++copyIdx)
				{
					g_debug.briefAges [copyIdx] = g_debug.briefAges [copyIdx + 1];
					memcpy (&g_debug.briefStrings [copyIdx][0], &g_debug.briefStrings [copyIdx + 1][0], g_debug.stringLengthMax);
				}
				--g_debug.briefStringCount;
				g_debug.briefStringOffsetY += lineHeight;
			}
			else
			{
				++idx;
			}
		}

		// render brief messages
		x = 20;
		y = 300 + g_debug.briefStringOffsetY;
		for (int idx = 0; idx < g_debug.briefStringCount; ++idx)
		{
			font->DrawString (g_debug.briefStrings [idx], x, y); 
			y += lineHeight;
		}

		if (g_debug.briefStringOffsetY > 0)
		{
			g_debug.briefStringOffsetY /= 2;
		}
	}

	void Debug_WriteLine (0, const char * fmt, ... )
	{
		static char buffer [2048];

		va_list argptr;
		va_start (argptr, fmt);
		vsprintf (buffer, fmt, argptr);
		va_end (argptr);

		std::cout << buffer << std::endl;
	}
	*/
}
