#include "stdafx.h"

#include <vector>
#include <sys/types.h>

namespace Utils
{
	Text File_NeatenFolder (TextRef folder)
	{
		Text result = folder.ReplaceStr ("\\", "/");
		if (result.Length () == 0 || result.Raw () [result.Length () - 1] != '/')
		{
			result += "/";
		}
		return result;
	}

	bool File_Exists (TextRef filename)
	{
		WIN32_FIND_DATA find;
		HANDLE handle = FindFirstFile (filename.Raw (), &find);
		if (handle != INVALID_HANDLE_VALUE)
		{
			FindClose (handle);
		}
		return (handle != INVALID_HANDLE_VALUE);
	}

	bool File_CreateDirTree (TextRef path)
	{
		bool isOk = true;
		int currentIdx = 0;

		while (currentIdx < path.Length ())
		{
			if (path.Raw () [currentIdx] == '/')
			{
				Text extract = path.SubStr (0, currentIdx);

				if (extract != "..")
				{
					LPSECURITY_ATTRIBUTES attr = NULL;
					BOOL success = CreateDirectory (extract.Raw (), attr);
					if (!success)
					{
						if (GetLastError () != ERROR_ALREADY_EXISTS)
						{
							isOk = false;
							break;
						}
					}
				}
			}

			++currentIdx;
		}

		return isOk;
	}

	bool File_CopyFile (TextRef destFile, TextRef srcFile, bool overwriteExisting)
	{
		int ret = CopyFile (srcFile.Raw (), destFile.Raw (), !overwriteExisting);
		return (ret != 0);
	}

	bool File_RecurseCopy (TextRef destFolder, TextRef srcFolder, bool overwriteExisting)
	{
		Text search (srcFolder + "*");

		std::vector <Text> folders;
		std::vector <Text> files;

		WIN32_FIND_DATA find;
		HANDLE handle = FindFirstFile (search.Raw (), &find);

		if (handle == INVALID_HANDLE_VALUE)
		{
			return false;
		}

		do
		{
			if (find.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
			{
				if (strcmp (find.cFileName, ".") != 0 && strcmp (find.cFileName, "..") != 0)
				{
					folders.push_back (find.cFileName);
				}
			}
			else
			{
				files.push_back (find.cFileName);
			}
		} while (FindNextFile (handle, &find) != 0);

		FindClose (handle);

		// recurse folders
		for (int idx = 0; idx < folders.size (); ++idx)
		{
			bool isOk = File_RecurseCopy (destFolder + folders [idx] + "/", srcFolder + folders [idx] + "/", overwriteExisting);
			if (!isOk)
			{
				return false;
			}
		}

		// create this folder only if its a leaf of the directory tree
		if (folders.size () == 0)
		{
			File_CreateDirTree (destFolder);
		}

		// copy files
		for (int idx = 0; idx < files.size (); ++idx)
		{
			bool isOk = File_CopyFile (destFolder + files [idx], srcFolder + files [idx], overwriteExisting);
			if (!isOk)
			{
				return false;
			}
		}

		return true;
	}

	bool File_CopyFolder (TextRef destFolder, TextRef srcFolder, bool overwriteExisting)
	{
		return File_RecurseCopy (destFolder, srcFolder, overwriteExisting);
	}

	void File_ListFolders (TextRef folderToScan, std::vector <Text> * results)
	{
		assert (results != NULL);

		Text search = folderToScan + "*.*";

		WIN32_FIND_DATA find;
		HANDLE handle = FindFirstFile (search.Raw (), &find);

		if (handle != INVALID_HANDLE_VALUE)
		{
			do
			{
				if (find.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
				{
					if (strcmp (find.cFileName, ".") != 0 && strcmp (find.cFileName, "..") != 0)
					{
						results->push_back (find.cFileName);
					}
				}
			} while (FindNextFile (handle, &find) != 0);

			FindClose (handle);
		}
	}
}
