#include "stdafx.h"

#include "Utils_Math.h"

namespace Utils
{
	int FloorDivideInt (int value, int divideBy)
	{
		int result = ((value >= 0) ? value / divideBy : (value + 1) / divideBy - 1);
		return result;
	}

	int LengthSqr (Int3 int3)
	{
		return (int3.x * int3.x + int3.y * int3.y + int3.z * int3.z);
	}

	float LengthSqr (Vec3 vec3)
	{
		return (vec3.x * vec3.x + vec3.y * vec3.y + vec3.z * vec3.z);
	}

	int CountBitsSet (unsigned int in)
	{
		int bitsSetCount = 0;
		for (; in; ++bitsSetCount)
		{
		  in &= in - 1; // clear the least significant bit set
		}
		return bitsSetCount;
	}

	bool CircleSquareIntersects (const Vec3 & csgCentre, float radius, const Vec3 & colCorner, const Vec3 & colSize)
	{
		const Vec3 colHalfSize (colSize.x * 0.5f, 0.0f, colSize.z * 0.5f);

		Vec3 circleDistance (
			abs (csgCentre.x - colCorner.x - colHalfSize.x),
			0.0f,
			abs (csgCentre.z - colCorner.z - colHalfSize.z));

		if (circleDistance.x > (colHalfSize.x + radius)) { return false; }
		if (circleDistance.z > (colHalfSize.z + radius)) { return false; }

		if (circleDistance.x <= (colHalfSize.x)) { return true; } 
		if (circleDistance.z <= (colHalfSize.z)) { return true; }

		float cornerDistance_sq = 
			(circleDistance.x - colHalfSize.x) * (circleDistance.x - colHalfSize.x) +
			(circleDistance.z - colHalfSize.z) * (circleDistance.z - colHalfSize.z);

		return (cornerDistance_sq <= (radius * radius));
	}

	bool CircleSquareIntersectsInt (const Int3 & csgCentre, int radius, const Int3 & colCorner, const Int3 & colSize)
	{
		const int radiusSqr = radius * radius;

		Int3 farCorner (colCorner + colSize);

		const bool overlapsX = (colCorner.x <= csgCentre.x && farCorner.x >= csgCentre.x);
		const bool overlapsZ = (colCorner.z <= csgCentre.z && farCorner.z >= csgCentre.z);
		if (overlapsX && overlapsZ)
		{
			return true;
		}

		Int3 nearestEdge (abs (colCorner.x - csgCentre.x), 0, abs (colCorner.z - csgCentre.z));

		if (abs (farCorner.x - csgCentre.x) < nearestEdge.x)
		{
			nearestEdge.x = abs (farCorner.x - csgCentre.x);
		}

		const int distSqrX = nearestEdge.x * nearestEdge.x;
		if (!overlapsX && distSqrX > radiusSqr)
		{
			return false;
		}

		if (abs (farCorner.z - csgCentre.z) < nearestEdge.z)
		{
			nearestEdge.z = abs (farCorner.z - csgCentre.z);
		}

		const int distSqrZ = nearestEdge.z * nearestEdge.z;
		if (!overlapsZ && distSqrZ > radiusSqr)
		{
			return false;
		}
		
		if (!overlapsX && !overlapsZ && distSqrX + distSqrZ > radiusSqr)
		{
			return false;
		}

		return true;
	}
}
