#pragma once

#include <string>

#define TEXT_DEBUG_SAFE_TYPE
//#include "RakNet_PC-4.022/Source/RakString.h"

#if defined (TEXT_DEBUG_SAFE_TYPE)

namespace RakNet
{
	namespace RakString
	{
		extern int ToInteger (const char * str);
	}
}

#endif

namespace Utils
{

#if defined (TEXT_DEBUG_SAFE_TYPE)

	class TextString : private std::string

#else

	class TextString : private RakNet::RakString

#endif 

	{
	public:

		enum VALIDATETYPES
		{
			VT_ALPHANUMERIC,	// upper and lower case and numerals only
			VT_FILENAME,
			VT_FILEPATH,
			VT_IP,
			VT_NUMBER,
			VT_TEXT,
		};

		static TextString NonVariadic (const char * str);
		static TextString Format (const char * fmt, ...);
		static TextString Format (const char * fmt, va_list args);

		TextString ();
		TextString (const char * fmt);

		~TextString ();

		operator const char * () const;

		void Clear ();
		void Set (const char * fmt, ...);
		void Set (const TextString &);
		void RawSet (const char * str);

		// Comparison
		bool operator < (const TextString & rhs) const;
		bool operator <= (const TextString & rhs) const;
		bool operator > (const TextString & rhs) const;
		bool operator >= (const TextString & rhs) const;

		void Truncate (int length);
		int Length () const;
		TextString SubStr (int pos, int len) const;
		TextString ReplaceStr (const char * find, const char * replace) const;

		const char * Raw () const;

		TextString operator + (const char * rhs) const;
		TextString operator + (const TextString & rhs) const;
		TextString & operator += (char rhs);
		TextString & operator += (const TextString & rhs);

		bool operator == (const char * rhs) const;
		bool operator == (const TextString & rhs) const;

		bool operator != (const char * rhs) const;
		bool operator != (const TextString & rhs) const;

		static bool ValidateChar (VALIDATETYPES type, char c);
		static TextString Validate (VALIDATETYPES type, const char * in, char replaceWith);
	};
}

//
//struct Text : std::string
//{
//	Text (const char * str)
//	{
//		*this = str;
//	}
//
//	Text (const char * strA, const char * strB)
//	{
//		*this = strA;
//		*this += strB;
//	}
//
//	void Truncate (int length)
//	{
//		if (size () > (size_t) length)
//		{
//			erase (length);
//		}
//	};
//
//	size_t GetLength () const
//	{
//		return size ();
//	}
//
//	const char * Raw () const
//	{
//		return &operator [] (0);
//	}
//
//	Text SubStr (int startIdx, int length) const
//	{
//		return *(Text *) &(substr (startIdx, length));
//	}
//};
//
//typedef const Text & TextRef;
//
//Text operator + (TextRef str, const char * add)
//{
//	Text newStr (str);
//	newStr += add;
//	return newStr;
//}
//
//#else
//
//#include "RakNet_PC-4.022/Source/RakString.h"
//typedef RakNet::RakString Text;
//typedef const RakNet::RakString & TextRef;
//
//#endif
